/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

#include "lcrzo_incl.h"


/*-------------------------------------------------------------*/
typedef struct {
  lcrzo_bool priority;
  lcrzo_device device;
  lcrzo_int16 hwtype;
  lcrzo_etha eth;
  lcrzo_ipl ip;
  lcrzo_ipl netmask;
  lcrzo_uint32 mtu;
  lcrzo_bool isup;
  lcrzo_bool isanalias;
  lcrzo_bool ispointtopoint;
  lcrzo_ipl pointtopointipl;
} lcrzo_priv_conf_devices;

typedef struct {
  lcrzo_bool priority;
  lcrzo_device device;
  lcrzo_etha eth;
  lcrzo_ipl ip;
  lcrzo_bool ispermanent;
  lcrzo_uint32 timeset;
} lcrzo_priv_conf_arp;

typedef struct {
  lcrzo_bool priority;
  lcrzo_device device;
  lcrzo_ipl dest;
  lcrzo_ipl mask;
  lcrzo_ipl ipsource;
  lcrzo_ipl gateway;
  lcrzo_uint32 metric;
  lcrzo_bool isup;
} lcrzo_priv_conf_routes;

typedef struct {
  lcrzo_uint32 number;
  lcrzo_device name;
} lcrzo_priv_conf_devicenumbername;

typedef struct {
  lcrzo_device device;
  lcrzo_etha eth;
} lcrzo_priv_conf_deviceeth;

#define LCRZO_PRIV_CONF_VALIDIPINFO_MAXGW 10
typedef struct {
  lcrzo_uint8 keyid[2];
  lcrzo_device device;
  lcrzo_etha eth;
  lcrzo_ipl ip;
  lcrzo_ipl mask;
  lcrzo_ipl gw[LCRZO_PRIV_CONF_VALIDIPINFO_MAXGW];
} lcrzo_priv_conf_validdhcpinfo;

typedef struct {
  lcrzo_bool devicenumberset;
  lcrzo_uint32 devicenumber;
  lcrzo_bool deviceset;
  lcrzo_device device;
  lcrzo_bool hwtypeset;
  lcrzo_int16 hwtype;
  lcrzo_bool ethset;
  lcrzo_etha eth;
  lcrzo_bool ipset;
  lcrzo_ipl ip;
  lcrzo_bool netmaskset;
  lcrzo_ipl netmask;
  lcrzo_bool mtuset;
  lcrzo_uint32 mtu;
  lcrzo_bool isupset;
  lcrzo_bool isup;
  lcrzo_bool isanaliasset;
  lcrzo_bool isanalias;
  lcrzo_bool ispointtopointset;
  lcrzo_bool ispointtopoint;
  lcrzo_bool pointtopointiplset;
  lcrzo_ipl pointtopointipl;
  lcrzo_int32 conflictpower;
} lcrzo_priv_conf_incomplete;

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_etha_init_cardnumberdevice(lcrzo_constdevice device,
                                               lcrzo_int32 devicenumber,
                                               lcrzo_int32 cardnumber,
                                               lcrzo_bool isanalias,
                                               lcrzo_int16 * phwtype,
                                               lcrzo_etha etha);
int lcrzo_priv_conf_localdevice_eventuallyinit(void);
int lcrzo_priv_conf_devicenumbername_item_add(lcrzo_uint32 devicenumber,
                                              lcrzo_device devicename);
int lcrzo_priv_conf_init_NTregistry(lcrzo_bool setdevices,
                                    lcrzo_bool setroutes,
                                    lcrzo_bool searchipl,
                                    lcrzo_ipl iplwanted,
                                    lcrzo_device founddevice);
int lcrzo_priv_conf_init_2kregistry(lcrzo_bool setdevices,
                                    lcrzo_bool setroutes,
                                    lcrzo_bool searchipl,
                                    lcrzo_ipl iplwanted,
                                    lcrzo_device founddevice);
int lcrzo_priv_conf_init_W9xregistry(lcrzo_bool setdevices,
                                     lcrzo_bool setroutes,
                                     lcrzo_bool searchipl,
                                     lcrzo_ipl iplwanted,
                                     lcrzo_device founddevice);

/*-------------------------------------------------------------*/
#define LCRZO_PRIV_CONF_WINRAS_MAXCON 10
#include "sysdep/priv_conf_incomplete.c"
#include "sysdep/priv_conf_winras.c"
#include "sysdep/priv_conf_device_init_da.c"
#include "sysdep/priv_conf_etha_init_cnd.c"
#include "sysdep/priv_conf_localdevice.c"
#include "sysdep/priv_conf_localeth.c"
#include "sysdep/priv_conf_localip.c"
#include "sysdep/priv_conf_validdhcpinfo.c"
#include "sysdep/priv_conf_device_init_ipl.c"
#include "sysdep/priv_conf_devicenumbername.c"
#include "sysdep/priv_hwtype_init_hwtype.c"
#include "sysdep/priv_conf_init_dev.c"
#include "sysdep/priv_conf_init_arpcache.c"
#include "sysdep/priv_conf_init_rarpcache.c"
#include "sysdep/priv_conf_init_routes.c"

/*-------------------------------------------------------------*/
int lcrzo_priv_confvars_init(lcrzo_priv_confvars * ptr)
{
  /* initialize the lists */
  lcrzo_er(lcrzo_list_init(&(ptr->list_system_devices),
                           sizeof(lcrzo_priv_conf_devices), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_system_arpperm),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_system_arpcache),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_system_routes),
                           sizeof(lcrzo_priv_conf_routes), NULL));

  lcrzo_er(lcrzo_list_init(&(ptr->list_systemdel_devices),
                           sizeof(lcrzo_priv_conf_devices), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_systemdel_arpperm),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_systemdel_arpcache),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_systemdel_routes),
                           sizeof(lcrzo_priv_conf_routes), NULL));

  lcrzo_er(lcrzo_list_init(&(ptr->list_user_devices),
                           sizeof(lcrzo_priv_conf_devices), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_user_arpperm),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_user_arpcache),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_user_routes),
                           sizeof(lcrzo_priv_conf_routes), NULL));

  lcrzo_er(lcrzo_list_init(&(ptr->list_sys_devices),
                           sizeof(lcrzo_priv_conf_devices), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_sys_arpperm),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_sys_arpcache),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_sys_routes),
                           sizeof(lcrzo_priv_conf_routes), NULL));

  lcrzo_er(lcrzo_list_init(&(ptr->list_conf_devices),
                           sizeof(lcrzo_priv_conf_devices), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_conf_arpperm),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_conf_arpcache),
                           sizeof(lcrzo_priv_conf_arp), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_conf_routes),
                           sizeof(lcrzo_priv_conf_routes), NULL));

  lcrzo_er(lcrzo_list_init(&(ptr->list_devicenumbername),
                           sizeof(lcrzo_priv_conf_devicenumbername), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_localdevice),
                           sizeof(lcrzo_priv_conf_deviceeth), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_localeth),
                           sizeof(lcrzo_priv_conf_deviceeth), NULL));
  lcrzo_er(lcrzo_iplist_init(&(ptr->list_localip)));
  lcrzo_er(lcrzo_list_init(&(ptr->list_validdhcpinfo),
                           sizeof(lcrzo_priv_conf_validdhcpinfo), NULL));
  lcrzo_er(lcrzo_list_init(&(ptr->list_incomplete),
                           sizeof(lcrzo_priv_conf_incomplete), NULL));

  ptr->sysrenew_allow = LCRZO_TRUE;
  ptr->sysrenew_allow_devices = LCRZO_TRUE;
  ptr->sysrenew_allow_arpperm = LCRZO_TRUE;
  ptr->sysrenew_allow_arpcache = LCRZO_TRUE;
  ptr->sysrenew_allow_routes = LCRZO_TRUE;

  ptr->sysrenew_now = LCRZO_TRUE;
  ptr->sysrenew_now_devices = LCRZO_FALSE;
  ptr->sysrenew_now_arpperm = LCRZO_FALSE;
  ptr->sysrenew_now_arpcache = LCRZO_FALSE;
  ptr->sysrenew_now_routes = LCRZO_FALSE;

  ptr->compute_sys_devices = LCRZO_FALSE;
  ptr->compute_sys_arpperm = LCRZO_FALSE;
  ptr->compute_sys_arpcache = LCRZO_FALSE;
  ptr->compute_sys_routes = LCRZO_FALSE;
  ptr->compute_conf_devices = LCRZO_FALSE;
  ptr->compute_conf_arpperm = LCRZO_FALSE;
  ptr->compute_conf_arpcache = LCRZO_FALSE;
  ptr->compute_conf_routes = LCRZO_FALSE;

  ptr->cacheentriestimeout = 120;

  ptr->permentriesperiod = 60;
  ptr->permentrieslastupdatetime = 0;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_confvars_close(lcrzo_priv_confvars * ptr)
{
  lcrzo_er(lcrzo_list_close(&(ptr->list_system_devices)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_system_arpperm)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_system_arpcache)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_system_routes)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_systemdel_devices)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_systemdel_arpperm)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_systemdel_arpcache)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_systemdel_routes)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_sys_devices)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_sys_arpperm)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_sys_arpcache)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_sys_routes)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_user_devices)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_user_arpperm)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_user_arpcache)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_user_routes)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_conf_devices)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_conf_arpperm)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_conf_arpcache)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_conf_routes)));

  lcrzo_er(lcrzo_list_close(&(ptr->list_devicenumbername)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_localdevice)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_localeth)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_localip)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_validdhcpinfo)));
  lcrzo_er(lcrzo_list_close(&(ptr->list_incomplete)));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_confvars_ptr_get(lcrzo_priv_confvars ** pptr)
{
  lcrzo_priv_globalvars *ptrglo;

  lcrzo_er(lcrzo_priv_globalvars_ptr_get(&ptrglo));

  if (pptr != NULL) {
    *pptr = &(ptrglo->confvars);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_list_copy(lcrzo_list * psrc,
                              lcrzo_list * pdst);
int lcrzo_priv_conf_list_copy(lcrzo_list * psrc,
                              lcrzo_list * pdst)
{
  lcrzo_int32 count, i;
  union {
    lcrzo_priv_conf_devices cd;
    lcrzo_priv_conf_arp ca;
    lcrzo_priv_conf_routes cr;
    lcrzo_priv_conf_devicenumbername dnn;
    lcrzo_priv_conf_deviceeth de;
    lcrzo_iplist il;
    lcrzo_priv_conf_validdhcpinfo vdi;
  } item,
   *pitem;

  pitem = &item;

  /* copy all items from src */
  lcrzo_er(lcrzo_list_count(psrc, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(psrc, i, pitem));
    lcrzo_er(lcrzo_list_add_last(pdst, pitem));
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_confvars_copy(lcrzo_priv_confvars * srcptr,
                             lcrzo_priv_confvars * dstptr)
{
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_system_devices, &dstptr->list_system_devices));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_system_arpperm, &dstptr->list_system_arpperm));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_system_arpcache, &dstptr->list_system_arpcache));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_system_routes, &dstptr->list_system_routes));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_systemdel_devices,
            &dstptr->list_systemdel_devices));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_systemdel_arpperm,
            &dstptr->list_systemdel_arpperm));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_systemdel_arpcache,
            &dstptr->list_systemdel_arpcache));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_systemdel_routes, &dstptr->list_systemdel_routes));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_user_devices, &dstptr->list_user_devices));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_user_arpperm, &dstptr->list_user_arpperm));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_user_arpcache, &dstptr->list_user_arpcache));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_user_routes, &dstptr->list_user_routes));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_sys_devices, &dstptr->list_sys_devices));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_sys_arpperm, &dstptr->list_sys_arpperm));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_sys_arpcache, &dstptr->list_sys_arpcache));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_sys_routes, &dstptr->list_sys_routes));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_conf_devices, &dstptr->list_conf_devices));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_conf_arpperm, &dstptr->list_conf_arpperm));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_conf_arpcache, &dstptr->list_conf_arpcache));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_conf_routes, &dstptr->list_conf_routes));

  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_devicenumbername, &dstptr->list_devicenumbername));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_localdevice, &dstptr->list_localdevice));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_localeth, &dstptr->list_localeth));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_localip, &dstptr->list_localip));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_localip, &dstptr->list_validdhcpinfo));
  lcrzo_er(lcrzo_priv_conf_list_copy
           (&srcptr->list_incomplete, &dstptr->list_incomplete));

  dstptr->sysrenew_allow = srcptr->sysrenew_allow;
  dstptr->sysrenew_allow_devices = srcptr->sysrenew_allow_devices;
  dstptr->sysrenew_allow_arpperm = srcptr->sysrenew_allow_arpperm;
  dstptr->sysrenew_allow_arpcache = srcptr->sysrenew_allow_arpcache;
  dstptr->sysrenew_allow_routes = srcptr->sysrenew_allow_routes;
  dstptr->sysrenew_now = srcptr->sysrenew_now;
  dstptr->sysrenew_now_devices = srcptr->sysrenew_now_devices;
  dstptr->sysrenew_now_arpperm = srcptr->sysrenew_now_arpperm;
  dstptr->sysrenew_now_arpcache = srcptr->sysrenew_now_arpcache;
  dstptr->sysrenew_now_routes = srcptr->sysrenew_now_routes;

  dstptr->compute_sys_devices = srcptr->compute_sys_devices;
  dstptr->compute_sys_arpperm = srcptr->compute_sys_arpperm;
  dstptr->compute_sys_arpcache = srcptr->compute_sys_arpcache;
  dstptr->compute_sys_routes = srcptr->compute_sys_routes;
  dstptr->compute_conf_devices = srcptr->compute_conf_devices;
  dstptr->compute_conf_arpperm = srcptr->compute_conf_arpperm;
  dstptr->compute_conf_arpcache = srcptr->compute_conf_arpcache;
  dstptr->compute_conf_routes = srcptr->compute_conf_routes;

  dstptr->cacheentriestimeout = srcptr->cacheentriestimeout;
  dstptr->permentriesperiod = srcptr->permentriesperiod;
  dstptr->permentrieslastupdatetime = srcptr->permentrieslastupdatetime;

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_priv_conf_devices_equal(lcrzo_constptr pelem,
                                  lcrzo_ptr pinfos,
                                  lcrzo_bool * pyes);
int lcrzo_priv_conf_devices_equal(lcrzo_constptr pelem,
                                  lcrzo_ptr pinfos,
                                  lcrzo_bool * pyes)
{
  const lcrzo_priv_conf_devices de1 = *(const lcrzo_priv_conf_devices *)pelem;
  lcrzo_priv_conf_devices de2 = *(lcrzo_priv_conf_devices *) pinfos;
  lcrzo_bool equal;

  if (de1.hwtype != de2.hwtype || de1.ip != de2.ip) {
    if (pyes != NULL)
      *pyes = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzo_device_equal2(de1.device, de2.device, &equal));
  if (!equal) {
    if (pyes != NULL)
      *pyes = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzo_etha_equal2(de1.eth, de2.eth, &equal));
  if (!equal) {
    if (pyes != NULL)
      *pyes = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  if (pyes != NULL)
    *pyes = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_conf_arp_equal(lcrzo_constptr pelem,
                              lcrzo_ptr pinfos,
                              lcrzo_bool * pyes);
int lcrzo_priv_conf_arp_equal(lcrzo_constptr pelem,
                              lcrzo_ptr pinfos,
                              lcrzo_bool * pyes)
{
  const lcrzo_priv_conf_arp de1 = *(const lcrzo_priv_conf_arp *)pelem;
  lcrzo_priv_conf_arp de2 = *(lcrzo_priv_conf_arp *) pinfos;
  lcrzo_bool equal;

  if (de1.ip != de2.ip) {
    if (pyes != NULL)
      *pyes = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzo_device_equal2(de1.device, de2.device, &equal));
  if (!equal) {
    if (pyes != NULL)
      *pyes = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzo_etha_equal2(de1.eth, de2.eth, &equal));
  if (!equal) {
    if (pyes != NULL)
      *pyes = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  if (pyes != NULL)
    *pyes = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_conf_arp_equalwdev(lcrzo_constptr pelem,
                                  lcrzo_ptr pinfos,
                                  lcrzo_bool * pyes);
int lcrzo_priv_conf_arp_equalwdev(lcrzo_constptr pelem,
                                  lcrzo_ptr pinfos,
                                  lcrzo_bool * pyes)
{
  const lcrzo_priv_conf_arp de1 = *(const lcrzo_priv_conf_arp *)pelem;
  lcrzo_priv_conf_arp de2 = *(lcrzo_priv_conf_arp *) pinfos;
  lcrzo_bool equal;

  if (de1.ip != de2.ip) {
    if (pyes != NULL)
      *pyes = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzo_etha_equal2(de1.eth, de2.eth, &equal));
  if (!equal) {
    if (pyes != NULL)
      *pyes = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  if (pyes != NULL)
    *pyes = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_conf_arp_purge(lcrzo_constptr pelem,
                              lcrzo_ptr pinfos,
                              lcrzo_bool * pyes);
int lcrzo_priv_conf_arp_purge(lcrzo_constptr pelem,
                              lcrzo_ptr pinfos,
                              lcrzo_bool * pyes)
{
  const lcrzo_priv_conf_arp de = *(const lcrzo_priv_conf_arp *)pelem;
  lcrzo_uint32 purgeolderthan = *(lcrzo_uint32 *) pinfos;
  lcrzo_bool yes;

  yes = LCRZO_FALSE;
  if (de.timeset < purgeolderthan) {
    yes = LCRZO_TRUE;
  }

  if (pyes != NULL)
    *pyes = yes;
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_conf_routes_equal(lcrzo_constptr pelem,
                                 lcrzo_ptr pinfos,
                                 lcrzo_bool * pyes);
int lcrzo_priv_conf_routes_equal(lcrzo_constptr pelem,
                                 lcrzo_ptr pinfos,
                                 lcrzo_bool * pyes)
{
  const lcrzo_priv_conf_routes de1 = *(const lcrzo_priv_conf_routes *)pelem;
  lcrzo_priv_conf_routes de2 = *(lcrzo_priv_conf_routes *) pinfos;
  lcrzo_bool yes;

  yes = LCRZO_FALSE;
  if (de1.dest == de2.dest &&
      de1.mask == de2.mask && de1.gateway == de2.gateway) {
    yes = LCRZO_TRUE;
  }

  if (pyes != NULL)
    *pyes = yes;
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_conf_routes_cmp(lcrzo_constptr peleminf,
                               lcrzo_constptr pelemsup,
                               lcrzo_ptr pinfos,
                               lcrzo_int8 * pcmp);
int lcrzo_priv_conf_routes_cmp(lcrzo_constptr peleminf,
                               lcrzo_constptr pelemsup,
                               lcrzo_ptr pinfos,
                               lcrzo_int8 * pcmp)
{
  const lcrzo_priv_conf_routes de1 =
    *(const lcrzo_priv_conf_routes *)peleminf;
  const lcrzo_priv_conf_routes de2 =
    *(const lcrzo_priv_conf_routes *)pelemsup;
  lcrzo_uint32 net1, net2;
  lcrzo_int8 cmp;

  net1 = ~de1.mask;
  net2 = ~de2.mask;

  cmp = 0;
  if (net1 == net2) {
    if (de1.dest == de2.dest) {
      if (de1.metric == de2.metric)
        cmp = 0;
      else if (de1.metric < de2.metric)
        cmp = -1;
      else
        cmp = +1;
    } else
      cmp = 0;
  } else if (net1 > net2)
    cmp = +1;
  else
    cmp = -1;

  if (pcmp != NULL)
    *pcmp = cmp;
  return (LCRZO_ERR_OK);
  pinfos = pinfos;              /*just for the compiler */
}

/*-------------------------------------------------------------*/
#include "sysdep/priv_conf_system_renew.c"

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_sys_renew(lcrzo_list * plist_system,
                              lcrzo_list * plist_systemdel,
                              lcrzo_list * plist_sys,
                              int (*pfunc_equal) (lcrzo_constptr pelem,
                                                  lcrzo_ptr pinfos,
                                                  lcrzo_bool * pequal));
int lcrzo_priv_conf_sys_renew(lcrzo_list * plist_system,
                              lcrzo_list * plist_systemdel,
                              lcrzo_list * plist_sys,
                              int (*pfunc_equal) (lcrzo_constptr pelem,
                                                  lcrzo_ptr pinfos,
                                                  lcrzo_bool * pequal))
{
  lcrzo_int32 count, i;
  lcrzo_priv_conf_devices cd;
  lcrzo_priv_conf_arp ca;
  lcrzo_priv_conf_routes cr;
  lcrzo_ptr pitem;

  /* select which item to use */
  if (pfunc_equal == &lcrzo_priv_conf_devices_equal)
    pitem = &cd;
  else if (pfunc_equal == &lcrzo_priv_conf_arp_equal)
    pitem = &ca;
  else if (pfunc_equal == &lcrzo_priv_conf_routes_equal)
    pitem = &cr;
  else {
    return (lcrzo_ie("lcrzo_priv_conf_compute",
                     "mauvaise adresse pour pfunc_equal",
                     "bad address for pfunc_equal"));
  }

  /* remove items in sys */
  lcrzo_er(lcrzo_list_remove_all(plist_sys));

  /* copy all items from system */
  lcrzo_er(lcrzo_list_count(plist_system, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist_system, i, pitem));
    lcrzo_er(lcrzo_list_add_last(plist_sys, pitem));
  }

  /* remove all items from systemdel */
  lcrzo_er(lcrzo_list_count(plist_systemdel, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist_systemdel, i, pitem));
    lcrzo_er(lcrzo_list_remove_criteria_all2(plist_sys, pfunc_equal, pitem));
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_conf_renew(lcrzo_list * plist_sys,
                               lcrzo_list * plist_user,
                               lcrzo_list * plist_conf,
                               int (*pfunc_equal) (lcrzo_constptr pelem,
                                                   lcrzo_ptr pinfos,
                                                   lcrzo_bool * pequal));
int lcrzo_priv_conf_conf_renew(lcrzo_list * plist_sys,
                               lcrzo_list * plist_user,
                               lcrzo_list * plist_conf,
                               int (*pfunc_equal) (lcrzo_constptr pelem,
                                                   lcrzo_ptr pinfos,
                                                   lcrzo_bool * pequal))
{
  lcrzo_int32 count, i, pos;
  lcrzo_priv_conf_devices cd;
  lcrzo_priv_conf_arp ca;
  lcrzo_priv_conf_routes cr;
  lcrzo_ptr pitem;
  int ret;

  /* select which item to use */
  if (pfunc_equal == &lcrzo_priv_conf_devices_equal)
    pitem = &cd;
  else if (pfunc_equal == &lcrzo_priv_conf_arp_equal)
    pitem = &ca;
  else if (pfunc_equal == &lcrzo_priv_conf_routes_equal)
    pitem = &cr;
  else {
    return (lcrzo_ie("lcrzo_priv_conf_compute",
                     "mauvaise adresse pour pfunc_equal",
                     "bad address for pfunc_equal"));
  }

  /* remove items in conf */
  lcrzo_er(lcrzo_list_remove_all(plist_conf));

  /* put low priority user items */
  lcrzo_er(lcrzo_list_count(plist_user, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist_user, i, pitem));
    if (((lcrzo_priv_conf_devices *) pitem)->priority)
      continue;
    lcrzo_er(lcrzo_list_add_last(plist_conf, pitem));
  }

  /* update with sys items */
  lcrzo_er(lcrzo_list_count(plist_sys, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist_sys, i, pitem));
    /* search it in the list */
    ret = lcrzo_list_search_all2(plist_conf, pfunc_equal, pitem, &pos, NULL);
    if (ret != LCRZO_ERR_OKSEARCHNOTFOUND && ret != LCRZO_ERR_OK)
      return (ret);
    if (ret == LCRZO_ERR_OK) {
      /* we found it, so we replace it */
      lcrzo_er(lcrzo_list_replace_pos(plist_conf, pos, pitem));
    } else {
      /* we add it to the list */
      lcrzo_er(lcrzo_list_add_last(plist_conf, pitem));
    }
  }

  /* update with high priority user items */
  lcrzo_er(lcrzo_list_count(plist_user, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist_user, i, pitem));
    if (!((lcrzo_priv_conf_devices *) pitem)->priority)
      continue;
    /* search it in the list */
    ret = lcrzo_list_search_all2(plist_conf, pfunc_equal, pitem, &pos, NULL);
    if (ret != LCRZO_ERR_OKSEARCHNOTFOUND && ret != LCRZO_ERR_OK)
      return (ret);
    if (ret == LCRZO_ERR_OK) {
      /* we found it, so we replace it */
      lcrzo_er(lcrzo_list_replace_pos(plist_conf, pos, pitem));
    } else {
      /* we add it to the list */
      lcrzo_er(lcrzo_list_add_last(plist_conf, pitem));
    }
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_confvar_print(lcrzo_conststring msg,
                             lcrzo_priv_confvars * ptr);
int lcrzo_priv_confvar_print(lcrzo_conststring msg,
                             lcrzo_priv_confvars * ptr)
{
  puts(msg);
  printf("sysrenew_allow %d dev=%d arpperm=%d arpcache=%d routes=%d\n",
         ptr->sysrenew_allow, ptr->sysrenew_allow_devices,
         ptr->sysrenew_allow_arpperm, ptr->sysrenew_allow_arpcache,
         ptr->sysrenew_allow_routes);
  printf("sysrenew_now %d dev=%d arpperm=%d arpcache=%d routes=%d\n",
         ptr->sysrenew_now, ptr->sysrenew_now_devices,
         ptr->sysrenew_now_arpperm, ptr->sysrenew_now_arpcache,
         ptr->sysrenew_now_routes);
  printf("compute_sys dev=%d arpperm=%d arpcache=%d routes=%d\n",
         ptr->compute_sys_devices, ptr->compute_sys_arpperm,
         ptr->compute_sys_arpcache, ptr->compute_sys_routes);
  printf("compute_conf dev=%d arpperm=%d arpcache=%d routes=%d\n",
         ptr->compute_conf_devices, ptr->compute_conf_arpperm,
         ptr->compute_conf_arpcache, ptr->compute_conf_routes);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_eventuallyupdate(lcrzo_priv_confvars * ptr);
int lcrzo_priv_conf_eventuallyupdate(lcrzo_priv_confvars * ptr)
{
  lcrzo_time currenttime;
  lcrzo_uint32 removeolderthan, currenttimesec;
  lcrzo_bool system_renew;

  /*lcrzo_er(lcrzo_priv_confvar_print("Step 1", ptr)); */
  /* check if we have to update system entries */
  lcrzo_er(lcrzo_time_init_currenttime(&currenttime));
  lcrzo_er(lcrzo_time_get_sec(&currenttime, &currenttimesec));
  system_renew = LCRZO_FALSE;
  if (ptr->sysrenew_allow) {
    if (ptr->sysrenew_now ||
        ptr->permentriesperiod == 0 ||
        ptr->permentriesperiod + ptr->permentrieslastupdatetime <
        currenttimesec) {
      if (ptr->sysrenew_allow_devices)
        ptr->compute_sys_devices = LCRZO_TRUE;
      if (ptr->sysrenew_allow_arpperm)
        ptr->compute_sys_arpperm = LCRZO_TRUE;
      if (ptr->sysrenew_allow_arpcache)
        ptr->compute_sys_arpcache = LCRZO_TRUE;
      if (ptr->sysrenew_allow_routes)
        ptr->compute_sys_routes = LCRZO_TRUE;
      system_renew = LCRZO_TRUE;
      /* unset for next call */
      ptr->sysrenew_now = LCRZO_FALSE;
    }
    if (ptr->sysrenew_now_devices && ptr->sysrenew_allow_devices) {
      ptr->compute_sys_devices = LCRZO_TRUE;
      system_renew = LCRZO_TRUE;
      /* unset for next call */
      ptr->sysrenew_now_devices = LCRZO_FALSE;
    }
    if (ptr->sysrenew_now_arpperm && ptr->sysrenew_allow_arpperm) {
      ptr->compute_sys_arpperm = LCRZO_TRUE;
      system_renew = LCRZO_TRUE;
      /* unset for next call */
      ptr->sysrenew_now_arpperm = LCRZO_FALSE;
    }
    if (ptr->sysrenew_now_arpcache && ptr->sysrenew_allow_arpcache) {
      ptr->compute_sys_arpcache = LCRZO_TRUE;
      system_renew = LCRZO_TRUE;
      /* unset for next call */
      ptr->sysrenew_now_arpcache = LCRZO_FALSE;
    }
    if (ptr->sysrenew_now_routes && ptr->sysrenew_allow_routes) {
      ptr->compute_sys_routes = LCRZO_TRUE;
      system_renew = LCRZO_TRUE;
      /* unset for next call */
      ptr->sysrenew_now_routes = LCRZO_FALSE;
    }
  }

  /* update system entries */
  /*lcrzo_er(lcrzo_priv_confvar_print("Step 2", ptr)); */
  if (system_renew) {
    lcrzo_er(lcrzo_priv_conf_system_renew(ptr));
    ptr->permentrieslastupdatetime = currenttimesec;
  }

  /* recompute new sys entries */
  /*lcrzo_er(lcrzo_priv_confvar_print("Step 3", ptr)); */
  if (ptr->compute_sys_devices &&
      ptr->sysrenew_allow && ptr->sysrenew_allow_devices) {
    lcrzo_er(lcrzo_priv_conf_sys_renew(&ptr->list_system_devices,
                                       &ptr->list_systemdel_devices,
                                       &ptr->list_sys_devices,
                                       &lcrzo_priv_conf_devices_equal));
    ptr->compute_conf_devices = LCRZO_TRUE;
    /* unset for next call */
    ptr->compute_sys_devices = LCRZO_FALSE;
  }
  if (ptr->compute_sys_arpperm &&
      ptr->sysrenew_allow && ptr->sysrenew_allow_arpperm) {
    lcrzo_er(lcrzo_priv_conf_sys_renew(&ptr->list_system_arpperm,
                                       &ptr->list_systemdel_arpperm,
                                       &ptr->list_sys_arpperm,
                                       &lcrzo_priv_conf_arp_equal));
    ptr->compute_conf_arpperm = LCRZO_TRUE;
    /* unset for next call */
    ptr->compute_sys_arpperm = LCRZO_FALSE;
  }
  if (ptr->compute_sys_arpcache &&
      ptr->sysrenew_allow && ptr->sysrenew_allow_arpcache) {
    lcrzo_er(lcrzo_priv_conf_sys_renew(&ptr->list_system_arpcache,
                                       &ptr->list_systemdel_arpcache,
                                       &ptr->list_sys_arpcache,
                                       &lcrzo_priv_conf_arp_equal));
    ptr->compute_conf_arpcache = LCRZO_TRUE;
    /* unset for next call */
    ptr->compute_sys_arpcache = LCRZO_FALSE;
  }
  if (ptr->compute_sys_routes &&
      ptr->sysrenew_allow && ptr->sysrenew_allow_routes) {
    lcrzo_er(lcrzo_priv_conf_sys_renew(&ptr->list_system_routes,
                                       &ptr->list_systemdel_routes,
                                       &ptr->list_sys_routes,
                                       &lcrzo_priv_conf_routes_equal));
    ptr->compute_conf_routes = LCRZO_TRUE;
    /* unset for next call */
    ptr->compute_sys_routes = LCRZO_FALSE;
  }

  /* recompute new conf items */
  /*lcrzo_er(lcrzo_priv_confvar_print("Step 4", ptr)); */
  if (ptr->compute_conf_devices) {
    lcrzo_er(lcrzo_priv_conf_conf_renew(&ptr->list_sys_devices,
                                        &ptr->list_user_devices,
                                        &ptr->list_conf_devices,
                                        &lcrzo_priv_conf_devices_equal));
    /* this is done */
    ptr->compute_conf_devices = LCRZO_FALSE;
  }
  if (ptr->compute_conf_arpperm) {
    lcrzo_er(lcrzo_priv_conf_conf_renew(&ptr->list_sys_arpperm,
                                        &ptr->list_user_arpperm,
                                        &ptr->list_conf_arpperm,
                                        &lcrzo_priv_conf_arp_equal));
    /* this is done */
    ptr->compute_conf_arpperm = LCRZO_FALSE;
  }
  if (ptr->compute_conf_arpcache) {
    lcrzo_er(lcrzo_priv_conf_conf_renew(&ptr->list_sys_arpcache,
                                        &ptr->list_user_arpcache,
                                        &ptr->list_conf_arpcache,
                                        &lcrzo_priv_conf_arp_equal));
    /* this is done */
    ptr->compute_conf_arpcache = LCRZO_FALSE;
  }
  if (ptr->compute_conf_routes) {
    lcrzo_er(lcrzo_priv_conf_conf_renew(&ptr->list_sys_routes,
                                        &ptr->list_user_routes,
                                        &ptr->list_conf_routes,
                                        &lcrzo_priv_conf_routes_equal));
    /* this is done */
    ptr->compute_conf_routes = LCRZO_FALSE;
  }

  /* remove old cache entries */
  if (ptr->cacheentriestimeout) {
    removeolderthan = currenttimesec - ptr->cacheentriestimeout;
    lcrzo_er(lcrzo_list_remove_criteria_all2(&ptr->list_conf_arpcache,
                                             &lcrzo_priv_conf_arp_purge,
                                             &removeolderthan));
  }

  /*lcrzo_er(lcrzo_priv_confvar_print("Step 5", ptr)); */
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
int lcrzo_conf_fprint(LCRZOFILE * pf)
{
  lcrzo_priv_confvars *ptr;
  lcrzo_bool sysrenew_allow_old;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptr));
  sysrenew_allow_old = ptr->sysrenew_allow;
  ptr->sysrenew_allow = LCRZO_FALSE;
  fprintf((FILE *) pf, " + Devices +\n");
  lcrzo_er(lcrzo_conf_devices_fprint(pf));
  fprintf((FILE *) pf, " + Arp +\n");
  lcrzo_er(lcrzo_conf_arp_fprint(pf));
  fprintf((FILE *) pf, " + Routes +\n");
  lcrzo_er(lcrzo_conf_routes_fprint(pf));
  ptr->sysrenew_allow = sysrenew_allow_old;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_devices_add_system(lcrzo_constdevice device,
                                       lcrzo_int16 hwtype,
                                       lcrzo_constetha etha,
                                       lcrzo_ipl ipl,
                                       lcrzo_ipl netmask,
                                       lcrzo_uint32 mtu,
                                       lcrzo_bool isup,
                                       lcrzo_bool isanalias,
                                       lcrzo_bool ispointtopoint,
                                       lcrzo_ipl pointtopointipl)
{
  lcrzo_priv_conf_devices cd;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_int32 pos;
  int ret;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));

  if (ipl == 0 && netmask == 0)
    isup = LCRZO_FALSE;

  /* initialize the structure */
  lcrzo_er(lcrzo_device_init(device, cd.device));
  cd.priority = LCRZO_FALSE;
  if (hwtype == LCRZO_DEVICE_HWTYPE_UNKNOWN && ispointtopoint) {        /* suppose it's ppp */
    cd.hwtype = LCRZO_DEVICE_HWTYPE_PPP;
  } else {
    cd.hwtype = hwtype;
  }
  memcpy(cd.eth, etha, LCRZO_ETHA_MAXBYTES);
  cd.ip = ipl;
  cd.netmask = netmask;
  cd.mtu = mtu;
  cd.isup = isup;
  cd.isanalias = isanalias;
  cd.ispointtopoint = ispointtopoint;
  cd.pointtopointipl = pointtopointipl;

  /* search it in the list */
  ret = lcrzo_list_search_all2(&ptrcv->list_system_devices,
                               &lcrzo_priv_conf_devices_equal,
                               &cd, &pos, NULL);
  if (ret != LCRZO_ERR_OKSEARCHNOTFOUND && ret != LCRZO_ERR_OK)
    return (ret);

  if (ret == LCRZO_ERR_OK) {
    /* we found it, so we replace it */
    lcrzo_er(lcrzo_list_replace_pos(&ptrcv->list_system_devices, pos, &cd));
  } else {
    /* we add it to the list */
    lcrzo_er(lcrzo_list_add_last(&ptrcv->list_system_devices, &cd));
  }

  if (isup) {
    /* add it in the ARP cache */
    if (hwtype == LCRZO_DEVICE_HWTYPE_ETHER) {
      lcrzo_er(lcrzo_priv_conf_arp_add_system(device, etha, ipl, LCRZO_TRUE));
    }
    /* add it in the routing table */
    lcrzo_er(lcrzo_priv_conf_routes_add_system(device, ipl, 0xFFFFFFFFu,
                                               0, 0, 0, isup));
    if (netmask != 0xFFFFFFFFu) {
      /* otherwise, add twice */
      if (ispointtopoint) {
        lcrzo_er(lcrzo_priv_conf_routes_add_system(device, ipl & netmask,
                                                   netmask,
                                                   ipl, pointtopointipl,
                                                   0, isup));
      } else {
        lcrzo_er(lcrzo_priv_conf_routes_add_system(device, ipl & netmask,
                                                   netmask, ipl, 0, 0, isup));
      }
    }
    /* say how to reach the peer */
    if (ispointtopoint) {
      lcrzo_er(lcrzo_priv_conf_routes_add_system(device, pointtopointipl,
                                                 0xFFFFFFFFu,
                                                 ipl, 0, 0, isup));
    }
  }

  ptrcv->compute_sys_devices = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_add_user(lcrzo_bool priorityoversystem,
                                lcrzo_constdevice device,
                                lcrzo_int16 hwtype,
                                lcrzo_constetha etha,
                                lcrzo_ipl ipl,
                                lcrzo_ipl netmask,
                                lcrzo_uint32 mtu,
                                lcrzo_bool isup,
                                lcrzo_bool isanalias,
                                lcrzo_bool ispointtopoint,
                                lcrzo_ipl pointtopointipl)
{
  lcrzo_priv_conf_devices cd;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_int32 pos;
  int ret;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));

  /* initialize the structure */
  lcrzo_er(lcrzo_device_init(device, cd.device));
  cd.priority = priorityoversystem;
  cd.hwtype = hwtype;
  memcpy(cd.eth, etha, LCRZO_ETHA_MAXBYTES);
  cd.ip = ipl;
  cd.netmask = netmask;
  cd.mtu = mtu;
  cd.isup = isup;
  cd.isanalias = isanalias;
  cd.ispointtopoint = ispointtopoint;
  cd.pointtopointipl = pointtopointipl;

  /* search it in the list */
  ret = lcrzo_list_search_all2(&ptrcv->list_user_devices,
                               &lcrzo_priv_conf_devices_equal,
                               &cd, &pos, NULL);
  if (ret != LCRZO_ERR_OKSEARCHNOTFOUND && ret != LCRZO_ERR_OK)
    return (ret);

  if (ret == LCRZO_ERR_OK) {
    /* we found it, so we replace it */
    lcrzo_er(lcrzo_list_replace_pos(&ptrcv->list_user_devices, pos, &cd));
  } else {
    /* we add it to the list */
    lcrzo_er(lcrzo_list_add_last(&ptrcv->list_user_devices, &cd));
  }

  ptrcv->compute_conf_devices = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_del_system(lcrzo_constdevice device,
                                  lcrzo_int16 hwtype,
                                  lcrzo_constetha etha,
                                  lcrzo_ipl ipl)
{
  lcrzo_priv_conf_devices cd;
  lcrzo_priv_confvars *ptrcv;

  /* initialize the structure */
  cd.priority = LCRZO_FALSE;
  lcrzo_er(lcrzo_string_init_coretext(device, LCRZO_DEVICE_MAXBYTES,
                                      cd.device));
  cd.hwtype = hwtype;
  memcpy(cd.eth, etha, LCRZO_ETHA_MAXBYTES);
  cd.ip = ipl;

  /* remove it from the list */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_add_last(&ptrcv->list_systemdel_devices, &cd));

  ptrcv->compute_sys_devices = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_del_user(lcrzo_constdevice device,
                                lcrzo_int16 hwtype,
                                lcrzo_constetha etha,
                                lcrzo_ipl ipl)
{
  lcrzo_priv_conf_devices cd;
  lcrzo_priv_confvars *ptrcv;

  /* initialize the structure */
  cd.priority = LCRZO_FALSE;
  lcrzo_er(lcrzo_string_init_coretext(device, LCRZO_DEVICE_MAXBYTES,
                                      cd.device));
  cd.hwtype = hwtype;
  memcpy(cd.eth, etha, LCRZO_ETHA_MAXBYTES);
  cd.ip = ipl;

  /* remove it from the list */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_criteria_all2(&ptrcv->list_user_devices,
                                           &lcrzo_priv_conf_devices_equal,
                                           &cd));
  ptrcv->compute_conf_devices = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_count(lcrzo_int32 * pcount)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));

  lcrzo_er(lcrzo_list_count(&ptrcv->list_conf_devices, pcount));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_value_pos(lcrzo_int32 pos,
                                 lcrzo_device device,
                                 lcrzo_int16 * phwtype,
                                 lcrzo_etha etha,
                                 lcrzo_ipl * pipl,
                                 lcrzo_ipl * pnetmask,
                                 lcrzo_uint32 * pmtu,
                                 lcrzo_bool * pisup,
                                 lcrzo_bool * pisanalias,
                                 lcrzo_bool * pispointtopoint,
                                 lcrzo_ipl * ppointtopointipl)
{
  lcrzo_priv_conf_devices cd;
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_value_pos(&ptrcv->list_conf_devices, pos, &cd));
  lcrzo_er(lcrzo_device_init(cd.device, device));
  if (phwtype != NULL)
    *phwtype = cd.hwtype;
  if (etha != NULL)
    memcpy(etha, cd.eth, LCRZO_ETHA_MAXBYTES);
  if (pipl != NULL)
    *pipl = cd.ip;
  if (pnetmask != NULL)
    *pnetmask = cd.netmask;
  if (pmtu != NULL)
    *pmtu = cd.mtu;
  if (pisup != NULL)
    *pisup = cd.isup;
  if (pisanalias != NULL)
    *pisanalias = cd.isanalias;
  if (pispointtopoint != NULL)
    *pispointtopoint = cd.ispointtopoint;
  if (ppointtopointipl != NULL)
    *ppointtopointipl = cd.pointtopointipl;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_fprint_cstsized(LCRZOFILE * pf,
                               const char *string,
                               lcrzo_int32 size);
int lcrzo_priv_fprint_cstsized(LCRZOFILE * pf,
                               const char *string,
                               lcrzo_int32 size)
{
  int spacenumber, i;

  fprintf((FILE *) pf, "%s", string);
  spacenumber = size - strlen(string);
  if (spacenumber <= 0)
    spacenumber = 1;
  for (i = 0; i < spacenumber; i++) {
    fputc(' ', (FILE *) pf);
  }
  return (LCRZO_ERR_OK);
}

int lcrzo_conf_devices_fprint(LCRZOFILE * pf)
{
  lcrzo_int32 count, i;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_priv_conf_devices cd;
  lcrzo_ips ips;
  lcrzo_int16 language;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));

  lcrzo_er(lcrzo_list_count(&ptrcv->list_conf_devices, &count));
  lcrzo_er(lcrzo_global_get_language(&language));
  if (count) {
    if (language == LCRZO_GLOBAL_LANG_FR) {
      fprintf((FILE *) pf, "device  ethernet.ou.hw    ip             ");
      fprintf((FILE *) pf, "/netmask          mtu\n");
    } else {
      fprintf((FILE *) pf, "device  ethernet.or.hw    ip             ");
      fprintf((FILE *) pf, "/netmask          mtu\n");
    }
  }
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(&ptrcv->list_conf_devices, i, &cd));
    if (strlen(cd.device) > 7) {
      fprintf((FILE *) pf, "%s\n", cd.device);
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, "...", 8));
    } else {
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, cd.device, 8));
    }
    if (cd.hwtype == LCRZO_DEVICE_HWTYPE_UNKNOWN) {
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, LCRZO_DEVICE_HWTYPE_UNKNOWN_txt,
                                          18));
    } else if (cd.hwtype == LCRZO_DEVICE_HWTYPE_ETHER) {
      lcrzo_er(lcrzo_etha_fprint(pf, "", cd.eth, " "));
    } else if (cd.hwtype == LCRZO_DEVICE_HWTYPE_LOOPBACK) {
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf,
                                          LCRZO_DEVICE_HWTYPE_LOOPBACK_txt,
                                          18));
    } else if (cd.hwtype == LCRZO_DEVICE_HWTYPE_PPP) {
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, LCRZO_DEVICE_HWTYPE_PPP_txt,
                                          18));
    } else if (cd.hwtype == LCRZO_DEVICE_HWTYPE_PLIP) {
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, LCRZO_DEVICE_HWTYPE_PLIP_txt,
                                          18));
    } else if (cd.hwtype == LCRZO_DEVICE_HWTYPE_SLIP) {
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, LCRZO_DEVICE_HWTYPE_SLIP_txt,
                                          18));
    } else {
      fprintf((FILE *) pf, "/\\Internal Error/\\");
    }
    lcrzo_er(lcrzo_ips_init_ipl(cd.ip, ips));
    lcrzo_er(lcrzo_priv_fprint_cstsized(pf, ips, 15));
    fprintf((FILE *) pf, "/");
    lcrzo_er(lcrzo_ips_init_ipl(cd.netmask, ips));
    lcrzo_er(lcrzo_priv_fprint_cstsized(pf, ips, 16));
    lcrzo_er(lcrzo_uint32_fprint(pf, "", cd.mtu, "%5u", " "));
    if (cd.isup)
      fprintf((FILE *) pf, "up");
    else
      fprintf((FILE *) pf, "down");
    if (cd.isanalias)
      fprintf((FILE *) pf, ",alias");
    fprintf((FILE *) pf, "\n");
    if (cd.isup && cd.ispointtopoint) {
      fprintf((FILE *) pf, "        ");
      if (language == LCRZO_GLOBAL_LANG_FR) {
        fprintf((FILE *) pf, "pointtopoint_avec ");
      } else {
        fprintf((FILE *) pf, "pointtopoint_with ");
      }
      lcrzo_er(lcrzo_ipl_fprint(pf, "", cd.pointtopointipl, "\n"));
    }
  }
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_arp_add_system(lcrzo_constdevice device,
                                   lcrzo_constetha etha,
                                   lcrzo_ipl ipl,
                                   lcrzo_bool ispermanent)
{
  lcrzo_priv_conf_arp ca, permfoundca, cachefoundca;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_time currenttime;
  lcrzo_int32 permpos, cachepos;
  int permret, cacheret;

  /* if theses case, we do not add it */
  if (!ispermanent) {
    if (ipl == 0)
      return (LCRZO_ERR_OK);
    if (ipl == 0xFFFFFFFFu)
      return (LCRZO_ERR_OK);
    if (etha[0] == 0 && etha[1] == 0 && etha[2] == 0 &&
        etha[3] == 0 && etha[4] == 0 && etha[5] == 0)
      return (LCRZO_ERR_OK);
    if (etha[0] == 0xFF && etha[1] == 0xFF && etha[2] == 0xFF &&
        etha[3] == 0xFF && etha[4] == 0xFF && etha[5] == 0xFF)
      return (LCRZO_ERR_OK);
  }

  /* initialize the structure */
  ca.priority = LCRZO_FALSE;
  memcpy(ca.eth, etha, LCRZO_ETHA_MAXBYTES);
  lcrzo_er(lcrzo_device_init(device, ca.device));
  ca.ip = ipl;
  ca.ispermanent = ispermanent;
  lcrzo_er(lcrzo_time_init_currenttime(&currenttime));
  lcrzo_er(lcrzo_time_get_sec(&currenttime, &(ca.timeset)));

  /* search it in the lists */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  permret = lcrzo_list_search_all2(&ptrcv->list_system_arpperm,
                                   &lcrzo_priv_conf_arp_equalwdev,
                                   &ca, &permpos, &permfoundca);
  if (permret != LCRZO_ERR_OKSEARCHNOTFOUND && permret != LCRZO_ERR_OK)
    return (permret);
  cacheret = lcrzo_list_search_all2(&ptrcv->list_system_arpcache,
                                    &lcrzo_priv_conf_arp_equalwdev,
                                    &ca, &cachepos, &cachefoundca);
  if (cacheret != LCRZO_ERR_OKSEARCHNOTFOUND && cacheret != LCRZO_ERR_OK)
    return (cacheret);

  /* if the user doesn't know the device, we look if we already know it */
  if (strlen(ca.device) == 0) {
    if (permret == LCRZO_ERR_OK && strlen(permfoundca.device) != 0) {
      lcrzo_er(lcrzo_device_init(permfoundca.device, ca.device));
    } else if (cacheret == LCRZO_ERR_OK && strlen(cachefoundca.device) != 0) {
      lcrzo_er(lcrzo_device_init(cachefoundca.device, ca.device));
    }
  }

  if (ispermanent) {
    ptrcv->compute_sys_arpperm = LCRZO_TRUE;
    if (permret == LCRZO_ERR_OK) {
      /* we found it, so we replace it */
      lcrzo_er(lcrzo_list_replace_pos(&ptrcv->list_system_arpperm,
                                      permpos, &ca));
    } else {
      /* we add it to the list */
      lcrzo_er(lcrzo_list_add_last(&ptrcv->list_system_arpperm, &ca));
    }
    if (cacheret == LCRZO_ERR_OK) {
      /* we remove it from the cache */
      lcrzo_er(lcrzo_list_remove_criteria_all2(&ptrcv->list_system_arpcache,
                                               &lcrzo_priv_conf_arp_equalwdev,
                                               &ca));
    }
  } else {
    ptrcv->compute_sys_arpcache = LCRZO_TRUE;
    if (cacheret == LCRZO_ERR_OK) {
      /* we found it, so we replace it */
      lcrzo_er(lcrzo_list_replace_pos(&ptrcv->list_system_arpcache,
                                      cachepos, &ca));
    } else {
      /* we add it to the list */
      lcrzo_er(lcrzo_list_add_last(&ptrcv->list_system_arpcache, &ca));
    }
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arp_add_user(lcrzo_bool priorityoversystem,
                            lcrzo_constdevice device,
                            lcrzo_constetha etha,
                            lcrzo_ipl ipl,
                            lcrzo_bool ispermanent)
{
  lcrzo_priv_conf_arp ca, permfoundca, cachefoundca;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_time currenttime;
  lcrzo_int32 permpos, cachepos;
  int permret, cacheret;

  /* initialize the structure */
  ca.priority = priorityoversystem;
  memcpy(ca.eth, etha, LCRZO_ETHA_MAXBYTES);
  lcrzo_er(lcrzo_device_init(device, ca.device));
  ca.ip = ipl;
  ca.ispermanent = ispermanent;
  lcrzo_er(lcrzo_time_init_currenttime(&currenttime));
  lcrzo_er(lcrzo_time_get_sec(&currenttime, &(ca.timeset)));

  /* search it in the lists */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  permret = lcrzo_list_search_all2(&ptrcv->list_user_arpperm,
                                   &lcrzo_priv_conf_arp_equalwdev,
                                   &ca, &permpos, &permfoundca);
  if (permret != LCRZO_ERR_OKSEARCHNOTFOUND && permret != LCRZO_ERR_OK)
    return (permret);
  cacheret = lcrzo_list_search_all2(&ptrcv->list_user_arpcache,
                                    &lcrzo_priv_conf_arp_equalwdev,
                                    &ca, &cachepos, &cachefoundca);
  if (cacheret != LCRZO_ERR_OKSEARCHNOTFOUND && cacheret != LCRZO_ERR_OK)
    return (cacheret);

  /* if the user doesn't know the device, we look if we already know it */
  if (strlen(ca.device) == 0) {
    if (permret == LCRZO_ERR_OK && strlen(permfoundca.device) != 0) {
      lcrzo_er(lcrzo_device_init(permfoundca.device, ca.device));
    } else if (cacheret == LCRZO_ERR_OK && strlen(cachefoundca.device) != 0) {
      lcrzo_er(lcrzo_device_init(cachefoundca.device, ca.device));
    }
  }

  if (ispermanent) {
    ptrcv->compute_conf_arpperm = LCRZO_TRUE;
    if (permret == LCRZO_ERR_OK) {
      /* we found it, so we replace it */
      lcrzo_er(lcrzo_list_replace_pos(&ptrcv->list_user_arpperm,
                                      permpos, &ca));
    } else {
      /* we add it to the list */
      lcrzo_er(lcrzo_list_add_last(&ptrcv->list_user_arpperm, &ca));
    }
    if (cacheret == LCRZO_ERR_OK) {
      /* we remove it from the cache */
      lcrzo_er(lcrzo_list_remove_criteria_all2(&ptrcv->list_user_arpcache,
                                               &lcrzo_priv_conf_arp_equalwdev,
                                               &ca));
    }
  } else {
    ptrcv->compute_conf_arpperm = LCRZO_TRUE;
    if (cacheret == LCRZO_ERR_OK) {
      /* we found it, so we replace it */
      lcrzo_er(lcrzo_list_replace_pos(&ptrcv->list_user_arpcache,
                                      cachepos, &ca));
    } else {
      /* we add it to the list */
      lcrzo_er(lcrzo_list_add_last(&ptrcv->list_user_arpcache, &ca));
    }
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arp_del_system(lcrzo_constdevice device,
                              lcrzo_constetha etha,
                              lcrzo_ipl ipl)
{
  lcrzo_priv_conf_arp ca;
  lcrzo_priv_confvars *ptrcv;

  /* initialize the structure */
  ca.priority = LCRZO_FALSE;
  lcrzo_er(lcrzo_device_init(device, ca.device));
  memcpy(ca.eth, etha, LCRZO_ETHA_MAXBYTES);
  ca.ip = ipl;

  /* add it from the list */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_add_last(&ptrcv->list_systemdel_arpperm, &ca));
  lcrzo_er(lcrzo_list_add_last(&ptrcv->list_systemdel_arpcache, &ca));
  ptrcv->compute_sys_arpperm = LCRZO_TRUE;
  ptrcv->compute_sys_arpcache = LCRZO_TRUE;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arp_del_user(lcrzo_constdevice device,
                            lcrzo_constetha etha,
                            lcrzo_ipl ipl)
{
  lcrzo_priv_conf_arp ca;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_int32 count1, count2;

  /* initialize the structure */
  ca.priority = LCRZO_FALSE;
  lcrzo_er(lcrzo_device_init(device, ca.device));
  memcpy(ca.eth, etha, LCRZO_ETHA_MAXBYTES);
  ca.ip = ipl;

  /* remove it from the list */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_count(&ptrcv->list_user_arpperm, &count1));
  lcrzo_er(lcrzo_list_remove_criteria_all2(&ptrcv->list_user_arpperm,
                                           &lcrzo_priv_conf_arp_equal, &ca));
  lcrzo_er(lcrzo_list_count(&ptrcv->list_user_arpperm, &count2));
  if (count1 != count2) {
    ptrcv->compute_conf_arpperm = LCRZO_TRUE;
  }

  lcrzo_er(lcrzo_list_count(&ptrcv->list_user_arpcache, &count1));
  lcrzo_er(lcrzo_list_remove_criteria_all2(&ptrcv->list_user_arpcache,
                                           &lcrzo_priv_conf_arp_equal, &ca));
  lcrzo_er(lcrzo_list_count(&ptrcv->list_user_arpcache, &count2));
  if (count1 != count2) {
    ptrcv->compute_conf_arpcache = LCRZO_TRUE;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arp_count(lcrzo_int32 * pcount)
{
  lcrzo_int32 count;
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));

  if (pcount != NULL) {
    lcrzo_er(lcrzo_list_count(&ptrcv->list_conf_arpperm, &count));
    *pcount = count;
    lcrzo_er(lcrzo_list_count(&ptrcv->list_conf_arpcache, &count));
    *pcount += count;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arp_value_pos(lcrzo_int32 pos,
                             lcrzo_device device,
                             lcrzo_etha etha,
                             lcrzo_ipl * pipl,
                             lcrzo_bool * pispermanent,
                             lcrzo_uint32 * ptimeset)
{
  lcrzo_priv_conf_arp ca;
  lcrzo_int32 permcount, cachecount;
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));

  /* calculate the position */
  lcrzo_er(lcrzo_list_count(&ptrcv->list_conf_arpperm, &permcount));
  lcrzo_er(lcrzo_list_count(&ptrcv->list_conf_arpcache, &cachecount));
  if (pos < 0) {
    pos = permcount + cachecount + 1 + pos;
    if (pos <= 0)
      return (LCRZO_ERR_PATOOLOW);
  }

  /* get the value from the correct list */
  if (pos <= permcount) {
    lcrzo_er(lcrzo_list_value_pos(&ptrcv->list_conf_arpperm, pos, &ca));
  } else {
    pos -= permcount;
    lcrzo_er(lcrzo_list_value_pos(&ptrcv->list_conf_arpcache, pos, &ca));
  }

  /* set the output parameters */
  lcrzo_er(lcrzo_device_init(ca.device, device));
  if (etha != NULL)
    memcpy(etha, ca.eth, LCRZO_ETHA_MAXBYTES);
  if (pipl != NULL)
    *pipl = ca.ip;
  if (pispermanent != NULL)
    *pispermanent = ca.ispermanent;
  if (ptimeset != NULL)
    *ptimeset = ca.timeset;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arp_fprint(LCRZOFILE * pf)
{
  lcrzo_int32 count, i;
  lcrzo_time currenttime;
  lcrzo_ips ips;
  lcrzo_device device;
  lcrzo_etha etha;
  lcrzo_ipl ipl;
  lcrzo_bool ispermanent;
  lcrzo_uint32 timeset, currenttimesec, devsize;

  lcrzo_er(lcrzo_conf_arp_count(&count));
  lcrzo_er(lcrzo_time_init_currenttime(&currenttime));
  lcrzo_er(lcrzo_time_get_sec(&currenttime, &currenttimesec));
#if defined LCRZODEF_SYSTEM_Unix
  devsize = 8;
#elif defined LCRZODEF_SYSTEM_Windows
  devsize = 28;
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
  if (count) {
    lcrzo_er(lcrzo_priv_fprint_cstsized(pf, "device", devsize));
    fprintf((FILE *) pf, "ethernet          ip              age\n");
  }
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_conf_arp_value_pos(i, device, etha, &ipl,
                                      &ispermanent, &timeset));
    if (strlen(device) > devsize - 1) {
      fprintf((FILE *) pf, "%s\n", device);
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, "...", devsize));
    } else {
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, device, devsize));
    }
    lcrzo_er(lcrzo_etha_fprint(pf, "", etha, " "));
    lcrzo_er(lcrzo_ips_init_ipl(ipl, ips));
    lcrzo_er(lcrzo_priv_fprint_cstsized(pf, ips, 16));
    if (ispermanent)
      fprintf((FILE *) pf, "(permanent)\n");
    else {
      lcrzo_er(lcrzo_uint32_fprint(pf, "(-", currenttimesec - timeset,
                                   "%u", "s)\n"));
    }
  }

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_priv_conf_routes_add_system(lcrzo_constdevice device,
                                      lcrzo_ipl dest,
                                      lcrzo_ipl mask,
                                      lcrzo_ipl ipsource,
                                      lcrzo_ipl gateway,
                                      lcrzo_uint32 metric,
                                      lcrzo_bool isup)
{
  lcrzo_priv_conf_routes cr;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_int32 pos;
  int ret;

  /* initialize the structure */
  cr.priority = LCRZO_FALSE;
  lcrzo_er(lcrzo_device_init(device, cr.device));
  cr.dest = dest;
  cr.mask = mask;
  cr.ipsource = ipsource;
  cr.gateway = gateway;
  cr.metric = metric;
  cr.isup = isup;

  /* search it in the list */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  ret = lcrzo_list_search_all2(&ptrcv->list_system_routes,
                               &lcrzo_priv_conf_routes_equal,
                               &cr, &pos, NULL);
  if (ret != LCRZO_ERR_OKSEARCHNOTFOUND && ret != LCRZO_ERR_OK)
    return (ret);

  if (ret == LCRZO_ERR_OK) {
    /* we found it, so we replace it */
    lcrzo_er(lcrzo_list_replace_pos(&ptrcv->list_system_routes, pos, &cr));
  } else {
    /* we add it to the list */
    lcrzo_er(lcrzo_list_add_last(&ptrcv->list_system_routes, &cr));
  }
  ptrcv->compute_sys_routes = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_routes_add_user(lcrzo_bool priorityoversystem,
                               lcrzo_constdevice device,
                               lcrzo_ipl dest,
                               lcrzo_ipl mask,
                               lcrzo_ipl ipsource,
                               lcrzo_ipl gateway,
                               lcrzo_uint32 metric,
                               lcrzo_bool isup)
{
  lcrzo_priv_conf_routes cr;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_int32 pos;
  int ret;

  /* initialize the structure */
  cr.priority = priorityoversystem;
  lcrzo_er(lcrzo_device_init(device, cr.device));
  cr.dest = dest;
  cr.mask = mask;
  cr.ipsource = ipsource;
  cr.gateway = gateway;
  cr.metric = metric;
  cr.isup = isup;

  /* search it in the list */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  ret = lcrzo_list_search_all2(&ptrcv->list_user_routes,
                               &lcrzo_priv_conf_routes_equal,
                               &cr, &pos, NULL);
  if (ret != LCRZO_ERR_OKSEARCHNOTFOUND && ret != LCRZO_ERR_OK)
    return (ret);

  if (ret == LCRZO_ERR_OK) {
    /* we found it, so we replace it */
    lcrzo_er(lcrzo_list_replace_pos(&ptrcv->list_user_routes, pos, &cr));
  } else {
    /* we add it to the list */
    lcrzo_er(lcrzo_list_add_last(&ptrcv->list_user_routes, &cr));
  }

  ptrcv->compute_conf_routes = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_routes_del_system(lcrzo_ipl dest,
                                 lcrzo_ipl mask,
                                 lcrzo_ipl gateway)
{
  lcrzo_priv_conf_routes cr;
  lcrzo_priv_confvars *ptrcv;

  /* initialize the structure */
  cr.priority = LCRZO_FALSE;
  cr.dest = dest;
  cr.mask = mask;
  cr.gateway = gateway;

  /* add it to the list */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_add_last(&ptrcv->list_systemdel_routes, &cr));
  ptrcv->compute_sys_routes = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_routes_del_user(lcrzo_ipl dest,
                               lcrzo_ipl mask,
                               lcrzo_ipl gateway)
{
  lcrzo_priv_conf_routes cr;
  lcrzo_priv_confvars *ptrcv;

  /* initialize the structure */
  cr.priority = LCRZO_FALSE;
  cr.dest = dest;
  cr.mask = mask;
  cr.gateway = gateway;

  /* remove it from the list */
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_criteria_all2(&ptrcv->list_user_routes,
                                           &lcrzo_priv_conf_routes_equal,
                                           &cr));
  ptrcv->compute_conf_routes = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_routes_count(lcrzo_int32 * pcount)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));

  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));

  lcrzo_er(lcrzo_list_count(&(ptrcv->list_conf_routes), pcount));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_routes_value_pos(lcrzo_int32 pos,
                                lcrzo_device device,
                                lcrzo_ipl * pdest,
                                lcrzo_ipl * pmask,
                                lcrzo_ipl * pipsource,
                                lcrzo_ipl * pgateway,
                                lcrzo_uint32 * pmetric,
                                lcrzo_bool * pisup)
{
  lcrzo_priv_conf_routes cr;
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_value_pos(&(ptrcv->list_conf_routes), pos, &cr));
  lcrzo_er(lcrzo_device_init(cr.device, device));
  if (pdest != NULL)
    *pdest = cr.dest;
  if (pmask != NULL)
    *pmask = cr.mask;
  if (pipsource != NULL)
    *pipsource = cr.ipsource;
  if (pgateway != NULL)
    *pgateway = cr.gateway;
  if (pmetric != NULL)
    *pmetric = cr.metric;
  if (pisup != NULL)
    *pisup = cr.isup;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_routes_fprint(LCRZOFILE * pf)
{
  lcrzo_int32 count, i;
  lcrzo_priv_conf_routes cr;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_ips ips;
  lcrzo_int16 language;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));

  lcrzo_er(lcrzo_list_count(&(ptrcv->list_conf_routes), &count));
  if (count) {
    lcrzo_er(lcrzo_global_get_language(&language));
    if (language == LCRZO_GLOBAL_LANG_FR) {
      fprintf((FILE *) pf, "device  destination    /netmask");
      fprintf((FILE *) pf, "         ip_source       routeur\n");
    } else {
      fprintf((FILE *) pf, "device  destination    /netmask");
      fprintf((FILE *) pf, "         source_ip       gateway\n");
    }
  }
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(&(ptrcv->list_conf_routes), i, &cr));
    if (strlen(cr.device) > 7) {
      fprintf((FILE *) pf, "%s\n", cr.device);
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, "...", 8));
    } else {
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, cr.device, 8));
    }
    lcrzo_er(lcrzo_ips_init_ipl(cr.dest, ips));
    lcrzo_er(lcrzo_priv_fprint_cstsized(pf, ips, 15));
    fprintf((FILE *) pf, "/");
    lcrzo_er(lcrzo_ips_init_ipl(cr.mask, ips));
    lcrzo_er(lcrzo_priv_fprint_cstsized(pf, ips, 16));
    if (cr.ipsource == 0 && cr.gateway == 0) {
      lcrzo_er(lcrzo_global_get_language(&language));
      if (language == LCRZO_GLOBAL_LANG_FR) {
        fprintf((FILE *) pf, "device_local                    ");
      } else {
        fprintf((FILE *) pf, "local_device                    ");
      }
    } else if (cr.gateway == 0) {
      lcrzo_er(lcrzo_ips_init_ipl(cr.ipsource, ips));
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, ips, 16));
      fprintf((FILE *) pf, "                ");
    } else {
      lcrzo_er(lcrzo_ips_init_ipl(cr.ipsource, ips));
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, ips, 16));
      lcrzo_er(lcrzo_ips_init_ipl(cr.gateway, ips));
      lcrzo_er(lcrzo_priv_fprint_cstsized(pf, ips, 16));
    }
    lcrzo_er(lcrzo_uint32_fprint(pf, "", cr.metric, "% 2u", ","));
    if (cr.isup)
      fprintf((FILE *) pf, "up");
    else
      fprintf((FILE *) pf, "down");
    fprintf((FILE *) pf, "\n");
  }
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_conf_set_sysrenew_allow(lcrzo_bool yes)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_allow = yes;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_get_sysrenew_allow(lcrzo_bool * pyes)
{
  lcrzo_priv_confvars *ptr;

  if (pyes != NULL) {
    lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
    *pyes = ptr->sysrenew_allow;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_set_sysrenew_allow_devices(lcrzo_bool yes)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_allow_devices = yes;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_get_sysrenew_allow_devices(lcrzo_bool * pyes)
{
  lcrzo_priv_confvars *ptr;

  if (pyes != NULL) {
    lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
    *pyes = ptr->sysrenew_allow_devices;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_set_sysrenew_allow_arpperm(lcrzo_bool yes)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_allow_arpperm = yes;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_get_sysrenew_allow_arpperm(lcrzo_bool * pyes)
{
  lcrzo_priv_confvars *ptr;

  if (pyes != NULL) {
    lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
    *pyes = ptr->sysrenew_allow_arpperm;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_set_sysrenew_allow_arpcache(lcrzo_bool yes)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_allow_arpcache = yes;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_get_sysrenew_allow_arpcache(lcrzo_bool * pyes)
{
  lcrzo_priv_confvars *ptr;

  if (pyes != NULL) {
    lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
    *pyes = ptr->sysrenew_allow_arpcache;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_set_sysrenew_allow_routes(lcrzo_bool yes)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_allow_routes = yes;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_get_sysrenew_allow_routes(lcrzo_bool * pyes)
{
  lcrzo_priv_confvars *ptr;

  if (pyes != NULL) {
    lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
    *pyes = ptr->sysrenew_allow_routes;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_set_permentriesperiod(lcrzo_uint32 period)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->permentriesperiod = period;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_get_permentriesperiod(lcrzo_uint32 * pperiod)
{
  lcrzo_priv_confvars *ptr;

  if (pperiod != NULL) {
    lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
    *pperiod = ptr->permentriesperiod;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_set_cacheentriestimeout(lcrzo_uint32 cachetimeout)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->cacheentriestimeout = cachetimeout;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_get_cacheentriestimeout(lcrzo_uint32 * ptimeout)
{
  lcrzo_priv_confvars *ptr;

  if (ptimeout != NULL) {
    lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
    *ptimeout = ptr->cacheentriestimeout;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_renewnow_system(void)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_now = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_renewnow_system(void)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_now_devices = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arpperm_renewnow_system(void)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_now_arpperm = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_conf_arpcache_renewnow_system(void)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_now_arpcache = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_routes_renewnow_system(void)
{
  lcrzo_priv_confvars *ptr;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptr));
  ptr->sysrenew_now_routes = LCRZO_TRUE;
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_conf_flush_system(void)
{
  lcrzo_er(lcrzo_conf_devices_flush_system());
  lcrzo_er(lcrzo_conf_arpperm_flush_system());
  lcrzo_er(lcrzo_conf_arpcache_flush_system());
  lcrzo_er(lcrzo_conf_routes_flush_system());
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_flush_system(void)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_all(&ptrcv->list_system_devices));
  ptrcv->compute_sys_devices = LCRZO_TRUE;
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arpperm_flush_system(void)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_all(&ptrcv->list_system_arpperm));
  ptrcv->compute_sys_arpperm = LCRZO_TRUE;
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arpcache_flush_system(void)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_all(&ptrcv->list_system_arpcache));
  ptrcv->compute_sys_arpcache = LCRZO_TRUE;
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_routes_flush_system(void)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_all(&ptrcv->list_system_routes));
  ptrcv->compute_sys_routes = LCRZO_TRUE;
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_flush_user(void)
{
  lcrzo_er(lcrzo_conf_devices_flush_user());
  lcrzo_er(lcrzo_conf_arpperm_flush_user());
  lcrzo_er(lcrzo_conf_arpcache_flush_user());
  lcrzo_er(lcrzo_conf_routes_flush_user());
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_flush_user(void)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_all(&ptrcv->list_user_devices));
  ptrcv->compute_conf_devices = LCRZO_TRUE;
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arpperm_flush_user(void)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_all(&ptrcv->list_user_arpperm));
  ptrcv->compute_conf_arpperm = LCRZO_TRUE;
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_arpcache_flush_user(void)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_all(&ptrcv->list_user_arpcache));
  ptrcv->compute_conf_arpcache = LCRZO_TRUE;
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_routes_flush_user(void)
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_list_remove_all(&ptrcv->list_user_routes));
  ptrcv->compute_conf_routes = LCRZO_TRUE;
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
int lcrzo_conf_route_to_host(lcrzo_ipl hostipl,
                             lcrzo_device device,
                             lcrzo_etha srcetha,
                             lcrzo_etha destetha,
                             lcrzo_ipl * psrcipl,
                             lcrzo_ipl * prouteripl)
{
  lcrzo_int32 count, i;
  lcrzo_priv_conf_routes cr;
  lcrzo_priv_confvars *ptrcv;
  lcrzo_ipl routenotetherip, routenotethermask;
  lcrzo_bool routenotetherfound;
  int ret;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));

  routenotetherfound = LCRZO_FALSE;
  routenotetherip = 0;          /* for the compiler warning */
  routenotethermask = 0;        /* for the compiler warning */
  lcrzo_er(lcrzo_list_count(&ptrcv->list_conf_routes, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(&ptrcv->list_conf_routes, i, &cr));
    /* if this route matches */
    if (cr.isup && ((hostipl & cr.mask) == (cr.dest & cr.mask))) {
      /* set info we know */
      lcrzo_er(lcrzo_device_init(cr.device, device));
      if (cr.ipsource == 0 && cr.gateway == 0) {
        /* it is a local device */
        return (LCRZO_ERR_OKROUTELOCALDEV);
      }
      if (psrcipl != NULL)
        *psrcipl = cr.ipsource;
      ret = lcrzo_etha_init_device(cr.device, srcetha);
      if (ret == LCRZO_ERR_SPDEVNOTETHER) {
        /* do not exit now because we can have several metrics */
        if (prouteripl != NULL)
          *prouteripl = cr.gateway;
        if (!routenotetherfound) {
          routenotetherip = cr.dest;    /* set only the first time */
          routenotethermask = cr.mask;  /* set only the first time */
          routenotetherfound = LCRZO_TRUE;
        }
      } else if (ret == LCRZO_ERR_OK) {
        /* check we have the same route */
        if (routenotetherfound) {
          if (routenotetherip != cr.dest || routenotethermask != cr.mask) {
            return (LCRZO_ERR_OKROUTENOTETHER);
          }
        }
        /* set more info */
        if (cr.gateway == 0) {
          /* it is on the lan */
          if (prouteripl != NULL)
            *prouteripl = 0;
          ret = lcrzo_etha_init_ipl(hostipl, destetha);
          if (ret == LCRZO_ERR_OK) {
          } else if (ret == LCRZO_ERR_OKUNRESOLVED) {
            return (LCRZO_ERR_OKROUTEHOSTUNREACH);
          } else
            return (ret);
        } else {
          /* it is after a router */
          if (prouteripl != NULL)
            *prouteripl = cr.gateway;
          ret = lcrzo_etha_init_ipl(cr.gateway, destetha);
          if (ret == LCRZO_ERR_OK) {
          } else if (ret == LCRZO_ERR_OKUNRESOLVED) {
            return (LCRZO_ERR_OKROUTEGWUNREACH);
          } else
            return (ret);
        }
        /* all the info are set */
        return (LCRZO_ERR_OK);
      } else {
        return (ret);
      }
    }
  }

  if (routenotetherfound)
    return (LCRZO_ERR_OKROUTENOTETHER);
  return (LCRZO_ERR_OKROUTENOTFOUND);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_route_to_host_ip(lcrzo_ipl hostipl,
                                lcrzo_device device,
                                lcrzo_ipl * psrcipl,
                                lcrzo_ipl * prouteripl)
{
  lcrzo_int32 count, i;
  lcrzo_priv_conf_routes cr;
  lcrzo_priv_confvars *ptrcv;

  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));
  lcrzo_er(lcrzo_priv_conf_eventuallyupdate(ptrcv));

  lcrzo_er(lcrzo_list_count(&ptrcv->list_conf_routes, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(&ptrcv->list_conf_routes, i, &cr));
    /* if this route matches */
    if (cr.isup && ((hostipl & cr.mask) == (cr.dest & cr.mask))) {
      /* set info we know */
      lcrzo_er(lcrzo_device_init(cr.device, device));
      if (cr.ipsource == 0 && cr.gateway == 0) {
        /* it is a local device */
        return (LCRZO_ERR_OKROUTELOCALDEV);
      }
      if (psrcipl != NULL)
        *psrcipl = cr.ipsource;
      if (prouteripl != NULL)
        *prouteripl = cr.gateway;
      return (LCRZO_ERR_OK);
    }
  }

  return (LCRZO_ERR_OKROUTENOTFOUND);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
int lcrzo_conf_debug_fprint(LCRZOFILE * pf)
{
#ifdef LCRZODEF_SYSTEM_Unix
#elif defined LCRZODEF_SYSTEM_Windows
  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf, "------------------------------------------------------------\n");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_localdevice_init_debug_fprint(pf),
                "lcrzo_priv_conf_localdevice_init_debug_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf, "------------------------------------------------------------\n");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_localeth_init_debug_fprint(pf),
                "lcrzo_priv_conf_localeth_init_debug_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_localip_init_debug_fprint(pf),
                "lcrzo_priv_conf_localip_init_debug_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_validdhcpinfo_init_debug_fprint(pf),
                "lcrzo_priv_conf_validdhcpinfo_init_debug_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_etha_init_cardnumber_debug_fprint(pf),
                "lcrzo_priv_conf_etha_init_cardnumber_debug_fprint");
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

  fprintf((FILE *) pf,
          "\n------------------------------------------------------------\n");
  fprintf((FILE *) pf, "Complete configuration\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");

  fprintf((FILE *) pf,
          "First correct device configuration (lcrzo_priv_conf_init_dev)\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_init_dev(), "lcrzo_priv_conf_init_dev");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  fprintf((FILE *) pf, "First correct arpcache configuration (lcrzo_priv_conf_init_arpcache)\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_init_arpcache(),
                "lcrzo_priv_conf_init_arpcache");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "First correct rarpcache configuration (lcrzo_priv_conf_init_rarpcache)\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw2o(pf, lcrzo_priv_conf_init_rarpcache(),
                "lcrzo_priv_conf_init_rarpcache", LCRZO_ERR_IEUNIMPLEMENTED);
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  fprintf((FILE *) pf, "First correct routes configuration (lcrzo_priv_conf_init_routes)\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_init_routes(),
                "lcrzo_priv_conf_init_routes");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  fprintf((FILE *) pf, "Device private configuration\n");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_dev_debug_fprint(pf),
                "lcrzo_priv_conf_dev_debug_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  fprintf((FILE *) pf, "Arp cache private configuration\n");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_arpcache_debug_fprint(pf),
                "lcrzo_priv_conf_arpcache_debug_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  fprintf((FILE *) pf, "Rarp cache private configuration\n");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_rarpcache_debug_fprint(pf),
                "lcrzo_priv_conf_rarpcache_debug_fprint");

  fprintf((FILE *) pf, "\n\n------------------------------------------------------------\n");
  fprintf((FILE *) pf,
          "------------------------------------------------------------\n");
  fprintf((FILE *) pf, "Routes private configuration\n");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_routes_debug_fprint(pf),
                "lcrzo_priv_conf_routes_debug_fprint");

  fprintf((FILE *) pf,
          "\n------------------------------------------------------------\n");
  return (LCRZO_ERR_OK);
}
