/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_incl.h"

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_fd2_wait(lcrzo_fd2 * pfd2,
                             lcrzo_time * pmaxwaitingtime,
                             lcrzo_bool * pevent)
{
  return (lcrzo_priv_wait_fd_wait(pfd2->fd, pmaxwaitingtime, pevent));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_fd2_read(lcrzo_fd2 * pfd2)
{
  return (lcrzo_priv_wait_fd_read(pfd2->fd, &(pfd2->storeddata),
                                  &(pfd2->storeddatasize)));
}

/*-------------------------------------------------------------*/
int lcrzo_fd2_init_fd(int fd,
                      lcrzo_fd2 * pfd2)
{
  if (pfd2 != NULL) {
    pfd2->fd = fd;
    pfd2->openforreading = LCRZO_TRUE;
    pfd2->openforwriting = LCRZO_TRUE;
    pfd2->dataend = LCRZO_FALSE;
    pfd2->storeddata = NULL;
    pfd2->storeddatasize = 0;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_fd2_close(lcrzo_fd2 * pfd2)
{
  if (pfd2 == NULL)
    return (LCRZO_ERR_OK);

  /* file can not be used */
  pfd2->fd = -1;
  pfd2->dataend = LCRZO_TRUE;
  pfd2->openforreading = LCRZO_FALSE;
  pfd2->openforwriting = LCRZO_FALSE;

  /* free data */
  if (pfd2->storeddata == NULL && pfd2->storeddatasize)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (pfd2->storeddatasize) {
    lcrzo_er(lcrzo_data_free2(&(pfd2->storeddata)));
    pfd2->storeddata = NULL;
    pfd2->storeddatasize = 0;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_fd_init_fd2(lcrzo_fd2 * pfd2,
                      int *pfd)
{
  /*parameters verification */
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pfd != NULL) {
    *pfd = pfd2->fd;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_fd2_write(lcrzo_fd2 * pfd2,
                    lcrzo_constdata data,
                    lcrzo_int32 datasize)
{
  /*parameters verification */
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (!pfd2->openforwriting)
    return (LCRZO_ERR_LONOTOPENFORWRITING);

  lcrzo_er(lcrzo_priv_fd_write(pfd2->fd, data, datasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_fd2_write_msg(lcrzo_fd2 * pfd2,
                        lcrzo_constdata data,
                        lcrzo_int32 datasize)
{
  /*parameters verification */
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (!pfd2->openforwriting)
    return (LCRZO_ERR_LONOTOPENFORWRITING);

  lcrzo_er(lcrzo_priv_fd_write_msg(pfd2->fd, data, datasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_fd2_readm(lcrzo_fd2 * pfd2,
                    lcrzo_time * pmaxwaitingtime,
                    lcrzo_data * pdataout,
                    lcrzo_int32 * pdataoutsize)
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_fd2_read(pfd2, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract
           (&(pfd2->storeddata), &(pfd2->storeddatasize), pfd2->dataend,
            pdataout, pdataoutsize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_fd2_readm_fixed(lcrzo_fd2 * pfd2,
                          lcrzo_time * pmaxwaitingtime,
                          lcrzo_int32 datasizetoread,
                          lcrzo_data * pdataout)
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_fd2_read_fixed(pfd2, datasizetoread, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract_fixed(&(pfd2->storeddata),
                                               &(pfd2->storeddatasize),
                                               pfd2->dataend, datasizetoread,
                                               pdataout));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_fd2_readm_line(lcrzo_fd2 * pfd2,
                         lcrzo_time * pmaxwaitingtime,
                         lcrzo_data * pdataout,
                         lcrzo_int32 * pdataoutsize)
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_fd2_read_line(pfd2, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract_line(&(pfd2->storeddata),
                                              &(pfd2->storeddatasize),
                                              pfd2->dataend, pdataout,
                                              pdataoutsize));

  return (LCRZO_ERR_OK);
}
