/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_incl.h"

/*-------------------------------------------------------------*/
int lcrzo_priv_stat_print(struct stat st);
int lcrzo_priv_stat_print(struct stat st)
{
  lcrzo_er(lcrzo_uint32_print("dev=", st.st_dev, "%u", " "));
  lcrzo_er(lcrzo_uint32_print("ino=", st.st_ino, "%u", " "));
  lcrzo_er(lcrzo_uint32_print("mode=", st.st_mode, "%u", " "));
  lcrzo_er(lcrzo_uint32_print("nlink=", st.st_nlink, "%u", " "));
  lcrzo_er(lcrzo_uint32_print("uid=", st.st_uid, "%u", " "));
  lcrzo_er(lcrzo_uint32_print("gid=", st.st_gid, "%u", " "));
  lcrzo_er(lcrzo_uint32_print("size=", st.st_size, "%u", "\n"));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#include "sysdep/priv_cleanpath_stat_get.c"

/*-------------------------------------------------------------*/
#include "sysdep/priv_fd_stat_get.c"

/*-------------------------------------------------------------*/
#include "sysdep/priv_stat_forbidschange.c"

/*-------------------------------------------------------------*/
#include "sysdep/priv_stat_forbidslink.c"

/*-------------------------------------------------------------*/
#include "sysdep/priv_stat_forbidsnotreg.c"

/*-------------------------------------------------------------*/
#include "sysdep/priv_stat_type.c"

/*-------------------------------------------------------------*/
int lcrzo_priv_stat_size(struct stat st,
                         lcrzo_int32 * psize);
int lcrzo_priv_stat_size(struct stat st,
                         lcrzo_int32 * psize)
{
  if (st.st_size > 0x7FFFFFFF)
    return (LCRZO_ERR_SPFILE2G);

  /*set the correct value */
  if (psize != NULL) {
    *psize = (lcrzo_int32) st.st_size;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_file_wait(lcrzo_file * pfile,
                              lcrzo_time * pmaxwaitingtime,
                              lcrzo_bool * pevent)
{
  return (lcrzo_priv_wait_fd_wait(pfile->fd, pmaxwaitingtime, pevent));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_file_read(lcrzo_file * pfile)
{
  return (lcrzo_priv_wait_fd_read(pfile->fd, &(pfile->storeddata),
                                  &(pfile->storeddatasize)));
}

/*-------------------------------------------------------------*/
#include "sysdep/priv_winpath240_check.c"

/*-------------------------------------------------------------*/
int lcrzo_priv_path_create_parentdirs(lcrzo_conststring pathname,
                                      lcrzo_bool pathnameisclean);
int lcrzo_priv_path_create_parentdirs(lcrzo_conststring pathname,
                                      lcrzo_bool pathnameisclean)
{
  int ret;
  lcrzo_string parent, start;
  lcrzo_bool yes;

  /* check if start directory exists */
  lcrzo_er(lcrzo_path_decode_start(pathname, pathnameisclean, &start));
  lcrzo_er(lcrzo_dir_exist(start, &yes));
  if (!yes) {
    lcrzo_er(lcrzo_priv_err_purge_int());
    lcrzo_er(lcrzo_priv_global_set_errmsglcrzo(start));
    lcrzo_er(lcrzo_string_free2(&start));
    return (LCRZO_ERR_OKFILENOTFOUND);
  }
  lcrzo_er(lcrzo_string_free2(&start));

  /* check if parent directory exists. Otherwise, create it */
  ret = lcrzo_path_decode_parentdir(pathname, pathnameisclean, &parent);
  if (ret == LCRZO_ERR_OK) {
    ret = lcrzo_dir_exist(parent, &yes);
    if (ret == LCRZO_ERR_OK && !yes) {
      ret = lcrzo_dir_create(parent);
    }
    lcrzo_debug_err_ok_activate(LCRZO_FALSE);
    lcrzo_er(lcrzo_string_free2(&parent));
    lcrzo_debug_err_ok_activate(LCRZO_TRUE);
  }

  return (ret);
}

/*-------------------------------------------------------------*/
#if ! defined O_BINARY
 /* should only be useful on Windows, so set 0 if not defined */
#define O_BINARY 0
#endif

/*-------------------------------------------------------------*/
int lcrzo_priv_path_clean_searchsmb(lcrzo_conststring pc,
                                    lcrzo_string * pstart,
                                    lcrzo_conststring * ppcstartanalyze);
int lcrzo_priv_path_clean_searchsmb(lcrzo_conststring pc,
                                    lcrzo_string * pstart,
                                    lcrzo_conststring * ppcstartanalyze)
{
  lcrzo_conststring pccurrent;
  lcrzo_string start;
  lcrzo_bool charfound;
  int i;
  char c;

  /* pc should contain "server[\/]+share[\/]*.*" */

  /* initialize the working zone (+3 to add the first \\ and the last \) */
  lcrzo_er(lcrzo_string_alloc(strlen(pc) + 3, &start));
  start[0] = '\\';
  start[1] = '\\';

  /* search the server */
  pccurrent = pc;
  i = 2;
  charfound = LCRZO_FALSE;
  while (LCRZO_TRUE) {
    c = *pccurrent;
    if (c == '\\' || c == '/') {
      if (!charfound) {
        /* path is "\\\" which is invalid because there is no server name */
        lcrzo_er(lcrzo_string_free2(&start));
        return (LCRZO_ERR_SPPATHNOTCLEANED);
      }
      pccurrent++;
      break;
    } else if (c == '\0') {
      /* path is "\\server" which is invalid because there is no share name */
      lcrzo_er(lcrzo_string_free2(&start));
      return (LCRZO_ERR_SPPATHNOTCLEANED);
    } else {
      start[i++] = c;
      charfound = LCRZO_TRUE;
    }
    pccurrent++;
  }
  start[i++] = '\\';

  /* skip / or \ */
  while (*pccurrent == '\\' || *pccurrent == '/')
    pccurrent++;

  /* now search the share */
  charfound = LCRZO_FALSE;
  while (LCRZO_TRUE) {
    c = *pccurrent;
    if (c == '\\' || c == '/') {
      break;
    } else if (c == '\0') {
      if (!charfound) {
        /* path is "\\s\" which is invalid because there is no share name */
        lcrzo_er(lcrzo_string_free2(&start));
        return (LCRZO_ERR_SPPATHNOTCLEANED);
      }
      break;
    } else {
      start[i++] = c;
      charfound = LCRZO_TRUE;
    }
    pccurrent++;
  }
  start[i++] = '\\';
  start[i] = '\0';

  /* set output */
  if (ppcstartanalyze != NULL)
    *ppcstartanalyze = pccurrent;
  if (pstart != NULL)
    *pstart = start;
  else {
    lcrzo_er(lcrzo_string_free2(&start));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_path_cleanunix(lcrzo_conststring pathname,
                              lcrzo_string * pcleanedpathname);
int lcrzo_priv_path_cleanunix(lcrzo_conststring pathname,
                              lcrzo_string * pcleanedpathname)
{
  lcrzo_string tmpstr;
  lcrzo_uint8 c, c_1;
  lcrzo_bool lastcharwasslash;
  int i, j, size, lastdirsize;
  char *pc;

  /* return empty filenames */
  if (strlen(pathname) == 0) {
    return (LCRZO_ERR_SPPATHNOTCLEANED);
  }

  /* eventually add "./" */
  if (pathname[0] == '/' || pathname[0] == '\\') {
    lcrzo_er(lcrzo_string_initm_text(pathname, &tmpstr));
  } else {
    /* add "./" */
    lcrzo_er(lcrzo_string_initm_text("./", &tmpstr));
    lcrzo_er(lcrzo_string_appendm_text(pathname, &tmpstr));
  }

  /* convert "\" to "/" */
  size = strlen(tmpstr);
  for (i = 0; i < size; i++) {
    if (tmpstr[i] == '\\')
      tmpstr[i] = '/';
  }

  /* convert "/.$" to "/" */
  size = strlen(tmpstr);
  if (size >= 2) {
    if (tmpstr[size - 2] == '/' && tmpstr[size - 1] == '.') {
      tmpstr[size - 1] = '\0';
    }
  }

  /* convert "/..$" to "/../" */
  size = strlen(tmpstr);
  if (size >= 3) {
    if (tmpstr[size - 3] == '/' && tmpstr[size - 2] == '.'
        && tmpstr[size - 1] == '.') {
      lcrzo_er(lcrzo_string_appendm_char('/', 1, &tmpstr));
    }
  }

  /* convert "/./" to "//" */
  size = strlen(tmpstr);
  for (i = 0, j = 0; i < size - 2; i++) {
    c = tmpstr[i];
    tmpstr[j++] = c;
    if (c == '/' && tmpstr[i + 1] == '.' && tmpstr[i + 2] == '/') {
      i++;
    }
  }
  if (i < size - 1)
    tmpstr[j++] = tmpstr[i++];
  if (i < size)
    tmpstr[j++] = tmpstr[i];
  tmpstr[j] = '\0';

  /* convert "//" to "/" */
  size = strlen(tmpstr);
  c_1 = '_';
  for (i = 0, j = 0; i < size; i++) {
    c = tmpstr[i];
    if (!(c_1 == '/' && c == '/')) {
      tmpstr[j++] = c;
    }
    c_1 = c;
  }
  tmpstr[j] = '\0';

  /* convert /aa/bb/../ to /aa/ */
  size = strlen(tmpstr);
  lastcharwasslash = LCRZO_TRUE;
  for (i = 0, j = 0; i < size - 2; i++) {
    c = tmpstr[i];
    if (lastcharwasslash && c == '.' 
        && tmpstr[i + 1] == '.' && tmpstr[i + 2] == '/') {
      /* go back to the last '/' */
      if (j > 0) {
        c_1 = tmpstr[j - 1];
        tmpstr[j - 1] = '\0';   /* just for strrchr */
        pc = strrchr(tmpstr, '/');
        tmpstr[j - 1] = c_1;
      } else
        pc = NULL;
      if (pc == NULL) {
        /* no '/' found */
        if (tmpstr[0] == '/') {
          /* can not have /.. because / is root */
          lcrzo_er(lcrzo_string_free2(&tmpstr));
          return (LCRZO_ERR_SPPATHROOTDOTDOT);
        } else {
          /* start with ./../, so add ../ */
          tmpstr[j++] = '.';
          tmpstr[j++] = '.';
          tmpstr[j++] = '/';
        }
      } else {
        /* './' found */
        c_1 = tmpstr[j - 1];
        tmpstr[j - 1] = '\0';   /* just for strlen */
        lastdirsize = strlen(tmpstr) - (pc - tmpstr);
        tmpstr[j - 1] = c_1;
        if (lastdirsize == 3 && pc[1] == '.' && pc[2] == '.') {
          /* tmpstr actually end with /../ (between the found character (pc)
             and the strlen of tmpstr */
          tmpstr[j++] = '.';
          tmpstr[j++] = '.';
          tmpstr[j++] = '/';
        } else {
          /* go back to the last '/' */
          j = pc - tmpstr + 1;
        }
      }
      i += 2;                   /* we skip two characters */
      lastcharwasslash = LCRZO_TRUE;
    } else {
      tmpstr[j++] = c;
      lastcharwasslash = (lcrzo_bool) ((c == '/') ? LCRZO_TRUE : LCRZO_FALSE);
    }
  }
  if (i < size - 1)
    tmpstr[j++] = tmpstr[i++];
  if (i < size)
    tmpstr[j++] = tmpstr[i];
  tmpstr[j] = '\0';

  /* convert "/$" to "$", except "^/$" */
  size = strlen(tmpstr);
  if (size >= 2) {
    if (tmpstr[size - 1] == '/') {
      tmpstr[size - 1] = '\0';
    }
  }

  /* set output value */
  if (pcleanedpathname != NULL) {
    if (!strncmp(tmpstr, "./../", 5) || !strcmp(tmpstr, "./..")) {
      /* because tmp always start with "./" so we can have "./../x",
         "./.." but not "./..x" */
      lcrzo_er(lcrzo_string_initm_text(tmpstr + 2, pcleanedpathname));
    } else {
      lcrzo_er(lcrzo_string_initm_text(tmpstr, pcleanedpathname));
    }
  }
  lcrzo_er(lcrzo_string_free2(&tmpstr));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_path_clean(lcrzo_conststring pathname,
                     lcrzo_string * pcleanedpathname)
{
  lcrzo_conststring pc, pcstartanalyze;
  lcrzo_string start, cleaned;
  char c1, c2, c3;
  int ret;

  /* pathname is empty */
  if (pathname[0] == '\0')
    return (LCRZO_ERR_SPPATHNOTCLEANED);

  /* first, we find the start */
  pc = pathname;
  c1 = *pc;
  c2 = *(pc + 1);               /* at the limit, it gets '\0' */
  if (c1 == '.') {
    lcrzo_er(lcrzo_string_initm_text(".", &start));
    pcstartanalyze = pc;
  } else if (c1 == '/' || (c1 == '\\' && c2 != '\\')) {
    lcrzo_er(lcrzo_string_initm_text("/", &start));
    pcstartanalyze = pc;
  } else if (c1 == '\\' && c2 == '\\') {
    /* search "s\sh" */
    ret = lcrzo_priv_path_clean_searchsmb(pc + 2, &start, &pcstartanalyze);
    if (ret == LCRZO_ERR_SPPATHNOTCLEANED) {
      lcrzo_er(lcrzo_string_initm_text("/", &start));
      pcstartanalyze = pc;
    } else if (ret != LCRZO_ERR_OK) {
      return (ret);
    }
  } else if (((c1 >= 'a' && c1 <= 'z') || (c1 >= 'A' && c1 <= 'Z'))
             && c2 == ':') {
    if (c1 >= 'A' && c1 <= 'Z')
      c1 = (char)(c1 - 'A' + 'a');
    lcrzo_er(lcrzo_string_initm_char(c1, 1, &start));
    c3 = *(pc + 2);             /* at the limit, it gets '\0' */
    if (c3 == '/' || c3 == '\\') {
      lcrzo_er(lcrzo_string_appendm_text(":/", &start));
    } else {
      lcrzo_er(lcrzo_string_appendm_text(":.", &start));
    }
    pcstartanalyze = pc + 2;
  } else {
    lcrzo_er(lcrzo_string_initm_text(".", &start));
    pcstartanalyze = pc;
  }

  /* if pcstartanalyze is at end, then there is only the start */
  if (*pcstartanalyze == '\0') {
    lcrzo_er(lcrzo_string_initm_text(start, pcleanedpathname));
    lcrzo_er(lcrzo_string_free2(&start));
    return (LCRZO_ERR_OK);
  }

  /* now, we clean the path */
  ret = lcrzo_priv_path_cleanunix(pcstartanalyze, &cleaned);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_string_free2(&start));
    return (ret);
  }

  /* now, we reconstruct the path */
  lcrzo_er(lcrzo_string_initm_text(start, pcleanedpathname));
  lcrzo_er(lcrzo_string_appendm_text(cleaned + 1, pcleanedpathname));
  lcrzo_er(lcrzo_string_free2(&start));
  lcrzo_er(lcrzo_string_free2(&cleaned));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_path_isabsolute(lcrzo_conststring pathname,
                          lcrzo_bool pathnameisclean,
                          lcrzo_bool * pyes)
{
  lcrzo_string tmpstr;

  lcrzo_er(lcrzo_path_decode_end(pathname, pathnameisclean, &tmpstr));
  if (pyes != NULL) {
    if (tmpstr[0] == '/')
      *pyes = LCRZO_TRUE;
    else
      *pyes = LCRZO_FALSE;
  }
  lcrzo_er(lcrzo_string_free2(&tmpstr));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_path_isrelative(lcrzo_conststring pathname,
                               lcrzo_bool * pyes);
int lcrzo_priv_path_isrelative(lcrzo_conststring pathname,
                               lcrzo_bool * pyes)
{
  if (pyes != NULL) {
    if (pathname[0] == '.')
      *pyes = LCRZO_TRUE;
    else
      *pyes = LCRZO_FALSE;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_path_isrelative(lcrzo_conststring pathname,
                          lcrzo_bool pathnameisclean,
                          lcrzo_bool * pyes)
{
  lcrzo_string clean;
  int ret;

  if (pathnameisclean) {
    return (lcrzo_priv_path_isrelative(pathname, pyes));
  }

  lcrzo_er(lcrzo_path_clean(pathname, &clean));
  ret = lcrzo_priv_path_isrelative(clean, pyes);
  lcrzo_er(lcrzo_string_free2(&clean));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_path_isroot(lcrzo_conststring pathname,
                      lcrzo_bool pathnameisclean,
                      lcrzo_bool * pyes)
{
  lcrzo_string tmpstr;

  lcrzo_er(lcrzo_path_decode_end(pathname, pathnameisclean, &tmpstr));
  if (pyes != NULL) {
    if (tmpstr[0] == '/' && tmpstr[1] == '\0')
      *pyes = LCRZO_TRUE;
    else
      *pyes = LCRZO_FALSE;
  }
  lcrzo_er(lcrzo_string_free2(&tmpstr));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_path_isstart(lcrzo_conststring pathname,
                       lcrzo_bool pathnameisclean,
                       lcrzo_bool * pyes)
{
  lcrzo_string tmpstr;

  lcrzo_er(lcrzo_path_decode_end(pathname, pathnameisclean, &tmpstr));
  if (pyes != NULL) {
    if ((tmpstr[0] == '/' || tmpstr[0] == '.') && tmpstr[1] == '\0')
      *pyes = LCRZO_TRUE;
    else
      *pyes = LCRZO_FALSE;
  }
  lcrzo_er(lcrzo_string_free2(&tmpstr));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_path_decode_root(lcrzo_conststring pathname,
                           lcrzo_bool pathnameisclean,
                           lcrzo_string * proot)
{
  lcrzo_string root;

  lcrzo_er(lcrzo_path_decode_start(pathname, pathnameisclean, &root));
  if (root[strlen(root) - 1] == '.') {
    lcrzo_er(lcrzo_string_free2(&root));
    return (LCRZO_ERR_SPPATHRELATIVE);
  }

  if (proot != NULL)
    *proot = root;
  else {
    lcrzo_er(lcrzo_string_free2(&root));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cleanpath_decode_startend(lcrzo_conststring pathname,
                                         lcrzo_string * pstart,
                                         lcrzo_string * pend);
int lcrzo_priv_cleanpath_decode_startend(lcrzo_conststring pathname,
                                         lcrzo_string * pstart,
                                         lcrzo_string * pend)
{
  lcrzo_conststring pbs;
  lcrzo_uintmax size;
  char c1, c2;

  /* suppose pathname is clean */

  /* first, we find the start */
  c1 = pathname[0];
  c2 = pathname[1];             /* at the limit, it gets '\0' */
  if (c1 == '.') {
    lcrzo_er(lcrzo_string_initm_text(".", pstart));
    lcrzo_er(lcrzo_string_initm_text(pathname, pend));
  } else if (c1 == '/') {
    lcrzo_er(lcrzo_string_initm_text("/", pstart));
    lcrzo_er(lcrzo_string_initm_text(pathname, pend));
  } else if (c1 == '\\' && c2 == '\\') {
    /* search "server\share\" */
    pbs = strchr(pathname + 2, '\\');
    if (pbs == NULL)
      return (LCRZO_ERR_SPPATHNOTCLEANED);
    pbs = strchr(pbs + 1, '\\');
    if (pbs == NULL)
      return (LCRZO_ERR_SPPATHNOTCLEANED);
    if (pstart != NULL) {
      size = pbs - pathname + 1;
      lcrzo_er(lcrzo_string_alloc(size + 1, pstart));
      memcpy(*pstart, pathname, size);
      (*pstart)[size] = '\0';
    }
    if (pend != NULL) {
      lcrzo_er(lcrzo_string_initm_text("/", pend));
      lcrzo_er(lcrzo_string_appendm_text(pbs + 1, pend));
    }
  } else if (isalpha((int)c1) && c2 == ':') {
    if (pstart != NULL) {
      lcrzo_er(lcrzo_string_alloc(4, pstart));
      (*pstart)[0] = pathname[0];
      (*pstart)[1] = pathname[1];
      (*pstart)[2] = pathname[2];
      (*pstart)[3] = '\0';
    }
    lcrzo_er(lcrzo_string_initm_text(pathname + 2, pend));
  } else {
    lcrzo_er(lcrzo_string_initm_text(".", pstart));
    lcrzo_er(lcrzo_string_initm_text(pathname, pend));
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_path_decode_start(lcrzo_conststring pathname,
                            lcrzo_bool pathnameisclean,
                            lcrzo_string * pstart)
{
  lcrzo_string clean;
  int ret;

  if (pathnameisclean) {
    return (lcrzo_priv_cleanpath_decode_startend(pathname, pstart, NULL));
  }

  lcrzo_er(lcrzo_path_clean(pathname, &clean));
  ret = lcrzo_priv_cleanpath_decode_startend(clean, pstart, NULL);
  lcrzo_er(lcrzo_string_free2(&clean));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_path_decode_end(lcrzo_conststring pathname,
                          lcrzo_bool pathnameisclean,
                          lcrzo_string * pend)
{
  lcrzo_string clean;
  int ret;

  if (pathnameisclean) {
    return (lcrzo_priv_cleanpath_decode_startend(pathname, NULL, pend));
  }

  lcrzo_er(lcrzo_path_clean(pathname, &clean));
  ret = lcrzo_priv_cleanpath_decode_startend(clean, NULL, pend);
  lcrzo_er(lcrzo_string_free2(&clean));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_path_decode_parentdir(lcrzo_conststring pathname,
                                lcrzo_bool pathnameisclean,
                                lcrzo_string * pparentdir)
{
  return (lcrzo_path_init_concat(pathname, pathnameisclean, LCRZO_TRUE,
                                 "..", LCRZO_TRUE, pparentdir));
}

/*-------------------------------------------------------------*/
int lcrzo_path_decode_lastitem(lcrzo_conststring pathname,
                               lcrzo_bool pathnameisclean,
                               lcrzo_string * plastitem)
{
  lcrzo_string tmpstr, pc;

  lcrzo_er(lcrzo_path_decode_end(pathname, pathnameisclean, &tmpstr));

  if (strlen(tmpstr) == 1) {
    /* is "." or "/" */
    if (plastitem != NULL)
      *plastitem = tmpstr;
    else {
      lcrzo_er(lcrzo_string_free2(&tmpstr));
    }
    return (LCRZO_ERR_OK);
  }

  /* search the last '/' */
  pc = strrchr(tmpstr, '/');
  if (pc == NULL) {
    lcrzo_er(lcrzo_string_free2(&tmpstr));
    return (lcrzo_ierp("lcrzo_path_decode_lastitem"));
  }

  lcrzo_er(lcrzo_string_initm_text(pc + 1, plastitem));
  lcrzo_er(lcrzo_string_free2(&tmpstr));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cleanpath_init_concat(lcrzo_conststring pathname1,
                                     lcrzo_conststring pathname2,
                                     lcrzo_string * pconcat);
int lcrzo_priv_cleanpath_init_concat(lcrzo_conststring pathname1,
                                     lcrzo_conststring pathname2,
                                     lcrzo_string * pconcat)
{
  lcrzo_bool b;
  lcrzo_string concat;
  int ret;

  /* there is not limit on pathname1
     pathname2 must start with . or /
     if pathname2 starts with /, pathname1 must be a root
   */
  if (pathname2[0] == '.') {
    /* always ok */
  } else if (pathname2[0] == '/') {
    lcrzo_er(lcrzo_path_isroot(pathname1, LCRZO_TRUE, &b));
    if (!b)
      return (LCRZO_ERR_SPPATHCANTINIT);
  } else {
    return (LCRZO_ERR_SPPATHCANTINIT);
  }

  lcrzo_er(lcrzo_string_initm_text(pathname1, &concat));
  lcrzo_er(lcrzo_string_appendm_text("/", &concat));
  lcrzo_er(lcrzo_string_appendm_text(pathname2, &concat));
  ret = lcrzo_path_clean(concat, pconcat);
  lcrzo_er(lcrzo_string_free2(&concat));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_path_init_concat(lcrzo_conststring pathname1,
                           lcrzo_bool pathname1isclean,
                           lcrzo_bool pathname1isadir,
                           lcrzo_conststring pathname2,
                           lcrzo_bool pathname2isclean,
                           lcrzo_string * pconcat)
{
  lcrzo_string pn1, pn2;
  int ret;

  if (!pathname1isadir) {
    lcrzo_er(lcrzo_path_decode_parentdir(pathname1, pathname1isclean, &pn1));
  } else if (!pathname1isclean) {
    lcrzo_er(lcrzo_path_clean(pathname1, &pn1));
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname1, &pn1));
  }

  if (!pathname2isclean) {
    ret = lcrzo_path_clean(pathname2, &pn2);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_string_free2(&pn1));
      return (ret);
    }
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname2, &pn2));
  }

  ret = lcrzo_priv_cleanpath_init_concat(pn1, pn2, pconcat);
  lcrzo_er(lcrzo_string_free2(&pn1));
  lcrzo_er(lcrzo_string_free2(&pn2));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cleanpath_init_abs(lcrzo_conststring pathname1,
                                  lcrzo_conststring pathname2,
                                  lcrzo_string * pabs);
int lcrzo_priv_cleanpath_init_abs(lcrzo_conststring pathname1,
                                  lcrzo_conststring pathname2,
                                  lcrzo_string * pabs)
{
  lcrzo_bool b;

  lcrzo_er(lcrzo_path_isabsolute(pathname2, LCRZO_TRUE, &b));
  if (b) {
    lcrzo_er(lcrzo_string_initm_text(pathname2, pabs));
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzo_path_isabsolute(pathname1, LCRZO_TRUE, &b));
  if (!b)
    return (LCRZO_ERR_SPPATHCANTINIT);

  lcrzo_er(lcrzo_priv_cleanpath_init_concat(pathname1, pathname2, pabs));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_path_init_abs(lcrzo_conststring pathname1,
                        lcrzo_bool pathname1isclean,
                        lcrzo_bool pathname1isadir,
                        lcrzo_conststring pathname2,
                        lcrzo_bool pathname2isclean,
                        lcrzo_string * pabs)
{
  lcrzo_string pn1, pn2;
  int ret;

  if (!pathname1isadir) {
    lcrzo_er(lcrzo_path_decode_parentdir(pathname1, pathname1isclean, &pn1));
  } else if (!pathname1isclean) {
    lcrzo_er(lcrzo_path_clean(pathname1, &pn1));
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname1, &pn1));
  }

  if (!pathname2isclean) {
    ret = lcrzo_path_clean(pathname2, &pn2);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_string_free2(&pn1));
      return (ret);
    }
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname2, &pn2));
  }

  ret = lcrzo_priv_cleanpath_init_abs(pn1, pn2, pabs);
  lcrzo_er(lcrzo_string_free2(&pn1));
  lcrzo_er(lcrzo_string_free2(&pn2));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cleanpath_init_rel(lcrzo_conststring end1,
                                  lcrzo_conststring end2,
                                  lcrzo_string * prel);
int lcrzo_priv_cleanpath_init_rel(lcrzo_conststring end1,
                                  lcrzo_conststring end2,
                                  lcrzo_string * prel)
{
  lcrzo_string tmpstr;
  lcrzo_conststring pc1, pc1lastslash, pc1startcount, pc2, pc2lastslash;

  /*printf("-> %s %s\n", end1, end2); */

  /* find the common part */
  pc1 = end1;
  pc1lastslash = end1;       /* init for relative cases (1="./" and 2="../") */
  pc1startcount = NULL;
  pc2 = end2;
  pc2lastslash = end2;       /* init for relative cases (1="./" and 2="../") */
  while (LCRZO_TRUE) {
    if (*pc1 == '\0') {
      if (*pc2 == '/') {
        pc1startcount = pc1;
        pc2lastslash = pc2;
      } else if (*pc2 == '\0')
        pc1startcount = pc1lastslash;
      else if (*(pc1 - 1) == '/')
        pc1startcount = pc1;
      else
        pc1startcount = pc1lastslash;
      break;
    }
    if (*pc2 == '\0') {
      pc1startcount = pc1lastslash;
      break;
    }
    if (*pc1 != *pc2) {
      pc1startcount = pc1lastslash;
      break;
    }
    if (*pc1 == '/')
      pc1lastslash = pc1;
    if (*pc2 == '/')
      pc2lastslash = pc2;
    pc1++;
    pc2++;
  }

  /* if pc1startcount points to "../xx", we cannot determine */
  if (strlen(pc1startcount) > 2) {
    if (*(pc1startcount) == '.' &&
        *(pc1startcount + 1) == '.' && *(pc1startcount + 2) == '/') {
      return (LCRZO_ERR_SPPATHCANTINIT);
    }
  }

  /* now loop till the end of pc1 to create "../../../" */
  lcrzo_er(lcrzo_string_initm_text("./", &tmpstr));
  pc1 = pc1startcount;
  while (*pc1 != '\0') {
    if (*pc1 == '/') {
      lcrzo_er(lcrzo_string_appendm_text("../", &tmpstr));
    }
    pc1++;
  }

  /* now concatene */
  /*printf("%s %s -> %s %s\n", end1, end2, tmpstr, pc2lastslash); */
  lcrzo_er(lcrzo_string_appendm_text(pc2lastslash, &tmpstr));
  lcrzo_er(lcrzo_path_clean(tmpstr, prel));
  lcrzo_er(lcrzo_string_free2(&tmpstr));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cleanpath_init_rela(lcrzo_conststring pathname1,
                                   lcrzo_conststring pathname2,
                                   lcrzo_string * prela);
int lcrzo_priv_cleanpath_init_rela(lcrzo_conststring pathname1,
                                   lcrzo_conststring pathname2,
                                   lcrzo_string * prela)
{
  lcrzo_string start1, start2, end1, end2;
  lcrzo_bool b;

  /* simple case */
  lcrzo_er(lcrzo_path_isrelative(pathname2, LCRZO_TRUE, &b));
  if (b) {
    lcrzo_er(lcrzo_string_initm_text(pathname2, prela));
    return (LCRZO_ERR_OK);
  }

  /* now we have to ensure starts are equal to continue */
  lcrzo_er(lcrzo_path_decode_start(pathname1, LCRZO_TRUE, &start1));
  lcrzo_er(lcrzo_path_decode_start(pathname2, LCRZO_TRUE, &start2));
  if (strcmp(start1, start2)) {
    lcrzo_er(lcrzo_string_free2(&start1));
    lcrzo_er(lcrzo_string_free2(&start2));
    return (LCRZO_ERR_SPPATHCANTINIT);
  }
  lcrzo_er(lcrzo_string_free2(&start1));
  lcrzo_er(lcrzo_string_free2(&start2));

  /* continue our work with ends only */
  lcrzo_er(lcrzo_path_decode_end(pathname1, LCRZO_TRUE, &end1));
  lcrzo_er(lcrzo_path_decode_end(pathname2, LCRZO_TRUE, &end2));

  /* simple case */
  lcrzo_er(lcrzo_path_isrelative(end2, LCRZO_TRUE, &b));
  if (b) {
    lcrzo_er(lcrzo_string_initm_text(end2, prela));
    lcrzo_er(lcrzo_string_free2(&end1));
    lcrzo_er(lcrzo_string_free2(&end2));
    return (LCRZO_ERR_OK);
  }

  /* we need both to be absolute */
  lcrzo_er(lcrzo_path_isabsolute(end1, LCRZO_TRUE, &b));
  if (!b) {
    lcrzo_er(lcrzo_string_free2(&end1));
    lcrzo_er(lcrzo_string_free2(&end2));
    return (LCRZO_ERR_SPPATHCANTINIT);
  }

  /* now, compute the relative path */
  lcrzo_er(lcrzo_priv_cleanpath_init_rel(end1, end2, prela));
  lcrzo_er(lcrzo_string_free2(&end1));
  lcrzo_er(lcrzo_string_free2(&end2));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_path_init_rela(lcrzo_conststring pathname1,
                         lcrzo_bool pathname1isclean,
                         lcrzo_bool pathname1isadir,
                         lcrzo_conststring pathname2,
                         lcrzo_bool pathname2isclean,
                         lcrzo_string * prel)
{
  lcrzo_string pn1, pn2;
  int ret;

  if (!pathname1isadir) {
    lcrzo_er(lcrzo_path_decode_parentdir(pathname1, pathname1isclean, &pn1));
  } else if (!pathname1isclean) {
    lcrzo_er(lcrzo_path_clean(pathname1, &pn1));
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname1, &pn1));
  }

  if (!pathname2isclean) {
    ret = lcrzo_path_clean(pathname2, &pn2);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_string_free2(&pn1));
      return (ret);
    }
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname2, &pn2));
  }

  ret = lcrzo_priv_cleanpath_init_rela(pn1, pn2, prel);
  lcrzo_er(lcrzo_string_free2(&pn1));
  lcrzo_er(lcrzo_string_free2(&pn2));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_path_init_rela2(lcrzo_conststring pathcommon,
                          lcrzo_bool pathcommonisclean,
                          lcrzo_conststring pathname1,
                          lcrzo_bool pathname1isclean,
                          lcrzo_bool pathname1isadir,
                          lcrzo_conststring pathname2,
                          lcrzo_bool pathname2isclean,
                          lcrzo_string * prela2)
{
  lcrzo_bool b;
  lcrzo_string np1;
  int ret;

  if (pathcommon != NULL) {
    /* we eventually use it */
    lcrzo_er(lcrzo_path_isrelative(pathname1, pathname1isclean, &b));
    if (b) {
      lcrzo_er(lcrzo_path_init_concat(pathcommon, pathcommonisclean,
                                      LCRZO_TRUE,
                                      pathname1, pathname1isclean, &np1));
      ret = lcrzo_path_init_rela(np1, LCRZO_TRUE, pathname1isadir,
                                 pathname2, pathname2isclean, prela2);
      lcrzo_er(lcrzo_string_free2(&np1));
      return (ret);
    }
  }

  /* if we are here, pathcommon was not used */
  lcrzo_er(lcrzo_path_init_rela(pathname1, pathname1isclean, pathname1isadir,
                                pathname2, pathname2isclean, prela2));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_cleanpath_init_relb(lcrzo_conststring pathname1,
                                   lcrzo_conststring pathname2,
                                   lcrzo_string * prelb);
int lcrzo_priv_cleanpath_init_relb(lcrzo_conststring pathname1,
                                   lcrzo_conststring pathname2,
                                   lcrzo_string * prelb)
{
  lcrzo_string start1, start2, end1, end2;
  lcrzo_bool b1, b2;
  int ret;

  /* now we have to ensure starts are equal to continue */
  lcrzo_er(lcrzo_path_decode_start(pathname1, LCRZO_TRUE, &start1));
  lcrzo_er(lcrzo_path_decode_start(pathname2, LCRZO_TRUE, &start2));
  if (strcmp(start1, start2)) {
    lcrzo_er(lcrzo_string_free2(&start1));
    lcrzo_er(lcrzo_string_free2(&start2));
    return (LCRZO_ERR_SPPATHCANTINIT);
  }
  lcrzo_er(lcrzo_string_free2(&start1));
  lcrzo_er(lcrzo_string_free2(&start2));

  /* continue our work with ends only */
  lcrzo_er(lcrzo_path_decode_end(pathname1, LCRZO_TRUE, &end1));
  lcrzo_er(lcrzo_path_decode_end(pathname2, LCRZO_TRUE, &end2));

  /* must be both relative or absolute */
  lcrzo_er(lcrzo_path_isrelative(end1, LCRZO_TRUE, &b1));
  lcrzo_er(lcrzo_path_isrelative(end2, LCRZO_TRUE, &b2));
  if ((b1 && !b2) || (!b1 && b2)) {
    lcrzo_er(lcrzo_string_free2(&end1));
    lcrzo_er(lcrzo_string_free2(&end2));
    return (LCRZO_ERR_SPPATHCANTINIT);
  }

  /* now, compute the relative path */
  ret = lcrzo_priv_cleanpath_init_rel(end1, end2, prelb);
  lcrzo_er(lcrzo_string_free2(&end1));
  lcrzo_er(lcrzo_string_free2(&end2));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_path_init_relb2(lcrzo_conststring pathcommon,
                               lcrzo_bool pathcommonisclean,
                               lcrzo_conststring pathname1,
                               lcrzo_conststring pathname2,
                               lcrzo_string * prelb);
int lcrzo_priv_path_init_relb2(lcrzo_conststring pathcommon,
                               lcrzo_bool pathcommonisclean,
                               lcrzo_conststring pathname1,
                               lcrzo_conststring pathname2,
                               lcrzo_string * prelb)
{
  lcrzo_string start1, start2, end1, end2;
  lcrzo_string pn1, pn2;
  lcrzo_bool b;
  int ret;

  /* use pathcommon if paths are relative */
  lcrzo_er(lcrzo_path_isrelative(pathname1, LCRZO_TRUE, &b));
  if (b) {
    lcrzo_er(lcrzo_path_init_concat(pathcommon, pathcommonisclean, LCRZO_TRUE,
                                    pathname1, LCRZO_TRUE, &pn1));
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname1, &pn1));
  }
  lcrzo_er(lcrzo_path_isrelative(pathname2, LCRZO_TRUE, &b));
  if (b) {
    lcrzo_er(lcrzo_path_init_concat(pathcommon, pathcommonisclean, LCRZO_TRUE,
                                    pathname2, LCRZO_TRUE, &pn2));
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname2, &pn2));
  }

  /* now we have to ensure starts are equal to continue */
  lcrzo_er(lcrzo_path_decode_start(pn1, LCRZO_TRUE, &start1));
  lcrzo_er(lcrzo_path_decode_start(pn2, LCRZO_TRUE, &start2));
  if (strcmp(start1, start2)) {
    lcrzo_er(lcrzo_string_free2(&start1));
    lcrzo_er(lcrzo_string_free2(&start2));
    return (LCRZO_ERR_SPPATHCANTINIT);
  }
  lcrzo_er(lcrzo_string_free2(&start1));
  lcrzo_er(lcrzo_string_free2(&start2));

  /* continue our work with ends only */
  lcrzo_er(lcrzo_path_decode_end(pn1, LCRZO_TRUE, &end1));
  lcrzo_er(lcrzo_path_decode_end(pn2, LCRZO_TRUE, &end2));
  lcrzo_er(lcrzo_string_free2(&pn1));
  lcrzo_er(lcrzo_string_free2(&pn2));

  /* now, compute the relative path */
  ret = lcrzo_priv_cleanpath_init_rel(end1, end2, prelb);
  lcrzo_er(lcrzo_string_free2(&end1));
  lcrzo_er(lcrzo_string_free2(&end2));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_path_init_relb(lcrzo_conststring pathname1,
                         lcrzo_bool pathname1isclean,
                         lcrzo_bool pathname1isadir,
                         lcrzo_conststring pathname2,
                         lcrzo_bool pathname2isclean,
                         lcrzo_string * prelb)
{
  return (lcrzo_path_init_relb2(NULL, LCRZO_FALSE,
                                pathname1, pathname1isclean, pathname1isadir,
                                pathname2, pathname2isclean, prelb));

}

/*-------------------------------------------------------------*/
int lcrzo_path_init_relb2(lcrzo_conststring pathcommon,
                          lcrzo_bool pathcommonisclean,
                          lcrzo_conststring pathname1,
                          lcrzo_bool pathname1isclean,
                          lcrzo_bool pathname1isadir,
                          lcrzo_conststring pathname2,
                          lcrzo_bool pathname2isclean,
                          lcrzo_string * prelb2)
{
  lcrzo_string pn1, pn2;
  int ret;

  if (!pathname1isadir) {
    lcrzo_er(lcrzo_path_decode_parentdir(pathname1, pathname1isclean, &pn1));
  } else if (!pathname1isclean) {
    lcrzo_er(lcrzo_path_clean(pathname1, &pn1));
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname1, &pn1));
  }

  if (!pathname2isclean) {
    ret = lcrzo_path_clean(pathname2, &pn2);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_string_free2(&pn1));
      return (ret);
    }
  } else {
    lcrzo_er(lcrzo_string_initm_text(pathname2, &pn2));
  }

  if (pathcommon == NULL) {
    ret = lcrzo_priv_cleanpath_init_relb(pn1, pn2, prelb2);
  } else {
    ret = lcrzo_priv_path_init_relb2(pathcommon, pathcommonisclean,
                                     pn1, pn2, prelb2);
  }

  lcrzo_er(lcrzo_string_free2(&pn1));
  lcrzo_er(lcrzo_string_free2(&pn2));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_path_type(lcrzo_conststring pathname,
                    lcrzo_int16 * ptype)
{
  struct stat st;

  /*get the stat */
  lcrzo_er(lcrzo_priv_path_stat_get(pathname, &st));
  lcrzo_er(lcrzo_priv_stat_type(st, ptype));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_path_exist(lcrzo_conststring pathname,
                     lcrzo_bool * pyes)
{
  int ret;
  struct stat st;
  lcrzo_bool fileexists;

  fileexists = LCRZO_FALSE;
  ret = lcrzo_priv_path_stat_get(pathname, &st);
  if (ret == LCRZO_ERR_OK) {
    fileexists = LCRZO_TRUE;
  } else if (ret == LCRZO_ERR_OKFILENOTFOUND) {
    fileexists = LCRZO_FALSE;
    lcrzo_er(lcrzo_priv_err_purge_text());
  } else {
    return (ret);
  }

  if (pyes != NULL) {
    *pyes = fileexists;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_open_read(lcrzo_conststring filename,
                         lcrzo_file * pfile)
{
  int ret, fd;
  struct stat st1, st2;

  /*parameters verification */
  if (filename == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*before opening, we verify if the file seems OK */
  lcrzo_er(lcrzo_priv_path_stat_get(filename, &st1));
  lcrzo_er(lcrzo_priv_stat_forbidslink(st1));
  lcrzo_er(lcrzo_priv_stat_forbidsnotreg(st1));

  /*open */
  fd = LAURENT_open(filename, O_RDONLY | O_BINARY);
  if (fd == -1) {
    lcrzo_er(lcrzo_priv_global_set_errmsglcrzo(filename));
    return (LCRZO_ERR_FUOPEN);
  }

  /*verify if the characteristics are the same */
  ret = lcrzo_priv_fd_stat_get(fd, &st2);
  if (ret != LCRZO_ERR_OK) {
    LAURENT_close(fd);
    return (ret);
  }
  ret = lcrzo_priv_stat_forbidschange(st1, st2);
  if (ret != LCRZO_ERR_OK) {
    LAURENT_close(fd);
    return (ret);
  }

  /*eventually close the file descriptor */
  if (pfile == NULL) {
    ret = LAURENT_close(fd);
    if (ret == -1)
      return (LCRZO_ERR_FUCLOSE);
  } else {
    pfile->fd = fd;
    pfile->canclosefd = LCRZO_TRUE;
    pfile->openforreading = LCRZO_TRUE;
    pfile->openforwriting = LCRZO_FALSE;
    pfile->dataend = LCRZO_FALSE;
    pfile->storeddata = NULL;
    pfile->storeddatasize = 0;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#include "sysdep/file_open_write.c"

/*-------------------------------------------------------------*/
#include "sysdep/file_open_append.c"

/*-------------------------------------------------------------*/
int lcrzo_file_close(lcrzo_file * pfile)
{
  int ret;

  if (pfile == NULL)
    return (LCRZO_ERR_OK);

  /* eventually close the file descriptor */
  if (pfile->canclosefd) {
    ret = LAURENT_close(pfile->fd);
    if (ret == -1)
      return (LCRZO_ERR_FUCLOSE);
    pfile->canclosefd = LCRZO_FALSE;
  }

  /* file can not be used */
  pfile->fd = -1;
  pfile->dataend = LCRZO_TRUE;
  pfile->openforreading = LCRZO_FALSE;
  pfile->openforwriting = LCRZO_FALSE;

  /* free data */
  if (pfile->storeddata == NULL && pfile->storeddatasize)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (pfile->storeddatasize) {
    lcrzo_er(lcrzo_data_free2(&(pfile->storeddata)));
    pfile->storeddata = NULL;
    pfile->storeddatasize = 0;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_fd_close(int fd)
{
  int ret;

  ret = LAURENT_close(fd);
  if (ret == -1)
    return (LCRZO_ERR_FUCLOSE);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_stream_close(LCRZOFILE * pstream)
{
  int ret;

  ret = fclose((FILE *) pstream);
  if (ret == -1)
    return (LCRZO_ERR_FUFCLOSE);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_init_fd(int fd,
                       lcrzo_file * pfile)
{
  if (pfile != NULL) {
    pfile->fd = fd;
    pfile->canclosefd = LCRZO_FALSE;
    pfile->openforreading = LCRZO_TRUE;
    pfile->openforwriting = LCRZO_TRUE;
    pfile->dataend = LCRZO_FALSE;
    pfile->storeddata = NULL;
    pfile->storeddatasize = 0;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_init_stream(LCRZOFILE * pstream,
                           lcrzo_file * pfile)
{
  if (pfile != NULL) {
    pfile->fd = LAURENT_fileno((FILE *) pstream);
    pfile->canclosefd = LCRZO_FALSE;
    pfile->openforreading = LCRZO_TRUE;
    pfile->openforwriting = LCRZO_TRUE;
    pfile->dataend = LCRZO_FALSE;
    pfile->storeddata = NULL;
    pfile->storeddatasize = 0;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_fd_init_file(lcrzo_file * pfile,
                       int *pfd)
{
  /* parameter verification */
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pfd != NULL) {
    *pfd = pfile->fd;
    pfile->canclosefd = LCRZO_FALSE; /* because it must be externally closed */
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_stream_init_file(lcrzo_file * pfile,
                           LCRZOFILE ** ppstream)
{
  /* parameter verification */
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (ppstream != NULL) {
    if (pfile->openforreading && !pfile->openforwriting) {
      *ppstream = LAURENT_fdopen(pfile->fd, "rb");
    } else if (!pfile->openforreading && pfile->openforwriting) {
      *ppstream = LAURENT_fdopen(pfile->fd, "wb");
    } else {                    /* don't know so try both */
      *ppstream = LAURENT_fdopen(pfile->fd, "r+wb");
      if (*ppstream == NULL) {
        *ppstream = LAURENT_fdopen(pfile->fd, "rb");
      }
      if (*ppstream == NULL) {
        *ppstream = LAURENT_fdopen(pfile->fd, "wb");
      }
    }
    if (*ppstream == NULL)
      return (LCRZO_ERR_FUFDOPEN);
    pfile->canclosefd = LCRZO_FALSE; /* because it must be externally closed */
  }
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_file_write(lcrzo_file * pfile,
                     lcrzo_constdata data,
                     lcrzo_int32 datasize)
{
  /*parameters verification */
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (!pfile->openforwriting)
    return (LCRZO_ERR_LONOTOPENFORWRITING);

  lcrzo_er(lcrzo_priv_fd_write(pfile->fd, data, datasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_write_msg(lcrzo_file * pfile,
                         lcrzo_constdata data,
                         lcrzo_int32 datasize)
{
  /*parameters verification */
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (!pfile->openforwriting)
    return (LCRZO_ERR_LONOTOPENFORWRITING);

  lcrzo_er(lcrzo_priv_fd_write_msg(pfile->fd, data, datasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_readm(lcrzo_file * pfile,
                     lcrzo_time * pmaxwaitingtime,
                     lcrzo_data * pdataout,
                     lcrzo_int32 * pdataoutsize)
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_file_read(pfile, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract
           (&(pfile->storeddata), &(pfile->storeddatasize), pfile->dataend,
            pdataout, pdataoutsize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_readm_fixed(lcrzo_file * pfile,
                           lcrzo_time * pmaxwaitingtime,
                           lcrzo_int32 datasizetoread,
                           lcrzo_data * pdataout)
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_file_read_fixed(pfile, datasizetoread, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract_fixed(&(pfile->storeddata),
                                               &(pfile->storeddatasize),
                                               pfile->dataend, datasizetoread,
                                               pdataout));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_readm_line(lcrzo_file * pfile,
                          lcrzo_time * pmaxwaitingtime,
                          lcrzo_data * pdataout,
                          lcrzo_int32 * pdataoutsize)
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_file_read_line(pfile, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract_line(&(pfile->storeddata),
                                              &(pfile->storeddatasize),
                                              pfile->dataend, pdataout,
                                              pdataoutsize));

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
#include "sysdep/file_flush.c"

/*-------------------------------------------------------------*/
int lcrzo_file_offset_set(lcrzo_file * pfile,
                          lcrzo_int32 offset)
{
  int ret;

  if (offset < 0)
    return (LCRZO_ERR_PATOOLOW);
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  ret = LAURENT_lseek(pfile->fd, offset, SEEK_SET);
  if (ret < 0)
    return (LCRZO_ERR_FULSEEK);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_offset_get(lcrzo_file * pfile,
                          lcrzo_int32 * poffset)
{
  lcrzo_intmax off;

  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  off = LAURENT_lseek(pfile->fd, 0, SEEK_CUR);
  if (off == -1)
    return (LCRZO_ERR_FULSEEK);
  if (off > 0x7FFFFFFF)
    return (LCRZO_ERR_SPFILE2G);

  if (poffset != NULL)
    *poffset = (lcrzo_int32) off;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_filename_size(lcrzo_conststring filename,
                        lcrzo_int32 * psize)
{
  struct stat st;

  /*get the stat */
  lcrzo_er(lcrzo_priv_path_stat_get(filename, &st));
  lcrzo_er(lcrzo_priv_stat_size(st, psize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_size(lcrzo_file * pfile,
                    lcrzo_int32 * psize)
{
  struct stat st;

  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*get the stat */
  lcrzo_er(lcrzo_priv_fd_stat_get(pfile->fd, &st));
  lcrzo_er(lcrzo_priv_stat_size(st, psize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#include "sysdep/filename_exist.c"

/*-------------------------------------------------------------*/
int lcrzo_filename_create(lcrzo_conststring filename)
{
  return (lcrzo_file_open_write(filename, LCRZO_FALSE, LCRZO_TRUE, NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_filename_remove(lcrzo_conststring filename)
{
  int ret;
  struct stat st1;

  /*parameters verification */
  if (filename == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*before removing, we verify if the file seems OK */
  lcrzo_er(lcrzo_priv_path_stat_get(filename, &st1));
  lcrzo_er(lcrzo_priv_stat_forbidslink(st1));
  lcrzo_er(lcrzo_priv_stat_forbidsnotreg(st1));

  /* unlink the file */
  ret = LAURENT_unlink(filename);
  if (ret == -1) {
    lcrzo_er(lcrzo_priv_global_set_errmsglcrzo(filename));
    return (LCRZO_ERR_FUUNLINK);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_filename_rename(lcrzo_conststring oldfilename,
                          lcrzo_conststring newfilename)
{
  lcrzo_er(lcrzo_filename_copy(oldfilename, newfilename));
  return (lcrzo_filename_remove(oldfilename));
}

/*-------------------------------------------------------------*/
int lcrzo_filename_copy(lcrzo_conststring oldfilename,
                        lcrzo_conststring newfilename)
{
  int ret;
  lcrzo_file filein, fileout;
  lcrzo_data data;
  lcrzo_int32 datasize;

  /*open files */
  lcrzo_er(lcrzo_file_open_read(oldfilename, &filein));
  ret =
    lcrzo_file_open_write(newfilename, LCRZO_FALSE, LCRZO_FALSE, &fileout);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_file_close(&filein));
    return (ret);
  }

  while (LCRZO_TRUE) {
    ret = lcrzo_file_readm(&filein, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret != LCRZO_ERR_OK)
      break;
    ret = lcrzo_file_write(&fileout, data, datasize);
    lcrzo_er(lcrzo_data_free2(&data));
    if (ret != LCRZO_ERR_OK)
      break;
  }
  lcrzo_er(lcrzo_file_close(&filein));
  lcrzo_er(lcrzo_file_close(&fileout));
  if (ret == LCRZO_ERR_OKDATAEND)
    ret = LCRZO_ERR_OK;
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_file_copy(lcrzo_file * pfile,
                    lcrzo_conststring newfilename)
{
  int ret;
  lcrzo_file fileout;
  lcrzo_data data;
  lcrzo_int32 datasize;

  /*open files */
  lcrzo_er(lcrzo_file_offset_set(pfile, 0));
  lcrzo_er(lcrzo_file_open_write(newfilename, LCRZO_FALSE, LCRZO_TRUE,
                                 &fileout));

  ret = LCRZO_ERR_OK;
  while (LCRZO_TRUE) {
    ret = lcrzo_file_readm(pfile, LCRZO_WAIT_INFINITE, &data, &datasize);
    if (ret != LCRZO_ERR_OK)
      break;
    ret = lcrzo_file_write(&fileout, data, datasize);
    lcrzo_er(lcrzo_data_free2(&data));
    if (ret != LCRZO_ERR_OK)
      break;
  }
  lcrzo_er(lcrzo_file_close(&fileout));

  if (ret == LCRZO_ERR_OKDATAEND)
    ret = LCRZO_ERR_OK;
  return (ret);
}

/*-------------------------------------------------------------*/
#include "sysdep/filename_truncate.c"

/*-------------------------------------------------------------*/
#include "sysdep/file_truncate.c"

/*-------------------------------------------------------------*/
int lcrzo_filename_read(lcrzo_conststring filename,
                        lcrzo_data * pdata,
                        lcrzo_int32 * pdatasize)
{
  int ret;
  lcrzo_file file;
  lcrzo_data data, tmpdata;
  lcrzo_int32 datasize, tmpdatasize;

  /* to check if size < 2G */
  lcrzo_er(lcrzo_filename_size(filename, NULL));

  datasize = 0;
  lcrzo_er(lcrzo_data_alloc(datasize, &data));

  /*open file */
  lcrzo_er(lcrzo_file_open_read(filename, &file));

  while (LCRZO_TRUE) {
    ret =
      lcrzo_file_readm(&file, LCRZO_WAIT_INFINITE, &tmpdata, &tmpdatasize);
    if (ret == LCRZO_ERR_OKDATAEND) {
      break;
    } else if (ret == LCRZO_ERR_OK) {
      ret = lcrzo_data_appendm_data(tmpdata, tmpdatasize,
                                    datasize, &data, &datasize);
      lcrzo_er(lcrzo_data_free2(&tmpdata));
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_file_close(&file));
        lcrzo_er(lcrzo_data_free2(&data));
        return (ret);
      }
    } else {
      lcrzo_er(lcrzo_file_close(&file));
      lcrzo_er(lcrzo_data_free2(&data));
      return (ret);
    }
  }
  lcrzo_er(lcrzo_file_close(&file));

  if (pdatasize != NULL)
    *pdatasize = datasize;
  if (pdata != NULL) {
    *pdata = data;
  } else {
    lcrzo_er(lcrzo_data_free2(&data));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_filename_write(lcrzo_conststring filename,
                         lcrzo_bool mustbeanexistentfile,
                         lcrzo_bool mustbeanewfile,
                         lcrzo_constdata data,
                         lcrzo_int32 datasize)
{
  int ret, ret2;
  lcrzo_file file;

  /*open file */
  lcrzo_er(lcrzo_file_open_write(filename, mustbeanexistentfile,
                                 mustbeanewfile, &file));
  ret = lcrzo_file_write(&file, data, datasize);
  ret2 = lcrzo_file_close(&file);
  if (ret == LCRZO_ERR_OK && ret2 != LCRZO_ERR_OK)
    return (ret2);
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_filename_append(lcrzo_conststring filename,
                          lcrzo_bool mustbeanexistentfile,
                          lcrzo_constdata data,
                          lcrzo_int32 datasize)
{
  int ret, ret2;
  lcrzo_file file;

  /*open file */
  lcrzo_er(lcrzo_file_open_append(filename, mustbeanexistentfile, &file));
  ret = lcrzo_file_write(&file, data, datasize);
  ret2 = lcrzo_file_close(&file);
  if (ret == LCRZO_ERR_OK && ret2 != LCRZO_ERR_OK)
    return (ret2);
  return (ret);
}

/*-------------------------------------------------------------*/
#include "sysdep/dir_exist.c"

/*-------------------------------------------------------------*/
#include "sysdep/dir_create.c"

/*-------------------------------------------------------------*/
int lcrzo_dir_remove(lcrzo_conststring dirname)
{
  int ret;
  lcrzo_string cleandirname;

  lcrzo_er(lcrzo_path_clean(dirname, &cleandirname));
  ret = LAURENT_rmdir(cleandirname);
  if (ret == -1) {
    lcrzo_debug_err_ok_activate(LCRZO_FALSE);
    lcrzo_er(lcrzo_priv_global_set_errmsglcrzo(cleandirname));
    lcrzo_er(lcrzo_string_free2(&cleandirname));
    lcrzo_debug_err_ok_activate(LCRZO_TRUE);
    if (errno == ENOENT) {
      lcrzo_er(lcrzo_priv_err_purge_int());
      return (LCRZO_ERR_OKFILENOTFOUND);
    }
    return (LCRZO_ERR_FURMDIR);
  }
  lcrzo_er(lcrzo_string_free2(&cleandirname));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_dir_rename(lcrzo_conststring olddirname,
                     lcrzo_conststring newdirname)
{
  int ret, err;
  lcrzo_string cleanolddirname, cleannewdirname;

  lcrzo_er(lcrzo_path_clean(olddirname, &cleanolddirname));
  ret = lcrzo_path_clean(newdirname, &cleannewdirname);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_string_free2(&cleanolddirname));
    return (ret);
  }

  err = rename(cleanolddirname, cleannewdirname);
  if (err == -1) {
    lcrzo_debug_err_ok_activate(LCRZO_FALSE);
    lcrzo_er(lcrzo_string_free2(&cleanolddirname));
    lcrzo_er(lcrzo_priv_global_set_errmsglcrzo(cleannewdirname));
    lcrzo_er(lcrzo_string_free2(&cleannewdirname));
    lcrzo_debug_err_ok_activate(LCRZO_TRUE);
    if (errno == ENOENT) {
      lcrzo_er(lcrzo_priv_err_purge_int());
      return (LCRZO_ERR_OKFILENOTFOUND);
    }
    return (LCRZO_ERR_FURENAME);
  }
  lcrzo_er(lcrzo_string_free2(&cleannewdirname));
  lcrzo_er(lcrzo_string_free2(&cleanolddirname));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#include "sysdep/dir_count.c"

/*-------------------------------------------------------------*/
#include "sysdep/dir_value_pos.c"
