/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_incl.h"

/*-------------------------------------------------------------*/
int lcrzo_hdrleth_initdefault(lcrzo_hdrleth * phdrleth)
{
  if (phdrleth != NULL) {
    lcrzo_er(lcrzo_etha_initdefault(phdrleth->dst));
    lcrzo_er(lcrzo_etha_initdefault(phdrleth->src));
    phdrleth->type = LCRZO_HDRLETH_TYPE_DEFVAL;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlip_initdefault(lcrzo_hdrlip * phdrlip)
{
  if (phdrlip != NULL) {
    phdrlip->version = 4;
    phdrlip->ihl = LCRZO_HDRLIP_IHL_DEFVAL;
    phdrlip->tos = 0;
    phdrlip->totlen = LCRZO_HDRLIP_TOTLEN_DEFVAL;
    lcrzo_er(lcrzo_uint16_init_rand(1, 0xFFFF, &(phdrlip->id)));
    phdrlip->reserve = 0;
    phdrlip->dontfrag = 0;
    phdrlip->morefrag = 0;
    phdrlip->offsetfrag = 0;
    phdrlip->ttl = 128;
    phdrlip->protocol = LCRZO_HDRLIP_PROTOCOL_DEFVAL;
    phdrlip->check = LCRZO_HDRLIP_CHECK_DEFVAL;
    phdrlip->saddr = 0;
    phdrlip->daddr = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrludp_initdefault(lcrzo_hdrludp * phdrludp)
{
  if (phdrludp != NULL) {
    phdrludp->sport = 0;
    phdrludp->dport = 0;
    phdrludp->len = LCRZO_HDRLUDP_LEN_DEFVAL;
    phdrludp->check = LCRZO_HDRLUDP_CHECK_DEFVAL;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrltcp_initdefault(lcrzo_hdrltcp * phdrltcp)
{
  if (phdrltcp != NULL) {
    phdrltcp->sport = 0;
    phdrltcp->dport = 0;
    lcrzo_er(lcrzo_uint32_init_rand(1, (lcrzo_uint32) 0xFFFFFFFFU,
                                    &(phdrltcp->seqnum)));
    phdrltcp->acknum = 0;
    phdrltcp->doff = LCRZO_HDRLTCP_DOFF_DEFVAL;
    phdrltcp->reserve = 0;
    phdrltcp->urg = 0;
    phdrltcp->ack = 0;
    phdrltcp->psh = 0;
    phdrltcp->rst = 0;
    phdrltcp->syn = 0;
    phdrltcp->fin = 0;
    phdrltcp->window = 1500;
    phdrltcp->check = LCRZO_HDRLTCP_CHECK_DEFVAL;
    phdrltcp->urgptr = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlicmp_initdefault(lcrzo_hdrlicmp * phdrlicmp)
{
  if (phdrlicmp != NULL) {
    phdrlicmp->type = 0;
    phdrlicmp->code = 0;
    phdrlicmp->check = LCRZO_HDRLICMP_CHECK_DEFVAL;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlarp_initdefault(lcrzo_hdrlarp * phdrlarp)
{
  if (phdrlarp != NULL) {
    phdrlarp->hw_type = 1;
    phdrlarp->prot_type = LCRZO_HDRLETH_TYPE_IP;
    phdrlarp->hw_size = 6;
    phdrlarp->prot_size = 4;
    phdrlarp->op = 0;
    lcrzo_er(lcrzo_etha_initdefault(phdrlarp->hw_src));
    lcrzo_er(lcrzo_ipa_initdefault(phdrlarp->prot_src));
    lcrzo_er(lcrzo_etha_initdefault(phdrlarp->hw_dst));
    lcrzo_er(lcrzo_ipa_initdefault(phdrlarp->prot_dst));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrleth_update(lcrzo_hdrleth fieldstomodify,
                         lcrzo_hdrleth * phdreth)
{
  lcrzo_hdrleth def;
  lcrzo_bool ethequal;

  if (phdreth == NULL)
    return (LCRZO_ERR_OK);

  lcrzo_er(lcrzo_hdrleth_initdefault(&def));

  lcrzo_er(lcrzo_etha_equal2(fieldstomodify.src, def.src, &ethequal));
  if (!ethequal)
    memcpy(phdreth->src, fieldstomodify.src, LCRZO_ETHA_MAXBYTES);
  lcrzo_er(lcrzo_etha_equal2(fieldstomodify.dst, def.dst, &ethequal));
  if (!ethequal)
    memcpy(phdreth->dst, fieldstomodify.dst, LCRZO_ETHA_MAXBYTES);
  if (fieldstomodify.type != def.type)
    phdreth->type = fieldstomodify.type;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlip_update(lcrzo_hdrlip fieldstomodify,
                        lcrzo_hdrlip * phdrip)
{
  lcrzo_hdrlip def;

  if (phdrip == NULL)
    return (LCRZO_ERR_OK);

  lcrzo_er(lcrzo_hdrlip_initdefault(&def));

  if (fieldstomodify.version != def.version)
    phdrip->version = fieldstomodify.version;
  if (fieldstomodify.ihl != def.ihl)
    phdrip->ihl = fieldstomodify.ihl;
  if (fieldstomodify.tos != def.tos)
    phdrip->tos = fieldstomodify.tos;
  if (fieldstomodify.totlen != def.totlen)
    phdrip->totlen = fieldstomodify.totlen;
  if (fieldstomodify.id != def.id)
    phdrip->id = fieldstomodify.id;
  if (fieldstomodify.reserve != def.reserve)
    phdrip->reserve = fieldstomodify.reserve;
  if (fieldstomodify.dontfrag != def.dontfrag)
    phdrip->dontfrag = fieldstomodify.dontfrag;
  if (fieldstomodify.morefrag != def.morefrag)
    phdrip->morefrag = fieldstomodify.morefrag;
  if (fieldstomodify.offsetfrag != def.offsetfrag)
    phdrip->offsetfrag = fieldstomodify.offsetfrag;
  if (fieldstomodify.ttl != def.ttl)
    phdrip->ttl = fieldstomodify.ttl;
  if (fieldstomodify.protocol != def.protocol)
    phdrip->protocol = fieldstomodify.protocol;
  phdrip->check = LCRZO_HDRLIP_CHECK_DEFVAL;
  if (fieldstomodify.saddr != def.saddr)
    phdrip->saddr = fieldstomodify.saddr;
  if (fieldstomodify.daddr != def.daddr)
    phdrip->daddr = fieldstomodify.daddr;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrludp_update(lcrzo_hdrludp fieldstomodify,
                         lcrzo_hdrludp * phdrudp)
{
  lcrzo_hdrludp def;

  if (phdrudp == NULL)
    return (LCRZO_ERR_OK);

  lcrzo_er(lcrzo_hdrludp_initdefault(&def));

  if (fieldstomodify.sport != def.sport)
    phdrudp->sport = fieldstomodify.sport;
  if (fieldstomodify.dport != def.dport)
    phdrudp->dport = fieldstomodify.dport;
  if (fieldstomodify.len != def.len)
    phdrudp->len = fieldstomodify.len;
  phdrudp->check = LCRZO_HDRLUDP_CHECK_DEFVAL;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrltcp_update(lcrzo_hdrltcp fieldstomodify,
                         lcrzo_hdrltcp * phdrtcp)
{
  lcrzo_hdrltcp def;

  if (phdrtcp == NULL)
    return (LCRZO_ERR_OK);

  lcrzo_er(lcrzo_hdrltcp_initdefault(&def));

  if (fieldstomodify.sport != def.sport)
    phdrtcp->sport = fieldstomodify.sport;
  if (fieldstomodify.dport != def.dport)
    phdrtcp->dport = fieldstomodify.dport;
  if (fieldstomodify.seqnum != def.seqnum)
    phdrtcp->seqnum = fieldstomodify.seqnum;
  if (fieldstomodify.acknum != def.acknum)
    phdrtcp->acknum = fieldstomodify.acknum;
  if (fieldstomodify.doff != def.doff)
    phdrtcp->doff = fieldstomodify.doff;
  if (fieldstomodify.reserve != def.reserve)
    phdrtcp->reserve = fieldstomodify.reserve;
  if (fieldstomodify.urg != def.urg)
    phdrtcp->urg = fieldstomodify.urg;
  if (fieldstomodify.ack != def.ack)
    phdrtcp->ack = fieldstomodify.ack;
  if (fieldstomodify.psh != def.psh)
    phdrtcp->psh = fieldstomodify.psh;
  if (fieldstomodify.rst != def.rst)
    phdrtcp->rst = fieldstomodify.rst;
  if (fieldstomodify.syn != def.syn)
    phdrtcp->syn = fieldstomodify.syn;
  if (fieldstomodify.fin != def.fin)
    phdrtcp->fin = fieldstomodify.fin;
  if (fieldstomodify.window != def.window)
    phdrtcp->window = fieldstomodify.window;
  phdrtcp->check = LCRZO_HDRLTCP_CHECK_DEFVAL;
  if (fieldstomodify.urgptr != def.urgptr)
    phdrtcp->urgptr = fieldstomodify.urgptr;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlicmp_update(lcrzo_hdrlicmp fieldstomodify,
                          lcrzo_hdrlicmp * phdricmp)
{
  lcrzo_hdrlicmp def;

  if (phdricmp == NULL)
    return (LCRZO_ERR_OK);

  lcrzo_er(lcrzo_hdrlicmp_initdefault(&def));

  if (fieldstomodify.type != def.type)
    phdricmp->type = fieldstomodify.type;
  if (fieldstomodify.code != def.code)
    phdricmp->code = fieldstomodify.code;
  phdricmp->check = LCRZO_HDRLICMP_CHECK_DEFVAL;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlarp_update(lcrzo_hdrlarp fieldstomodify,
                         lcrzo_hdrlarp * phdrarp)
{
  lcrzo_hdrlarp def;
  lcrzo_bool equal;

  if (phdrarp == NULL)
    return (LCRZO_ERR_OK);

  lcrzo_er(lcrzo_hdrlarp_initdefault(&def));

  if (fieldstomodify.hw_type != def.hw_type)
    phdrarp->hw_type = fieldstomodify.hw_type;
  if (fieldstomodify.prot_type != def.prot_type)
    phdrarp->prot_type = fieldstomodify.prot_type;
  if (fieldstomodify.hw_size != def.hw_size)
    phdrarp->hw_size = fieldstomodify.hw_size;
  if (fieldstomodify.prot_size != def.prot_size)
    phdrarp->prot_size = fieldstomodify.prot_size;
  if (fieldstomodify.op != def.op)
    phdrarp->op = fieldstomodify.op;

  lcrzo_er(lcrzo_etha_equal2(fieldstomodify.hw_src, def.hw_src, &equal));
  if (!equal)
    memcpy(phdrarp->hw_src, fieldstomodify.hw_src, LCRZO_ETHA_MAXBYTES);
  lcrzo_er(lcrzo_etha_equal2(fieldstomodify.hw_dst, def.hw_dst, &equal));
  if (!equal)
    memcpy(phdrarp->hw_dst, fieldstomodify.hw_dst, LCRZO_ETHA_MAXBYTES);
  lcrzo_er(lcrzo_ipa_equal2(fieldstomodify.prot_src, def.prot_src, &equal));
  if (!equal)
    memcpy(phdrarp->prot_src, fieldstomodify.prot_src, LCRZO_IPA_MAXBYTES);
  lcrzo_er(lcrzo_ipa_equal2(fieldstomodify.prot_dst, def.prot_dst, &equal));
  if (!equal)
    memcpy(phdrarp->prot_dst, fieldstomodify.prot_dst, LCRZO_IPA_MAXBYTES);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrpeth2_init_hdrleth(lcrzo_hdrleth hdrleth,
                                lcrzo_hdrpeth2 hdrpeth)
{
  /* create the physical header */
  if (hdrpeth != NULL) {
    hdrpeth[0] = hdrleth.dst[0];
    hdrpeth[1] = hdrleth.dst[1];
    hdrpeth[2] = hdrleth.dst[2];
    hdrpeth[3] = hdrleth.dst[3];
    hdrpeth[4] = hdrleth.dst[4];
    hdrpeth[5] = hdrleth.dst[5];
    hdrpeth[6] = hdrleth.src[0];
    hdrpeth[7] = hdrleth.src[1];
    hdrpeth[8] = hdrleth.src[2];
    hdrpeth[9] = hdrleth.src[3];
    hdrpeth[10] = hdrleth.src[4];
    hdrpeth[11] = hdrleth.src[5];
    hdrpeth[12] = (lcrzo_uint8) (hdrleth.type >> 8);
    hdrpeth[13] = (lcrzo_uint8) (hdrleth.type & 0xFF);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrleth_init_hdrpeth2(lcrzo_consthdrpeth2 hdrpeth,
                                lcrzo_hdrleth * phdrleth)
{
  /* check parameters */
  if (hdrpeth == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* set output */
  if (phdrleth != NULL) {
    phdrleth->dst[0] = hdrpeth[0];
    phdrleth->dst[1] = hdrpeth[1];
    phdrleth->dst[2] = hdrpeth[2];
    phdrleth->dst[3] = hdrpeth[3];
    phdrleth->dst[4] = hdrpeth[4];
    phdrleth->dst[5] = hdrpeth[5];
    phdrleth->src[0] = hdrpeth[6];
    phdrleth->src[1] = hdrpeth[7];
    phdrleth->src[2] = hdrpeth[8];
    phdrleth->src[3] = hdrpeth[9];
    phdrleth->src[4] = hdrpeth[10];
    phdrleth->src[5] = hdrpeth[11];
    phdrleth->type = (lcrzo_uint16) ((hdrpeth[12] << 8) | hdrpeth[13]);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrpip2_init_hdrlip(lcrzo_hdrlip hdrlip,
                              lcrzo_hdrpip2 hdrpip)
{
  lcrzo_uint32 fragoff;

  if (hdrlip.version > 0xF)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrlip.ihl > 0xF)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);

  if (hdrlip.reserve != 0 && hdrlip.reserve != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrlip.dontfrag != 0 && hdrlip.dontfrag != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrlip.morefrag != 0 && hdrlip.morefrag != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrlip.offsetfrag > 0x1FFF)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  fragoff = (lcrzo_uint16) ((hdrlip.reserve << 15) |
                            (hdrlip.dontfrag << 14) |
                            (hdrlip.morefrag << 13) | hdrlip.offsetfrag);

  /* create the physical header */
  if (hdrpip != NULL) {
    hdrpip[0] = (lcrzo_uint8) ((hdrlip.version << 4) | hdrlip.ihl);
    hdrpip[1] = hdrlip.tos;
    hdrpip[2] = (lcrzo_uint8) (hdrlip.totlen >> 8);
    hdrpip[3] = (lcrzo_uint8) (hdrlip.totlen & 0xFF);
    hdrpip[4] = (lcrzo_uint8) (hdrlip.id >> 8);
    hdrpip[5] = (lcrzo_uint8) (hdrlip.id & 0xFF);
    hdrpip[6] = (lcrzo_uint8) (fragoff >> 8);
    hdrpip[7] = (lcrzo_uint8) (fragoff & 0xFF);
    hdrpip[8] = hdrlip.ttl;
    hdrpip[9] = hdrlip.protocol;
    hdrpip[10] = (lcrzo_uint8) (hdrlip.check >> 8);
    hdrpip[11] = (lcrzo_uint8) (hdrlip.check & 0xFF);
    hdrpip[12] = (lcrzo_uint8) (hdrlip.saddr >> 24);
    hdrpip[13] = (lcrzo_uint8) (0xFF & (hdrlip.saddr >> 16));
    hdrpip[14] = (lcrzo_uint8) (0xFF & (hdrlip.saddr >> 8));
    hdrpip[15] = (lcrzo_uint8) (0xFF & hdrlip.saddr);
    hdrpip[16] = (lcrzo_uint8) (hdrlip.daddr >> 24);
    hdrpip[17] = (lcrzo_uint8) (0xFF & (hdrlip.daddr >> 16));
    hdrpip[18] = (lcrzo_uint8) (0xFF & (hdrlip.daddr >> 8));
    hdrpip[19] = (lcrzo_uint8) (0xFF & hdrlip.daddr);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlip_init_hdrpip2(lcrzo_consthdrpip2 hdrpip,
                              lcrzo_hdrlip * phdrlip)
{
  /* check parameters */
  if (hdrpip == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* set output */
  if (phdrlip != NULL) {
    phdrlip->version = (lcrzo_uint8) (hdrpip[0] >> 4);
    phdrlip->ihl = (lcrzo_uint8) (hdrpip[0] & 0xF);
    phdrlip->tos = hdrpip[1];
    phdrlip->totlen = (lcrzo_uint16) ((hdrpip[2] << 8) | hdrpip[3]);
    phdrlip->id = (lcrzo_uint16) ((hdrpip[4] << 8) | hdrpip[5]);
    phdrlip->reserve = (lcrzo_uint8) (hdrpip[6] >> 7);
    phdrlip->dontfrag = (lcrzo_uint8) ((hdrpip[6] >> 6) & 1);
    phdrlip->morefrag = (lcrzo_uint8) ((hdrpip[6] >> 5) & 1);
    phdrlip->offsetfrag =
      (lcrzo_uint16) (((hdrpip[6] & 0x1F) << 8) | hdrpip[7]);
    phdrlip->ttl = hdrpip[8];
    phdrlip->protocol = hdrpip[9];
    phdrlip->check = (lcrzo_uint16) ((hdrpip[10] << 8) | hdrpip[11]);
    phdrlip->saddr = (lcrzo_uint32) ((lcrzo_uint32) (hdrpip[12] << 24) |
                                     (lcrzo_uint32) (hdrpip[13] << 16) |
                                     (lcrzo_uint32) (hdrpip[14] << 8) |
                                     (lcrzo_uint32) hdrpip[15]);
    phdrlip->daddr = (lcrzo_uint32) ((lcrzo_uint32) (hdrpip[16] << 24) |
                                     (lcrzo_uint32) (hdrpip[17] << 16) |
                                     (lcrzo_uint32) (hdrpip[18] << 8) |
                                     (lcrzo_uint32) hdrpip[19]);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrpudp2_init_hdrludp(lcrzo_hdrludp hdrludp,
                                lcrzo_hdrpudp2 hdrpudp)
{
  /* create the physical header */
  if (hdrpudp != NULL) {
    hdrpudp[0] = (lcrzo_uint8) (hdrludp.sport >> 8);
    hdrpudp[1] = (lcrzo_uint8) (hdrludp.sport & 0xFF);
    hdrpudp[2] = (lcrzo_uint8) (hdrludp.dport >> 8);
    hdrpudp[3] = (lcrzo_uint8) (hdrludp.dport & 0xFF);
    hdrpudp[4] = (lcrzo_uint8) (hdrludp.len >> 8);
    hdrpudp[5] = (lcrzo_uint8) (hdrludp.len & 0xFF);
    hdrpudp[6] = (lcrzo_uint8) (hdrludp.check >> 8);
    hdrpudp[7] = (lcrzo_uint8) (hdrludp.check & 0xFF);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrludp_init_hdrpudp2(lcrzo_consthdrpudp2 hdrpudp,
                                lcrzo_hdrludp * phdrludp)
{
  /* check parameters */
  if (hdrpudp == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* set output */
  if (phdrludp != NULL) {
    phdrludp->sport = (lcrzo_uint16) ((hdrpudp[0] << 8) | hdrpudp[1]);
    phdrludp->dport = (lcrzo_uint16) ((hdrpudp[2] << 8) | hdrpudp[3]);
    phdrludp->len = (lcrzo_uint16) ((hdrpudp[4] << 8) | hdrpudp[5]);
    phdrludp->check = (lcrzo_uint16) ((hdrpudp[6] << 8) | hdrpudp[7]);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrptcp2_init_hdrltcp(lcrzo_hdrltcp hdrltcp,
                                lcrzo_hdrptcp2 hdrptcp)
{
  lcrzo_uint8 flag1, flag2;

  if (hdrltcp.reserve > 0x3F)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.urg != 0 && hdrltcp.urg != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.ack != 0 && hdrltcp.ack != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.psh != 0 && hdrltcp.psh != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.rst != 0 && hdrltcp.rst != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.syn != 0 && hdrltcp.syn != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.fin != 0 && hdrltcp.fin != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  flag1 = (lcrzo_uint8) ((hdrltcp.doff << 4) | (hdrltcp.reserve >> 2));
  flag2 = (lcrzo_uint8) (((hdrltcp.reserve & 0x3) << 6) | hdrltcp.urg << 5 |
                         hdrltcp.ack << 4 | hdrltcp.psh << 3 | hdrltcp.
                         rst << 2 | hdrltcp.syn << 1 | hdrltcp.fin);

  /* create the physical header */
  if (hdrptcp != NULL) {
    hdrptcp[0] = (lcrzo_uint8) (hdrltcp.sport >> 8);
    hdrptcp[1] = (lcrzo_uint8) (hdrltcp.sport & 0xFF);
    hdrptcp[2] = (lcrzo_uint8) (hdrltcp.dport >> 8);
    hdrptcp[3] = (lcrzo_uint8) (hdrltcp.dport & 0xFF);
    hdrptcp[4] = (lcrzo_uint8) (hdrltcp.seqnum >> 24);
    hdrptcp[5] = (lcrzo_uint8) (0xFF & (hdrltcp.seqnum >> 16));
    hdrptcp[6] = (lcrzo_uint8) (0xFF & (hdrltcp.seqnum >> 8));
    hdrptcp[7] = (lcrzo_uint8) (0xFF & hdrltcp.seqnum);
    hdrptcp[8] = (lcrzo_uint8) (hdrltcp.acknum >> 24);
    hdrptcp[9] = (lcrzo_uint8) (0xFF & (hdrltcp.acknum >> 16));
    hdrptcp[10] = (lcrzo_uint8) (0xFF & (hdrltcp.acknum >> 8));
    hdrptcp[11] = (lcrzo_uint8) (0xFF & hdrltcp.acknum);
    hdrptcp[12] = flag1;
    hdrptcp[13] = flag2;
    hdrptcp[14] = (lcrzo_uint8) (hdrltcp.window >> 8);
    hdrptcp[15] = (lcrzo_uint8) (hdrltcp.window & 0xFF);
    hdrptcp[16] = (lcrzo_uint8) (hdrltcp.check >> 8);
    hdrptcp[17] = (lcrzo_uint8) (hdrltcp.check & 0xFF);
    hdrptcp[18] = (lcrzo_uint8) (hdrltcp.urgptr >> 8);
    hdrptcp[19] = (lcrzo_uint8) (hdrltcp.urgptr & 0xFF);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrltcp_init_hdrptcp2(lcrzo_consthdrptcp2 hdrptcp,
                                lcrzo_hdrltcp * phdrltcp)
{
  /* check parameters */
  if (hdrptcp == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* set output */
  if (phdrltcp != NULL) {
    phdrltcp->sport = (lcrzo_uint16) ((hdrptcp[0] << 8) | hdrptcp[1]);
    phdrltcp->dport = (lcrzo_uint16) ((hdrptcp[2] << 8) | hdrptcp[3]);
    phdrltcp->seqnum = (lcrzo_uint32) ((lcrzo_uint32) (hdrptcp[4] << 24) |
                                       (lcrzo_uint32) (hdrptcp[5] << 16) |
                                       (lcrzo_uint32) (hdrptcp[6] << 8) |
                                       (lcrzo_uint32) hdrptcp[7]);
    phdrltcp->acknum = (lcrzo_uint32) ((lcrzo_uint32) (hdrptcp[8] << 24) |
                                       (lcrzo_uint32) (hdrptcp[9] << 16) |
                                       (lcrzo_uint32) (hdrptcp[10] << 8) |
                                       (lcrzo_uint32) hdrptcp[11]);
    phdrltcp->doff = (lcrzo_uint8) (hdrptcp[12] >> 4);
    phdrltcp->reserve = (lcrzo_uint8) (((0xF & hdrptcp[12]) << 2) |
                                       hdrptcp[13] >> 6);
    phdrltcp->urg = (lcrzo_int8) ((hdrptcp[13] >> 5) & 1);
    phdrltcp->ack = (lcrzo_int8) ((hdrptcp[13] >> 4) & 1);
    phdrltcp->psh = (lcrzo_int8) ((hdrptcp[13] >> 3) & 1);
    phdrltcp->rst = (lcrzo_int8) ((hdrptcp[13] >> 2) & 1);
    phdrltcp->syn = (lcrzo_int8) ((hdrptcp[13] >> 1) & 1);
    phdrltcp->fin = (lcrzo_int8) (hdrptcp[13] & 1);
    phdrltcp->window = (lcrzo_uint16) ((hdrptcp[14] << 8) | hdrptcp[15]);
    phdrltcp->check = (lcrzo_uint16) ((hdrptcp[16] << 8) | hdrptcp[17]);
    phdrltcp->urgptr = (lcrzo_uint16) ((hdrptcp[18] << 8) | hdrptcp[19]);

  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrpicmp2_init_hdrlicmp(lcrzo_hdrlicmp hdrlicmp,
                                  lcrzo_hdrpicmp2 hdrpicmp)
{
  /* create the physical header */
  if (hdrpicmp != NULL) {
    hdrpicmp[0] = hdrlicmp.type;
    hdrpicmp[1] = hdrlicmp.code;
    hdrpicmp[2] = (lcrzo_uint8) (hdrlicmp.check >> 8);
    hdrpicmp[3] = (lcrzo_uint8) (hdrlicmp.check & 0xFF);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlicmp_init_hdrpicmp2(lcrzo_consthdrpicmp2 hdrpicmp,
                                  lcrzo_hdrlicmp * phdrlicmp)
{
  /* check parameters */
  if (hdrpicmp == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* set output */
  if (phdrlicmp != NULL) {
    phdrlicmp->type = hdrpicmp[0];
    phdrlicmp->code = hdrpicmp[1];
    phdrlicmp->check = (lcrzo_uint16) ((hdrpicmp[2] << 8) | hdrpicmp[3]);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrparp2_init_hdrlarp(lcrzo_hdrlarp hdrlarp,
                                lcrzo_hdrparp2 hdrparp)
{
  /* create the physical header */
  if (hdrparp != NULL) {
    hdrparp[0] = (lcrzo_uint8) (hdrlarp.hw_type >> 8);
    hdrparp[1] = (lcrzo_uint8) (hdrlarp.hw_type & 0xFF);
    hdrparp[2] = (lcrzo_uint8) (hdrlarp.prot_type >> 8);
    hdrparp[3] = (lcrzo_uint8) (hdrlarp.prot_type & 0xFF);
    hdrparp[4] = hdrlarp.hw_size;
    hdrparp[5] = hdrlarp.prot_size;
    hdrparp[6] = (lcrzo_uint8) (hdrlarp.op >> 8);
    hdrparp[7] = (lcrzo_uint8) (hdrlarp.op & 0xFF);
    hdrparp[8] = hdrlarp.hw_src[0];
    hdrparp[9] = hdrlarp.hw_src[1];
    hdrparp[10] = hdrlarp.hw_src[2];
    hdrparp[11] = hdrlarp.hw_src[3];
    hdrparp[12] = hdrlarp.hw_src[4];
    hdrparp[13] = hdrlarp.hw_src[5];
    hdrparp[14] = hdrlarp.prot_src[0];
    hdrparp[15] = hdrlarp.prot_src[1];
    hdrparp[16] = hdrlarp.prot_src[2];
    hdrparp[17] = hdrlarp.prot_src[3];
    hdrparp[18] = hdrlarp.hw_dst[0];
    hdrparp[19] = hdrlarp.hw_dst[1];
    hdrparp[20] = hdrlarp.hw_dst[2];
    hdrparp[21] = hdrlarp.hw_dst[3];
    hdrparp[22] = hdrlarp.hw_dst[4];
    hdrparp[23] = hdrlarp.hw_dst[5];
    hdrparp[24] = hdrlarp.prot_dst[0];
    hdrparp[25] = hdrlarp.prot_dst[1];
    hdrparp[26] = hdrlarp.prot_dst[2];
    hdrparp[27] = hdrlarp.prot_dst[3];
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlarp_init_hdrparp2(lcrzo_consthdrparp2 hdrparp,
                                lcrzo_hdrlarp * phdrlarp)
{
  /* check parameters */
  if (hdrparp == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* set output */
  if (phdrlarp != NULL) {
    phdrlarp->hw_type = (lcrzo_uint16) ((hdrparp[0] << 8) | hdrparp[1]);
    phdrlarp->prot_type = (lcrzo_uint16) ((hdrparp[2] << 8) | hdrparp[3]);
    phdrlarp->hw_size = hdrparp[4];
    phdrlarp->prot_size = hdrparp[5];
    phdrlarp->op = (lcrzo_uint16) ((hdrparp[6] << 8) | hdrparp[7]);
    phdrlarp->hw_src[0] = hdrparp[8];
    phdrlarp->hw_src[1] = hdrparp[9];
    phdrlarp->hw_src[2] = hdrparp[10];
    phdrlarp->hw_src[3] = hdrparp[11];
    phdrlarp->hw_src[4] = hdrparp[12];
    phdrlarp->hw_src[5] = hdrparp[13];
    phdrlarp->prot_src[0] = hdrparp[14];
    phdrlarp->prot_src[1] = hdrparp[15];
    phdrlarp->prot_src[2] = hdrparp[16];
    phdrlarp->prot_src[3] = hdrparp[17];
    phdrlarp->hw_dst[0] = hdrparp[18];
    phdrlarp->hw_dst[1] = hdrparp[19];
    phdrlarp->hw_dst[2] = hdrparp[20];
    phdrlarp->hw_dst[3] = hdrparp[21];
    phdrlarp->hw_dst[4] = hdrparp[22];
    phdrlarp->hw_dst[5] = hdrparp[23];
    phdrlarp->prot_dst[0] = hdrparp[24];
    phdrlarp->prot_dst[1] = hdrparp[25];
    phdrlarp->prot_dst[2] = hdrparp[26];
    phdrlarp->prot_dst[3] = hdrparp[27];
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrpip2_get_version(lcrzo_hdrpip2 hdrpip2,
                                   lcrzo_uint8 * pversion)
{
  if (hdrpip2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pversion != NULL) {
    *pversion = (lcrzo_uint8) (hdrpip2[0] >> 4);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrpip2_get_ihl(lcrzo_hdrpip2 hdrpip2,
                               lcrzo_uint8 * pihl)
{
  if (hdrpip2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pihl != NULL) {
    *pihl = (lcrzo_uint8) (hdrpip2[0] & 0xF);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrpip2_get_totlen(lcrzo_hdrpip2 hdrpip2,
                                  lcrzo_uint16 * ptotlen)
{
  if (hdrpip2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (ptotlen != NULL) {
    *ptotlen = (lcrzo_uint16) ((hdrpip2[2] << 8) | hdrpip2[3]);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrpip2_get_check(lcrzo_hdrpip2 hdrpip2,
                                 lcrzo_uint16 * pcheck)
{
  if (hdrpip2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pcheck != NULL) {
    *pcheck = (lcrzo_uint16) ((hdrpip2[10] << 8) | hdrpip2[11]);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrpip2_set_check(lcrzo_hdrpip2 hdrpip2,
                                 lcrzo_uint16 check)
{
  if (hdrpip2 != NULL) {
    hdrpip2[10] = (lcrzo_uint8) (check >> 8);
    hdrpip2[11] = (lcrzo_uint8) (check & 0xFF);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrpudp2_get_check(lcrzo_hdrpudp2 hdrpudp2,
                                  lcrzo_uint16 * pcheck)
{
  if (hdrpudp2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pcheck != NULL) {
    *pcheck = (lcrzo_uint16) ((hdrpudp2[6] << 8) | hdrpudp2[7]);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrpudp2_set_check(lcrzo_hdrpudp2 hdrpudp2,
                                  lcrzo_uint16 check)
{
  if (hdrpudp2 != NULL) {
    hdrpudp2[6] = (lcrzo_uint8) (check >> 8);
    hdrpudp2[7] = (lcrzo_uint8) (check & 0xFF);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrptcp2_get_doff(lcrzo_hdrptcp2 hdrptcp2,
                                 lcrzo_uint8 * pdoff)
{
  if (hdrptcp2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pdoff != NULL) {
    *pdoff = (lcrzo_uint8) (hdrptcp2[12] >> 4);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrptcp2_get_check(lcrzo_hdrptcp2 hdrptcp2,
                                  lcrzo_uint16 * pcheck)
{
  if (hdrptcp2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pcheck != NULL) {
    *pcheck = (lcrzo_uint16) ((hdrptcp2[16] << 8) | hdrptcp2[17]);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrptcp2_set_check(lcrzo_hdrptcp2 hdrptcp2,
                                  lcrzo_uint16 check)
{
  if (hdrptcp2 != NULL) {
    hdrptcp2[16] = (lcrzo_uint8) (check >> 8);
    hdrptcp2[17] = (lcrzo_uint8) (check & 0xFF);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrpicmp2_get_check(lcrzo_hdrpicmp2 hdrpicmp2,
                                   lcrzo_uint16 * pcheck)
{
  if (hdrpicmp2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pcheck != NULL) {
    *pcheck = (lcrzo_uint16) ((hdrpicmp2[2] << 8) | hdrpicmp2[3]);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrpicmp2_set_check(lcrzo_hdrpicmp2 hdrpicmp2,
                                   lcrzo_uint16 check)
{
  if (hdrpicmp2 != NULL) {
    hdrpicmp2[2] = (lcrzo_uint8) (check >> 8);
    hdrpicmp2[3] = (lcrzo_uint8) (check & 0xFF);
  }
  return (LCRZO_ERR_OK);
}
