/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_incl.h"

/*-------------------------------------------------------------*/
#include "sysdep/priv_wait_kbd_wait.c"

/*-------------------------------------------------------------*/
#include "sysdep/priv_wait_kbd_read.c"

/*-------------------------------------------------------------*/
#include "sysdep/priv_kbd_key_echo.c"
#include "sysdep/priv_kbd_key_noecho.c"
#include "sysdep/priv_kbd_key_press.c"
#include "sysdep/priv_kbd_line_echo.c"
#include "sysdep/priv_kbd_line_noecho.c"

/*-------------------------------------------------------------*/
#include "sysdep/priv_kbd_set_echo.c"

/*-------------------------------------------------------------*/
#include "sysdep/kbd_init.c"

/*-------------------------------------------------------------*/
int lcrzo_kbd_close(lcrzo_kbd * pkbd)
{
  if (pkbd == NULL)
    return (LCRZO_ERR_OK);

  /* kbd can not be used */
  pkbd->fd = -1;
  pkbd->thehandle = NULL;
  lcrzo_debug_notset_int16(pkbd->type);
  lcrzo_debug_notset_bool(pkbd->echokeypresses);
  pkbd->dataend = LCRZO_TRUE;
  pkbd->openforreading = LCRZO_FALSE;
  pkbd->openforwriting = LCRZO_FALSE;

  /* free data */
  if (pkbd->storeddata == NULL && pkbd->storeddatasize)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (pkbd->storeddatasize) {
    lcrzo_er(lcrzo_data_free2(&(pkbd->storeddata)));
    pkbd->storeddata = NULL;
    pkbd->storeddatasize = 0;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_kbd_readm(lcrzo_kbd * pkbd,
                    lcrzo_time * pmaxwaitingtime,
                    lcrzo_data * pdataout,
                    lcrzo_int32 * pdataoutsize)
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pkbd == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_kbd_read(pkbd, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract(&(pkbd->storeddata),
                                         &(pkbd->storeddatasize),
                                         pkbd->dataend,
                                         pdataout, pdataoutsize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_kbd_readm_fixed(lcrzo_kbd * pkbd,
                          lcrzo_time * pmaxwaitingtime,
                          lcrzo_int32 datasizetoread,
                          lcrzo_data * pdataout)
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pkbd == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_kbd_read_fixed(pkbd, datasizetoread, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract_fixed(&(pkbd->storeddata),
                                               &(pkbd->storeddatasize),
                                               pkbd->dataend, datasizetoread,
                                               pdataout));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_kbd_readm_line(lcrzo_kbd * pkbd,
                         lcrzo_time * pmaxwaitingtime,
                         lcrzo_data * pdataout,
                         lcrzo_int32 * pdataoutsize)
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pkbd == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_kbd_read_line(pkbd, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract_line(&(pkbd->storeddata),
                                              &(pkbd->storeddatasize),
                                              pkbd->dataend, pdataout,
                                              pdataoutsize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#include "sysdep/kbd_purge.c"

/*-------------------------------------------------------------*/
int lcrzo_kbddefault_purge(void)
{
  lcrzo_kbd kbd;

  lcrzo_er(lcrzo_kbd_initdefault(&kbd));
  lcrzo_er(lcrzo_kbd_purge(&kbd));
  lcrzo_er(lcrzo_kbd_close(&kbd));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_kbd_set_echo(lcrzo_kbd * pkbd,
                       lcrzo_bool echokeypresses)
{
  /* parameter verification */
  if (pkbd == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* no need */
  if (pkbd->type != LCRZO_PRIV_HANDLE_TYPE_CONSOLE) {
    return (LCRZO_ERR_OK);
  }

  if (pkbd->echokeypresses && echokeypresses) {
    return (LCRZO_ERR_OK);
  }
  if (!pkbd->echokeypresses && !echokeypresses) {
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzo_priv_kbd_set_echo(pkbd, echokeypresses));

  return (LCRZO_ERR_OK);
}
