/*en    Laurent Constantin's network library (lcrzo)
                 lcrzo_kbd module

  Functions herein allow to read data from keyboard.
*/
/*fr    Bibliotheque reseau de Laurent Constantin (lcrzo)
                 Module lcrzo_kbd

  Les fonctions presentes dans ce module assurent la gestion
du clavier.
*/

/*-------------------------------------------------------------*/
/*en definition of a lcrzo_kbd. You should not use its
   fields directly. */
/*fr definition d'un lcrzo_kbd. Vous ne devriez pas utiliser
   directement ses champs. */
typedef struct
{ int fd;
  LCRZOHANDLE thehandle;
  lcrzo_int16 type;
  lcrzo_bool openforreading;
  lcrzo_bool openforwriting;
  lcrzo_bool dataend;
  lcrzo_data storeddata;
  lcrzo_int32 storeddatasize;
  lcrzo_bool echokeypresses;
} lcrzo_kbd;

/*-------------------------------------------------------------*/
/*en Name : lcrzo_kbd_initdefault
   Description :
     Initialize a lcrzo_kbd. Under unix, it attachs it to the
     file descriptor 0. Under windows, it is attached to the
     console.
   Input parameter(s) :
   Input/output parameter(s) :
   Output parameter(s) :
     *pkbd : lcrzo_kbd which is initialized 
   Normal return values :
     LCRZO_ERR_OK : ok
*/
/*fr Nom : lcrzo_kbd_initdefault
   Description :
     Initialise un lcrzo_kbd. Sous Unix, il est attache au
     descripteur 0. Sous Windows, il est attache a la console.
   Parametre(s) d'entree :
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
     *pkbd : lcrzo_kbd qui est initialise
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
*/
int lcrzo_kbd_initdefault(lcrzo_kbd *pkbd);

/*-------------------------------------------------------------*/
/*en Name : lcrzo_kbd_init_fd
   Description :
     Initialize a lcrzo_kbd associated to the file descriptor fd.
     This function is only valid under Unix.
   Input parameter(s) :
     fd : file descriptor attached to a tty.
   Input/output parameter(s) :
   Output parameter(s) :
     *pkbd : lcrzo_kbd which is initialized 
   Normal return values :
     LCRZO_ERR_OK : ok
*/
/*fr Nom : lcrzo_kbd_init_fd
   Description :
     Initialise un lcrzo_kbd attache au descripteur fd.
     Cette fonction n'est valide que sous Unix.
   Parametre(s) d'entree :
     fd : descripteur de fichier associe au tty.
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
     *pkbd : lcrzo_kbd qui est initialise
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
*/
int lcrzo_kbd_init_fd(int fd,
		      lcrzo_kbd *pkbd);

/*-------------------------------------------------------------*/
/*en Name : lcrzo_kbd_init_handle
   Description :
     Initialize a lcrzo_kbd associated to a HANDLE.
     This function is only valid under Windows.
   Input parameter(s) :
     thehandle : HANDLE associated to console.
   Input/output parameter(s) :
   Output parameter(s) :
     *pkbd : lcrzo_kbd which is initialized 
   Normal return values :
     LCRZO_ERR_OK : ok
*/
/*fr Nom : lcrzo_kbd_init_handle
   Description :
     Initialise un lcrzo_kbd attache a un HANDLE.
     Cette fonction n'est valide que sous Windows.
   Parametre(s) d'entree :
     thehandle : HANDLE associe a la console.
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
     *pkbd : lcrzo_kbd qui est initialise
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
*/
int lcrzo_kbd_init_handle(LCRZOHANDLE thehandle,
			  lcrzo_kbd *pkbd);

/*-------------------------------------------------------------*/
/*en Name : lcrzo_kbd_close
   Description :
     Close a kbd.
   Input parameter(s) :
     pkbd : lcrzo_kbd to close
   Input/output parameter(s) :
   Output parameter(s) :
   Normal return values :
     LCRZO_ERR_OK : ok
*/
/*fr Nom : lcrzo_kbd_close
   Description :
     Ferme un kbd.
   Parametre(s) d'entree :
     pkbd : lcrzo_kbd a fermer
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
*/
int lcrzo_kbd_close(lcrzo_kbd *pkbd);

/*-------------------------------------------------------------*/
/*en Name : lcrzo_kbd_readm
   Description :
     Allocate a memory array, and initialize it with
     data read from a kbd.
   Input parameter(s) :
     *pkbd : lcrzo_kbd where to read
     *pmaxwaitingtime : maximum waiting time (can be a lcrzo_time,
                        LCRZO_WAIT_NODELAY or LCRZO_WAIT_INFINITE)
   Input/output parameter(s) :
   Output parameter(s) :
     *pdataout : pointer which will be malloced (so, the
                 memory will have to be freed by the
                 user with 'free(*pdataout)').
                 The allocated memory will be set with read data
     *pdataoutsize : reached size of *pdataout
   Normal return values :
     LCRZO_ERR_OK : ok
     LCRZO_ERR_OKTEMPDATAEND : the function received nothing
                               before pmaxwaitingtime, so it 
			       exited without reading data
*/
/*fr Nom : lcrzo_kbd_readm
   Description :
     Alloue une zone de memoire et l'initialise avec
     des donnees lues depuis un fichier.
   Parametre(s) d'entree :
     *pkbd : lcrzo_kbd ou lire
     *pmaxwaitingtime : temps maximum d'attente (peut etre un
                        lcrzo_time, LCRZO_WAIT_NODELAY ou
			LCRZO_WAIT_INFINITE)
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
     *pdataout : pointeur qui sera alloue (la memoire
                 devra etre liberee par l'utilisateur 
                 avec 'free(*pdataout)').
                 Cette memoire allouee recevra les donnees lues
     *pdataoutsize : taille atteinte par *pdataout
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
     LCRZO_ERR_OKTEMPDATAEND : la fonction n'a pas recu de donnees
                               avant pmaxwaitingtime, donc on 
                               retourne sans donnees.
*/
int lcrzo_kbd_readm(lcrzo_kbd *pkbd,
		    lcrzo_time *pmaxwaitingtime, 
		    lcrzo_data *pdataout,
		    lcrzo_int32 *pdataoutsize);

/*en Name : lcrzo_kbd_readm_fixed
   Description :
     Allocate a memory array, and initialize it with
     data of fixed size read from a kbd.
   Input parameter(s) :
     *pkbd : lcrzo_kbd where to read
     *pmaxwaitingtime : maximum waiting time (can be a lcrzo_time,
                        LCRZO_WAIT_NODELAY or LCRZO_WAIT_INFINITE)
     datasizetoread : size to read
   Input/output parameter(s) :
   Output parameter(s) :
     *pdataout : pointer which will be malloced (so, the
                 memory will have to be freed by the
                 user with 'free(*pdataout)').
                 The allocated memory will be set with read data
   Normal return values :
     LCRZO_ERR_OK : ok
     LCRZO_ERR_OKTEMPDATAEND : the function received nothing
                               (or no sufficient data) before
                               pmaxwaitingtime, so it 
			       exited without reading data
*/
/*fr Nom : lcrzo_kbd_readm_fixed
   Description :
     Alloue une zone de memoire et l'initialise avec
     des donnees de taille constante lues depuis un fichier.
   Parametre(s) d'entree :
     *pkbd : lcrzo_kbd ou lire
     *pmaxwaitingtime : temps maximum d'attente (peut etre un
                        lcrzo_time, LCRZO_WAIT_NODELAY ou
			LCRZO_WAIT_INFINITE)
     datasizetoread : taille des donnees a lire
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
     *pdataout : pointeur qui sera alloue (la memoire
                 devra etre liberee par l'utilisateur 
                 avec 'free(*pdataout)').
                 Cette memoire allouee recevra les donnees lues
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
     LCRZO_ERR_OKTEMPDATAEND : la fonction n'a pas recu de donnees
                               (ou pas assez de donnees) avant
                               pmaxwaitingtime, donc on 
                               retourne sans donnees.
*/
int lcrzo_kbd_readm_fixed(lcrzo_kbd *pkbd,
			  lcrzo_time *pmaxwaitingtime, 
			  lcrzo_int32 datasizetoread,
			  lcrzo_data *pdataout);

/*en Name : lcrzo_kbd_readm_line
   Description :
     Allocate a memory array, and initialize it with
     data read from a kbd. We stop at first '\r' or '\n' (which
     is not stored in data).
   Input parameter(s) :
     *pkbd : lcrzo_kbd where to read
     *pmaxwaitingtime : maximum waiting time (can be a lcrzo_time,
                        LCRZO_WAIT_NODELAY or LCRZO_WAIT_INFINITE)
   Input/output parameter(s) :
   Output parameter(s) :
     *pdataout : pointer which will be malloced (so, the
                 memory will have to be freed by the
                 user with 'free(*pdataout)').
                 The allocated memory will be set with read data
     *pdataoutsize : reached size of *pdataout
   Normal return values :
     LCRZO_ERR_OK : ok
     LCRZO_ERR_OKTEMPDATAEND : the function received nothing
                               (or data containing no end of line)
                               before pmaxwaitingtime, so it 
			       exited without reading data
*/
/*fr Nom : lcrzo_kbd_readm_line
   Description :
     Alloue une zone de memoire et l'initialise avec
     des donnees lues depuis un fichier. On s'arrete de lire
     au premier '\r' ou '\n' (qui n'est pas stocke dans
     *pdataout).
   Parametre(s) d'entree :
     *pkbd : lcrzo_kbd ou lire
     *pmaxwaitingtime : temps maximum d'attente (peut etre un
                        lcrzo_time, LCRZO_WAIT_NODELAY ou
			LCRZO_WAIT_INFINITE)
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
     *pdataout : pointeur qui sera alloue (la memoire
                 devra etre liberee par l'utilisateur 
                 avec 'free(*pdataout)').
                 Cette memoire allouee recevra les donnees lues
     *pdataoutsize : taille atteinte par *pdataout
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
     LCRZO_ERR_OKTEMPDATAEND : la fonction n'a pas recu de donnees
                               (ou des donnes de ccontenant pas de
                               saut de ligne) avant pmaxwaitingtime,
			       donc on retourne sans donnees.
*/
int lcrzo_kbd_readm_line(lcrzo_kbd *pkbd,
			 lcrzo_time *pmaxwaitingtime, 
			 lcrzo_data *pdataout,
			 lcrzo_int32 *pdataoutsize);

/*-------------------------------------------------------------*/
/*en Name : lcrzo_kbd_purge
   Description :
     Empty key presses stored in the buffer.
   Input parameter(s) :
     *pkbd : lcrzo_kbd
   Input/output parameter(s) :
   Output parameter(s) :
   Normal return values :
     LCRZO_ERR_OK : ok
*/
/*fr Nom : lcrzo_kbd_purge
   Description :
     Purge les touches stockees dans le buffer.
   Parametre(s) d'entree :
     *pkbd : lcrzo_kbd
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
*/
int lcrzo_kbd_purge(lcrzo_kbd *pkbd);

/*-------------------------------------------------------------*/
/*en Name : lcrzo_kbddefault_purge
   Description :
     Empty key presses stored in the buffer.
   Input parameter(s) :
   Input/output parameter(s) :
   Output parameter(s) :
   Normal return values :
     LCRZO_ERR_OK : ok
*/
/*fr Nom : lcrzo_kbddefault_purge
   Description :
     Purge les touches stockees dans le buffer.
   Parametre(s) d'entree :
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
*/
int lcrzo_kbddefault_purge(void);

/*-------------------------------------------------------------*/
/*en Name : lcrzo_kbd_set_echo
   Description :
     Indicates if key pressed have to be shown.
   Input parameter(s) :
     *pkbd : lcrzo_kbd
     echokeypresses : true if display it
   Input/output parameter(s) :
   Output parameter(s) :
   Normal return values :
     LCRZO_ERR_OK : ok
*/
/*fr Nom : lcrzo_kbd_set_echo
   Description :
     Indique les les touches pressees doivent etre affichees.
   Parametre(s) d'entree :
     *pkbd : lcrzo_kbd
     echokeypresses : vrai si il faut afficher
   Parametre(s) d'entree-sortie :
   Parametre(s) de sortie :
   Valeurs de retour normales :
     LCRZO_ERR_OK : ok
*/
int lcrzo_kbd_set_echo(lcrzo_kbd *pkbd,
		       lcrzo_bool echokeypresses);

