/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_incl.h"

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_LIST_MEMLEAK==1
void *lcrzo_debug_list_malloc(lcrzo_int32 size);
void *lcrzo_debug_list_malloc(lcrzo_int32 size)
{
  lcrzo_ptr ptr;
  int ret;

  ret = lcrzo_ptr_alloc(size, &ptr);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_err_print(ret);
    lcrzo_process_current_exit(ret);
  }

  return (ptr);
}
#else
 #define lcrzo_debug_list_malloc(s) malloc(s)
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_LIST_MEMLEAK==1
/* not : this does not work on multiprocess, because the child
   might try to free me allocated in lcrzo_conf of the father */
void lcrzo_debug_list_free(lcrzo_ptr ptr);
void lcrzo_debug_list_free(lcrzo_ptr ptr)
{
  int ret;

  ret = lcrzo_ptr_free2(&ptr);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_err_print(ret);
    lcrzo_process_current_exit(ret);
  }
}
#else
 #define lcrzo_debug_list_free(s) free(s)
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_LIST_CHECK==1
int lcrzo_debug_list_check_workpos(lcrzo_conststring functionname,
                                   lcrzo_list * plist,
                                   lcrzo_int32 workpos,
                                   lcrzo_listitem * pwork);
int lcrzo_debug_list_check_workpos(lcrzo_conststring functionname,
                                   lcrzo_list * plist,
                                   lcrzo_int32 workpos,
                                   lcrzo_listitem * pwork)
{
  lcrzo_int32 itemcountplusone;

  itemcountplusone = plist->itemcount + 1;

  if (workpos < 0) {
    return (lcrzo_ie(functionname, "workpos<0", "workpos<0"));
  }
  if (workpos > itemcountplusone) {
    return (lcrzo_ie(functionname, "workpos>m", "workpos>m"));
  }
  if (pwork == NULL && workpos != 0) {
    return (lcrzo_ie(functionname,
                     "pwork==NULL workpos!=0", "pwork==NULL workpos!=0"));
  }
  if (pwork == (lcrzo_listitem *) plist) {
    if (workpos != 0 && workpos != itemcountplusone) {
      return (lcrzo_ie(functionname,
                       "pwork==plist workpos!=0_max",
                       "pwork==plist workpos!=0_max"));
    }
  } else {
    if (workpos == 0 || workpos == itemcountplusone) {
      return (lcrzo_ie(functionname,
                       "pwork!=plist workpos==0_max",
                       "pwork!=plist workpos==0_max"));
    }
  }

  return (LCRZO_ERR_OK);
}

 #define lcrzo__debug_list_check_workpos(a,b,c,d) {lcrzo_er(lcrzo_debug_list_check_workpos(a,b,c,d));}
#else
 #define lcrzo__debug_list_check_workpos(a,b,c,d) {}
#endif

/*-------------------------------------------------------------*/
#if LCRZO_DEBUG_LIST_CHECK==1
int lcrzo_debug_list_check(lcrzo_conststring functionname,
                           lcrzo_list * plist);
int lcrzo_debug_list_check(lcrzo_conststring functionname,
                           lcrzo_list * plist)
{
  lcrzo_int32 i;
  lcrzo_listitem *pcurrent, *pnext;
  lcrzo_bool founderror;

  /* check working pointers */
  if (plist->work1pos < 0) {
    return (lcrzo_ie2(functionname, "plist->work1pos<0"));
  }
  if (plist->work2pos < 0) {
    return (lcrzo_ie2(functionname, "plist->work2pos<0"));
  }
  if (plist->pwork1 == NULL && plist->work1pos != 0) {
    return (lcrzo_ie2
            (functionname, "plist->pwork1==NULL plist->work1pos!=0"));
  }
  if (plist->pwork1 != NULL && plist->work1pos == 0) {
    return (lcrzo_ie2
            (functionname, "plist->pwork1!=NULL plist->work1pos==0"));
  }
  if (plist->pwork2 == NULL && plist->work2pos != 0) {
    return (lcrzo_ie2
            (functionname, "plist->pwork2==NULL plist->work2pos!=0"));
  }
  if (plist->pwork2 != NULL && plist->work2pos == 0) {
    return (lcrzo_ie2
            (functionname, "plist->pwork2!=NULL plist->work2pos==0"));
  }

  /* check the list by looping through it */
  pcurrent = (lcrzo_listitem *) plist;
  founderror = LCRZO_FALSE;
  for (i = 0; i < plist->itemcount; i++) {
    pnext = pcurrent->pnext;
    if (pnext->pprevious != pcurrent) {
      lcrzo_er(lcrzo_int32_print
               ("Error : pnext->pprevious != pcurrent[", i + 1, "%d", "]\n"));
      founderror = LCRZO_TRUE;
    }
    if (i + 1 == plist->work1pos && pnext != plist->pwork1) {
      lcrzo_er(lcrzo_int32_print("Error : i==", i + 1, "%d",
                                 " && pnext!=pwork1\n"));
      founderror = LCRZO_TRUE;
    }
    if (i + 1 == plist->work2pos && pnext != plist->pwork2) {
      lcrzo_er(lcrzo_int32_print("Error : i==", i + 1, "%d",
                                 " && pnext!=pwork2\n"));
      founderror = LCRZO_TRUE;
    }
    if (pnext == plist->pwork1 && i + 1 != plist->work1pos) {
      lcrzo_er(lcrzo_int32_print
               ("Error : pnext==pwork1 && i==", i + 1, "%d", ""));
      lcrzo_er(lcrzo_int32_print
               (" instead of ", plist->work1pos, "%d", "\n"));
      founderror = LCRZO_TRUE;
    }
    if (pnext == plist->pwork2 && i + 1 != plist->work2pos) {
      lcrzo_er(lcrzo_int32_print
               ("Error : pnext==pwork2 && i==", i + 1, "%d", ""));
      lcrzo_er(lcrzo_int32_print
               (" instead of ", plist->work2pos, "%d", "\n"));
      founderror = LCRZO_TRUE;
    }
    pcurrent = pnext;
  }
  if (plist->pwork1 == (lcrzo_listitem *) plist) {
    printf("Error : pwork1==plist");
    founderror = LCRZO_TRUE;
  }
  if (plist->pwork2 == (lcrzo_listitem *) plist) {
    printf("Error : pwork2==plist");
    founderror = LCRZO_TRUE;
  }
  if (founderror) {
    return (lcrzo_ie2(functionname, "pwork1 or pwork2 (see message)"));
  }
  if (pcurrent->pnext != (lcrzo_listitem *) plist) {
    return (lcrzo_ie2(functionname, "plast->pnext != plist"));
  }
  if (plist->pprevious != pcurrent) {
    return (lcrzo_ie2(functionname, "plist->pprevious != plast"));
  }

  return (LCRZO_ERR_OK);
}

 #define lcrzo__debug_list_check(a,b) {lcrzo_er(lcrzo_debug_list_check(a,b));}
#else
 #define lcrzo__debug_list_check(a,b) {}
#endif

/*-------------------------------------------------------------*/
int lcrzo_priv_list_pos_save1(lcrzo_list * plist,
                              lcrzo_int32 work1pos,
                              lcrzo_listitem * pwork1);
int lcrzo_priv_list_pos_save1(lcrzo_list * plist,
                              lcrzo_int32 work1pos,
                              lcrzo_listitem * pwork1)
{
  lcrzo_int32 itemcountplusone;

  itemcountplusone = plist->itemcount + 1;

#if LCRZO_DEBUG_LIST_GOTO==1
  lcrzo_int32_print("lcrzo_priv_list_pos_save1 : ", work1pos, "%d", "-");
  printf("%p\n", pwork1);
#endif

  lcrzo__debug_list_check_workpos("lcrzo_priv_list_pos_save1", plist,
                                  work1pos, pwork1);

  /* treat it as a valid case to not bother doing it in the calling function */
  if (work1pos == 0) {
    pwork1 = NULL;
  }
  if (work1pos == itemcountplusone) {
    pwork1 = NULL;
    work1pos = 0;
  }

  /* store */
  plist->pwork1 = pwork1;
  plist->pwork2 = NULL;
  plist->work1pos = work1pos;
  plist->work2pos = 0;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_pos_save1keepother(lcrzo_list * plist,
                                       lcrzo_int32 workpos,
                                       lcrzo_listitem * pwork);
int lcrzo_priv_list_pos_save1keepother(lcrzo_list * plist,
                                       lcrzo_int32 workpos,
                                       lcrzo_listitem * pwork)
{
  lcrzo_int32 itemcountplusone, dist1, dist2;

  itemcountplusone = plist->itemcount + 1;

#if LCRZO_DEBUG_LIST_GOTO==1
  lcrzo_int32_print("lcrzo_priv_list_pos_save1keepother : ", workpos,
                    "%d", "-");
  printf("%p\n", pwork);
#endif

  lcrzo__debug_list_check_workpos("lcrzo_priv_list_pos_save1keepother", plist,
                                  workpos, pwork);

  /* treat it as a valid case to not bother doing it in the calling function */
  if (workpos == 0 || workpos == itemcountplusone) {    /* nothing to save */
    return (LCRZO_ERR_OK);
  }

  if (plist->pwork1 == NULL) {  /* save here */
    plist->work1pos = workpos;
    plist->pwork1 = pwork;
    return (LCRZO_ERR_OK);
  }
  if (plist->pwork2 == NULL) {  /* save here */
    plist->work2pos = workpos;
    plist->pwork2 = pwork;
    return (LCRZO_ERR_OK);
  }

  /* now, save in the near position */
  dist1 = workpos - plist->work1pos;
  if (dist1 < 0)
    dist1 = -dist1;
  dist2 = workpos - plist->work2pos;
  if (dist2 < 0)
    dist2 = -dist2;
  if (dist1 < dist2) {
    plist->work1pos = workpos;
    plist->pwork1 = pwork;
  } else {
    plist->work2pos = workpos;
    plist->pwork2 = pwork;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_pos_save2(lcrzo_list * plist,
                              lcrzo_int32 work1pos,
                              lcrzo_listitem * pwork1,
                              lcrzo_int32 work2pos,
                              lcrzo_listitem * pwork2);
int lcrzo_priv_list_pos_save2(lcrzo_list * plist,
                              lcrzo_int32 work1pos,
                              lcrzo_listitem * pwork1,
                              lcrzo_int32 work2pos,
                              lcrzo_listitem * pwork2)
{
  lcrzo_int32 itemcountplusone;

  itemcountplusone = plist->itemcount + 1;

#if LCRZO_DEBUG_LIST_GOTO==1
  lcrzo_int32_print("lcrzo_priv_list_pos_save2 : ", work1pos, "%d", "-");
  printf("%p\t", pwork1);
  lcrzo_int32_print("", work2pos, "%d", "-");
  printf("%p\n", pwork2);
#endif

  lcrzo__debug_list_check_workpos("lcrzo_priv_list_pos_save2 1", plist,
                                  work1pos, pwork1);
  lcrzo__debug_list_check_workpos("lcrzo_priv_list_pos_save2 2", plist,
                                  work2pos, pwork2);

  /* treat it as a valid case to not bother doing it in the calling function */
  if (work1pos == 0) {
    pwork1 = NULL;
  }
  if (work2pos == 0) {
    pwork2 = NULL;
  }
  if (work1pos == itemcountplusone) {
    pwork1 = NULL;
    work1pos = 0;
  }
  if (work2pos == itemcountplusone) {
    pwork2 = NULL;
    work2pos = 0;
  }

  /* store */
  plist->pwork1 = pwork1;
  plist->pwork2 = pwork2;
  plist->work1pos = work1pos;
  plist->work2pos = work2pos;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_pos_goto(lcrzo_list * plist,
                             lcrzo_int32 position,
                             lcrzo_bool wanttoinsertitem,
                             lcrzo_int32 * ppospositive,
                             lcrzo_listitem ** pplistitem);
int lcrzo_priv_list_pos_goto(lcrzo_list * plist,
                             lcrzo_int32 position,
                             lcrzo_bool wanttoinsertitem,
                             lcrzo_int32 * ppospositive,
                             lcrzo_listitem ** pplistitem)
{
  lcrzo_int32 i, hops, minhops;
  lcrzo_listitem *pli;
  lcrzo_int32 pospositive, posmax;
  lcrzo_int8 direction;

  /* check if position is valid */
  if (position == 0)
    return (LCRZO_ERR_PONOZERO);
  posmax = plist->itemcount + (wanttoinsertitem ? 1 : 0);
  if (position > 0) {
    if (position > posmax)
      return (LCRZO_ERR_POTOOBIG);
    pospositive = position;
  } else {
    if (position < -posmax)
      return (LCRZO_ERR_POTOOLOW);
    pospositive = posmax + 1 + position;
  }
  if (ppospositive != NULL)
    *ppospositive = pospositive;

  /* specials cases of very short tests : no need to compute */
  if (pospositive == 1) {
    if (pplistitem != NULL)
      *pplistitem = plist->pnext;
#if LCRZO_DEBUG_LIST_GOTO==1
    puts("lcrzo_priv_list_pos_goto : exit pospositive==1");
#endif
    return (LCRZO_ERR_OK);
  }
  if (pospositive == plist->itemcount) {
    if (pplistitem != NULL)
      *pplistitem = plist->pprevious;
#if LCRZO_DEBUG_LIST_GOTO==1
    puts("lcrzo_priv_list_pos_goto : exit plist->itemcount");
#endif
    return (LCRZO_ERR_OK);
  }
  if (pospositive == plist->itemcount + 1) {    /* this case is useful when adding a new member at the end */
    if (pplistitem != NULL)
      *pplistitem = (lcrzo_listitem *) plist;
#if LCRZO_DEBUG_LIST_GOTO==1
    puts("lcrzo_priv_list_pos_goto : exit plist->itemcount+1");
#endif
    return (LCRZO_ERR_OK);
  }
  if (plist->pwork1 != NULL && pospositive == plist->work1pos) {
    if (pplistitem != NULL)
      *pplistitem = plist->pwork1;
#if LCRZO_DEBUG_LIST_GOTO==1
    lcrzo_int32_print("lcrzo_priv_list_pos_goto : exit plist->work1pos (",
                      plist->work1pos, "%d", "-");
    printf("%p)\n", plist->pwork1);
#endif
    return (LCRZO_ERR_OK);
  }
  if (plist->pwork2 != NULL && pospositive == plist->work2pos) {
    if (pplistitem != NULL)
      *pplistitem = plist->pwork2;
#if LCRZO_DEBUG_LIST_GOTO==1
    lcrzo_int32_print("lcrzo_priv_list_pos_goto : exit plist->work2pos (",
                      plist->work2pos, "%d", "-");
    printf("%p)\n", plist->pwork2);
#endif
    return (LCRZO_ERR_OK);
  }

  /* determine number of hops and direction */
  /* from start */
  minhops = pospositive - 1;
  direction = LCRZO_PRIV_INCREASE;
  pli = plist->pnext;
  /* from end */
  hops = plist->itemcount - pospositive;
  if (hops < minhops) {
    minhops = hops;
    direction = LCRZO_PRIV_DECREASE;
    pli = plist->pprevious;
  }
  /* from pwork1 */
  if (plist->pwork1 != NULL) {
    hops = pospositive - plist->work1pos;
    if (hops > 0) {
      if (hops < minhops) {
        minhops = hops;
        direction = LCRZO_PRIV_INCREASE;
        pli = plist->pwork1;
      }
    } else {
      hops = -hops;
      if (hops < minhops) {
        minhops = hops;
        direction = LCRZO_PRIV_DECREASE;
        pli = plist->pwork1;
      }
    }
  }
  /* from pwork2 */
  if (plist->pwork2 != NULL) {
    hops = pospositive - plist->work2pos;
    if (hops > 0) {
      if (hops < minhops) {
        minhops = hops;
        direction = LCRZO_PRIV_INCREASE;
        pli = plist->pwork2;
      }
    } else {
      hops = -hops;
      if (hops < minhops) {
        minhops = hops;
        direction = LCRZO_PRIV_DECREASE;
        pli = plist->pwork2;
      }
    }
  }

  /* now, go to the wanted position */
  if (direction == LCRZO_PRIV_INCREASE) {
    for (i = 0; i < minhops; i++) {
      pli = pli->pnext;
    }
  } else {
    for (i = 0; i < minhops; i++) {
      pli = pli->pprevious;
    }
  }

  if (pplistitem != NULL)
    *pplistitem = pli;
#if LCRZO_DEBUG_LIST_GOTO==1
  lcrzo_int32_print("lcrzo_priv_list_pos_goto : ", pospositive, "%d", "-");
  printf("%p\n", pli);
#endif
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_pos_goto2(lcrzo_list * plist,
                              lcrzo_int32 position1,
                              lcrzo_int32 position2,
                              lcrzo_int32 * ppos1positive,
                              lcrzo_int32 * ppos2positive,
                              lcrzo_listitem ** pplistitem1,
                              lcrzo_listitem ** pplistitem2);
int lcrzo_priv_list_pos_goto2(lcrzo_list * plist,
                              lcrzo_int32 position1,
                              lcrzo_int32 position2,
                              lcrzo_int32 * ppos1positive,
                              lcrzo_int32 * ppos2positive,
                              lcrzo_listitem ** pplistitem1,
                              lcrzo_listitem ** pplistitem2)
{
  /* This function could be optimized :
     - find if it's shorter to go to 1 or 2
     - go to the shorter
     - find the shortest way to go to the other :
     start, pwork1, pwork2, end AND from_shorter
     - go to the other
  */
  lcrzo_er(lcrzo_priv_list_pos_goto(plist, position1, LCRZO_FALSE,
                                    ppos1positive, pplistitem1));
  lcrzo_er(lcrzo_priv_list_pos_goto(plist, position2, LCRZO_FALSE,
                                    ppos2positive, pplistitem2));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_range_goto(lcrzo_list * plist,
                               lcrzo_int32 positionstart,
                               lcrzo_int32 positionend,
                               lcrzo_int32 * pposstartpositive,
                               lcrzo_int32 * pposendpositive,
                               lcrzo_listitem ** pplistitemstart,
                               lcrzo_int8 * pdirectiontoend,
                               lcrzo_int32 * phopstoend);
int lcrzo_priv_list_range_goto(lcrzo_list * plist,
                               lcrzo_int32 positionstart,
                               lcrzo_int32 positionend,
                               lcrzo_int32 * pposstartpositive,
                               lcrzo_int32 * pposendpositive,
                               lcrzo_listitem ** pplistitemstart,
                               lcrzo_int8 * pdirectiontoend,
                               lcrzo_int32 * phopstoend)
{
  lcrzo_int32 posstartpositive, posendpositive, posmax, hopstoend;
  lcrzo_int8 directiontoend;

  /* check if positionend is valid */
  if (positionend == 0)
    return (LCRZO_ERR_PONOZERO);
  posmax = plist->itemcount;
  if (positionend > 0) {
    if (positionend > posmax)
      return (LCRZO_ERR_POTOOBIG);
    posendpositive = positionend;
  } else {
    if (positionend < -posmax)
      return (LCRZO_ERR_POTOOLOW);
    posendpositive = posmax + 1 + positionend;
  }
  if (pposendpositive != NULL)
    *pposendpositive = posendpositive;

  /* now, go to start */
  lcrzo_er(lcrzo_priv_list_pos_goto(plist, positionstart, LCRZO_FALSE,
                                    &posstartpositive, pplistitemstart));
  if (pposstartpositive != NULL)
    *pposstartpositive = posstartpositive;

  /* compute distance to the second item */
  hopstoend = posendpositive - posstartpositive;
  if (hopstoend < 0)
    hopstoend = -hopstoend;
  if (phopstoend != NULL)
    *phopstoend = hopstoend;

  /* compute direction to the second item */
  directiontoend = LCRZO_PRIV_INCREASE;
  if (posstartpositive > posendpositive)
    directiontoend = LCRZO_PRIV_DECREASE;
  if (pdirectiontoend != NULL)
    *pdirectiontoend = directiontoend;

#if LCRZO_DEBUG_LIST_GOTO==1
  lcrzo_int32_print("lcrzo_priv_list_range_goto : ", positionstart, "%d",
                    ":");
  lcrzo_int32_print("", positionstart, "%d", "\t");
  lcrzo_int32_print("", posstartapositive, "%d", ":");
  lcrzo_int32_print("", posstartpositive, "%d", "\t");
  lcrzo_uint8_print("", directiontoend, "%u", ":");
  lcrzo_int32_print("", hopstoend, "%d", "\t");
  printf("%p\n", pli);
#endif
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_range_gotofast(lcrzo_list * plist,
                                   lcrzo_int32 position1,
                                   lcrzo_int32 position2,
                                   lcrzo_int32 * pposapositive,
                                   lcrzo_int32 * pposbpositive,
                                   lcrzo_listitem ** pplistitema,
                                   lcrzo_int8 * pdirectiontob,
                                   lcrzo_int32 * phopstob);
int lcrzo_priv_list_range_gotofast(lcrzo_list * plist,
                                   lcrzo_int32 position1,
                                   lcrzo_int32 position2,
                                   lcrzo_int32 * pposapositive,
                                   lcrzo_int32 * pposbpositive,
                                   lcrzo_listitem ** pplistitema,
                                   lcrzo_int8 * pdirectiontob,
                                   lcrzo_int32 * phopstob)
{
  lcrzo_int32 i, hops, minhops, hopstob;
  lcrzo_listitem *pli;
  lcrzo_int32 pos1positive, pos2positive, posmax;
  lcrzo_int8 direction, from, directiontob;

  /* check if positions are valid */
  if (position1 == 0)
    return (LCRZO_ERR_PONOZERO);
  if (position2 == 0)
    return (LCRZO_ERR_PONOZERO);
  posmax = plist->itemcount;
  if (position1 > 0) {
    if (position1 > posmax)
      return (LCRZO_ERR_POTOOBIG);
    pos1positive = position1;
  } else {
    if (position1 < -posmax)
      return (LCRZO_ERR_POTOOLOW);
    pos1positive = posmax + 1 + position1;
  }
  if (position2 > 0) {
    if (position2 > posmax)
      return (LCRZO_ERR_POTOOBIG);
    pos2positive = position2;
  } else {
    if (position2 < -posmax)
      return (LCRZO_ERR_POTOOLOW);
    pos2positive = posmax + 1 + position2;
  }

  /* determine number of hops and direction to shortest item */
  /* from start */
  minhops = pos1positive - 1;
  from = 1;
  direction = LCRZO_PRIV_INCREASE;
  pli = plist->pnext;
  hops = pos2positive - 1;
  if (hops < minhops) {
    minhops = hops;
    from = 2;
  }
  /* from end */
  hops = plist->itemcount - pos1positive;
  if (hops < minhops) {
    minhops = hops;
    from = 1;
    direction = LCRZO_PRIV_DECREASE;
    pli = plist->pprevious;
  }
  hops = plist->itemcount - pos2positive;
  if (hops < minhops) {
    minhops = hops;
    from = 2;
    direction = LCRZO_PRIV_DECREASE;
    pli = plist->pprevious;
  }
  /* from pwork1 */
  if (plist->pwork1 != NULL) {
    hops = pos1positive - plist->work1pos;
    if (hops > 0) {
      if (hops < minhops) {
        minhops = hops;
        from = 1;
        direction = LCRZO_PRIV_INCREASE;
        pli = plist->pwork1;
      }
    } else {
      hops = -hops;
      if (hops < minhops) {
        minhops = hops;
        from = 1;
        direction = LCRZO_PRIV_DECREASE;
        pli = plist->pwork1;
      }
    }
    hops = pos2positive - plist->work1pos;
    if (hops > 0) {
      if (hops < minhops) {
        minhops = hops;
        from = 2;
        direction = LCRZO_PRIV_INCREASE;
        pli = plist->pwork1;
      }
    } else {
      hops = -hops;
      if (hops < minhops) {
        minhops = hops;
        from = 2;
        direction = LCRZO_PRIV_DECREASE;
        pli = plist->pwork1;
      }
    }
  }
  /* from pwork2 */
  if (plist->pwork2 != NULL) {
    hops = pos1positive - plist->work2pos;
    if (hops > 0) {
      if (hops < minhops) {
        minhops = hops;
        from = 1;
        direction = LCRZO_PRIV_INCREASE;
        pli = plist->pwork2;
      }
    } else {
      hops = -hops;
      if (hops < minhops) {
        minhops = hops;
        from = 1;
        direction = LCRZO_PRIV_DECREASE;
        pli = plist->pwork2;
      }
    }
    hops = pos2positive - plist->work2pos;
    if (hops > 0) {
      if (hops < minhops) {
        minhops = hops;
        from = 2;
        direction = LCRZO_PRIV_INCREASE;
        pli = plist->pwork2;
      }
    } else {
      hops = -hops;
      if (hops < minhops) {
        minhops = hops;
        from = 2;
        direction = LCRZO_PRIV_DECREASE;
        pli = plist->pwork2;
      }
    }
  }

  /* set position */
  if (pposapositive != NULL) {
    if (from == 1)
      *pposapositive = pos1positive;
    else if (from == 2)
      *pposapositive = pos2positive;
    else
      return (lcrzo_ieuv("lcrzo_priv_list_range_gotofast : ", "from"));
  }
  if (pposbpositive != NULL) {
    if (from == 1)
      *pposbpositive = pos2positive;
    else if (from == 2)
      *pposbpositive = pos1positive;
    else
      return (lcrzo_ieuv("lcrzo_priv_list_range_gotofast : ", "from"));
  }

  /* now, go to the wanted position */
  if (direction == LCRZO_PRIV_INCREASE) {
    for (i = 0; i < minhops; i++) {
      pli = pli->pnext;
    }
  } else {
    for (i = 0; i < minhops; i++) {
      pli = pli->pprevious;
    }
  }
  if (pplistitema != NULL)
    *pplistitema = pli;

  /* compute distance to the second item */
  hopstob = pos2positive - pos1positive;
  if (hopstob < 0)
    hopstob = -hopstob;
  if (phopstob != NULL)
    *phopstob = hopstob;

  /* compute direction to the second item */
  directiontob = LCRZO_PRIV_INCREASE;
  if ((from == 1 && pos1positive > pos2positive) ||
      (from == 2 && pos1positive < pos2positive)) {
    directiontob = LCRZO_PRIV_DECREASE;
  }
  if (pdirectiontob != NULL)
    *pdirectiontob = directiontob;

#if LCRZO_DEBUG_LIST_GOTO==1
  lcrzo_int32_print("lcrzo_priv_list_range_gotofast : ", position1, "%d",
                    ":");
  lcrzo_int32_print("", position2, "%d", "\t");
  lcrzo_int32_print("", pos1apositive, "%d", ":");
  lcrzo_int32_print("", pos2positive, "%d", "\t");
  lcrzo_uint8_print("", directiontob, "%u", ":");
  lcrzo_int32_print("", hopstob, "%d", "\t");
  printf("%p\n", pli);
#endif
  return (LCRZO_ERR_OK);
}



/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

/*-------------------------------------------------------------*/
int lcrzo_list_init(lcrzo_list * plist,
                    lcrzo_uint32 itemsize,
                    int (*pfunc_erase) (lcrzo_ptr pitem))
{
  /*parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);

  plist->pnext = (lcrzo_listitem *) plist;
  plist->pprevious = (lcrzo_listitem *) plist;
  plist->itemsize = itemsize;
  plist->itemcount = 0;
  plist->pfunc_erase = pfunc_erase;
  plist->pwork1 = NULL;
  plist->pwork2 = NULL;
  plist->work1pos = 0;
  plist->work2pos = 0;

  lcrzo__debug_list_check("lcrzo_list_init", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_close(lcrzo_list * plist)
{
  lcrzo__debug_list_check("lcrzo_list_close a", plist);
  lcrzo_er(lcrzo_list_remove_all(plist));
  lcrzo__debug_list_check("lcrzo_list_close b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_add_pos(lcrzo_list * plist,
                       lcrzo_int32 position,
                       lcrzo_constptr pitem)
{
  lcrzo_int32 pospos;
  lcrzo_listitem *pnew, *pwork1, *pwork2;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pitem == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (plist->itemcount >= 0x7FFFFFFF)
    return (LCRZO_ERR_SPLISTFULL);
  lcrzo__debug_list_check("lcrzo_list_add_pos a", plist);

  /* go to the future position */
  lcrzo_er(lcrzo_priv_list_pos_goto(plist, position, LCRZO_TRUE,
                                    &pospos, &pwork2));

  /* alloc memory for the item */
  pnew = (lcrzo_listitem *) lcrzo_debug_list_malloc(sizeof(lcrzo_listitem));
  if (pnew == NULL)
    return (LCRZO_ERR_FUMALLOC);
  pnew->pitem = lcrzo_debug_list_malloc(plist->itemsize);
  if (pnew->pitem == NULL) {
    lcrzo_debug_list_free(pnew);
    return (LCRZO_ERR_FUMALLOC);
  }
  memcpy(pnew->pitem, pitem, plist->itemsize);

  /* insert it */
  pwork1 = pwork2->pprevious;
  pnew->pnext = pwork2;
  pnew->pprevious = pwork1;
  pwork2->pprevious = pnew;
  pwork1->pnext = pnew;

  /* we added one item */
  plist->itemcount++;

  /* save working positions */
  lcrzo_er(lcrzo_priv_list_pos_save2(plist, pospos - 1, pwork1,
                                     pospos + 1, pwork2));

  lcrzo__debug_list_check("lcrzo_list_add_pos b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_add_first(lcrzo_list * plist,
                         lcrzo_constptr pitem)
{
  return (lcrzo_list_add_pos(plist, +1, pitem));
}

/*-------------------------------------------------------------*/
int lcrzo_list_add_last(lcrzo_list * plist,
                        lcrzo_constptr pitem)
{
  return (lcrzo_list_add_pos(plist, -1, pitem));
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_pos(lcrzo_list * plist,
                          lcrzo_int32 position)
{
  lcrzo_int32 pospos;
  lcrzo_listitem *pold, *pwork1, *pwork2;
  int ret;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_remove_pos a", plist);

  /* go to the position */
  lcrzo_er(lcrzo_priv_list_pos_goto(plist, position, LCRZO_FALSE,
                                    &pospos, &pold));

  /* save previous and next */
  pwork1 = pold->pprevious;
  pwork2 = pold->pnext;

  /* free the memory */
  if (plist->pfunc_erase != NULL) {
    /* we have a removing function */
    ret = (*plist->pfunc_erase) (pold->pitem);
    if (ret != LCRZO_ERR_OK) {
      /* return without unlinking the item */
      return (ret);
    }
  }
  lcrzo_debug_list_free(pold->pitem);
  lcrzo_debug_list_free(pold);

  /* unlink the item */
  pwork2->pprevious = pwork1;
  pwork1->pnext = pwork2;

  /* one less */
  plist->itemcount--;

  /* save working positions */
  lcrzo_er(lcrzo_priv_list_pos_save2(plist, pospos - 1, pwork1,
                                     pospos, pwork2));

  lcrzo__debug_list_check("lcrzo_list_remove_pos b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_first(lcrzo_list * plist)
{
  return (lcrzo_list_remove_pos(plist, +1));
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_last(lcrzo_list * plist)
{
  return (lcrzo_list_remove_pos(plist, -1));
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_range(lcrzo_list * plist,
                            lcrzo_int32 startpos,
                            lcrzo_int32 endpos)
{
  lcrzo_int32 posapositive, posbpositive;
  lcrzo_listitem *pitema, *pwork1, *pwork2, *pwork3;
  lcrzo_int8 directiontob;
  lcrzo_int32 hopstob, i;
  int ret;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_remove_range a", plist);

  /* special case */
  if (plist->itemcount == 0 &&
      ((startpos == +1 && endpos == -1) || (startpos == -1 && endpos == +1)))
  {
    return (LCRZO_ERR_OK);
  }

  /* go to the near position */
  lcrzo_er(lcrzo_priv_list_range_gotofast(plist, startpos, endpos,
                                          &posapositive, &posbpositive,
                                          &pitema, &directiontob, &hopstob));

  /* loop through each item to remove */
  if (directiontob == LCRZO_PRIV_INCREASE) {
    /* save start */
    pwork1 = pitema->pprevious;
    pwork2 = pitema;
    /* remove items */
    for (i = 0; i <= hopstob; i++) {
      /* free the memory */
      if (plist->pfunc_erase != NULL) {
        /* have a removing function */
        ret = (*plist->pfunc_erase) (pwork2->pitem);
        /* on error, relink the list */
        if (ret != LCRZO_ERR_OK) {
          pwork2->pprevious = pwork1;
          pwork1->pnext = pwork2;
          plist->itemcount -= i;
          lcrzo_er(lcrzo_priv_list_pos_save2(plist, posapositive - 1, pwork1,
                                             posapositive, pwork2));
          lcrzo__debug_list_check("lcrzo_list_remove_range c", plist);
          return (ret);
        }
      }
      lcrzo_debug_list_free(pwork2->pitem);
      /* skip to next item */
      pwork3 = pwork2->pnext;
      lcrzo_debug_list_free(pwork2);
      pwork2 = pwork3;
    }
    /* relink the list */
    pwork1->pnext = pwork2;
    pwork2->pprevious = pwork1;
    plist->itemcount -= hopstob + 1;
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save1(plist, posapositive - 1, pwork1));

  } else {
    /* save start */
    pwork2 = pitema->pnext;
    pwork1 = pitema;
    /* remove items */
    for (i = 0; i <= hopstob; i++) {
      /* free the memory */
      if (plist->pfunc_erase != NULL) {
        /* have a removing function */
        ret = (*plist->pfunc_erase) (pwork1->pitem);
        /* on error, relink the list */
        if (ret != LCRZO_ERR_OK) {
          pwork1->pnext = pwork2;
          pwork2->pprevious = pwork1;
          plist->itemcount -= i;
          lcrzo_er(lcrzo_priv_list_pos_save2
                   (plist, posapositive - i + 1, pwork2, posapositive - i,
                    pwork1));
          lcrzo__debug_list_check("lcrzo_list_remove_range d", plist);
          return (ret);
        }
      }
      lcrzo_debug_list_free(pwork1->pitem);
      /* skip to next item */
      pwork3 = pwork1->pprevious;
      lcrzo_debug_list_free(pwork1);
      pwork1 = pwork3;
    }
    /* relink the list */
    pwork2->pprevious = pwork1;
    pwork1->pnext = pwork2;
    plist->itemcount -= hopstob + 1;
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save1(plist, posbpositive - 1, pwork1));
  }

  lcrzo__debug_list_check("lcrzo_list_remove_range b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_all(lcrzo_list * plist)
{
  lcrzo_er(lcrzo_list_remove_range(plist, 1, -1));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_remove_duplicate_range(lcrzo_list * plist,
                                           lcrzo_int32 startpos,
                                           lcrzo_int32 endpos,
                                           lcrzo_bool usefunc1,
                                           lcrzo_int8(*pfunc_compare1)
                                           (lcrzo_constptr piteminf,
                                            lcrzo_constptr pitemsup,
                                            lcrzo_ptr pinfos),
                                           int (*pfunc_compare2)
                                           (lcrzo_constptr piteminf,
                                            lcrzo_constptr pitemsup,
                                            lcrzo_ptr pinfos,
                                            lcrzo_int8 * pcmp),
                                           lcrzo_ptr pinfos)
{
  lcrzo_int32 posstartpositive, posendpositive, i, j;
  lcrzo_listitem *pitemstart, *pwork1, *pwork2, *pwork3;
  lcrzo_listitem *pwork4, *pwork5, *pwork6;
  lcrzo_int8 directiontoend;
  lcrzo_int32 hopstoend, numremoved;
  lcrzo_int8 cmp;
  int ret;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (usefunc1 && pfunc_compare1 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (!usefunc1 && pfunc_compare2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_remove_duplicate_range a", plist);

  /* special case */
  if (plist->itemcount == 0 &&
      ((startpos == +1 && endpos == -1) || (startpos == -1 && endpos == +1)))
  {
    return (LCRZO_ERR_OK);
  }

  /* go to the start position */
  lcrzo_er(lcrzo_priv_list_range_goto(plist, startpos, endpos,
                                      &posstartpositive, &posendpositive,
                                      &pitemstart,
                                      &directiontoend, &hopstoend));

  numremoved = 0;
  if (directiontoend == LCRZO_PRIV_INCREASE) {
    /* save start */
    pwork1 = pitemstart;
    pwork3 = pitemstart->pprevious;
    pwork4 = NULL;
    /* loop */
    for (i = 0; i <= hopstoend - numremoved; i++) {
      pwork3 = pwork3->pnext;
      pwork4 = pwork3;
      for (j = i + 1; j <= hopstoend - numremoved; j++) {
        pwork5 = pwork4->pnext;
        if (usefunc1) {
          cmp = (*pfunc_compare1) (pwork3->pitem, pwork5->pitem, pinfos);
        } else {
          ret =
            (*pfunc_compare2) (pwork3->pitem, pwork5->pitem, pinfos, &cmp);
          if (ret != LCRZO_ERR_OK) {
            /* do not unlink the item */
            plist->itemcount -= numremoved;
            lcrzo_er(lcrzo_priv_list_pos_save1(plist, posstartpositive,
                                               pwork1));
            lcrzo__debug_list_check("lcrzo_list_remove_duplicate_range c",
                                    plist);
            return (ret);
          }
        }
        if (cmp == 0) {
          /* remove the item */
          if (plist->pfunc_erase != NULL) {
            ret = (*plist->pfunc_erase) (pwork5->pitem);
            if (ret != LCRZO_ERR_OK) {
              /* do not unlink the item */
              plist->itemcount -= numremoved;
              lcrzo_er(lcrzo_priv_list_pos_save1(plist, posstartpositive,
                                                 pwork1));
              lcrzo__debug_list_check("lcrzo_list_remove_duplicate_range d",
                                      plist);
              return (ret);
            }
          }
          lcrzo_debug_list_free(pwork5->pitem);
          pwork6 = pwork5->pnext;
          lcrzo_debug_list_free(pwork5);
          numremoved++;
          pwork4->pnext = pwork6;
          pwork6->pprevious = pwork4;
        } else {
          /* do not remove the item */
          pwork4 = pwork5;
        }
      }
    }
    /* save end */
    pwork2 = pwork4;
    plist->itemcount -= numremoved;
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                       posendpositive - numremoved, pwork2));
  } else {
    /* save start */
    pwork2 = pitemstart;
    pwork3 = pitemstart->pnext;
    pwork4 = NULL;
    /* loop */
    for (i = 0; i <= hopstoend - numremoved; i++) {
      pwork3 = pwork3->pprevious;
      pwork4 = pwork3;
      for (j = i + 1; j <= hopstoend - numremoved; j++) {
        pwork5 = pwork4->pprevious;
        if (usefunc1) {
          cmp = (*pfunc_compare1) (pwork3->pitem, pwork5->pitem, pinfos);
        } else {
          ret =
            (*pfunc_compare2) (pwork3->pitem, pwork5->pitem, pinfos, &cmp);
          if (ret != LCRZO_ERR_OK) {
            /* do not unlink the item */
            plist->itemcount -= numremoved;
            lcrzo_er(lcrzo_priv_list_pos_save1(plist,
                                               posstartpositive - numremoved,
                                               pwork2));
            lcrzo__debug_list_check("lcrzo_list_remove_duplicate_range e",
                                    plist);
            return (ret);
          }
        }
        if (cmp == 0) {
          /* remove the item */
          if (plist->pfunc_erase != NULL) {
            ret = (*plist->pfunc_erase) (pwork5->pitem);
            if (ret != LCRZO_ERR_OK) {
              /* do not unlink the item */
              plist->itemcount -= numremoved;
              lcrzo_er(lcrzo_priv_list_pos_save1(plist,
                                                 posstartpositive -
                                                 numremoved, pwork2));
              lcrzo__debug_list_check("lcrzo_list_remove_duplicate_range f",
                                      plist);
              return (ret);
            }
          }
          lcrzo_debug_list_free(pwork5->pitem);
          pwork6 = pwork5->pprevious;
          lcrzo_debug_list_free(pwork5);
          numremoved++;
          pwork4->pprevious = pwork6;
          pwork6->pnext = pwork4;
        } else {
          /* do not remove the item */
          pwork4 = pwork5;
        }
      }
    }
    /* save end */
    pwork1 = pwork4;
    plist->itemcount -= numremoved;
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist,
                                       posendpositive, pwork1,
                                       posstartpositive - numremoved,
                                       pwork2));
  }

  lcrzo__debug_list_check("lcrzo_list_remove_duplicate_range b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_duplicate_range2(lcrzo_list * plist,
                                       lcrzo_int32 startpos,
                                       lcrzo_int32 endpos,
                                       int (*pfunc_compare) (lcrzo_constptr
                                                             piteminf,
                                                             lcrzo_constptr
                                                             pitemsup,
                                                             lcrzo_ptr pinfos,
                                                             lcrzo_int8 *
                                                             pcmp),
                                       lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_remove_duplicate_range(plist, startpos, endpos,
                                                  LCRZO_FALSE,
                                                  NULL,
                                                  pfunc_compare, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_duplicate_all2(lcrzo_list * plist,
                                     int (*pfunc_compare) (lcrzo_constptr
                                                           piteminf,
                                                           lcrzo_constptr
                                                           pitemsup,
                                                           lcrzo_ptr pinfos,
                                                           lcrzo_int8 * pcmp),
                                     lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_remove_duplicate_range(plist, +1, -1,
                                                  LCRZO_FALSE,
                                                  NULL,
                                                  pfunc_compare, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_remove_criteria_range(lcrzo_list * plist,
                                          lcrzo_int32 startpos,
                                          lcrzo_int32 endpos,
                                          lcrzo_bool usefunc1,
                                          lcrzo_bool(*pfunc_criteria1)
                                          (lcrzo_constptr pitem,
                                           lcrzo_ptr pinfos),
                                          int (*pfunc_criteria2)
                                          (lcrzo_constptr pitem,
                                           lcrzo_ptr pinfos,
                                           lcrzo_bool * prem),
                                          lcrzo_ptr pinfos)
{
  lcrzo_int32 posstartpositive, posendpositive, i;
  lcrzo_listitem *pitemstart, *pwork1, *pwork2, *pwork3, *pwork4, *pwork5;
  lcrzo_int8 directiontoend;
  lcrzo_int32 hopstoend, numremoved;
  lcrzo_bool b;
  int ret;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (usefunc1 && pfunc_criteria1 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (!usefunc1 && pfunc_criteria2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_remove_criteria_range a", plist);

  /* special case */
  if (plist->itemcount == 0 &&
      ((startpos == +1 && endpos == -1) || (startpos == -1 && endpos == +1)))
  {
    return (LCRZO_ERR_OK);
  }

  /* go to the start position */
  /* we do not use gotofast, because pfunc_criteria might do things
     depending on previous item in the range */
  lcrzo_er(lcrzo_priv_list_range_goto(plist, startpos, endpos,
                                      &posstartpositive, &posendpositive,
                                      &pitemstart,
                                      &directiontoend, &hopstoend));

  numremoved = 0;
  if (directiontoend == LCRZO_PRIV_INCREASE) {
    /* save start */
    pwork3 = pitemstart->pprevious;
    pwork1 = pwork3;
    /* loop */
    for (i = 0; i <= hopstoend; i++) {
      pwork4 = pwork3->pnext;
      if (usefunc1) {
        b = (*pfunc_criteria1) (pwork4->pitem, pinfos);
      } else {
        ret = (*pfunc_criteria2) (pwork4->pitem, pinfos, &b);
        if (ret != LCRZO_ERR_OK) {
          /* do not unlink the item */
          plist->itemcount -= numremoved;
          lcrzo_er(lcrzo_priv_list_pos_save1(plist, posstartpositive - 1,
                                             pwork1));
          lcrzo__debug_list_check("lcrzo_list_remove_criteria_range c",
                                  plist);
          return (ret);
        }
      }
      if (b) {
        /* remove the item */
        if (plist->pfunc_erase != NULL) {
          ret = (*plist->pfunc_erase) (pwork4->pitem);
          if (ret != LCRZO_ERR_OK) {
            /* do not unlink the item */
            plist->itemcount -= numremoved;
            lcrzo_er(lcrzo_priv_list_pos_save1(plist, posstartpositive - 1,
                                               pwork1));
            lcrzo__debug_list_check("lcrzo_list_remove_criteria_range d",
                                    plist);
            return (ret);
          }
        }
        lcrzo_debug_list_free(pwork4->pitem);
        pwork5 = pwork4->pnext;
        lcrzo_debug_list_free(pwork4);
        numremoved++;
        pwork3->pnext = pwork5;
        pwork5->pprevious = pwork3;
      } else {
        /* do not remove the item */
        pwork3 = pwork4;
      }
    }
    /* save end */
    pwork2 = pwork3;
    plist->itemcount -= numremoved;
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive - 1, pwork1,
                                       posendpositive - numremoved, pwork2));
  } else {
    /* save start */
    pwork3 = pitemstart->pnext;
    pwork2 = pwork3;
    /* loop */
    for (i = 0; i <= hopstoend; i++) {
      pwork4 = pwork3->pprevious;
      if (usefunc1) {
        b = (*pfunc_criteria1) (pwork4->pitem, pinfos);
      } else {
        ret = (*pfunc_criteria2) (pwork4->pitem, pinfos, &b);
        if (ret != LCRZO_ERR_OK) {
          /* do not unlink the item */
          plist->itemcount -= numremoved;
          lcrzo_er(lcrzo_priv_list_pos_save1(plist,
                                             posstartpositive - numremoved +
                                             1, pwork2));
          lcrzo__debug_list_check("lcrzo_list_remove_criteria_range e",
                                  plist);
          return (ret);
        }
      }
      if (b) {
        /* remove the item */
        if (plist->pfunc_erase != NULL) {
          ret = (*plist->pfunc_erase) (pwork4->pitem);
          if (ret != LCRZO_ERR_OK) {
            /* do not unlink the item */
            plist->itemcount -= numremoved;
            lcrzo_er(lcrzo_priv_list_pos_save1(plist,
                                               posstartpositive - numremoved +
                                               1, pwork2));
            lcrzo__debug_list_check("lcrzo_list_remove_criteria_range f",
                                    plist);
            return (ret);
          }
        }
        lcrzo_debug_list_free(pwork4->pitem);
        pwork5 = pwork4->pprevious;
        lcrzo_debug_list_free(pwork4);
        numremoved++;
        pwork3->pprevious = pwork5;
        pwork5->pnext = pwork3;
      } else {
        /* do not remove the item */
        pwork3 = pwork4;
      }
    }
    /* save end */
    pwork1 = pwork3;
    plist->itemcount -= numremoved;
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist,
                                       posendpositive, pwork1,
                                       posstartpositive - numremoved + 1,
                                       pwork2));
  }

  lcrzo__debug_list_check("lcrzo_list_remove_criteria_range b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_criteria_range2(lcrzo_list * plist,
                                      lcrzo_int32 startpos,
                                      lcrzo_int32 endpos,
                                      int (*pfunc_criteria) (lcrzo_constptr
                                                             pitem,
                                                             lcrzo_ptr pinfos,
                                                             lcrzo_bool *
                                                             prem),
                                      lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_remove_criteria_range(plist, startpos, endpos,
                                                 LCRZO_FALSE,
                                                 NULL,
                                                 pfunc_criteria, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_criteria_all2(lcrzo_list * plist,
                                    int (*pfunc_criteria) (lcrzo_constptr
                                                           pitem,
                                                           lcrzo_ptr pinfos,
                                                           lcrzo_bool * prem),
                                    lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_remove_criteria_range(plist, +1, -1,
                                                 LCRZO_FALSE,
                                                 NULL,
                                                 pfunc_criteria, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_replace_pos(lcrzo_list * plist,
                           lcrzo_int32 position,
                           lcrzo_constptr pitem)
{
  lcrzo_int32 pospos;
  lcrzo_listitem *pwork1;
  int ret;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_replace_pos a", plist);

  /* go to the position */
  lcrzo_er(lcrzo_priv_list_pos_goto(plist, position, LCRZO_FALSE,
                                    &pospos, &pwork1));

  /*replace */
  if (plist->pfunc_erase != NULL) {
    ret = (*plist->pfunc_erase) (pwork1->pitem);
    if (ret != LCRZO_ERR_OK) {
      /* return without unlinking the item */
      return (ret);
    }
  }
  memcpy(pwork1->pitem, pitem, plist->itemsize);

  /* save working positions */
  lcrzo_er(lcrzo_priv_list_pos_save1keepother(plist, pospos, pwork1));

  lcrzo__debug_list_check("lcrzo_list_replace_pos b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_replace_first(lcrzo_list * plist,
                             lcrzo_constptr pitem)
{
  return (lcrzo_list_replace_pos(plist, +1, pitem));
}

/*-------------------------------------------------------------*/
int lcrzo_list_replace_last(lcrzo_list * plist,
                            lcrzo_constptr pitem)
{
  return (lcrzo_list_replace_pos(plist, -1, pitem));
}

/*-------------------------------------------------------------*/
int lcrzo_list_switch(lcrzo_list * plist,
                      lcrzo_int32 position1,
                      lcrzo_int32 position2)
{
  lcrzo_int32 pos1pos, pos2pos;
  lcrzo_listitem *pwork1, *pwork2;
  lcrzo_listitem *pwork1next, *pwork1prev, *pwork2prev, *pwork2next;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_switch a", plist);

  /* go to the positions */
  lcrzo_er(lcrzo_priv_list_pos_goto2(plist, position1, position2,
                                     &pos1pos, &pos2pos, &pwork1, &pwork2));

  /* no need to switch */
  if (pos1pos == pos2pos) {
    lcrzo_er(lcrzo_priv_list_pos_save1keepother(plist, pos1pos, pwork1));
    return (LCRZO_ERR_OK);
  }

  if (pos1pos + 1 == pos2pos) {
    /* pwork2 is just after pwork1 */
    pwork2next = pwork2->pnext;
    pwork1prev = pwork1->pprevious;
    pwork2next->pprevious = pwork1;
    pwork1prev->pnext = pwork2;
    pwork2->pnext = pwork1;
    pwork1->pprevious = pwork2;
    pwork2->pprevious = pwork1prev;
    pwork1->pnext = pwork2next;
  } else if (pos2pos + 1 == pos1pos) {
    /* pwork1 is just after pwork2 */
    pwork1next = pwork1->pnext;
    pwork2prev = pwork2->pprevious;
    pwork1next->pprevious = pwork2;
    pwork2prev->pnext = pwork1;
    pwork1->pnext = pwork2;
    pwork2->pprevious = pwork1;
    pwork1->pprevious = pwork2prev;
    pwork2->pnext = pwork1next;
  } else {
    /* they are not near */
    pwork1next = pwork1->pnext;
    pwork1prev = pwork1->pprevious;
    pwork2next = pwork2->pnext;
    pwork2prev = pwork2->pprevious;
    pwork1next->pprevious = pwork2;
    pwork1prev->pnext = pwork2;
    pwork2next->pprevious = pwork1;
    pwork2prev->pnext = pwork1;
    pwork1->pprevious = pwork2prev;
    pwork1->pnext = pwork2next;
    pwork2->pprevious = pwork1prev;
    pwork2->pnext = pwork1next;
  }

  lcrzo_er(lcrzo_priv_list_pos_save2
           (plist, pos1pos, pwork2, pos2pos, pwork1));
  lcrzo__debug_list_check("lcrzo_list_switch b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_invert_range(lcrzo_list * plist,
                            lcrzo_int32 position1,
                            lcrzo_int32 position2)
{
  lcrzo_int32 pos1pos, pos2pos, dist, numhops, i;
  lcrzo_listitem *pwork1, *pwork2;
  lcrzo_listitem *pworkax, *pworkay, *pworkaz;
  lcrzo_listitem *pworkbx, *pworkby, *pworkbz;
  lcrzo_bool endwithtworelink;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_invert_range a", plist);

  /* special case */
  if (plist->itemcount == 0 &&
      ((position1 == +1 && position2 == -1)
       || (position1 == -1 && position2 == +1))) {
    return (LCRZO_ERR_OK);
  }

  /* go to the positions */
  lcrzo_er(lcrzo_priv_list_pos_goto2(plist, position1, position2,
                                     &pos1pos, &pos2pos, &pwork1, &pwork2));

  /* no need to invert */
  dist = pos2pos - pos1pos;
  if (dist == 0) {
    lcrzo_er(lcrzo_priv_list_pos_save1keepother(plist, pos1pos, pwork1));
    return (LCRZO_ERR_OK);
  }

  /* initialize start */
  if (dist > 0) {
    pworkax = pwork1->pprevious;
    pworkay = pwork1;
    pworkaz = pwork1->pnext;
    pworkbx = pwork2->pnext;
    pworkby = pwork2;
    pworkbz = pwork2->pprevious;
  } else {
    pworkax = pwork2->pprevious;
    pworkay = pwork2;
    pworkaz = pwork2->pnext;
    pworkbx = pwork1->pnext;
    pworkby = pwork1;
    pworkbz = pwork1->pprevious;
    dist = -dist;
  }

  /* compute the number of hops */
  if (dist & 1) {
    endwithtworelink = LCRZO_TRUE;
    numhops = (dist - 1) / 2;
  } else {
    endwithtworelink = LCRZO_FALSE;
    numhops = dist / 2;
  }

  /* exchange during numhops */
  for (i = 0; i < numhops; i++) {
    /* half link */
    pworkay->pnext = pworkbx;
    pworkbx->pprevious = pworkay;
    pworkby->pprevious = pworkax;
    pworkax->pnext = pworkby;
    /* skip */
    pworkax = pworkby;
    pworkbx = pworkay;
    pworkay = pworkaz;
    pworkby = pworkbz;
    pworkaz = pworkaz->pnext;
    pworkbz = pworkbz->pprevious;
  }

  /* relink the last */
  if (endwithtworelink) {
    pworkax->pnext = pworkaz;
    pworkaz->pnext = pworkay;
    pworkay->pnext = pworkbx;
    pworkbx->pprevious = pworkbz;
    pworkbz->pprevious = pworkby;
    pworkby->pprevious = pworkax;
  } else {
    pworkay->pnext = pworkbx;
    pworkax->pnext = pworkby;
    pworkbx->pprevious = pworkay;
    pworkby->pprevious = pworkax;
  }

  /* save pos */
  lcrzo_er(lcrzo_priv_list_pos_save2
           (plist, pos1pos, pwork2, pos2pos, pwork1));
  lcrzo__debug_list_check("lcrzo_list_invert_range b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_invert_all(lcrzo_list * plist)
{
  lcrzo_er(lcrzo_list_invert_range(plist, 1, -1));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_value_pos(lcrzo_list * plist,
                         lcrzo_int32 position,
                         lcrzo_ptr pitem)
{
  lcrzo_int32 pospos;
  lcrzo_listitem *pwork;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_value_pos a", plist);

  /* go to the position */
  lcrzo_er(lcrzo_priv_list_pos_goto(plist, position, LCRZO_FALSE,
                                    &pospos, &pwork));

  /* get it */
  if (pitem != NULL)
    memcpy(pitem, pwork->pitem, plist->itemsize);

  /* save position */
  lcrzo_er(lcrzo_priv_list_pos_save1keepother(plist, pospos, pwork));
  lcrzo__debug_list_check("lcrzo_list_value_pos b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_value_first(lcrzo_list * plist,
                           lcrzo_ptr pitem)
{
  return (lcrzo_list_value_pos(plist, +1, pitem));
}

/*-------------------------------------------------------------*/
int lcrzo_list_value_last(lcrzo_list * plist,
                          lcrzo_ptr pitem)
{
  return (lcrzo_list_value_pos(plist, -1, pitem));
}

/*-------------------------------------------------------------*/
int lcrzo_list_count(lcrzo_list * plist,
                     lcrzo_int32 * pnumberofitem)
{
  if (pnumberofitem != NULL)
    *pnumberofitem = plist->itemcount;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_search_range(lcrzo_list * plist,
                                 lcrzo_int32 startpos,
                                 lcrzo_int32 endpos,
                                 lcrzo_bool usefunc1,
                                 lcrzo_bool(*pfunc_search1) (lcrzo_constptr
                                                             pitem,
                                                             lcrzo_ptr
                                                             pinfos),
                                 int (*pfunc_search2) (lcrzo_constptr pitem,
                                                       lcrzo_ptr pinfos,
                                                       lcrzo_bool * pfound),
                                 lcrzo_ptr pinfos,
                                 lcrzo_int32 * ppositem,
                                 lcrzo_ptr pitem)
{
  lcrzo_int32 posstartpositive, posendpositive, i, positem;
  lcrzo_listitem *pitemstart, *pwork1, *pwork2;
  lcrzo_int8 directiontoend;
  lcrzo_int32 hopstoend;
  lcrzo_bool b;
  int ret;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (usefunc1 && pfunc_search1 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (!usefunc1 && pfunc_search2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_search_range a", plist);

  /* special case */
  if (plist->itemcount == 0 &&
      ((startpos == +1 && endpos == -1) || (startpos == -1 && endpos == +1)))
  {
    return (LCRZO_ERR_OKSEARCHNOTFOUND);
  }

  /* go to the start position */
  lcrzo_er(lcrzo_priv_list_range_goto(plist, startpos, endpos,
                                      &posstartpositive, &posendpositive,
                                      &pitemstart,
                                      &directiontoend, &hopstoend));

  /* save start */
  pwork1 = pitemstart;
  pwork2 = pwork1;
  if (directiontoend == LCRZO_PRIV_INCREASE) {
    for (i = 0; i <= hopstoend; i++) {
      if (usefunc1) {
        b = (*pfunc_search1) (pwork2->pitem, pinfos);
      } else {
        ret = (*pfunc_search2) (pwork2->pitem, pinfos, &b);
        if (ret != LCRZO_ERR_OK) {
          lcrzo_er(lcrzo_priv_list_pos_save1keepother(plist,
                                                      posstartpositive + i,
                                                      pwork2));
          lcrzo__debug_list_check("lcrzo_list_search_range c", plist);
          return (ret);
        }
      }
      if (b) {
        positem = posstartpositive + i;
        if (ppositem != NULL)
          *ppositem = positem;
        if (pitem != NULL)
          memcpy(pitem, pwork2->pitem, plist->itemsize);
        /* save working positions */
        lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                           positem, pwork2));
        lcrzo__debug_list_check("lcrzo_list_search_range d", plist);
        return (LCRZO_ERR_OK);
      }
      pwork2 = pwork2->pnext;
    }
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                       posendpositive + 1, pwork2));
  } else {
    for (i = 0; i <= hopstoend; i++) {
      if (usefunc1) {
        b = (*pfunc_search1) (pwork2->pitem, pinfos);
      } else {
        ret = (*pfunc_search2) (pwork2->pitem, pinfos, &b);
        if (ret != LCRZO_ERR_OK) {
          lcrzo_er(lcrzo_priv_list_pos_save1keepother(plist,
                                                      posstartpositive - i,
                                                      pwork2));
          lcrzo__debug_list_check("lcrzo_list_search_range e", plist);
          return (ret);
        }
      }
      if (b) {
        positem = posstartpositive - i;
        if (ppositem != NULL)
          *ppositem = positem;
        if (pitem != NULL)
          memcpy(pitem, pwork2->pitem, plist->itemsize);
        /* save working positions */
        lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                           positem, pwork2));
        lcrzo__debug_list_check("lcrzo_list_search_range e", plist);
        return (LCRZO_ERR_OK);
      }
      pwork2 = pwork2->pprevious;
    }
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                       posendpositive - 1, pwork2));
  }

  lcrzo__debug_list_check("lcrzo_list_search_range b", plist);
  return (LCRZO_ERR_OKSEARCHNOTFOUND);
}

/*-------------------------------------------------------------*/
int lcrzo_list_search_range2(lcrzo_list * plist,
                             lcrzo_int32 startpos,
                             lcrzo_int32 endpos,
                             int (*pfunc_search) (lcrzo_constptr pitem,
                                                  lcrzo_ptr pinfos,
                                                  lcrzo_bool * pfound),
                             lcrzo_ptr pinfos,
                             lcrzo_int32 * ppositem,
                             lcrzo_ptr pitem)
{
  lcrzo_er(lcrzo_priv_list_search_range(plist, startpos, endpos,
                                        LCRZO_FALSE,
                                        NULL,
                                        pfunc_search,
                                        pinfos, ppositem, pitem));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_search_all2(lcrzo_list * plist,
                           int (*pfunc_search) (lcrzo_constptr pitem,
                                                lcrzo_ptr pinfos,
                                                lcrzo_bool * pfound),
                           lcrzo_ptr pinfos,
                           lcrzo_int32 * ppositem,
                           lcrzo_ptr pitem)
{
  lcrzo_er(lcrzo_priv_list_search_range(plist, +1, -1,
                                        LCRZO_FALSE,
                                        NULL,
                                        pfunc_search,
                                        pinfos, ppositem, pitem));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_searchitem_range(lcrzo_list * plist,
                                lcrzo_int32 startpos,
                                lcrzo_int32 endpos,
                                lcrzo_constptr pitemtofind,
                                lcrzo_int32 * ppositemfound)
{
  lcrzo_int32 posstartpositive, posendpositive, i, positem;
  lcrzo_listitem *pitemstart, *pwork1, *pwork2;
  lcrzo_int8 directiontoend;
  lcrzo_int32 hopstoend;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_searchitem_range a", plist);

  /* special case */
  if (plist->itemcount == 0 &&
      ((startpos == +1 && endpos == -1) || (startpos == -1 && endpos == +1)))
  {
    return (LCRZO_ERR_OKSEARCHNOTFOUND);
  }

  /* go to the start position */
  lcrzo_er(lcrzo_priv_list_range_goto(plist, startpos, endpos,
                                      &posstartpositive, &posendpositive,
                                      &pitemstart,
                                      &directiontoend, &hopstoend));

  /* save start */
  pwork1 = pitemstart;
  pwork2 = pwork1;
  if (directiontoend == LCRZO_PRIV_INCREASE) { 
    for (i = 0; i <= hopstoend; i++) {
      if (!memcmp(pwork2->pitem, pitemtofind, plist->itemsize)) {
        positem = posstartpositive + i;
        if (ppositemfound != NULL)
          *ppositemfound = positem;
        /* save working positions */
        lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                           positem, pwork2));
        lcrzo__debug_list_check("lcrzo_list_searchitem_range c", plist);
        return (LCRZO_ERR_OK);
      }
      pwork2 = pwork2->pnext;
    }
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                       posendpositive + 1, pwork2));
  } else {
    for (i = 0; i <= hopstoend; i++) {
      if (!memcmp(pwork2->pitem, pitemtofind, plist->itemsize)) {
        positem = posstartpositive - i;
        if (ppositemfound != NULL)
          *ppositemfound = positem;
        /* save working positions */
        lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                           positem, pwork2));
        lcrzo__debug_list_check("lcrzo_list_searchitem_range d", plist);
        return (LCRZO_ERR_OK);
      }
      pwork2 = pwork2->pprevious;
    }
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                       posendpositive - 1, pwork2));
  }

  lcrzo__debug_list_check("lcrzo_list_searchitem_range b", plist);
  return (LCRZO_ERR_OKSEARCHNOTFOUND);
}

/*-------------------------------------------------------------*/
int lcrzo_list_searchitem_all(lcrzo_list * plist,
                              lcrzo_constptr pitemtofind,
                              lcrzo_int32 * ppositemfound)
{
  lcrzo_er(lcrzo_list_searchitem_range(plist, 1, -1, pitemtofind,
                                       ppositemfound));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_loop_range(lcrzo_list * plist,
                          lcrzo_int32 startpos,
                          lcrzo_int32 endpos,
                          int (*pfunc) (lcrzo_constptr pitem,
                                        lcrzo_ptr pinfos),
                          lcrzo_ptr pinfos)
{
  lcrzo_int32 posstartpositive, posendpositive, i;
  lcrzo_listitem *pitemstart, *pwork1, *pwork2;
  lcrzo_int8 directiontoend;
  lcrzo_int32 hopstoend;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_loop_range a", plist);

  /* special case */
  if (plist->itemcount == 0 &&
      ((startpos == +1 && endpos == -1) || (startpos == -1 && endpos == +1)))
  {
    return (LCRZO_ERR_OK);
  }

  /* go to the start position */
  lcrzo_er(lcrzo_priv_list_range_goto(plist, startpos, endpos,
                                      &posstartpositive, &posendpositive,
                                      &pitemstart,
                                      &directiontoend, &hopstoend));

  /* save start */
  pwork1 = pitemstart;
  pwork2 = pwork1;
  if (directiontoend == LCRZO_PRIV_INCREASE) {
    for (i = 0; i <= hopstoend; i++) {
      lcrzo_er(((*pfunc) (pwork2->pitem, pinfos)));
      pwork2 = pwork2->pnext;
    }
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                       posendpositive + 1, pwork2));
  } else {
    for (i = 0; i <= hopstoend; i++) {
      lcrzo_er(((*pfunc) (pwork2->pitem, pinfos)));
      pwork2 = pwork2->pprevious;
    }
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive, pwork1,
                                       posendpositive - 1, pwork2));
  }

  lcrzo__debug_list_check("lcrzo_list_loop_range b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_loop_all(lcrzo_list * plist,
                        int (*pfonc) (lcrzo_constptr pitem,
                                      lcrzo_ptr pinfos),
                        lcrzo_ptr pinfos)
{
  return (lcrzo_list_loop_range(plist, 1, -1, pfonc, pinfos));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_list_sort_range(lcrzo_list * plist,
                               lcrzo_int32 startpos,
                               lcrzo_int32 endpos,
                               lcrzo_bool usefunc1,
                               lcrzo_int8(*pfunc_compare1) (lcrzo_constptr
                                                            piteminf,
                                                            lcrzo_constptr
                                                            pitemsup,
                                                            lcrzo_ptr pinfos),
                               int (*pfunc_compare2) (lcrzo_constptr piteminf,
                                                      lcrzo_constptr pitemsup,
                                                      lcrzo_ptr pinfos,
                                                      lcrzo_int8 * pcmp),
                               lcrzo_ptr pinfos)
{
  lcrzo_int32 posstartpositive, posendpositive, i, j, k;
  lcrzo_listitem *pitemstart, *pwork1, *pwork2, *psorted, *pl1, *pl2,
    *pl1last;
  lcrzo_int8 directiontoend, cmp;
  lcrzo_int32 hopstoend, numloops, tmpvar, numinhalflist, numinhalflist2;
  lcrzo_int32 itemstosort, numsubloops, l1size, l2size, l1l2size;
  lcrzo_bool comefroml1;
  int ret;

  /* parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (usefunc1 && pfunc_compare1 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (!usefunc1 && pfunc_compare2 == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo__debug_list_check("lcrzo_list_list_sort_range a", plist);

  /* special case */
  if (plist->itemcount == 0 &&
      ((startpos == +1 && endpos == -1) || (startpos == -1 && endpos == +1)))
  {
    return (LCRZO_ERR_OK);
  }

  /* go to the start position */
  lcrzo_er(lcrzo_priv_list_range_goto(plist, startpos, endpos,
                                      &posstartpositive, &posendpositive,
                                      &pitemstart,
                                      &directiontoend, &hopstoend));
  itemstosort = hopstoend + 1;

  /* special case */
  if (hopstoend == 0) {
    return (LCRZO_ERR_OK);
  }

  /* determine the number of loops */
  tmpvar = 1;
  numloops = 0;
  while (tmpvar < itemstosort) {
    numloops++;
    tmpvar <<= 1;
  }

  if (directiontoend == LCRZO_PRIV_INCREASE) {
    /* save start */
    pwork1 = pitemstart->pprevious;
    pwork2 = NULL;              /* for compiler warning */
    /* loop */
    for (i = 0, numinhalflist = 1; i < numloops; i++, numinhalflist <<= 1) {
      psorted = pwork1;
      /* compute the number of subloops */
      numinhalflist2 = numinhalflist << 1;
      numsubloops = (itemstosort + numinhalflist2 - 1) / numinhalflist2;
      /* sub-loop */
      for (j = 0; j < numsubloops; j++) {
        /* determine halflist sizes and go to working positions */
        if (j < numsubloops - 1) {
          l1size = numinhalflist;
          l2size = numinhalflist;
        } else {
          tmpvar = itemstosort - j * numinhalflist2;
          if (tmpvar > numinhalflist) {
            l1size = numinhalflist;
            l2size = tmpvar - numinhalflist;
          } else {
            l1size = tmpvar;
            l2size = 0;
          }
        }
        pl1 = psorted->pnext;
        pl1last = pl1;
        for (k = 1; k < l1size; k++)
          pl1last = pl1last->pnext;
        if (l2size) {
          pl2 = pl1last->pnext;
          l1l2size = l1size + l2size;
        } else {
          pl2 = NULL;
          l1l2size = l1size;
        }
        /* rearrange the list using pl1 and pl2 */
        for (k = 0; k < l1l2size; k++) {
          /* find from where to come */
          comefroml1 = LCRZO_FALSE;
          if (l1size == 0) {
            if (l2size == 0)
              return (LCRZO_ERR_IEINTERNALERROR);
            /* comefroml1=LCRZO_FALSE; already set */
          } else if (l2size == 0) {
            if (l1size == 0)
              return (LCRZO_ERR_IEINTERNALERROR);
            comefroml1 = LCRZO_TRUE;
          } else {
            if (usefunc1) {
              cmp = (*pfunc_compare1) (pl1->pitem, pl2->pitem, pinfos);
            } else {
              ret = (*pfunc_compare2) (pl1->pitem, pl2->pitem, pinfos, &cmp);
              if (ret != LCRZO_ERR_OK) {
                /* need to relink the list */
                if (l1size != 0) {
                  psorted->pnext = pl1;
                  pl1->pprevious = psorted;
                  psorted = pl1last;
                }
                if (l2size != 0) {
                  psorted->pnext = pl2;
                  pl2->pprevious = psorted;
                }
                lcrzo_er(lcrzo_priv_list_pos_save1
                         (plist, posstartpositive - 1, pwork1));
                lcrzo__debug_list_check("lcrzo_list_list_sort_range c",
                                        plist);
                return (ret);
              }
            }
            if (cmp <= 0)
              comefroml1 = LCRZO_TRUE;
            /* else comefroml1=LCRZO_FALSE; already set */
          }
          /* append the good item */
          if (comefroml1) {
            psorted->pnext = pl1;
            pl1->pprevious = psorted;
            pl1 = pl1->pnext;
            l1size--;
            if (pl2 == NULL && l1size == 0)
              pwork2 = pl1;     /* save end */
          } else {
            psorted->pnext = pl2;
            pl2->pprevious = psorted;
            pl2 = pl2->pnext;
            l2size--;
            if (l2size == 0)
              pwork2 = pl2;     /* save end */
          }
          psorted = psorted->pnext;
        }
        if (l1size || l2size)
          return (LCRZO_ERR_IEINTERNALERROR);
        /* just link the last item */
        psorted->pnext = pwork2;
        pwork2->pprevious = psorted;
#if LCRZO_DEBUG_LIST_CHECK==1
        /* to debug : the list should be OK at this point */
        lcrzo_er(lcrzo_priv_list_pos_save1
                 (plist, 0, (lcrzo_listitem *) plist));
        lcrzo__debug_list_check("lcrzo_list_list_sort_range z1", plist);
#endif
      }
    }
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive - 1, pwork1,
                                       posendpositive + 1, pwork2));
  } else {
    /* save start */
    pwork1 = pitemstart->pnext;
    pwork2 = NULL;              /* for compiler warning */
    /* loop */
    for (i = 0, numinhalflist = 1; i < numloops; i++, numinhalflist <<= 1) {
      psorted = pwork1;
      /* compute the number of subloops */
      numinhalflist2 = numinhalflist << 1;
      numsubloops = (itemstosort + numinhalflist2 - 1) / numinhalflist2;
      /* sub-loop */
      for (j = 0; j < numsubloops; j++) {
        /* determine halflist sizes and go to working positions */
        if (j < numsubloops - 1) {
          l1size = numinhalflist;
          l2size = numinhalflist;
        } else {
          tmpvar = itemstosort - j * numinhalflist2;
          if (tmpvar > numinhalflist) {
            l1size = numinhalflist;
            l2size = tmpvar - numinhalflist;
          } else {
            l1size = tmpvar;
            l2size = 0;
          }
        }
        pl1 = psorted->pprevious;
        pl1last = pl1;
        for (k = 1; k < l1size; k++)
          pl1last = pl1last->pprevious;
        if (l2size) {
          pl2 = pl1last->pprevious;
          l1l2size = l1size + l2size;
        } else {
          pl2 = NULL;
          l1l2size = l1size;
        }
        /* rearrange the list using pl1 and pl2 */
        for (k = 0; k < l1l2size; k++) {
          /* find from where to come */
          comefroml1 = LCRZO_FALSE;
          if (l1size == 0) {
            if (l2size == 0)
              return (LCRZO_ERR_IEINTERNALERROR);
            /* comefroml1=LCRZO_FALSE; already set */
          } else if (l2size == 0) {
            if (l1size == 0)
              return (LCRZO_ERR_IEINTERNALERROR);
            comefroml1 = LCRZO_TRUE;
          } else {
            if (usefunc1) {
              cmp = (*pfunc_compare1) (pl1->pitem, pl2->pitem, pinfos);
            } else {
              ret = (*pfunc_compare2) (pl1->pitem, pl2->pitem, pinfos, &cmp);
              if (ret != LCRZO_ERR_OK) {
                /* need to relink the list */
                if (l1size != 0) {
                  psorted->pprevious = pl1;
                  pl1->pnext = psorted;
                  psorted = pl1last;
                }
                if (l2size != 0) {
                  psorted->pprevious = pl2;
                  pl2->pnext = psorted;
                }
                lcrzo_er(lcrzo_priv_list_pos_save1
                         (plist, posstartpositive + 1, pwork1));
                lcrzo__debug_list_check("lcrzo_list_list_sort_range c",
                                        plist);
                return (ret);
              }
            }
            if (cmp <= 0)
              comefroml1 = LCRZO_TRUE;
            /* else comefroml1=LCRZO_FALSE; already set */
          }
          /* append the good item */
          if (comefroml1) {
            psorted->pprevious = pl1;
            pl1->pnext = psorted;
            pl1 = pl1->pprevious;
            l1size--;
            if (pl2 == NULL && l1size == 0)
              pwork2 = pl1;     /* save end */
          } else {
            psorted->pprevious = pl2;
            pl2->pnext = psorted;
            pl2 = pl2->pprevious;
            l2size--;
            if (l2size == 0)
              pwork2 = pl2;     /* save end */
          }
          psorted = psorted->pprevious;
        }
        if (l1size || l2size)
          return (LCRZO_ERR_IEINTERNALERROR);
        /* just link the last item */
        psorted->pprevious = pwork2;
        pwork2->pnext = psorted;
#if LCRZO_DEBUG_LIST_CHECK==1
        /* to debug : the list should be OK at this point */
        lcrzo_er(lcrzo_priv_list_pos_save1
                 (plist, 0, (lcrzo_listitem *) plist));
        lcrzo__debug_list_check("lcrzo_list_list_sort_range z2", plist);
#endif
      }
    }
    /* save working positions */
    lcrzo_er(lcrzo_priv_list_pos_save2(plist, posstartpositive + 1, pwork1,
                                       posendpositive - 1, pwork2));
  }

  lcrzo__debug_list_check("lcrzo_list_list_sort_range b", plist);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_sort_range2(lcrzo_list * plist,
                           lcrzo_int32 startpos,
                           lcrzo_int32 endpos,
                           int (*pfunc_compare) (lcrzo_constptr piteminf,
                                                 lcrzo_constptr pitemsup,
                                                 lcrzo_ptr pinfos,
                                                 lcrzo_int8 * pcmp),
                           lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_sort_range(plist, startpos, endpos,
                                      LCRZO_FALSE,
                                      NULL, pfunc_compare, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_sort_all2(lcrzo_list * plist,
                         int (*pfunc_compare) (lcrzo_constptr piteminf,
                                               lcrzo_constptr pitemsup,
                                               lcrzo_ptr pinfos,
                                               lcrzo_int8 * pcmp),
                         lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_sort_range(plist, +1, -1,
                                      LCRZO_FALSE,
                                      NULL, pfunc_compare, pinfos));
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#define LCRZO_DATALIST_MIX_PATTERN 0x90ABCDEFu
int lcrzo_priv_datalist_free(lcrzo_ptr pitem);
int lcrzo_priv_datalist_free(lcrzo_ptr pitem)
{
  lcrzo_datalist *pdatalist;

  pdatalist = (lcrzo_datalist *) pitem;
#if LCRZO_DEBUG_DATASTRINGLIST_MIX==1
  if (pdatalist->pattern != LCRZO_DATALIST_MIX_PATTERN) {
    return (LCRZO_ERR_PAUNKNOWNTYPE);
  }
#endif
  lcrzo_er(lcrzo_data_free2(&(pdatalist->pdata)));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_datalist_init(lcrzo_list * plist)
{
  return (lcrzo_list_init(plist, sizeof(lcrzo_datalist),
                          &lcrzo_priv_datalist_free));
}

/*-------------------------------------------------------------*/
int lcrzo_datalist_add_pos(lcrzo_list * plist,
                           lcrzo_int32 pos,
                           lcrzo_uint32 datatype,
                           lcrzo_constdata data,
                           lcrzo_int32 datasize)
{
  lcrzo_datalist datalist;

  /*parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (data == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*create the data */
#if LCRZO_DEBUG_DATASTRINGLIST_MIX==1
  datalist.pattern = LCRZO_DATALIST_MIX_PATTERN;
#endif
  datalist.datatype = datatype;
  datalist.datasize = datasize;
  lcrzo_er(lcrzo_data_alloc(datasize, &(datalist.pdata)));
  memcpy(datalist.pdata, data, datasize);

  /*add the data */
  lcrzo_er(lcrzo_list_add_pos(plist, pos, &datalist));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_datalist_valuem_pos(lcrzo_list * plist,
                              lcrzo_int32 pos,
                              lcrzo_uint32 * pdatatype,
                              lcrzo_data * pdata,
                              lcrzo_int32 * pdatasize)
{
  lcrzo_datalist datalist;

  /*get the value */
  lcrzo_er(lcrzo_list_value_pos(plist, pos, &datalist));

#if LCRZO_DEBUG_DATASTRINGLIST_MIX==1
  if (datalist.pattern != LCRZO_DATALIST_MIX_PATTERN) {
    return (LCRZO_ERR_PAUNKNOWNTYPE);
  }
#endif

  if (pdatatype != NULL)
    *pdatatype = datalist.datatype;
  return (lcrzo_data_initm_data
          (datalist.pdata, datalist.datasize, pdata, pdatasize));
}

/*-------------------------------------------------------------*/
#define LCRZO_STRINGLIST_MIX_PATTERN 0x12345678u
int lcrzo_priv_stringlist_free(lcrzo_ptr pitem);
int lcrzo_priv_stringlist_free(lcrzo_ptr pitem)
{
  lcrzo_stringlist *pstringlist;

  pstringlist = (lcrzo_stringlist *) pitem;
#if LCRZO_DEBUG_DATASTRINGLIST_MIX==1
  if (pstringlist->pattern != LCRZO_STRINGLIST_MIX_PATTERN) {
    return (LCRZO_ERR_PAUNKNOWNTYPE);
  }
#endif
  lcrzo_er(lcrzo_string_free2(&(pstringlist->pstring)));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_stringlist_init(lcrzo_list * plist)
{
  return (lcrzo_list_init(plist, sizeof(lcrzo_stringlist),
                          &lcrzo_priv_stringlist_free));
}

/*-------------------------------------------------------------*/
int lcrzo_stringlist_add_pos(lcrzo_list * plist,
                             lcrzo_int32 pos,
                             lcrzo_uint32 stringtype,
                             lcrzo_conststring string)
{
  lcrzo_stringlist stringlist;

  /*parameters verification */
  if (plist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (string == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*create the string */
  stringlist.stringtype = stringtype;
  lcrzo_er(lcrzo_string_initm_text(string, &(stringlist.pstring)));
#if LCRZO_DEBUG_DATASTRINGLIST_MIX==1
  stringlist.pattern = LCRZO_STRINGLIST_MIX_PATTERN;
#endif

  /*add the string */
  lcrzo_er(lcrzo_list_add_pos(plist, pos, &stringlist));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_stringlist_valuem_pos(lcrzo_list * plist,
                                lcrzo_int32 pos,
                                lcrzo_uint32 * pstringtype,
                                lcrzo_string * pstring)
{
  lcrzo_stringlist stringlist;

  /*get the value */
  lcrzo_er(lcrzo_list_value_pos(plist, pos, &stringlist));

#if LCRZO_DEBUG_DATASTRINGLIST_MIX==1
  if (stringlist.pattern != LCRZO_STRINGLIST_MIX_PATTERN) {
    return (LCRZO_ERR_PAUNKNOWNTYPE);
  }
#endif

  if (pstringtype != NULL)
    *pstringtype = stringlist.stringtype;
  lcrzo_er(lcrzo_string_initm_text(stringlist.pstring, pstring));

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_portlist_init(lcrzo_list * plist)
{
  return (lcrzo_list_init(plist, sizeof(lcrzo_portlist), NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_portlist_add_range(lcrzo_list * plist,
                             lcrzo_uint16 infport,
                             lcrzo_uint16 support)
{
  lcrzo_int32 count, i;
  lcrzo_portlist foundportlist, newportlist;

  if (infport > support)
    return (LCRZO_ERR_PAINFHIGHERSUP);

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &foundportlist));
    if (foundportlist.inf > support) {
      /* add it just before */
      newportlist.inf = infport;
      newportlist.sup = support;
      lcrzo_er(lcrzo_list_add_pos(plist, i, &newportlist));
      return (LCRZO_ERR_OK);
    }
    if (infport < foundportlist.inf && support <= foundportlist.sup) {
      /* extend current item */
      foundportlist.inf = infport;
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundportlist));
      return (LCRZO_ERR_OK);
    }
    if (infport >= foundportlist.inf && support <= foundportlist.sup) {
      /* inside current item */
      return (LCRZO_ERR_OK);
    }
    if (infport <= foundportlist.sup && support > foundportlist.sup) {
      /* extend current item */
      foundportlist.sup = support;
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundportlist));
      return (LCRZO_ERR_OK);
    }
  }

  /* add it at the end */
  newportlist.inf = infport;
  newportlist.sup = support;
  lcrzo_er(lcrzo_list_add_last(plist, &newportlist));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_portlist_del_range(lcrzo_list * plist,
                             lcrzo_uint16 infport,
                             lcrzo_uint16 support)
{
  lcrzo_int32 count, i;
  lcrzo_portlist foundportlist, newportlist;

  if (infport > support)
    return (LCRZO_ERR_PAINFHIGHERSUP);

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &foundportlist));
    if (foundportlist.inf > support) {
      /* nothing to remove */
      return (LCRZO_ERR_OK);
    } else if (infport <= foundportlist.inf && support < foundportlist.sup) {
      /* truncate current item */
      foundportlist.inf = (lcrzo_uint16) (support + 1);
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundportlist));
      return (LCRZO_ERR_OK);
    } else if (infport <= foundportlist.inf && support >= foundportlist.sup) {
      /* remove current item */
      lcrzo_er(lcrzo_list_remove_pos(plist, i));
      /* eventually continue to remove */
      i--;
      count--;
      continue;
    } else if (infport > foundportlist.inf && support < foundportlist.sup) {
      /* cut the current item in two */
      newportlist.inf = foundportlist.inf;
      foundportlist.inf = (lcrzo_uint16) (support + 1);
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundportlist));
      newportlist.sup = (lcrzo_uint16) (infport - 1);
      lcrzo_er(lcrzo_list_add_pos(plist, i, &newportlist));
      return (LCRZO_ERR_OK);
    } else if (infport <= foundportlist.sup && support >= foundportlist.sup) {
      /* truncate current item */
      foundportlist.sup = (lcrzo_uint16) (infport - 1);
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundportlist));
      /* eventually continue to remove */
      continue;
    }
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_portlist_init_text(lcrzo_conststring porttext,
                             lcrzo_list * plist)
{
  lcrzo_string porttext1, porttext2;
  int ret;
  lcrzo_int32 indic, startpos, endpos;
  lcrzo_uint16 infport, support;

  /* suppress unwanted characters */
  lcrzo_er(lcrzo_string_replacem_nchar(porttext, LCRZO_FALSE, +1, -1, 0,
                                       " \t", "", &porttext1));
  ret = lcrzo_string_replacem_nchar(porttext1, LCRZO_FALSE, +1, -1, 0,
                                    "\r\n", ",", &porttext2);
  lcrzo_er(lcrzo_string_free2(&porttext1));
  if (ret != LCRZO_ERR_OK)
    return (ret);

  /* construct the list */
  ret = lcrzo_portlist_init(plist);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_string_free2(&porttext2));
    return (ret);
  }
  indic = 0;
  ret = lcrzo_string_token_char(porttext2, LCRZO_FALSE, +1, -1, ',',
                                &indic, NULL, &startpos, NULL, &endpos);
  while (ret == LCRZO_ERR_OK) {
    ret = lcrzo_string_initm_range(porttext2, startpos, endpos, &porttext1);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_string_free2(&porttext2));
      return (ret);
    }
    /* treat special cases */
    if (strlen(porttext1) == 0) {       /* do nothing */
    } else if (!lcrzo_priv_strcasecmp(porttext1, "any")
               || !lcrzo_priv_strcasecmp(porttext1, "all")) {
      /* add all range */
      ret = lcrzo_portlist_add_range(plist, 0, 0xFFFF);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&porttext1));
        lcrzo_er(lcrzo_string_free2(&porttext2));
        return (ret);
      }
    } else if (porttext1[0] == '!') {
      /* remove this range */
      ret = lcrzo_port_init_portrange(porttext1 + 1, &infport, &support);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&porttext1));
        lcrzo_er(lcrzo_string_free2(&porttext2));
        return (ret);
      }
      ret = lcrzo_portlist_del_range(plist, infport, support);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&porttext1));
        lcrzo_er(lcrzo_string_free2(&porttext2));
        return (ret);
      }
    } else {
      /* add this range */
      ret = lcrzo_port_init_portrange(porttext1, &infport, &support);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&porttext1));
        lcrzo_er(lcrzo_string_free2(&porttext2));
        return (ret);
      }
      ret = lcrzo_portlist_add_range(plist, infport, support);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&porttext1));
        lcrzo_er(lcrzo_string_free2(&porttext2));
        return (ret);
      }
    }

    /* get next token */
    lcrzo_er(lcrzo_string_free2(&porttext1));
    ret = lcrzo_string_token_char(porttext2, LCRZO_FALSE, +1, -1, ',',
                                  &indic, NULL, &startpos, NULL, &endpos);
  }

  lcrzo_er(lcrzo_string_free2(&porttext2));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_string_initm_portlist(lcrzo_list * plist,
                                lcrzo_string * pporttext)
{
  lcrzo_int32 count, i;
  lcrzo_portlist portlist;
  lcrzo_portrange range;
  lcrzo_bool firstrange;

  lcrzo_er(lcrzo_list_count(plist, &count));
  lcrzo_er(lcrzo_string_initm_text("", pporttext));
  firstrange = LCRZO_TRUE;
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &portlist));
    lcrzo_er(lcrzo_portrange_init_port(portlist.inf, portlist.sup, range));
    if (!firstrange) {
      lcrzo_er(lcrzo_string_appendm_text(",", pporttext));
    }
    firstrange = LCRZO_FALSE;
    lcrzo_er(lcrzo_string_appendm_text(range, pporttext));
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_portlist_isinlist(lcrzo_list * plist,
                            lcrzo_uint16 port,
                            lcrzo_bool * pyes)
{
  lcrzo_int32 count, i;
  lcrzo_portlist portlist;

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &portlist));
    if (port >= portlist.inf && port <= portlist.sup) {
      if (pyes != NULL)
        *pyes = LCRZO_TRUE;
      return (LCRZO_ERR_OK);
    }
  }
  if (pyes != NULL)
    *pyes = LCRZO_FALSE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_portlist_value_first(lcrzo_list * plist,
                               lcrzo_uint16 * pport)
{
  lcrzo_int32 count;
  lcrzo_portlist portlist;

  lcrzo_er(lcrzo_list_count(plist, &count));
  if (count == 0)
    return (LCRZO_ERR_OKDATAEND);

  lcrzo_er(lcrzo_list_value_first(plist, &portlist));
  if (pport != NULL)
    *pport = portlist.inf;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_portlist_value_next(lcrzo_list * plist,
                              lcrzo_uint16 port,
                              lcrzo_uint16 * pnextport)
{
  lcrzo_int32 count, i;
  lcrzo_portlist portlist;

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &portlist));
    if (port < portlist.inf) {
      if (pnextport != NULL)
        *pnextport = portlist.inf;
      return (LCRZO_ERR_OK);
    }
    if (port >= portlist.inf && port < portlist.sup) {
      if (pnextport != NULL)
        *pnextport = (lcrzo_uint16) (port + 1);
      return (LCRZO_ERR_OK);
    }
  }

  return (LCRZO_ERR_OKDATAEND);
}


/*-------------------------------------------------------------*/
int lcrzo_iplist_init(lcrzo_list * plist)
{
  return (lcrzo_list_init(plist, sizeof(lcrzo_iplist), NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_iplist_add_range(lcrzo_list * plist,
                           lcrzo_ipl infipl,
                           lcrzo_ipl supipl)
{
  lcrzo_int32 count, i;
  lcrzo_iplist foundiplist, newiplist;

  if (infipl > supipl)
    return (LCRZO_ERR_PAINFHIGHERSUP);

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &foundiplist));
    if (foundiplist.inf > supipl) {
      /* add it just before */
      newiplist.inf = infipl;
      newiplist.sup = supipl;
      lcrzo_er(lcrzo_list_add_pos(plist, i, &newiplist));
      return (LCRZO_ERR_OK);
    }
    if (infipl < foundiplist.inf && supipl <= foundiplist.sup) {
      /* extend current item */
      foundiplist.inf = infipl;
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundiplist));
      return (LCRZO_ERR_OK);
    }
    if (infipl >= foundiplist.inf && supipl <= foundiplist.sup) {
      /* inside current item */
      return (LCRZO_ERR_OK);
    }
    if (infipl <= foundiplist.sup && supipl > foundiplist.sup) {
      /* extend current item */
      foundiplist.sup = supipl;
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundiplist));
      return (LCRZO_ERR_OK);
    }
  }

  /* add it at the end */
  newiplist.inf = infipl;
  newiplist.sup = supipl;
  lcrzo_er(lcrzo_list_add_last(plist, &newiplist));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_iplist_del_range(lcrzo_list * plist,
                           lcrzo_ipl infipl,
                           lcrzo_ipl supipl)
{
  lcrzo_int32 count, i;
  lcrzo_iplist foundiplist, newiplist;

  if (infipl > supipl)
    return (LCRZO_ERR_PAINFHIGHERSUP);

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &foundiplist));
    if (foundiplist.inf > supipl) {
      /* nothing to remove */
      return (LCRZO_ERR_OK);
    } else if (infipl <= foundiplist.inf && supipl < foundiplist.sup) {
      /* truncate current item */
      foundiplist.inf = supipl + 1;
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundiplist));
      return (LCRZO_ERR_OK);
    } else if (infipl <= foundiplist.inf && supipl >= foundiplist.sup) {
      /* remove current item */
      lcrzo_er(lcrzo_list_remove_pos(plist, i));
      /* eventually continue to remove */
      i--;
      count--;
      continue;
    } else if (infipl > foundiplist.inf && supipl < foundiplist.sup) {
      /* cut the current item in two */
      newiplist.inf = foundiplist.inf;
      foundiplist.inf = supipl + 1;
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundiplist));
      newiplist.sup = infipl - 1;
      lcrzo_er(lcrzo_list_add_pos(plist, i, &newiplist));
      return (LCRZO_ERR_OK);
    } else if (infipl <= foundiplist.sup && supipl >= foundiplist.sup) {
      /* truncate current item */
      foundiplist.sup = infipl - 1;
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundiplist));
      /* eventually continue to remove */
      continue;
    }
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_iplist_init_text(lcrzo_conststring ipltext,
                           lcrzo_list * plist)
{
  lcrzo_string ipltext1, ipltext2;
  int ret;
  lcrzo_int32 indic, startpos, endpos;
  lcrzo_ipl infipl, supipl;

  /* suppress unwanted characters */
  lcrzo_er(lcrzo_string_replacem_nchar(ipltext, LCRZO_FALSE, +1, -1, 0,
                                       " \t", "", &ipltext1));
  ret = lcrzo_string_replacem_nchar(ipltext1, LCRZO_FALSE, +1, -1, 0,
                                    "\r\n", ",", &ipltext2);
  lcrzo_er(lcrzo_string_free2(&ipltext1));
  if (ret != LCRZO_ERR_OK)
    return (ret);

  /* construct the list */
  ret = lcrzo_iplist_init(plist);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_string_free2(&ipltext2));
    return (ret);
  }
  indic = 0;
  ret = lcrzo_string_token_char(ipltext2, LCRZO_FALSE, +1, -1, ',',
                                &indic, NULL, &startpos, NULL, &endpos);
  while (ret == LCRZO_ERR_OK) {
    ret = lcrzo_string_initm_range(ipltext2, startpos, endpos, &ipltext1);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_string_free2(&ipltext2));
      return (ret);
    }
    /* treat special cases */
    if (strlen(ipltext1) == 0) {
      /* do nothing */
    } else if (!lcrzo_priv_strcasecmp(ipltext1, "any")
               || !lcrzo_priv_strcasecmp(ipltext1, "all")) {
      /* add all range */
      ret = lcrzo_iplist_add_range(plist, 0, 0xFFFFFFFFu);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ipltext1));
        lcrzo_er(lcrzo_string_free2(&ipltext2));
        return (ret);
      }
    } else if (ipltext1[0] == '!') {
      /* remove this range */
      ret = lcrzo_ipl_init_iprange(ipltext1 + 1, &infipl, &supipl);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ipltext1));
        lcrzo_er(lcrzo_string_free2(&ipltext2));
        return (ret);
      }
      ret = lcrzo_iplist_del_range(plist, infipl, supipl);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ipltext1));
        lcrzo_er(lcrzo_string_free2(&ipltext2));
        return (ret);
      }
    } else {
      /* add this range */
      ret = lcrzo_ipl_init_iprange(ipltext1, &infipl, &supipl);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ipltext1));
        lcrzo_er(lcrzo_string_free2(&ipltext2));
        return (ret);
      }
      ret = lcrzo_iplist_add_range(plist, infipl, supipl);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ipltext1));
        lcrzo_er(lcrzo_string_free2(&ipltext2));
        return (ret);
      }
    }

    /* get next token */
    lcrzo_er(lcrzo_string_free2(&ipltext1));
    ret = lcrzo_string_token_char(ipltext2, LCRZO_FALSE, +1, -1, ',',
                                  &indic, NULL, &startpos, NULL, &endpos);
  }

  lcrzo_er(lcrzo_string_free2(&ipltext2));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_string_initm_iplist(lcrzo_list * plist,
                              lcrzo_string * pipltext)
{
  lcrzo_int32 count, i;
  lcrzo_iplist iplist;
  lcrzo_iprange range;
  lcrzo_bool firstrange;

  lcrzo_er(lcrzo_list_count(plist, &count));
  lcrzo_er(lcrzo_string_initm_text("", pipltext));
  firstrange = LCRZO_TRUE;
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &iplist));
    lcrzo_er(lcrzo_iprange_init_ipl(iplist.inf, iplist.sup, range));
    if (!firstrange) {
      lcrzo_er(lcrzo_string_appendm_text(",", pipltext));
    }
    firstrange = LCRZO_FALSE;
    lcrzo_er(lcrzo_string_appendm_text(range, pipltext));
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_iplist_isinlist(lcrzo_list * plist,
                          lcrzo_ipl ipl,
                          lcrzo_bool * pyes)
{
  lcrzo_int32 count, i;
  lcrzo_iplist iplist;

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &iplist));
    if (ipl >= iplist.inf && ipl <= iplist.sup) {
      if (pyes != NULL)
        *pyes = LCRZO_TRUE;
      return (LCRZO_ERR_OK);
    }
  }
  if (pyes != NULL)
    *pyes = LCRZO_FALSE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_iplist_value_first(lcrzo_list * plist,
                             lcrzo_ipl * pipl)
{
  lcrzo_int32 count;
  lcrzo_iplist iplist;

  lcrzo_er(lcrzo_list_count(plist, &count));
  if (count == 0)
    return (LCRZO_ERR_OKDATAEND);

  lcrzo_er(lcrzo_list_value_first(plist, &iplist));
  if (pipl != NULL)
    *pipl = iplist.inf;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_iplist_value_next(lcrzo_list * plist,
                            lcrzo_ipl ipl,
                            lcrzo_ipl * pnextipl)
{
  lcrzo_int32 count, i;
  lcrzo_iplist iplist;

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &iplist));
    if (ipl < iplist.inf) {
      if (pnextipl != NULL)
        *pnextipl = iplist.inf;
      return (LCRZO_ERR_OK);
    }
    if (ipl >= iplist.inf && ipl < iplist.sup) {
      if (pnextipl != NULL)
        *pnextipl = ipl + 1;
      return (LCRZO_ERR_OK);
    }
  }

  return (LCRZO_ERR_OKDATAEND);
}


/*-------------------------------------------------------------*/
int lcrzo_ethlist_init(lcrzo_list * plist)
{
  return (lcrzo_list_init(plist, sizeof(lcrzo_ethlist), NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_ethlist_add_range(lcrzo_list * plist,
                            lcrzo_etha infetha,
                            lcrzo_etha supetha)
{
  lcrzo_int32 count, i;
  lcrzo_ethlist foundethlist, newethlist;
  lcrzo_bool b1, b2;

  lcrzo_er(lcrzo_priv_etha_gt(infetha, supetha, &b1));
  if (b1)
    return (LCRZO_ERR_PAINFHIGHERSUP);

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &foundethlist));
    lcrzo_er(lcrzo_priv_etha_gt(foundethlist.inf, supetha, &b1));
    if (b1) {
      /* add it just before */
      memcpy(newethlist.inf, infetha, LCRZO_ETHA_MAXBYTES);
      memcpy(newethlist.sup, supetha, LCRZO_ETHA_MAXBYTES);
      lcrzo_er(lcrzo_list_add_pos(plist, i, &newethlist));
      return (LCRZO_ERR_OK);
    }
    lcrzo_er(lcrzo_priv_etha_lt(infetha, foundethlist.inf, &b1));
    lcrzo_er(lcrzo_priv_etha_le(supetha, foundethlist.sup, &b2));
    if (b1 && b2) {
      /* extend current item */
      memcpy(foundethlist.inf, infetha, LCRZO_ETHA_MAXBYTES);
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundethlist));
      return (LCRZO_ERR_OK);
    }
    lcrzo_er(lcrzo_priv_etha_ge(infetha, foundethlist.inf, &b1));
    lcrzo_er(lcrzo_priv_etha_le(supetha, foundethlist.sup, &b2));
    if (b1 && b2) {
      /* inside current item */
      return (LCRZO_ERR_OK);
    }
    lcrzo_er(lcrzo_priv_etha_le(infetha, foundethlist.sup, &b1));
    lcrzo_er(lcrzo_priv_etha_gt(supetha, foundethlist.sup, &b2));
    if (b1 && b2) {
      /* extend current item */
      memcpy(foundethlist.sup, supetha, LCRZO_ETHA_MAXBYTES);
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundethlist));
      return (LCRZO_ERR_OK);
    }
  }

  /* add it at the end */
  memcpy(newethlist.inf, infetha, LCRZO_ETHA_MAXBYTES);
  memcpy(newethlist.sup, supetha, LCRZO_ETHA_MAXBYTES);
  lcrzo_er(lcrzo_list_add_last(plist, &newethlist));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_ethlist_del_range(lcrzo_list * plist,
                            lcrzo_etha infetha,
                            lcrzo_etha supetha)
{
  lcrzo_int32 count, i;
  lcrzo_ethlist foundethlist, newethlist;
  lcrzo_bool b1, b2;

  lcrzo_er(lcrzo_priv_etha_gt(infetha, supetha, &b1));
  if (b1)
    return (LCRZO_ERR_PAINFHIGHERSUP);

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &foundethlist));
    lcrzo_er(lcrzo_priv_etha_gt(foundethlist.inf, supetha, &b1));
    if (b1) {
      /* nothing to remove */
      return (LCRZO_ERR_OK);
    }
    lcrzo_er(lcrzo_priv_etha_le(infetha, foundethlist.inf, &b1));
    lcrzo_er(lcrzo_priv_etha_lt(supetha, foundethlist.sup, &b2));
    if (b1 && b2) {
      /* truncate current item */
      lcrzo_er(lcrzo_priv_etha_inc(supetha, foundethlist.inf));
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundethlist));
      return (LCRZO_ERR_OK);
    }
    lcrzo_er(lcrzo_priv_etha_le(infetha, foundethlist.inf, &b1));
    lcrzo_er(lcrzo_priv_etha_ge(supetha, foundethlist.sup, &b2));
    if (b1 && b2) {
      /* remove current item */
      lcrzo_er(lcrzo_list_remove_pos(plist, i));
      /* eventually continue to remove */
      i--;
      count--;
      continue;
    }
    lcrzo_er(lcrzo_priv_etha_gt(infetha, foundethlist.inf, &b1));
    lcrzo_er(lcrzo_priv_etha_lt(supetha, foundethlist.sup, &b2));
    if (b1 && b2) {
      /* cut the current item in two */
      memcpy(newethlist.inf, foundethlist.inf, LCRZO_ETHA_MAXBYTES);
      lcrzo_er(lcrzo_priv_etha_inc(supetha, foundethlist.inf));
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundethlist));
      lcrzo_er(lcrzo_priv_etha_dec(infetha, newethlist.sup));
      lcrzo_er(lcrzo_list_add_pos(plist, i, &newethlist));
      return (LCRZO_ERR_OK);
    }
    lcrzo_er(lcrzo_priv_etha_le(infetha, foundethlist.sup, &b1));
    lcrzo_er(lcrzo_priv_etha_ge(supetha, foundethlist.sup, &b2));
    if (b1 && b2) {
      /* truncate current item */
      lcrzo_er(lcrzo_priv_etha_dec(infetha, foundethlist.sup));
      lcrzo_er(lcrzo_list_replace_pos(plist, i, &foundethlist));
      /* eventually continue to remove */
      continue;
    }
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_ethlist_init_text(lcrzo_conststring ethatext,
                            lcrzo_list * plist)
{
  lcrzo_string ethatext1, ethatext2;
  int ret;
  lcrzo_int32 indic, startpos, endpos;
  lcrzo_etha infetha, supetha;

  /* suppress unwanted characters */
  lcrzo_er(lcrzo_string_replacem_nchar(ethatext, LCRZO_FALSE, +1, -1, 0,
                                       " \t", "", &ethatext1));
  ret = lcrzo_string_replacem_nchar(ethatext1, LCRZO_FALSE, +1, -1, 0,
                                    "\r\n", ",", &ethatext2);
  lcrzo_er(lcrzo_string_free2(&ethatext1));
  if (ret != LCRZO_ERR_OK)
    return (ret);

  /* construct the list */
  ret = lcrzo_ethlist_init(plist);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_string_free2(&ethatext2));
    return (ret);
  }
  indic = 0;
  ret = lcrzo_string_token_char(ethatext2, LCRZO_FALSE, +1, -1, ',',
                                &indic, NULL, &startpos, NULL, &endpos);
  while (ret == LCRZO_ERR_OK) {
    ret = lcrzo_string_initm_range(ethatext2, startpos, endpos, &ethatext1);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_string_free2(&ethatext2));
      return (ret);
    }
    /* treat special cases */
    if (strlen(ethatext1) == 0) {
      /* do nothing */
    } else if (!lcrzo_priv_strcasecmp(ethatext1, "any")
               || !lcrzo_priv_strcasecmp(ethatext1, "all")) {
      /* add all range */
      ret = lcrzo_etha_init(0, 0, 0, 0, 0, 0, infetha);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ethatext1));
        lcrzo_er(lcrzo_string_free2(&ethatext2));
        return (ret);
      }
      ret = lcrzo_etha_init(0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, supetha);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ethatext1));
        lcrzo_er(lcrzo_string_free2(&ethatext2));
        return (ret);
      }
      ret = lcrzo_ethlist_add_range(plist, infetha, supetha);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ethatext1));
        lcrzo_er(lcrzo_string_free2(&ethatext2));
        return (ret);
      }
    } else if (ethatext1[0] == '!') {
      /* remove this range */
      ret = lcrzo_etha_init_ethrange(ethatext1 + 1, infetha, supetha);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ethatext1));
        lcrzo_er(lcrzo_string_free2(&ethatext2));
        return (ret);
      }
      ret = lcrzo_ethlist_del_range(plist, infetha, supetha);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ethatext1));
        lcrzo_er(lcrzo_string_free2(&ethatext2));
        return (ret);
      }
    } else {
      /* add this range */
      ret = lcrzo_etha_init_ethrange(ethatext1, infetha, supetha);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ethatext1));
        lcrzo_er(lcrzo_string_free2(&ethatext2));
        return (ret);
      }
      ret = lcrzo_ethlist_add_range(plist, infetha, supetha);
      if (ret != LCRZO_ERR_OK) {
        lcrzo_er(lcrzo_string_free2(&ethatext1));
        lcrzo_er(lcrzo_string_free2(&ethatext2));
        return (ret);
      }
    }

    /* get next token */
    lcrzo_er(lcrzo_string_free2(&ethatext1));
    ret = lcrzo_string_token_char(ethatext2, LCRZO_FALSE, +1, -1, ',',
                                  &indic, NULL, &startpos, NULL, &endpos);
  }

  lcrzo_er(lcrzo_string_free2(&ethatext2));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_string_initm_ethlist(lcrzo_list * plist,
                               lcrzo_string * pethatext)
{
  lcrzo_int32 count, i;
  lcrzo_ethlist ethlist;
  lcrzo_ethrange range;
  lcrzo_bool firstrange;

  lcrzo_er(lcrzo_list_count(plist, &count));
  lcrzo_er(lcrzo_string_initm_text("", pethatext));
  firstrange = LCRZO_TRUE;
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &ethlist));
    lcrzo_er(lcrzo_ethrange_init_etha(ethlist.inf, ethlist.sup, range));
    if (!firstrange) {
      lcrzo_er(lcrzo_string_appendm_text(",", pethatext));
    }
    firstrange = LCRZO_FALSE;
    lcrzo_er(lcrzo_string_appendm_text(range, pethatext));
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_ethlist_isinlist(lcrzo_list * plist,
                           lcrzo_etha etha,
                           lcrzo_bool * pyes)
{
  lcrzo_int32 count, i;
  lcrzo_ethlist ethlist;
  lcrzo_bool b1, b2;

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &ethlist));
    lcrzo_er(lcrzo_priv_etha_ge(etha, ethlist.inf, &b1));
    lcrzo_er(lcrzo_priv_etha_le(etha, ethlist.sup, &b2));
    if (b1 && b2) {
      if (pyes != NULL)
        *pyes = LCRZO_TRUE;
      return (LCRZO_ERR_OK);
    }
  }
  if (pyes != NULL)
    *pyes = LCRZO_FALSE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_ethlist_value_first(lcrzo_list * plist,
                              lcrzo_etha etha)
{
  lcrzo_int32 count;
  lcrzo_ethlist ethlist;

  lcrzo_er(lcrzo_list_count(plist, &count));
  if (count == 0)
    return (LCRZO_ERR_OKDATAEND);

  lcrzo_er(lcrzo_list_value_first(plist, &ethlist));
  if (etha != NULL) {
    memcpy(etha, ethlist.inf, LCRZO_ETHA_MAXBYTES);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_ethlist_value_next(lcrzo_list * plist,
                             lcrzo_etha etha,
                             lcrzo_etha nextetha)
{
  lcrzo_int32 count, i;
  lcrzo_ethlist ethlist;
  lcrzo_bool b1, b2;

  lcrzo_er(lcrzo_list_count(plist, &count));
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_list_value_pos(plist, i, &ethlist));
    lcrzo_er(lcrzo_priv_etha_lt(etha, ethlist.inf, &b1));
    if (b1) {
      if (nextetha != NULL) {
        memcpy(nextetha, ethlist.inf, LCRZO_ETHA_MAXBYTES);
      }
      return (LCRZO_ERR_OK);
    }
    lcrzo_er(lcrzo_priv_etha_ge(etha, ethlist.inf, &b1));
    lcrzo_er(lcrzo_priv_etha_lt(etha, ethlist.sup, &b2));
    if (b1 && b2) {
      lcrzo_er(lcrzo_priv_etha_inc(etha, nextetha));
      return (LCRZO_ERR_OK);
    }
  }

  return (LCRZO_ERR_OKDATAEND);
}

/*-------------------------------------------------------------*/
int lcrzo_ethlist_init_stdin(lcrzo_conststring message,
                             lcrzo_conststring defaultlist,
                             lcrzo_list * plist)
{
  char prompt;
  lcrzo_string usertext;
  lcrzo_bool defautisvalid;
  lcrzo_list tmplist;
  int ret;

  /*parameters verification */
  defautisvalid = LCRZO_FALSE;
  if (defaultlist != NULL) {
    if (strlen(defaultlist)) {
      ret = lcrzo_ethlist_init_text(defaultlist, &tmplist);
      if (ret == LCRZO_ERR_OK) {
        defautisvalid = LCRZO_TRUE;
        lcrzo_er(lcrzo_list_close(&tmplist));
      }
    }
  }

  prompt = ':';
  while (LCRZO_TRUE) {
    /*affichage du message */
    if (message != NULL)
      if (message[0] != '\0') {
        printf("%s ", message);
        if (defautisvalid)
          printf("[%s]", defaultlist);
        printf("%c ", prompt);
        fflush(stdout);
      }
    prompt = '>';
    /*saisie de la chaine de l'utilisateur */
    lcrzo_er(lcrzo_priv_kbddefault_line_echo(&usertext));
    /*si rien n'a ete entre, on met le defaut */
    if (strlen(usertext) == 0 && defautisvalid) {
      lcrzo_er(lcrzo_string_free2(&usertext));
      lcrzo_er(lcrzo_ethlist_init_text(defaultlist, plist));
      break;
    }
    /*conversion */
    ret = lcrzo_ethlist_init_text(usertext, plist);
    lcrzo_er(lcrzo_string_free2(&usertext));
    if (ret == LCRZO_ERR_OK)
      break;
    lcrzo_er(lcrzo_priv_err_purge_text());
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_iplist_init_stdin(lcrzo_conststring message,
                            lcrzo_conststring defaultlist,
                            lcrzo_list * plist)
{
  char prompt;
  lcrzo_string usertext;
  lcrzo_bool defautisvalid;
  lcrzo_list tmplist;
  int ret;

  /*parameters verification */
  defautisvalid = LCRZO_FALSE;
  if (defaultlist != NULL) {
    if (strlen(defaultlist)) {
      ret = lcrzo_iplist_init_text(defaultlist, &tmplist);
      if (ret == LCRZO_ERR_OK) {
        defautisvalid = LCRZO_TRUE;
        lcrzo_er(lcrzo_list_close(&tmplist));
      }
    }
  }

  prompt = ':';
  while (LCRZO_TRUE) {
    /*affichage du message */
    if (message != NULL)
      if (message[0] != '\0') {
        printf("%s ", message);
        if (defautisvalid)
          printf("[%s]", defaultlist);
        printf("%c ", prompt);
        fflush(stdout);
      }
    prompt = '>';
    /*saisie de la chaine de l'utilisateur */
    lcrzo_er(lcrzo_priv_kbddefault_line_echo(&usertext));
    /*si rien n'a ete entre, on met le defaut */
    if (strlen(usertext) == 0 && defautisvalid) {
      lcrzo_er(lcrzo_string_free2(&usertext));
      lcrzo_er(lcrzo_iplist_init_text(defaultlist, plist));
      break;
    }
    /*conversion */
    ret = lcrzo_iplist_init_text(usertext, plist);
    lcrzo_er(lcrzo_string_free2(&usertext));
    if (ret == LCRZO_ERR_OK)
      break;
    lcrzo_er(lcrzo_priv_err_purge_text());
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_portlist_init_stdin(lcrzo_conststring message,
                              lcrzo_conststring defaultlist,
                              lcrzo_list * plist)
{
  char prompt;
  lcrzo_string usertext;
  lcrzo_bool defautisvalid;
  lcrzo_list tmplist;
  int ret;

  /*parameters verification */
  defautisvalid = LCRZO_FALSE;
  if (defaultlist != NULL) {
    if (strlen(defaultlist)) {
      ret = lcrzo_portlist_init_text(defaultlist, &tmplist);
      if (ret == LCRZO_ERR_OK) {
        defautisvalid = LCRZO_TRUE;
        lcrzo_er(lcrzo_list_close(&tmplist));
      }
    }
  }

  prompt = ':';
  while (LCRZO_TRUE) {
    /*affichage du message */
    if (message != NULL)
      if (message[0] != '\0') {
        printf("%s ", message);
        if (defautisvalid)
          printf("[%s]", defaultlist);
        printf("%c ", prompt);
        fflush(stdout);
      }
    prompt = '>';
    /*saisie de la chaine de l'utilisateur */
    lcrzo_er(lcrzo_priv_kbddefault_line_echo(&usertext));
    /*si rien n'a ete entre, on met le defaut */
    if (strlen(usertext) == 0 && defautisvalid) {
      lcrzo_er(lcrzo_string_free2(&usertext));
      lcrzo_er(lcrzo_portlist_init_text(defaultlist, plist));
      break;
    }
    /*conversion */
    ret = lcrzo_portlist_init_text(usertext, plist);
    lcrzo_er(lcrzo_string_free2(&usertext));
    if (ret == LCRZO_ERR_OK)
      break;
    lcrzo_er(lcrzo_priv_err_purge_text());
  }
  return (LCRZO_ERR_OK);
}
