/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_incl.h"

#if LCRZO_DEBUG_OBSOLETE_REJECT==0

/*-------------------------------------------------------------*/
int lcrzo_hdrpeth_init_hdrleth(lcrzo_hdrleth hdrleth,
                               lcrzo_hdrpeth * phdrpeth)
{
  if (phdrpeth != NULL) {
    memcpy(phdrpeth->dst, hdrleth.dst, LCRZO_ETHA_MAXBYTES);
    memcpy(phdrpeth->src, hdrleth.src, LCRZO_ETHA_MAXBYTES);
    phdrpeth->type = lcrzo_htons(hdrleth.type);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrleth_init_hdrpeth(lcrzo_hdrpeth hdrpeth,
                               lcrzo_hdrleth * phdrleth)
{
  if (phdrleth != NULL) {
    memcpy(phdrleth->dst, hdrpeth.dst, LCRZO_ETHA_MAXBYTES);
    memcpy(phdrleth->src, hdrpeth.src, LCRZO_ETHA_MAXBYTES);
    phdrleth->type = lcrzo_ntohs(hdrpeth.type);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrpip_init_hdrlip(lcrzo_hdrlip hdrlip,
                             lcrzo_hdrpip * phdrpip)
{
  if (hdrlip.reserve != 0 && hdrlip.reserve != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrlip.dontfrag != 0 && hdrlip.dontfrag != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrlip.morefrag != 0 && hdrlip.morefrag != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrlip.offsetfrag > 0x1FFF)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);

  if (phdrpip != NULL) {
    phdrpip->version = hdrlip.version;
    phdrpip->ihl = hdrlip.ihl;
    phdrpip->tos = hdrlip.tos;
    phdrpip->totlen = lcrzo_htons(hdrlip.totlen);
    phdrpip->id = lcrzo_htons(hdrlip.id);
    phdrpip->fragoff = (lcrzo_uint16) (LCRZO_HDRPIP_RF_HN(hdrlip.reserve) |
                                       LCRZO_HDRPIP_DF_HN(hdrlip.dontfrag) |
                                       LCRZO_HDRPIP_MF_HN(hdrlip.morefrag) |
                                       LCRZO_HDRPIP_OFF_HN(hdrlip.
                                                           offsetfrag));
    phdrpip->ttl = hdrlip.ttl;
    phdrpip->protocol = hdrlip.protocol;
    phdrpip->check = lcrzo_htons(hdrlip.check);
    phdrpip->saddr = lcrzo_htonl(hdrlip.saddr);
    phdrpip->daddr = lcrzo_htonl(hdrlip.daddr);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlip_init_hdrpip(lcrzo_hdrpip hdrpip,
                             lcrzo_hdrlip * phdrlip)
{
  if (phdrlip != NULL) {
    phdrlip->version = (lcrzo_uint8) hdrpip.version;
    phdrlip->ihl = (lcrzo_uint8) hdrpip.ihl;
    phdrlip->tos = (lcrzo_uint8) hdrpip.tos;
    phdrlip->totlen = lcrzo_ntohs(hdrpip.totlen);
    phdrlip->id = lcrzo_ntohs(hdrpip.id);
    phdrlip->reserve = LCRZO_HDRPIP_RF_NH(hdrpip.fragoff);
    phdrlip->dontfrag = LCRZO_HDRPIP_DF_NH(hdrpip.fragoff);
    phdrlip->morefrag = LCRZO_HDRPIP_MF_NH(hdrpip.fragoff);
    phdrlip->offsetfrag = LCRZO_HDRPIP_OFF_NH(hdrpip.fragoff);
    phdrlip->ttl = hdrpip.ttl;
    phdrlip->protocol = hdrpip.protocol;
    phdrlip->check = lcrzo_ntohs(hdrpip.check);
    phdrlip->saddr = lcrzo_ntohl(hdrpip.saddr);
    phdrlip->daddr = lcrzo_ntohl(hdrpip.daddr);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrpudp_init_hdrludp(lcrzo_hdrludp hdrludp,
                               lcrzo_hdrpudp * phdrpudp)
{
  if (phdrpudp != NULL) {
    phdrpudp->sport = lcrzo_htons(hdrludp.sport);
    phdrpudp->dport = lcrzo_htons(hdrludp.dport);
    phdrpudp->len = lcrzo_htons(hdrludp.len);
    phdrpudp->check = lcrzo_htons(hdrludp.check);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrludp_init_hdrpudp(lcrzo_hdrpudp hdrpudp,
                               lcrzo_hdrludp * phdrludp)
{
  if (phdrludp != NULL) {
    phdrludp->sport = lcrzo_ntohs(hdrpudp.sport);
    phdrludp->dport = lcrzo_ntohs(hdrpudp.dport);
    phdrludp->len = lcrzo_ntohs(hdrpudp.len);
    phdrludp->check = lcrzo_ntohs(hdrpudp.check);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrptcp_init_hdrltcp(lcrzo_hdrltcp hdrltcp,
                               lcrzo_hdrptcp * phdrptcp)
{

  if (hdrltcp.reserve > 0x3F)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.urg != 0 && hdrltcp.urg != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.ack != 0 && hdrltcp.ack != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.psh != 0 && hdrltcp.psh != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.rst != 0 && hdrltcp.rst != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.syn != 0 && hdrltcp.syn != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);
  if (hdrltcp.fin != 0 && hdrltcp.fin != 1)
    return (LCRZO_ERR_SPTOOBIGFORHDRL);

  if (phdrptcp != NULL) {
    phdrptcp->sport = lcrzo_htons(hdrltcp.sport);
    phdrptcp->dport = lcrzo_htons(hdrltcp.dport);
    phdrptcp->seqnum = lcrzo_htonl(hdrltcp.seqnum);
    phdrptcp->acknum = lcrzo_htonl(hdrltcp.acknum);
    phdrptcp->doff = hdrltcp.doff;
    phdrptcp->res1 = (lcrzo_uint8) (hdrltcp.reserve >> 2);
    phdrptcp->res2 = (lcrzo_uint8) (hdrltcp.reserve & 0x3);
    phdrptcp->urg = hdrltcp.urg;
    phdrptcp->ack = hdrltcp.ack;
    phdrptcp->psh = hdrltcp.psh;
    phdrptcp->rst = hdrltcp.rst;
    phdrptcp->syn = hdrltcp.syn;
    phdrptcp->fin = hdrltcp.fin;
    phdrptcp->window = lcrzo_htons(hdrltcp.window);
    phdrptcp->check = lcrzo_htons(hdrltcp.check);
    phdrptcp->urgptr = lcrzo_htons(hdrltcp.urgptr);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrltcp_init_hdrptcp(lcrzo_hdrptcp hdrptcp,
                               lcrzo_hdrltcp * phdrltcp)
{
  if (phdrltcp != NULL) {
    phdrltcp->sport = lcrzo_ntohs(hdrptcp.sport);
    phdrltcp->dport = lcrzo_ntohs(hdrptcp.dport);
    phdrltcp->seqnum = lcrzo_ntohl(hdrptcp.seqnum);
    phdrltcp->acknum = lcrzo_ntohl(hdrptcp.acknum);
    phdrltcp->doff = (lcrzo_uint8) hdrptcp.doff;
    phdrltcp->reserve = (lcrzo_uint8) ((hdrptcp.res1 << 2) | hdrptcp.res2);
    phdrltcp->urg = (lcrzo_int8) (hdrptcp.urg & 1);
    phdrltcp->ack = (lcrzo_int8) (hdrptcp.ack & 1);
    phdrltcp->psh = (lcrzo_int8) (hdrptcp.psh & 1);
    phdrltcp->rst = (lcrzo_int8) (hdrptcp.rst & 1);
    phdrltcp->syn = (lcrzo_int8) (hdrptcp.syn & 1);
    phdrltcp->fin = (lcrzo_int8) (hdrptcp.fin & 1);
    phdrltcp->window = lcrzo_ntohs(hdrptcp.window);
    phdrltcp->check = lcrzo_ntohs(hdrptcp.check);
    phdrltcp->urgptr = lcrzo_ntohs(hdrptcp.urgptr);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrpicmp_init_hdrlicmp(lcrzo_hdrlicmp hdrlicmp,
                                 lcrzo_hdrpicmp * phdrpicmp)
{
  if (phdrpicmp != NULL) {
    phdrpicmp->type = hdrlicmp.type;
    phdrpicmp->code = hdrlicmp.code;
    phdrpicmp->check = lcrzo_htons(hdrlicmp.check);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlicmp_init_hdrpicmp(lcrzo_hdrpicmp hdrpicmp,
                                 lcrzo_hdrlicmp * phdrlicmp)
{
  if (phdrlicmp != NULL) {
    phdrlicmp->type = hdrpicmp.type;
    phdrlicmp->code = hdrpicmp.code;
    phdrlicmp->check = lcrzo_ntohs(hdrpicmp.check);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrparp_init_hdrlarp(lcrzo_hdrlarp hdrlarp,
                               lcrzo_hdrparp * phdrparp)
{
  if (phdrparp != NULL) {
    phdrparp->hw_type = lcrzo_htons(hdrlarp.hw_type);
    phdrparp->prot_type = lcrzo_htons(hdrlarp.prot_type);
    phdrparp->hw_size = hdrlarp.hw_size;
    phdrparp->prot_size = hdrlarp.prot_size;
    phdrparp->op = lcrzo_htons(hdrlarp.op);
    memcpy(phdrparp->hw_src, hdrlarp.hw_src, LCRZO_ETHA_MAXBYTES);
    memcpy(phdrparp->prot_src, hdrlarp.prot_src, LCRZO_IPA_MAXBYTES);
    memcpy(phdrparp->hw_dst, hdrlarp.hw_dst, LCRZO_ETHA_MAXBYTES);
    memcpy(phdrparp->prot_dst, hdrlarp.prot_dst, LCRZO_IPA_MAXBYTES);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlarp_init_hdrparp(lcrzo_hdrparp hdrparp,
                               lcrzo_hdrlarp * phdrlarp)
{
  if (phdrlarp != NULL) {
    phdrlarp->hw_type = lcrzo_ntohs(hdrparp.hw_type);
    phdrlarp->prot_type = lcrzo_ntohs(hdrparp.prot_type);
    phdrlarp->hw_size = hdrparp.hw_size;
    phdrlarp->prot_size = hdrparp.prot_size;
    phdrlarp->op = lcrzo_ntohs(hdrparp.op);
    memcpy(phdrlarp->hw_src, hdrparp.hw_src, LCRZO_ETHA_MAXBYTES);
    memcpy(phdrlarp->prot_src, hdrparp.prot_src, LCRZO_IPA_MAXBYTES);
    memcpy(phdrlarp->hw_dst, hdrparp.hw_dst, LCRZO_ETHA_MAXBYTES);
    memcpy(phdrlarp->prot_dst, hdrparp.prot_dst, LCRZO_IPA_MAXBYTES);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_add_user_406(lcrzo_bool priorityoversystem,
                                    lcrzo_constdevice device,
                                    lcrzo_constetha etha,
                                    lcrzo_ipl ipl,
                                    lcrzo_ipl netmask,
                                    lcrzo_uint32 mtu,
                                    lcrzo_bool isup,
                                    lcrzo_bool isanalias)
{
  return (lcrzo_conf_devices_add_user(priorityoversystem,
                                      device,
                                      LCRZO_DEVICE_HWTYPE_ETHER,
                                      etha,
                                      ipl,
                                      netmask,
                                      mtu, isup, isanalias, LCRZO_FALSE, 0));
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_del_system_406(lcrzo_constdevice device,
                                      lcrzo_constetha etha,
                                      lcrzo_ipl ipl)
{
  return (lcrzo_conf_devices_del_system(device,
                                        LCRZO_DEVICE_HWTYPE_ETHER,
                                        etha, ipl));
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_del_user_406(lcrzo_constdevice device,
                                    lcrzo_constetha etha,
                                    lcrzo_ipl ipl)
{
  return (lcrzo_conf_devices_del_user(device,
                                      LCRZO_DEVICE_HWTYPE_ETHER, etha, ipl));
}

/*-------------------------------------------------------------*/
int lcrzo_conf_devices_value_pos_406(lcrzo_int32 pos,
                                     lcrzo_device device,
                                     lcrzo_etha etha,
                                     lcrzo_ipl * pipl,
                                     lcrzo_ipl * pnetmask,
                                     lcrzo_uint32 * pmtu,
                                     lcrzo_bool * pisup,
                                     lcrzo_bool * pisanalias)
{
  return (lcrzo_conf_devices_value_pos(pos,
                                       device,
                                       NULL,
                                       etha,
                                       pipl,
                                       pnetmask,
                                       pmtu, pisup, pisanalias, NULL, NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_data_free(lcrzo_data ptr)
{
  return (lcrzo_data_free2(&ptr));
}

/*-------------------------------------------------------------*/
int lcrzo_string_free(lcrzo_string ptr)
{
  return (lcrzo_string_free2(&ptr));
}


/*-------------------------------------------------------------*/
lcrzo_bool lcrzo_device_equal(lcrzo_constdevice device1,
                              lcrzo_constdevice device2)
{
  lcrzo_bool areequal;
  int ret;

  ret = lcrzo_device_equal2(device1, device2, &areequal);
  if (ret != LCRZO_ERR_OK)
    return (LCRZO_FALSE);
  return (areequal);
}

/*-------------------------------------------------------------*/
lcrzo_int8 lcrzo_device_cmp(lcrzo_constdevice device1,
                            lcrzo_constdevice device2)
{
  lcrzo_int8 cmp;
  int ret;

  ret = lcrzo_device_cmp2(device1, device2, &cmp);
  if (ret != LCRZO_ERR_OK)
    return (+2);
  return (cmp);
}

/*-------------------------------------------------------------*/
lcrzo_bool lcrzo_hs_equal(lcrzo_consths hs1,
                          lcrzo_consths hs2)
{
  lcrzo_bool areequal;
  int ret;

  ret = lcrzo_hs_equal2(hs1, hs2, &areequal);
  if (ret != LCRZO_ERR_OK)
    return (LCRZO_FALSE);
  return (areequal);
}

/*-------------------------------------------------------------*/
lcrzo_int8 lcrzo_hs_cmp(lcrzo_consths hs1,
                        lcrzo_consths hs2)
{
  lcrzo_int8 cmp;
  int ret;

  ret = lcrzo_hs_cmp2(hs1, hs2, &cmp);
  if (ret != LCRZO_ERR_OK)
    return (+2);
  return (cmp);
}

/*-------------------------------------------------------------*/
lcrzo_bool lcrzo_ips_equal(lcrzo_constips ips1,
                           lcrzo_constips ips2)
{
  lcrzo_bool areequal;
  int ret;

  ret = lcrzo_ips_equal2(ips1, ips2, &areequal);
  if (ret != LCRZO_ERR_OK)
    return (LCRZO_FALSE);
  return (areequal);
}

/*-------------------------------------------------------------*/
lcrzo_int8 lcrzo_ips_cmp(lcrzo_constips ips1,
                         lcrzo_constips ips2)
{
  lcrzo_int8 cmp;
  int ret;

  ret = lcrzo_ips_cmp2(ips1, ips2, &cmp);
  if (ret != LCRZO_ERR_OK)
    return (+2);
  return (cmp);
}

/*-------------------------------------------------------------*/
lcrzo_bool lcrzo_ipa_equal(lcrzo_constipa ipa1,
                           lcrzo_constipa ipa2)
{
  lcrzo_bool areequal;
  int ret;

  ret = lcrzo_ipa_equal2(ipa1, ipa2, &areequal);
  if (ret != LCRZO_ERR_OK)
    return (LCRZO_FALSE);
  return (areequal);
}

/*-------------------------------------------------------------*/
lcrzo_int8 lcrzo_ipa_cmp(lcrzo_constipa ipa1,
                         lcrzo_constipa ipa2)
{
  lcrzo_int8 cmp;
  int ret;

  ret = lcrzo_ipa_cmp2(ipa1, ipa2, &cmp);
  if (ret != LCRZO_ERR_OK)
    return (+2);
  return (cmp);
}

/*-------------------------------------------------------------*/
lcrzo_bool lcrzo_ipl_equal(lcrzo_ipl ipl1,
                           lcrzo_ipl ipl2)
{
  lcrzo_bool areequal;
  int ret;

  ret = lcrzo_ipl_equal2(ipl1, ipl2, &areequal);
  if (ret != LCRZO_ERR_OK)
    return (LCRZO_FALSE);
  return (areequal);
}

/*-------------------------------------------------------------*/
lcrzo_int8 lcrzo_ipl_cmp(lcrzo_ipl ipl1,
                         lcrzo_ipl ipl2)
{
  lcrzo_int8 cmp;
  int ret;

  ret = lcrzo_ipl_cmp2(ipl1, ipl2, &cmp);
  if (ret != LCRZO_ERR_OK)
    return (+2);
  return (cmp);
}

/*-------------------------------------------------------------*/
lcrzo_bool lcrzo_eths_equal(lcrzo_consteths eths1,
                            lcrzo_consteths eths2)
{
  lcrzo_bool areequal;
  int ret;

  ret = lcrzo_eths_equal2(eths1, eths2, &areequal);
  if (ret != LCRZO_ERR_OK)
    return (LCRZO_FALSE);
  return (areequal);
}

/*-------------------------------------------------------------*/
lcrzo_int8 lcrzo_eths_cmp(lcrzo_consteths eths1,
                          lcrzo_consteths eths2)
{
  lcrzo_int8 cmp;
  int ret;

  ret = lcrzo_eths_cmp2(eths1, eths2, &cmp);
  if (ret != LCRZO_ERR_OK)
    return (+2);
  return (cmp);
}

/*-------------------------------------------------------------*/
lcrzo_bool lcrzo_etha_equal(lcrzo_constetha etha1,
                            lcrzo_constetha etha2)
{
  lcrzo_bool areequal;
  int ret;

  ret = lcrzo_etha_equal2(etha1, etha2, &areequal);
  if (ret != LCRZO_ERR_OK)
    return (LCRZO_FALSE);
  return (areequal);
}

/*-------------------------------------------------------------*/
lcrzo_int8 lcrzo_etha_cmp(lcrzo_constetha etha1,
                          lcrzo_constetha etha2)
{
  lcrzo_int8 cmp;
  int ret;

  ret = lcrzo_etha_cmp2(etha1, etha2, &cmp);
  if (ret != LCRZO_ERR_OK)
    return (+2);
  return (cmp);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_duplicate_range(lcrzo_list * plist,
                                      lcrzo_int32 startpos,
                                      lcrzo_int32 endpos,
                                      lcrzo_int8(*pfunc_compare)
                                      (lcrzo_constptr peleminf,
                                       lcrzo_constptr pelemsup,
                                       lcrzo_ptr pinfos),
                                      lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_remove_duplicate_range(plist, startpos, endpos,
                                                  LCRZO_TRUE,
                                                  pfunc_compare,
                                                  NULL, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_duplicate_all(lcrzo_list * plist,
                                    lcrzo_int8(*pfunc_compare) (lcrzo_constptr
                                                                peleminf,
                                                                lcrzo_constptr
                                                                pelemsup,
                                                                lcrzo_ptr
                                                                pinfos),
                                    lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_remove_duplicate_range(plist, +1, -1,
                                                  LCRZO_TRUE,
                                                  pfunc_compare,
                                                  NULL, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_criteria_range(lcrzo_list * plist,
                                     lcrzo_int32 startpos,
                                     lcrzo_int32 endpos,
                                     lcrzo_bool(*pfunc_criteria)
                                     (lcrzo_constptr pelem,
                                      lcrzo_ptr pinfos),
                                     lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_remove_criteria_range(plist, startpos, endpos,
                                                 LCRZO_TRUE,
                                                 pfunc_criteria,
                                                 NULL, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_remove_criteria_all(lcrzo_list * plist,
                                   lcrzo_bool(*pfunc_criteria) (lcrzo_constptr
                                                                pelem,
                                                                lcrzo_ptr
                                                                pinfos),
                                   lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_remove_criteria_range(plist, +1, -1,
                                                 LCRZO_TRUE,
                                                 pfunc_criteria,
                                                 NULL, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_sort_range(lcrzo_list * plist,
                          lcrzo_int32 startpos,
                          lcrzo_int32 endpos,
                          lcrzo_int8(*pfunc_compare) (lcrzo_constptr peleminf,
                                                      lcrzo_constptr pelemsup,
                                                      lcrzo_ptr pinfos),
                          lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_sort_range(plist, startpos, endpos,
                                      LCRZO_TRUE,
                                      pfunc_compare, NULL, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_sort_all(lcrzo_list * plist,
                        lcrzo_int8(*pfunc_compare) (lcrzo_constptr peleminf,
                                                    lcrzo_constptr pelemsup,
                                                    lcrzo_ptr pinfos),
                        lcrzo_ptr pinfos)
{
  lcrzo_er(lcrzo_priv_list_sort_range(plist, +1, -1,
                                      LCRZO_TRUE,
                                      pfunc_compare, NULL, pinfos));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_search_range(lcrzo_list * plist,
                            lcrzo_int32 startpos,
                            lcrzo_int32 endpos,
                            lcrzo_bool(*pfunc_search) (lcrzo_constptr pelem,
                                                       lcrzo_ptr pinfos),
                            lcrzo_ptr pinfos,
                            lcrzo_int32 * pposelem,
                            lcrzo_ptr pelem)
{
  lcrzo_er(lcrzo_priv_list_search_range(plist, startpos, endpos,
                                        LCRZO_TRUE,
                                        pfunc_search,
                                        NULL, pinfos, pposelem, pelem));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_search_all(lcrzo_list * plist,
                          lcrzo_bool(*pfunc_search) (lcrzo_constptr pelem,
                                                     lcrzo_ptr pinfos),
                          lcrzo_ptr pinfos,
                          lcrzo_int32 * pposelem,
                          lcrzo_ptr pelem)
{
  lcrzo_er(lcrzo_priv_list_search_range(plist, +1, -1,
                                        LCRZO_TRUE,
                                        pfunc_search,
                                        NULL, pinfos, pposelem, pelem));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_list_searchelem_range(lcrzo_list * plist,
                                lcrzo_int32 startpos,
                                lcrzo_int32 endpos,
                                lcrzo_constptr pitemtofind,
                                lcrzo_int32 * ppositemfound,
                                lcrzo_ptr pitemfound)
{
  int ret;

  ret = lcrzo_list_searchitem_range(plist, startpos, endpos,
                                    pitemtofind, ppositemfound);
  if (ret == LCRZO_ERR_OK) {
    if (pitemfound != NULL)
      memcpy(pitemfound, pitemtofind, plist->itemsize);
  }
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_list_searchelem_all(lcrzo_list * plist,
                              lcrzo_constptr pitemtofind,
                              lcrzo_int32 * ppositemfound,
                              lcrzo_ptr pitemfound)
{
  return (lcrzo_list_searchitem_all(plist, pitemtofind, ppositemfound));
  pitemfound = pitemfound;
}


/*-------------------------------------------------------------*/
int lcrzo_data_init_int(lcrzo_int32 integer,
                        lcrzo_conststring format,
                        lcrzo_int32 dataoutmaxsize,
                        lcrzo_data dataout,
                        lcrzo_int32 * pdataoutsize)
{
  return (lcrzo_data_append_int(integer, format, 0, dataoutmaxsize,
                                dataout, pdataoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_data_initm_int(lcrzo_int32 integer,
                         lcrzo_conststring format,
                         lcrzo_data * pdataout,
                         lcrzo_int32 * pdataoutsize)
{
  lcrzo_data ptr;

  ptr = NULL;
  lcrzo_er(lcrzo_data_appendm_int(integer, format, 0, &ptr, pdataoutsize));
  if (pdataout != NULL)
    *pdataout = ptr;
  else {
    lcrzo_er(lcrzo_data_free2(&ptr));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_data_append_int(lcrzo_int32 integer,
                          lcrzo_conststring format,
                          lcrzo_int32 dataoutcurrentsize,
                          lcrzo_int32 dataoutmaxsize,
                          lcrzo_data dataout,
                          lcrzo_int32 * pdataoutsize)
{
  char tmp[101];              /*le format maxi est %+99X -> 100 octets maxi */
  lcrzo_string newfmt;
  lcrzo_bool addal;

  /*parameters verification */
  if (format == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (dataoutcurrentsize < 0)
    return (LCRZO_ERR_PATOOLOW);
  if (dataoutmaxsize < 0)
    return (LCRZO_ERR_PATOOLOW);
  if (dataoutcurrentsize > dataoutmaxsize)
    return (LCRZO_ERR_PADATACURSUPMAX);

  /*copy the integer */
#if LCRZODEF_INT32_ARELONGS == 1
  addal = LCRZO_TRUE;
#else
  addal = LCRZO_FALSE;
#endif
  lcrzo_er(lcrzo_priv_int_fmt_check(format, LCRZO_FALSE, LCRZO_FALSE,
                                    addal, &newfmt));
  sprintf(tmp, newfmt, integer);
  lcrzo_er(lcrzo_string_free2(&newfmt));
  return (lcrzo_data_append_data((lcrzo_constdata) tmp, strlen(tmp),
                                 dataoutcurrentsize,
                                 dataoutmaxsize, dataout, pdataoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_data_appendm_int(lcrzo_int32 integer,
                           lcrzo_conststring format,
                           lcrzo_int32 dataoutcurrentsize,
                           lcrzo_data * pdataout,
                           lcrzo_int32 * pdataoutsize)
{
  char tmp[101];              /*le format maxi est %+99X -> 100 octets maxi */
  lcrzo_string newfmt;
  lcrzo_bool addal;

  /*parameters verification */
  if (format == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (dataoutcurrentsize < 0)
    return (LCRZO_ERR_PATOOLOW);

  /*copy the integer */
#if LCRZODEF_INT32_ARELONGS == 1
  addal = LCRZO_TRUE;
#else
  addal = LCRZO_FALSE;
#endif
  lcrzo_er(lcrzo_priv_int_fmt_check(format, LCRZO_FALSE, LCRZO_FALSE,
                                    addal, &newfmt));
  sprintf(tmp, newfmt, integer);
  lcrzo_er(lcrzo_string_free2(&newfmt));
  return (lcrzo_data_appendm_data((lcrzo_constdata) tmp, strlen(tmp),
                                  dataoutcurrentsize,
                                  pdataout, pdataoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_string_init_int(lcrzo_int32 integer,
                          lcrzo_conststring format,
                          lcrzo_int32 stringoutmaxsize,
                          lcrzo_string stringout)
{
  lcrzo_int32 dataoutsize;
  int ret;

  ret = lcrzo_data_append_int(integer, format, 0,
                              stringoutmaxsize, (lcrzo_data) stringout,
                              &dataoutsize);
  if (ret != LCRZO_ERR_OK && ret != LCRZO_ERR_OKDATATRUNCATED)
    return (ret);

  if (stringout != NULL) {
    stringout[dataoutsize] = '\0';
  }
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_string_initm_int(lcrzo_int32 integer,
                           lcrzo_conststring format,
                           lcrzo_string * pstringout)
{
  lcrzo_data ptr;
  lcrzo_int32 dataoutsize;

  ptr = NULL;
  lcrzo_er(lcrzo_data_appendm_int(integer, format, 0, &ptr, &dataoutsize));

  if (pstringout != NULL) {
    ptr[dataoutsize] = '\0';
    *pstringout = (lcrzo_string) ptr;
  } else {
    lcrzo_er(lcrzo_data_free2(&ptr));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_string_append_int(lcrzo_int32 integer,
                            lcrzo_conststring format,
                            lcrzo_int32 stringoutmaxsize,
                            lcrzo_string stringout)
{
  lcrzo_int32 dataoutsize;
  int ret;

  if (stringout == NULL)
    return (LCRZO_ERR_PANULLPTR);
  ret = lcrzo_data_append_int(integer, format, strlen(stringout),
                              stringoutmaxsize, (lcrzo_data) stringout,
                              &dataoutsize);
  if (ret != LCRZO_ERR_OK && ret != LCRZO_ERR_OKDATATRUNCATED)
    return (ret);
  stringout[dataoutsize] = '\0';

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_string_appendm_int(lcrzo_int32 integer,
                             lcrzo_conststring format,
                             lcrzo_string * pstringout)
{
  lcrzo_string ptr;
  lcrzo_int32 dataoutsize;

  if (pstringout == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo_er(lcrzo_data_appendm_int(integer, format, strlen(*pstringout),
                                  (lcrzo_data *) pstringout, &dataoutsize));
  ptr = *pstringout;
  ptr[dataoutsize] = '\0';
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_filename_clean(lcrzo_conststring filename,
                         lcrzo_string * pcleanedfilename)
{
  lcrzo_string tmp, root;
  lcrzo_uint8 c, c_1;
  lcrzo_bool lastcharwasslash;
  int i, j, size, lastdirsize;
  char *pc;

  /* return empty filenames */
  if (strlen(filename) == 0) {
    lcrzo_er(lcrzo_string_initm_text("", pcleanedfilename));
    return (LCRZO_ERR_OK);
  }

  /* eventually add "./", or replace "x:\"by"/", or replace "\\comp\"by"/" */
  if (filename[0] == '/') {
    lcrzo_er(lcrzo_string_initm_text("/", &root));
    lcrzo_er(lcrzo_string_initm_text(filename, &tmp));
  } else {
    /* search "x:\" */
    if (strlen(filename) >= 3 && isalpha((int)filename[0]) &&
        filename[1] == ':' && (filename[2] == '\\' || filename[2] == '/')) {
      lcrzo_er(lcrzo_string_initm_data((lcrzo_constdata) filename, 3, &root));
      root[3] = '\0';
      if (filename[2] == '\\')
        root[2] = '/';
      lcrzo_er(lcrzo_string_initm_text(filename + 2, &tmp));    
      /* \ will be replaced by / */
    }
    /* search "x:" */
    else if (strlen(filename) >= 2 && isalpha((int)filename[0]) &&
             filename[1] == ':') {
      lcrzo_er(lcrzo_string_initm_data((lcrzo_constdata) filename, 2, &root));
      root[2] = '\0';
      lcrzo_er(lcrzo_string_initm_text("./", &tmp));
      lcrzo_er(lcrzo_string_appendm_text(filename + 2, &tmp));
    }
    /* search "\\comp\" */
    else if (strlen(filename) >= 2 && filename[0] == '\\' &&
             filename[1] == '\\') {
      pc = strchr(filename + 2, '\\');
      if (pc == NULL)
        pc = strchr(filename + 2, '/');
      if (pc == NULL) {
        /* file is only "\\comp", so there is nothing to clean */
        lcrzo_er(lcrzo_string_initm_text(filename, pcleanedfilename));
        return (LCRZO_ERR_OK);
      }
      size = (pc - filename);
      lcrzo_er(lcrzo_string_initm_data
               ((lcrzo_constdata) filename, size, &root));
      root[size] = '\0';
      lcrzo_er(lcrzo_string_appendm_text("\\", &root));
      lcrzo_er(lcrzo_string_initm_text(pc, &tmp));
      /* \ will be replaced by / */
    }
    /* search "\rep" */
    else if (strlen(filename) >= 1 && filename[0] == '\\') {
      lcrzo_er(lcrzo_string_initm_text("/", &root));
      lcrzo_er(lcrzo_string_initm_text(filename, &tmp));
      /* \ will be replaced by / */
    } else {
      /* add "./" */
      lcrzo_er(lcrzo_string_initm_text("", &root));
      lcrzo_er(lcrzo_string_initm_text("./", &tmp));
      lcrzo_er(lcrzo_string_appendm_text(filename, &tmp));
    }
  }
  /*printf("==%s ==%s\n", root, tmp); */

  /* convert "\" to "/" */
  size = strlen(tmp);
  for (i = 0; i < size; i++) {
    if (tmp[i] == '\\')
      tmp[i] = '/';
  }

  /* convert "/.$" to "/" */
  size = strlen(tmp);
  if (size >= 2) {
    if (tmp[size - 2] == '/' && tmp[size - 1] == '.') {
      tmp[size - 1] = '\0';
    }
  }

  /* convert "/..$" to "/../" */
  size = strlen(tmp);
  if (size >= 3) {
    if (tmp[size - 3] == '/' && tmp[size - 2] == '.' && tmp[size - 1] == '.') {
      lcrzo_er(lcrzo_string_appendm_char('/', 1, &tmp));
    }
  }

  /* convert "/./" to "//" */
  size = strlen(tmp);
  i = 0;
  j = 0;
  for (; i < size - 2; i++) {
    c = tmp[i];
    tmp[j++] = c;
    if (c == '/' && tmp[i + 1] == '.' && tmp[i + 2] == '/') {
      i++;
    }
  }
  if (i < size - 1)
    tmp[j++] = tmp[i++];
  if (i < size)
    tmp[j++] = tmp[i];
  tmp[j] = '\0';

  /* convert "//" to "/" */
  size = strlen(tmp);
  j = 0;
  c_1 = '_';
  for (i = 0; i < size; i++) {
    c = tmp[i];
    if (!(c_1 == '/' && c == '/')) {
      tmp[j++] = c;
    }
    c_1 = c;
  }
  tmp[j] = '\0';

  /* convert /aa/bb/../ to /aa/ */
  size = strlen(tmp);
  i = 0;
  j = 0;
  lastcharwasslash = LCRZO_TRUE;
  for (; i < size - 2; i++) {
    c = tmp[i];
    if (lastcharwasslash && c == '.'
        && tmp[i + 1] == '.' && tmp[i + 2] == '/') {
      /* go back to the last '/' */
      if (j > 0) {
        c_1 = tmp[j - 1];
        tmp[j - 1] = '\0';      /* just for strrchr */
        pc = strrchr(tmp, '/');
        tmp[j - 1] = c_1;
      } else
        pc = NULL;
      if (pc == NULL) {         /* no '/' found */
        if (tmp[0] == '/') {    /* can not have /.. because / is root */
          lcrzo_er(lcrzo_string_free2(&tmp));
          lcrzo_er(lcrzo_string_free2(&root));
          return (LCRZO_ERR_SPPATHROOTDOTDOT);
        } else {                /* start with ./../, so add ../ */
          tmp[j++] = '.';
          tmp[j++] = '.';
          tmp[j++] = '/';
        }
      } else {                  /* './' found */
        c_1 = tmp[j - 1];
        tmp[j - 1] = '\0';      /* just for strlen */
        lastdirsize = strlen(tmp) - (pc - tmp);
        tmp[j - 1] = c_1;
        if (lastdirsize == 3 && pc[1] == '.' && pc[2] == '.') {
          /* tmp actually end with /../ (between the found character (pc)
             and the strlen of tmp */
          tmp[j++] = '.';
          tmp[j++] = '.';
          tmp[j++] = '/';
        } else {                /* go back to the last '/' */
          j = pc - tmp + 1;
        }
      }
      i += 2;                   /* we skip two characters */
      lastcharwasslash = LCRZO_TRUE;
    } else {
      tmp[j++] = c;
      lastcharwasslash = (lcrzo_bool) ((c == '/') ? LCRZO_TRUE : LCRZO_FALSE);
    }
  }
  if (i < size - 1)
    tmp[j++] = tmp[i++];
  if (i < size)
    tmp[j++] = tmp[i];
  tmp[j] = '\0';

  /* set output value */
  /*printf("==%s ==%s\n", root, tmp); */
  if (pcleanedfilename != NULL) {
    if (tmp[0] == '/') {
      lcrzo_er(lcrzo_string_initm_text(root, pcleanedfilename));
      lcrzo_er(lcrzo_string_appendm_text(tmp + 1, pcleanedfilename));
    } else {
      lcrzo_er(lcrzo_string_initm_text(root, pcleanedfilename));
      if (strlen(root) || !strncmp(tmp, "./../", 5)) {
        /* because tmp always start with "./" */
        lcrzo_er(lcrzo_string_appendm_text(tmp + 2, pcleanedfilename));
      } else {
        lcrzo_er(lcrzo_string_appendm_text(tmp, pcleanedfilename));
      }
    }
  }
  lcrzo_er(lcrzo_string_free2(&root));
  lcrzo_er(lcrzo_string_free2(&tmp));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_filename_parentdir(lcrzo_conststring filename,
                             lcrzo_string * pparentdir)
{
  lcrzo_string tmp;
  int ret;

  /* special case for empty filenames */
  if (strlen(filename) == 0) {
    lcrzo_er(lcrzo_string_initm_text("./", pparentdir));
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzo_string_initm_text(filename, &tmp));
  lcrzo_er(lcrzo_string_appendm_text("/../", &tmp));
  ret = lcrzo_filename_clean(tmp, pparentdir);
  lcrzo_er(lcrzo_string_free2(&tmp));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_filename_lastfile(lcrzo_conststring filename,
                            lcrzo_string * plastfile)
{
  lcrzo_string tmp;
  char *pc;
  int size, ret;

  lcrzo_er(lcrzo_filename_clean(filename, &tmp));
  /* special case of root : try to go up */
  ret = lcrzo_filename_parentdir(tmp, NULL);
  if (ret == LCRZO_ERR_SPPATHROOTDOTDOT) {
    lcrzo_er(lcrzo_string_initm_text(tmp, plastfile));
    lcrzo_er(lcrzo_string_free2(&tmp));
    return (LCRZO_ERR_OK);
  } else if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_string_free2(&tmp));
    return (ret);
  }

  /* suppress the ending '/' */
  size = strlen(tmp);
  if (size > 0) {
    if (tmp[size - 1] == '/') {
      tmp[size - 1] = '\0';
    }
  }

  /* find the last '/' */
  pc = strrchr(tmp, '/');
  if (pc == NULL)
    pc = strrchr(tmp, '\\');
  if (pc == NULL) {
    lcrzo_er(lcrzo_string_initm_text(tmp, plastfile));
  } else {
    lcrzo_er(lcrzo_string_initm_text(pc + 1, plastfile));
  }
  lcrzo_er(lcrzo_string_free2(&tmp));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_filename_type(lcrzo_conststring filename,
                        lcrzo_int16 * ptype)
{
  struct stat st;

  /*get the stat */
  lcrzo_er(lcrzo_priv_path_stat_get(filename, &st));
  lcrzo_er(lcrzo_priv_stat_type(st, ptype));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_file_type(lcrzo_file * pfile,
                    lcrzo_int16 * ptype)
{
  struct stat st;

  /*get the stat */
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo_er(lcrzo_priv_fd_stat_get(pfile->fd, &st));
  lcrzo_er(lcrzo_priv_stat_type(st, ptype));

  return (LCRZO_ERR_OK);
}


#else
 /* fake function for Microsoft Visual compiler */
int lcrzo_priv_fakefunction(void);
int lcrzo_priv_fakefunction(void)
{
  return (LCRZO_ERR_OK);
}
#endif
