/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_incl.h"

/*-------------------------------------------------------------*/
/* pseudo header utilise pour le calcul des checksums */
typedef struct {
  lcrzo_uint32 saddr;
  lcrzo_uint32 daddr;
  lcrzo_uint8 zeros;
  lcrzo_uint8 protocol;
  lcrzo_uint16 len;
} lcrzo_hdrppseudo;

#define LCRZO_HDRPPSEUDO2_MAXBYTES 12
typedef lcrzo_uint8 lcrzo_hdrppseudo2[LCRZO_HDRPPSEUDO2_MAXBYTES];
int lcrzo_hdrppseudo2_init(lcrzo_uint32 saddr,
                           lcrzo_uint32 daddr,
                           lcrzo_uint8 protocol,
                           lcrzo_uint16 len,
                           lcrzo_hdrppseudo2 hdrppseudo2);
int lcrzo_hdrppseudo2_init(lcrzo_uint32 saddr,
                           lcrzo_uint32 daddr,
                           lcrzo_uint8 protocol,
                           lcrzo_uint16 len,
                           lcrzo_hdrppseudo2 hdrppseudo2)
{
  if (hdrppseudo2 != NULL) {
    hdrppseudo2[0] = (lcrzo_uint8) (saddr >> 24);
    hdrppseudo2[1] = (lcrzo_uint8) (0xFF & (saddr >> 16));
    hdrppseudo2[2] = (lcrzo_uint8) (0xFF & (saddr >> 8));
    hdrppseudo2[3] = (lcrzo_uint8) (0xFF & saddr);
    hdrppseudo2[4] = (lcrzo_uint8) (daddr >> 24);
    hdrppseudo2[5] = (lcrzo_uint8) (0xFF & (daddr >> 16));
    hdrppseudo2[6] = (lcrzo_uint8) (0xFF & (daddr >> 8));
    hdrppseudo2[7] = (lcrzo_uint8) (0xFF & daddr);
    hdrppseudo2[8] = 0;
    hdrppseudo2[9] = protocol;
    hdrppseudo2[10] = (lcrzo_uint8) (0xFF & (len >> 8));
    hdrppseudo2[11] = (lcrzo_uint8) (0xFF & len);
  }

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_packet_initm_ipoptdata(lcrzo_hdrlip hdrlip,
                                 lcrzo_constipopt ipopt,
                                 lcrzo_uint8 nboctipopt,
                                 lcrzo_constdata data,
                                 lcrzo_uint16 nboctdata,
                                 lcrzo_data * pdataout,
                                 lcrzo_int32 * pdataoutsize)
{
  lcrzo_hdrpip2 hdrpip2;
  lcrzo_bool hdr_compfields, hdr_compfield_iptotlen, hdr_compfield_ipihl,
    hdr_compfield_ipcheck;
  lcrzo_uint16 check;

  /*parameters verification */
  if (ipopt == NULL && nboctipopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (data == NULL && nboctdata)
    return (LCRZO_ERR_PANULLPTRSIZE);
  lcrzo_ipopt_verifbof(nboctipopt);
  /*verifie que la taille des options soit un multiple de 4 */
  if (nboctipopt % 4)
    return (LCRZO_ERR_SPOPTNOTX4);
  if (sizeof(lcrzo_hdrpip2) + nboctipopt + nboctdata > 0xFFFF)
    return (LCRZO_ERR_PATOOBIG);

  /*met eventuellement a jour totlen */
  lcrzo_er(lcrzo_global_get_hdr_compfields(&hdr_compfields));
  lcrzo_er(lcrzo_global_get_hdr_cf_iptotlen(&hdr_compfield_iptotlen));
  if (hdrlip.totlen == LCRZO_HDRLIP_TOTLEN_DEFVAL &&
      hdr_compfields && hdr_compfield_iptotlen) {
    hdrlip.totlen =
      (lcrzo_uint16) (sizeof(lcrzo_hdrpip2) + nboctipopt + nboctdata);
  }

  /*met eventuellement a jour ihl */
  lcrzo_er(lcrzo_global_get_hdr_cf_ipihl(&hdr_compfield_ipihl));
  if (hdrlip.ihl == LCRZO_HDRLIP_IHL_DEFVAL &&
      hdr_compfields && hdr_compfield_ipihl) {
    hdrlip.ihl = (lcrzo_uint8) ((sizeof(lcrzo_hdrpip2) + nboctipopt) / 4);
  }

  /*conversion de l'entete logique en entete physique */
  lcrzo_er(lcrzo_hdrpip2_init_hdrlip(hdrlip, hdrpip2));

  /*met eventuellement a jour check */
  lcrzo_er(lcrzo_global_get_hdr_cf_ipcheck(&hdr_compfield_ipcheck));
  lcrzo_er(lcrzo_priv_hdrpip2_get_check(hdrpip2, &check));
  if (check == LCRZO_HDRLIP_CHECK_DEFVAL &&
      hdr_compfields && hdr_compfield_ipcheck) {
    lcrzo_uint8 tmpbuf[sizeof(lcrzo_hdrpip2) + sizeof(lcrzo_ipopt)];
    lcrzo_uint16 ipcheck;

    /*pour le calcul, le checksum doit etre nul */
    lcrzo_er(lcrzo_priv_hdrpip2_set_check(hdrpip2, 0));
    memcpy(tmpbuf, hdrpip2, sizeof(lcrzo_hdrpip2));
    memcpy(tmpbuf + sizeof(lcrzo_hdrpip2), ipopt, nboctipopt);
    lcrzo_er(lcrzo_data_checksum(tmpbuf, sizeof(lcrzo_hdrpip2) + nboctipopt,
                                 &ipcheck));
    lcrzo_er(lcrzo_priv_hdrpip2_set_check(hdrpip2, ipcheck));
  }

  /*compose le packet */
  lcrzo_er(lcrzo_data_initm_data((lcrzo_data) hdrpip2, sizeof(lcrzo_hdrpip2),
                                 pdataout, pdataoutsize));
  lcrzo_er(lcrzo_data_appendm_data(ipopt, nboctipopt, sizeof(lcrzo_hdrpip2),
                                   pdataout, pdataoutsize));
  lcrzo_er(lcrzo_data_appendm_data(data, nboctdata,
                                   sizeof(lcrzo_hdrpip2) + nboctipopt,
                                   pdataout, pdataoutsize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
lcrzo_ipl lcrzo_priv_addsthdrpseudo(lcrzo_ipl hdrip_daddr,
                                    lcrzo_constipopt ipopt,
                                    lcrzo_uint8 nboctipopt);
lcrzo_ipl lcrzo_priv_addsthdrpseudo(lcrzo_ipl hdrip_daddr,
                                    lcrzo_constipopt ipopt,
                                    lcrzo_uint8 nboctipopt)
{
  int retour;
  lcrzo_uint8 nbiplaffectes;
  lcrzo_uint8 pointeur;
  lcrzo_ipl ipltab[9];

  /* il n'y a pas d'options IP */
  if (!nboctipopt) {
    return (hdrip_daddr);
  }

  /*cherche loose source routing */
  retour = lcrzo_ipopt_decode_lsrr(ipopt, nboctipopt,
                                   &nbiplaffectes, &pointeur,
                                   &ipltab[0], &ipltab[1], &ipltab[2],
                                   &ipltab[3], &ipltab[4], &ipltab[5],
                                   &ipltab[6], &ipltab[7], &ipltab[8]);
  if (retour != LCRZO_ERR_OK) {
    /*cherche strict source routing */
    retour = lcrzo_ipopt_decode_ssrr(ipopt, nboctipopt,
                                     &nbiplaffectes, &pointeur,
                                     &ipltab[0], &ipltab[1], &ipltab[2],
                                     &ipltab[3], &ipltab[4], &ipltab[5],
                                     &ipltab[6], &ipltab[7], &ipltab[8]);
    if (retour != LCRZO_ERR_OK) {
      /*on n'a rien trouve, donc on retourne l'adresse de l'entete IP */
      return (hdrip_daddr);
    }
  }

  /*on a trouve une option lsrr ou ssrr que l'on analyse */
  if (nbiplaffectes >= 1 && nbiplaffectes <= 9) {
    if (pointeur / 4 > nbiplaffectes) {
      /*est arrive au destinataire */
      return (hdrip_daddr);
    } else {
      /*n'a pas encore atteint le destinataire */
      return (ipltab[nbiplaffectes - 1]);
    }
  }

  /*on retourne l'adresse de l'entete IP */
  return (hdrip_daddr);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_initm_ipoptudpdata(lcrzo_hdrlip hdrlip,
                                    lcrzo_constipopt ipopt,
                                    lcrzo_uint8 nboctipopt,
                                    lcrzo_hdrludp hdrludp,
                                    lcrzo_constdata data,
                                    lcrzo_uint16 nboctdata,
                                    lcrzo_data * pdataout,
                                    lcrzo_int32 * pdataoutsize)
{
  lcrzo_data tmpbuf;
  lcrzo_uint16 tmpbuflen, check;
  lcrzo_hdrpudp2 hdrpudp2;
  lcrzo_bool hdr_compfields, hdr_compfield_ipprotocol, hdr_compfield_udplen,
    hdr_compfield_udpcheck;
  int ret;

  /*parameters verification */
  if (ipopt == NULL && nboctipopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (data == NULL && nboctdata)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (sizeof(lcrzo_hdrpip2) + nboctipopt +
      sizeof(lcrzo_hdrpudp2) + nboctdata > 0xFFFF)
    return (LCRZO_ERR_PATOOBIG);

  /*met eventuellement a jour protocole udp */
  lcrzo_er(lcrzo_global_get_hdr_compfields(&hdr_compfields));
  lcrzo_er(lcrzo_global_get_hdr_cf_ipprotocol(&hdr_compfield_ipprotocol));
  if (hdrlip.protocol == LCRZO_HDRLIP_PROTOCOL_DEFVAL &&
      hdr_compfields && hdr_compfield_ipprotocol) {
    hdrlip.protocol = LCRZO_HDRLIP_PROTOCOL_UDP;
  }

  /*met eventuellement a jour len */
  lcrzo_er(lcrzo_global_get_hdr_cf_udplen(&hdr_compfield_udplen));
  if (hdrludp.len == LCRZO_HDRLUDP_LEN_DEFVAL &&
      hdr_compfields && hdr_compfield_udplen) {
    hdrludp.len = (lcrzo_uint16) (sizeof(lcrzo_hdrpudp2) + nboctdata);
  }

  /*conversion de l'entete logique en entete physique */
  lcrzo_er(lcrzo_hdrpudp2_init_hdrludp(hdrludp, hdrpudp2));

  /*calcule le checksum que si il n'est pas deja affecte */
  lcrzo_er(lcrzo_global_get_hdr_cf_udpcheck(&hdr_compfield_udpcheck));
  lcrzo_er(lcrzo_priv_hdrpudp2_get_check(hdrpudp2, &check));
  if (check == LCRZO_HDRLUDP_CHECK_DEFVAL &&
      hdr_compfields && hdr_compfield_udpcheck) {
    lcrzo_uint16 udpcheck;

    tmpbuflen = (lcrzo_uint16) (sizeof(lcrzo_hdrppseudo2) +
                                sizeof(lcrzo_hdrpudp2) + nboctdata);
    lcrzo_er(lcrzo_data_alloc(tmpbuflen, &tmpbuf));
    lcrzo_er(lcrzo_hdrppseudo2_init(hdrlip.saddr,
                                    lcrzo_priv_addsthdrpseudo(hdrlip.daddr,
                                                              ipopt,
                                                              nboctipopt),
                                    hdrlip.protocol,
                                    (lcrzo_uint16) (sizeof(lcrzo_hdrpudp2) +
                                                    nboctdata), tmpbuf));
    memcpy(tmpbuf + sizeof(lcrzo_hdrppseudo2), hdrpudp2,
           sizeof(lcrzo_hdrpudp2));
    memcpy(tmpbuf + sizeof(lcrzo_hdrppseudo2) + sizeof(lcrzo_hdrpudp2), data,
           nboctdata);
    ret = lcrzo_data_checksum(tmpbuf, tmpbuflen, &udpcheck);
    lcrzo_er(lcrzo_data_free2(&tmpbuf));
    lcrzo_er(ret);
    lcrzo_er(lcrzo_priv_hdrpudp2_set_check(hdrpudp2, udpcheck));
  }

  /*compose le packet udp */
  tmpbuflen = (lcrzo_uint16) (sizeof(lcrzo_hdrpudp2) + nboctdata);
  lcrzo_er(lcrzo_data_alloc(tmpbuflen, &tmpbuf));
  memcpy(tmpbuf, hdrpudp2, sizeof(lcrzo_hdrpudp2));
  memcpy(tmpbuf + sizeof(lcrzo_hdrpudp2), data, nboctdata);

  ret = lcrzo_packet_initm_ipoptdata(hdrlip, ipopt, nboctipopt,
                                     tmpbuf, tmpbuflen, pdataout,
                                     pdataoutsize);
  lcrzo_er(lcrzo_data_free2(&tmpbuf));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_initm_ipopttcpoptdata(lcrzo_hdrlip hdrlip,
                                       lcrzo_constipopt ipopt,
                                       lcrzo_uint8 nboctipopt,
                                       lcrzo_hdrltcp hdrltcp,
                                       lcrzo_consttcpopt tcpopt,
                                       lcrzo_uint8 nbocttcpopt,
                                       lcrzo_constdata data,
                                       lcrzo_uint16 nboctdata,
                                       lcrzo_data * pdataout,
                                       lcrzo_int32 * pdataoutsize)
{
  lcrzo_data tmpbuf;
  lcrzo_uint16 tmpbuflen, check;
  lcrzo_hdrptcp2 hdrptcp2;
  lcrzo_bool hdr_compfields, hdr_compfield_ipprotocol, hdr_compfield_tcpdoff,
    hdr_compfield_tcpcheck;
  int ret;

  /*parameters verification */
  if (ipopt == NULL && nboctipopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (tcpopt == NULL && nbocttcpopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (data == NULL && nboctdata)
    return (LCRZO_ERR_PANULLPTRSIZE);
  lcrzo_tcpopt_verifbof(nbocttcpopt);
  if (sizeof(lcrzo_hdrpip2) + nboctipopt +
      sizeof(lcrzo_hdrptcp2) + nbocttcpopt + nboctdata > 0xFFFF)
    return (LCRZO_ERR_PATOOBIG);

  /*verifie que la taille des options soit un multiple de 4 */
  if (nbocttcpopt % 4)
    return (LCRZO_ERR_SPOPTNOTX4);

  /*met eventuellement a jour protocole tcp */
  lcrzo_er(lcrzo_global_get_hdr_compfields(&hdr_compfields));
  lcrzo_er(lcrzo_global_get_hdr_cf_ipprotocol(&hdr_compfield_ipprotocol));
  if (hdrlip.protocol == LCRZO_HDRLIP_PROTOCOL_DEFVAL &&
      hdr_compfields && hdr_compfield_ipprotocol) {
    hdrlip.protocol = LCRZO_HDRLIP_PROTOCOL_TCP;
  }

  /*met eventuellement a jour dataoff */
  lcrzo_er(lcrzo_global_get_hdr_cf_tcpdoff(&hdr_compfield_tcpdoff));
  if (hdrltcp.doff == LCRZO_HDRLTCP_DOFF_DEFVAL &&
      hdr_compfields && hdr_compfield_tcpdoff) {
    hdrltcp.doff = (lcrzo_uint8) ((sizeof(lcrzo_hdrptcp2) + nbocttcpopt) / 4);
  }

  /*conversion de l'entete logique en entete physique */
  lcrzo_er(lcrzo_hdrptcp2_init_hdrltcp(hdrltcp, hdrptcp2));

  /*calcule le checksum que si il n'est pas deja affecte */
  lcrzo_er(lcrzo_global_get_hdr_cf_tcpcheck(&hdr_compfield_tcpcheck));
  lcrzo_er(lcrzo_priv_hdrptcp2_get_check(hdrptcp2, &check));
  if (check == LCRZO_HDRLTCP_CHECK_DEFVAL &&
      hdr_compfields && hdr_compfield_tcpcheck) {
    lcrzo_uint16 tcpcheck;

    tmpbuflen = (lcrzo_uint16) (sizeof(lcrzo_hdrppseudo2) +
                                sizeof(lcrzo_hdrptcp2) + nbocttcpopt +
                                nboctdata);
    lcrzo_er(lcrzo_data_alloc(tmpbuflen, &tmpbuf));
    lcrzo_er(lcrzo_hdrppseudo2_init(hdrlip.saddr,
                                    lcrzo_priv_addsthdrpseudo(hdrlip.daddr,
                                                              ipopt,
                                                              nboctipopt),
                                    hdrlip.protocol,
                                    (lcrzo_uint16) (sizeof(lcrzo_hdrptcp2) +
                                                    nbocttcpopt + nboctdata),
                                    tmpbuf));
    memcpy(tmpbuf + sizeof(lcrzo_hdrppseudo2), hdrptcp2,
           sizeof(lcrzo_hdrptcp2));
    memcpy(tmpbuf + sizeof(lcrzo_hdrppseudo2) + sizeof(lcrzo_hdrptcp2),
           tcpopt, nbocttcpopt);
    memcpy(tmpbuf + sizeof(lcrzo_hdrppseudo2) + sizeof(lcrzo_hdrptcp2) +
           nbocttcpopt, data, nboctdata);
    ret = lcrzo_data_checksum(tmpbuf, tmpbuflen, &tcpcheck);
    lcrzo_er(lcrzo_data_free2(&tmpbuf));
    lcrzo_er(ret);
    lcrzo_er(lcrzo_priv_hdrptcp2_set_check(hdrptcp2, tcpcheck));
  }

  /*compose le packet tcp */
  tmpbuflen =
    (lcrzo_uint16) (sizeof(lcrzo_hdrptcp2) + nbocttcpopt + nboctdata);
  lcrzo_er(lcrzo_data_alloc(tmpbuflen, &tmpbuf));
  memcpy(tmpbuf, hdrptcp2, sizeof(lcrzo_hdrptcp2));
  memcpy(tmpbuf + sizeof(lcrzo_hdrptcp2), tcpopt, nbocttcpopt);
  memcpy(tmpbuf + sizeof(lcrzo_hdrptcp2) + nbocttcpopt, data, nboctdata);

  ret = lcrzo_packet_initm_ipoptdata(hdrlip, ipopt, nboctipopt,
                                     tmpbuf, tmpbuflen, pdataout,
                                     pdataoutsize);
  lcrzo_er(lcrzo_data_free2(&tmpbuf));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_initm_icmpdata(lcrzo_hdrlicmp hdrlicmp,
                                lcrzo_constdata icmpdata,
                                lcrzo_uint16 icmpdatasize,
                                lcrzo_data * pdataout,
                                lcrzo_int32 * pdataoutsize)
{
  lcrzo_hdrpicmp2 hdrpicmp2;
  lcrzo_uint16 icmpcheck;
  lcrzo_data tmpbuf;
  lcrzo_uint16 tmpbuflen, check;
  lcrzo_bool hdr_compfields, hdr_compfield_icmpcheck;
  int ret;

  /*parameters verification */
  if (icmpdata == NULL && icmpdatasize)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (sizeof(lcrzo_hdrpip2) + icmpdatasize > 0xFFFF)
    return (LCRZO_ERR_PATOOBIG);

  /* convert logical header to physical header */
  lcrzo_er(lcrzo_hdrpicmp2_init_hdrlicmp(hdrlicmp, hdrpicmp2));

  /* compute icmp checksum */
  lcrzo_er(lcrzo_global_get_hdr_compfields(&hdr_compfields));
  lcrzo_er(lcrzo_global_get_hdr_cf_icmpcheck(&hdr_compfield_icmpcheck));
  lcrzo_er(lcrzo_priv_hdrpicmp2_get_check(hdrpicmp2, &check));
  if (check == LCRZO_HDRLICMP_CHECK_DEFVAL &&
      hdr_compfields && hdr_compfield_icmpcheck) {
    tmpbuflen = (lcrzo_uint16) (sizeof(lcrzo_hdrpicmp2) + icmpdatasize);
    lcrzo_er(lcrzo_data_alloc(tmpbuflen, &tmpbuf));
    memcpy(tmpbuf, hdrpicmp2, sizeof(lcrzo_hdrpicmp2));
    memcpy(tmpbuf + sizeof(lcrzo_hdrpicmp2), icmpdata, icmpdatasize);
    ret = lcrzo_data_checksum(tmpbuf, tmpbuflen, &icmpcheck);
    lcrzo_er(lcrzo_data_free2(&tmpbuf));
    lcrzo_er(ret);
    lcrzo_er(lcrzo_priv_hdrpicmp2_set_check(hdrpicmp2, icmpcheck));
  }

  /* compose the packet */
  tmpbuflen = (lcrzo_uint16) (sizeof(lcrzo_hdrpicmp2) + icmpdatasize);
  lcrzo_er(lcrzo_data_alloc(tmpbuflen, &tmpbuf));
  memcpy(tmpbuf, hdrpicmp2, sizeof(lcrzo_hdrpicmp2));
  memcpy(tmpbuf + sizeof(lcrzo_hdrpicmp2), icmpdata, icmpdatasize);

  /* set output values */
  if (pdataoutsize != NULL)
    *pdataoutsize = tmpbuflen;
  if (pdataout != NULL) {
    *pdataout = tmpbuf;
  } else {
    lcrzo_er(lcrzo_data_free2(&tmpbuf));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_initm_ipopticmpdata(lcrzo_hdrlip hdrlip,
                                     lcrzo_constipopt ipopt,
                                     lcrzo_uint8 nboctipopt,
                                     lcrzo_hdrlicmp hdrlicmp,
                                     lcrzo_constdata data,
                                     lcrzo_uint16 nboctdata,
                                     lcrzo_data * pdataout,
                                     lcrzo_int32 * pdataoutsize)
{
  lcrzo_hdrpicmp2 hdrpicmp2;
  lcrzo_uint16 icmpcheck;
  lcrzo_data tmpbuf;
  lcrzo_uint16 tmpbuflen, check;
  lcrzo_bool hdr_compfields, hdr_compfield_ipprotocol,
    hdr_compfield_icmpcheck;
  int ret;

  /*parameters verification */
  if (ipopt == NULL && nboctipopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (data == NULL && nboctdata)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (sizeof(lcrzo_hdrpip2) + nboctipopt +
      sizeof(lcrzo_hdrpicmp2) + nboctdata > 0xFFFF)
    return (LCRZO_ERR_PATOOBIG);

  /*met eventuellement a jour protocole icp */
  lcrzo_er(lcrzo_global_get_hdr_compfields(&hdr_compfields));
  lcrzo_er(lcrzo_global_get_hdr_cf_ipprotocol(&hdr_compfield_ipprotocol));
  if (hdrlip.protocol == LCRZO_HDRLIP_PROTOCOL_DEFVAL &&
      hdr_compfields && hdr_compfield_ipprotocol) {
    hdrlip.protocol = LCRZO_HDRLIP_PROTOCOL_ICMP;
  }

  /*conversion de l'entete logique en entete physique */
  lcrzo_er(lcrzo_hdrpicmp2_init_hdrlicmp(hdrlicmp, hdrpicmp2));

  /*calcule le checksum que si il n'est pas deja affecte */
  lcrzo_er(lcrzo_global_get_hdr_cf_icmpcheck(&hdr_compfield_icmpcheck));
  lcrzo_er(lcrzo_priv_hdrpicmp2_get_check(hdrpicmp2, &check));
  if (check == LCRZO_HDRLICMP_CHECK_DEFVAL &&
      hdr_compfields && hdr_compfield_icmpcheck) {
    tmpbuflen = (lcrzo_uint16) (sizeof(lcrzo_hdrpicmp2) + nboctdata);
    lcrzo_er(lcrzo_data_alloc(tmpbuflen, &tmpbuf));
    memcpy(tmpbuf, hdrpicmp2, sizeof(lcrzo_hdrpicmp2));
    memcpy(tmpbuf + sizeof(lcrzo_hdrpicmp2), data, nboctdata);
    ret = lcrzo_data_checksum(tmpbuf, sizeof(lcrzo_hdrpicmp2) +
                              nboctdata, &icmpcheck);
    lcrzo_er(lcrzo_data_free2(&tmpbuf));
    lcrzo_er(ret);
    lcrzo_er(lcrzo_priv_hdrpicmp2_set_check(hdrpicmp2, icmpcheck));
  }

  /*compose le packet icmp */
  tmpbuflen = (lcrzo_uint16) (sizeof(lcrzo_hdrpicmp2) + nboctdata);
  lcrzo_er(lcrzo_data_alloc(tmpbuflen, &tmpbuf));
  memcpy(tmpbuf, hdrpicmp2, sizeof(lcrzo_hdrpicmp2));
  memcpy(tmpbuf + sizeof(lcrzo_hdrpicmp2), data, nboctdata);

  ret = lcrzo_packet_initm_ipoptdata(hdrlip, ipopt, nboctipopt,
                                     tmpbuf, tmpbuflen, pdataout,
                                     pdataoutsize);
  lcrzo_er(lcrzo_data_free2(&tmpbuf));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_initm_ethdata(lcrzo_hdrleth hdrleth,
                               lcrzo_constdata data,
                               lcrzo_uint16 nboctdata,
                               lcrzo_data * pdataout,
                               lcrzo_int32 * pdataoutsize)
{
  lcrzo_hdrpeth2 hdrpeth2;

  /*parameters verification */
  if (data == NULL && nboctdata)
    return (LCRZO_ERR_PANULLPTRSIZE);

  /*conversion de l'entete logique en entete physique */
  lcrzo_er(lcrzo_hdrpeth2_init_hdrleth(hdrleth, hdrpeth2));

  /*compose le packet */
  lcrzo_er(lcrzo_data_initm_data
           ((lcrzo_data) hdrpeth2, sizeof(lcrzo_hdrpeth2), pdataout,
            pdataoutsize));
  lcrzo_er(lcrzo_data_appendm_data
           (data, nboctdata, sizeof(lcrzo_hdrpeth2), pdataout, pdataoutsize));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_initm_ethipoptdata(lcrzo_hdrleth hdrleth,
                                    lcrzo_hdrlip hdrlip,
                                    lcrzo_constipopt ipopt,
                                    lcrzo_uint8 nboctipopt,
                                    lcrzo_constdata data,
                                    lcrzo_uint16 nboctdata,
                                    lcrzo_data * pdataout,
                                    lcrzo_int32 * pdataoutsize)
{
  lcrzo_data tmpbuf;
  lcrzo_int32 tmpbuflen;
  lcrzo_bool hdr_compfields, hdr_compfield_ethtype;
  int ret;

  /*parameters verification */
  if (ipopt == NULL && nboctipopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (data == NULL && nboctdata)
    return (LCRZO_ERR_PANULLPTRSIZE);
  lcrzo_ipopt_verifbof(nboctipopt);

  /*ajout eventuel du type */
  lcrzo_er(lcrzo_global_get_hdr_compfields(&hdr_compfields));
  lcrzo_er(lcrzo_global_get_hdr_cf_ethtype(&hdr_compfield_ethtype));
  if (hdrleth.type == LCRZO_HDRLETH_TYPE_DEFVAL &&
      hdr_compfields && hdr_compfield_ethtype) {
    hdrleth.type = LCRZO_HDRLETH_TYPE_IP;
  }

  /*compose le packet */
  lcrzo_er(lcrzo_packet_initm_ipoptdata(hdrlip, ipopt, nboctipopt,
                                        data, nboctdata,
                                        &tmpbuf, &tmpbuflen));
  if (tmpbuflen > 0xFFFF)
    return (LCRZO_ERR_PATOOBIG);
  ret = lcrzo_packet_initm_ethdata(hdrleth, tmpbuf, (lcrzo_uint16) tmpbuflen,
                                   pdataout, pdataoutsize);
  lcrzo_er(lcrzo_data_free2(&tmpbuf));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_initm_ethipoptudpdata(lcrzo_hdrleth hdrleth,
                                       lcrzo_hdrlip hdrlip,
                                       lcrzo_constipopt ipopt,
                                       lcrzo_uint8 nboctipopt,
                                       lcrzo_hdrludp hdrludp,
                                       lcrzo_constdata data,
                                       lcrzo_uint16 nboctdata,
                                       lcrzo_data * pdataout,
                                       lcrzo_int32 * pdataoutsize)
{
  lcrzo_data tmpbuf;
  lcrzo_int32 tmpbuflen;
  lcrzo_bool hdr_compfields, hdr_compfield_ethtype;
  int ret;

  /*parameters verification */
  if (ipopt == NULL && nboctipopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (data == NULL && nboctdata)
    return (LCRZO_ERR_PANULLPTRSIZE);
  lcrzo_ipopt_verifbof(nboctipopt);

  /*ajout eventuel du type */
  lcrzo_er(lcrzo_global_get_hdr_compfields(&hdr_compfields));
  lcrzo_er(lcrzo_global_get_hdr_cf_ethtype(&hdr_compfield_ethtype));
  if (hdrleth.type == LCRZO_HDRLETH_TYPE_DEFVAL &&
      hdr_compfields && hdr_compfield_ethtype) {
    hdrleth.type = LCRZO_HDRLETH_TYPE_IP;
  }

  /*compose le packet */
  lcrzo_er(lcrzo_packet_initm_ipoptudpdata(hdrlip, ipopt, nboctipopt, hdrludp,
                                           data, nboctdata,
                                           &tmpbuf, &tmpbuflen));
  if (tmpbuflen > 0xFFFF)
    return (LCRZO_ERR_PATOOBIG);
  ret = lcrzo_packet_initm_ethdata(hdrleth, tmpbuf, (lcrzo_uint16) tmpbuflen,
                                   pdataout, pdataoutsize);
  lcrzo_er(lcrzo_data_free2(&tmpbuf));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_initm_ethipopttcpoptdata(lcrzo_hdrleth hdrleth,
                                          lcrzo_hdrlip hdrlip,
                                          lcrzo_constipopt ipopt,
                                          lcrzo_uint8 nboctipopt,
                                          lcrzo_hdrltcp hdrltcp,
                                          lcrzo_consttcpopt tcpopt,
                                          lcrzo_uint8 nbocttcpopt,
                                          lcrzo_constdata data,
                                          lcrzo_uint16 nboctdata,
                                          lcrzo_data * pdataout,
                                          lcrzo_int32 * pdataoutsize)
{
  lcrzo_data tmpbuf;
  lcrzo_int32 tmpbuflen;
  lcrzo_bool hdr_compfields, hdr_compfield_ethtype;
  int ret;

  /*parameters verification */
  if (ipopt == NULL && nboctipopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (tcpopt == NULL && nbocttcpopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (data == NULL && nboctdata)
    return (LCRZO_ERR_PANULLPTRSIZE);
  lcrzo_ipopt_verifbof(nboctipopt);
  lcrzo_tcpopt_verifbof(nbocttcpopt);

  /*ajout eventuel du type */
  lcrzo_er(lcrzo_global_get_hdr_compfields(&hdr_compfields));
  lcrzo_er(lcrzo_global_get_hdr_cf_ethtype(&hdr_compfield_ethtype));
  if (hdrleth.type == LCRZO_HDRLETH_TYPE_DEFVAL &&
      hdr_compfields && hdr_compfield_ethtype) {
    hdrleth.type = LCRZO_HDRLETH_TYPE_IP;
  }

  /*compose le packet */
  lcrzo_er(lcrzo_packet_initm_ipopttcpoptdata(hdrlip, ipopt, nboctipopt,
                                              hdrltcp,
                                              tcpopt, nbocttcpopt,
                                              data, nboctdata,
                                              &tmpbuf, &tmpbuflen));
  if (tmpbuflen > 0xFFFF)
    return (LCRZO_ERR_PATOOBIG);
  ret = lcrzo_packet_initm_ethdata(hdrleth, tmpbuf, (lcrzo_uint16) tmpbuflen,
                                   pdataout, pdataoutsize);
  lcrzo_er(lcrzo_data_free2(&tmpbuf));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_initm_ethipopticmpdata(lcrzo_hdrleth hdrleth,
                                        lcrzo_hdrlip hdrlip,
                                        lcrzo_constipopt ipopt,
                                        lcrzo_uint8 nboctipopt,
                                        lcrzo_hdrlicmp hdrlicmp,
                                        lcrzo_constdata data,
                                        lcrzo_uint16 nboctdata,
                                        lcrzo_data * pdataout,
                                        lcrzo_int32 * pdataoutsize)
{
  lcrzo_data tmpbuf;
  lcrzo_int32 tmpbuflen;
  lcrzo_bool hdr_compfields, hdr_compfield_ethtype;
  int ret;

  /*parameters verification */
  if (ipopt == NULL && nboctipopt)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (data == NULL && nboctdata)
    return (LCRZO_ERR_PANULLPTRSIZE);
  lcrzo_ipopt_verifbof(nboctipopt);

  /*ajout eventuel du type */
  lcrzo_er(lcrzo_global_get_hdr_compfields(&hdr_compfields));
  lcrzo_er(lcrzo_global_get_hdr_cf_ethtype(&hdr_compfield_ethtype));
  if (hdrleth.type == LCRZO_HDRLETH_TYPE_DEFVAL &&
      hdr_compfields && hdr_compfield_ethtype) {
    hdrleth.type = LCRZO_HDRLETH_TYPE_IP;
  }

  /*compose le packet */
  lcrzo_er(lcrzo_packet_initm_ipopticmpdata(hdrlip, ipopt, nboctipopt,
                                            hdrlicmp,
                                            data, nboctdata,
                                            &tmpbuf, &tmpbuflen));
  if (tmpbuflen > 0xFFFF)
    return (LCRZO_ERR_PATOOBIG);
  ret = lcrzo_packet_initm_ethdata(hdrleth, tmpbuf, (lcrzo_uint16) tmpbuflen,
                                   pdataout, pdataoutsize);
  lcrzo_er(lcrzo_data_free2(&tmpbuf));
  return (ret);
}


/*-------------------------------------------------------------*/
int lcrzo_packet_initm_etharp(lcrzo_hdrleth hdrleth,
                              lcrzo_hdrlarp hdrlarp,
                              lcrzo_data * pdataout,
                              lcrzo_int32 * pdataoutsize)
{
  lcrzo_hdrpeth2 hdrpeth2;
  lcrzo_hdrparp2 hdrparp2;

  /*conversion de l'entete logique en entete physique */
  lcrzo_er(lcrzo_hdrpeth2_init_hdrleth(hdrleth, hdrpeth2));
  lcrzo_er(lcrzo_hdrparp2_init_hdrlarp(hdrlarp, hdrparp2));

  /*compose le packet */
  lcrzo_er(lcrzo_data_initm_data
           ((lcrzo_data) hdrpeth2, sizeof(lcrzo_hdrpeth2), pdataout,
            pdataoutsize));
  lcrzo_er(lcrzo_data_appendm_data
           ((lcrzo_data) hdrparp2, sizeof(lcrzo_hdrparp2),
            sizeof(lcrzo_hdrpeth2), pdataout, pdataoutsize));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_ipoptdata(lcrzo_constdata packet,
                                   lcrzo_int32 nboctpacket,
                                   lcrzo_hdrlip * phdrlip,
                                   lcrzo_ipopt ipopt,
                                   lcrzo_uint8 * pnboctipopt,
                                   lcrzo_data * pdata,
                                   lcrzo_uint16 * pdatasize)
{
  lcrzo_hdrpip2 hdrpip2;
  lcrzo_uint8 nboctipopt, ihl, version;
  lcrzo_uint16 nboctdata, totlen;

  /*parameters verification */
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);
  /*entete incomplet */
  if (nboctpacket < 0)
    return (LCRZO_ERR_PATOOLOW);
  if ((lcrzo_uint32) nboctpacket < sizeof(lcrzo_hdrpip2))
    return (LCRZO_ERR_OKNOTDECODED);

  /*entete IP */
  memcpy(hdrpip2, packet, sizeof(lcrzo_hdrpip2));
  lcrzo_er(lcrzo_priv_hdrpip2_get_version(hdrpip2, &version));
  if (version != 4)
    return (LCRZO_ERR_OKNOTDECODED);
  if (phdrlip != NULL) {
    lcrzo_er(lcrzo_hdrlip_init_hdrpip2(hdrpip2, phdrlip));
  }

  /*correction de l'entete IP */
  lcrzo_er(lcrzo_priv_hdrpip2_get_totlen(hdrpip2, &totlen));
  if (totlen < nboctpacket) {
    /*les packets sniffes contiennent des octets inutiles en fin */
    nboctpacket = totlen;
  }

  /*options IP */
  lcrzo_er(lcrzo_priv_hdrpip2_get_ihl(hdrpip2, &ihl));
  if (ihl > 5) {
    nboctipopt = (lcrzo_uint8) ((ihl - 5) * 4);
    if ((lcrzo_int32) (nboctipopt + sizeof(lcrzo_hdrpip2)) > nboctpacket)
      return (LCRZO_ERR_OKNOTDECODED);
    if (ipopt != NULL)
      memcpy(ipopt, packet + sizeof(lcrzo_hdrpip2), nboctipopt);
  } else if (ihl < 5) {
    return (LCRZO_ERR_OKNOTDECODED);
  } else {
    nboctipopt = 0;
  }
  if (pnboctipopt != NULL)
    *pnboctipopt = nboctipopt;

  /*donnees IP */
  if (nboctpacket - 4 * ihl > 0xFFFF)
    return (LCRZO_ERR_OKNOTDECODED);
  else
    nboctdata = (lcrzo_uint16) (nboctpacket - 4 * ihl);
  lcrzo_er(lcrzo_priv_data_u16_initm_data(packet + 4 * ihl, nboctdata,
                                          pdata, pdatasize));

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_udpdata(lcrzo_constdata packet,
                                 lcrzo_int32 nboctpacket,
                                 lcrzo_hdrludp * phdrludp,
                                 lcrzo_data * pdata,
                                 lcrzo_uint16 * pdatasize)
{
  lcrzo_uint16 nboctdata;

  /*parameters verification */
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);
  /*entete incomplet */
  if (nboctpacket < 0)
    return (LCRZO_ERR_PATOOLOW);
  if ((lcrzo_uint32) nboctpacket < sizeof(lcrzo_hdrpudp2))
    return (LCRZO_ERR_OKNOTDECODED);

  /*affectation de l'entete UDP */
  if (phdrludp != NULL) {
    lcrzo_hdrpudp2 hdrpudp2;

    memcpy(hdrpudp2, packet, sizeof(lcrzo_hdrpudp2));
    lcrzo_er(lcrzo_hdrludp_init_hdrpudp2(hdrpudp2, phdrludp));
  }

  /*donnees UDP */
  if (nboctpacket - sizeof(lcrzo_hdrpudp2) > 0xFFFF)
    return (LCRZO_ERR_OKNOTDECODED);
  else
    nboctdata = (lcrzo_uint16) (nboctpacket - sizeof(lcrzo_hdrpudp2));
  lcrzo_er(lcrzo_priv_data_u16_initm_data(packet + sizeof(lcrzo_hdrpudp2),
                                          nboctdata, pdata, pdatasize));

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_tcpoptdata(lcrzo_constdata packet,
                                    lcrzo_int32 nboctpacket,
                                    lcrzo_hdrltcp * phdrltcp,
                                    lcrzo_tcpopt tcpopt,
                                    lcrzo_uint8 * pnbocttcpopt,
                                    lcrzo_data * pdata,
                                    lcrzo_uint16 * pdatasize)
{
  lcrzo_hdrptcp2 hdrptcp2;
  lcrzo_uint8 nbocttcpopt, doff;
  lcrzo_uint16 nboctdata;

  /*parameters verification */
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);
  /*entete incomplet */
  if (nboctpacket < 0)
    return (LCRZO_ERR_PATOOLOW);
  if ((lcrzo_uint32) nboctpacket < sizeof(lcrzo_hdrptcp2))
    return (LCRZO_ERR_OKNOTDECODED);

  /*entete TCP */
  memcpy(hdrptcp2, packet, sizeof(lcrzo_hdrptcp2));
  if (phdrltcp != NULL) {
    lcrzo_er(lcrzo_hdrltcp_init_hdrptcp2(hdrptcp2, phdrltcp));
  }

  /*options TCP */
  lcrzo_er(lcrzo_priv_hdrptcp2_get_doff(hdrptcp2, &doff));
  if (doff > 5) {
    nbocttcpopt = (lcrzo_uint8) ((doff - 5) * 4);
    if ((lcrzo_int32) (nbocttcpopt + sizeof(lcrzo_hdrptcp2)) > nboctpacket)
      return (LCRZO_ERR_OKNOTDECODED);
    if (tcpopt != NULL)
      memcpy(tcpopt, packet + sizeof(lcrzo_hdrptcp2), nbocttcpopt);
  } else if (doff < 5) {
    return (LCRZO_ERR_OKNOTDECODED);
  } else {
    nbocttcpopt = 0;
  }
  if (pnbocttcpopt != NULL)
    *pnbocttcpopt = nbocttcpopt;

  /*donnees TCP */
  if (nboctpacket - 4 * doff > 0xFFFF)
    return (LCRZO_ERR_OKNOTDECODED);
  else
    nboctdata = (lcrzo_uint16) (nboctpacket - 4 * doff);
  lcrzo_er(lcrzo_priv_data_u16_initm_data(packet + 4 * doff, nboctdata,
                                          pdata, pdatasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_icmpdata(lcrzo_constdata packet,
                                  lcrzo_int32 nboctpacket,
                                  lcrzo_hdrlicmp * phdrlicmp,
                                  lcrzo_data * pdata,
                                  lcrzo_uint16 * pdatasize)
{
  lcrzo_uint16 nboctdata;

  /*parameters verification */
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);
  /*entete incomplet */
  if (nboctpacket < 0)
    return (LCRZO_ERR_PATOOLOW);
  if ((lcrzo_uint32) nboctpacket < sizeof(lcrzo_hdrpicmp2))
    return (LCRZO_ERR_OKNOTDECODED);

  /*affectation de l'entete ICMP */
  if (phdrlicmp != NULL) {
    lcrzo_hdrpicmp2 hdrpicmp2;

    memcpy(hdrpicmp2, packet, sizeof(lcrzo_hdrpicmp2));
    lcrzo_er(lcrzo_hdrlicmp_init_hdrpicmp2(hdrpicmp2, phdrlicmp));
  }

  /*donnees ICMP */
  if (nboctpacket - sizeof(lcrzo_hdrpicmp2) > 0xFFFF)
    return (LCRZO_ERR_OKNOTDECODED);
  else
    nboctdata = (lcrzo_uint16) (nboctpacket - sizeof(lcrzo_hdrpicmp2));
  lcrzo_er(lcrzo_priv_data_u16_initm_data(packet + sizeof(lcrzo_hdrpicmp2),
                                          nboctdata, pdata, pdatasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_ethdata(lcrzo_constdata packet,
                                 lcrzo_int32 nboctpacket,
                                 lcrzo_hdrleth * phdrleth,
                                 lcrzo_data * pdata,
                                 lcrzo_uint16 * pdatasize)
{
  lcrzo_uint16 nboctdata;

  /*parameters verification */
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);
  /*entete incomplet */
  if (nboctpacket < 0)
    return (LCRZO_ERR_PATOOLOW);
  if ((lcrzo_uint32) nboctpacket < sizeof(lcrzo_hdrpeth2))
    return (LCRZO_ERR_OKNOTDECODED);

  /*affectation de l'entete ETH */
  if (phdrleth != NULL) {
    lcrzo_hdrpeth2 hdrpeth2;

    memcpy(hdrpeth2, packet, sizeof(lcrzo_hdrpeth2));
    lcrzo_er(lcrzo_hdrleth_init_hdrpeth2(hdrpeth2, phdrleth));
  }

  /*donnees ETH */
  if (nboctpacket - sizeof(lcrzo_hdrpeth2) > 0xFFFF)
    return (LCRZO_ERR_OKNOTDECODED);
  else
    nboctdata = (lcrzo_uint16) (nboctpacket - sizeof(lcrzo_hdrpeth2));
  lcrzo_er(lcrzo_priv_data_u16_initm_data(packet + sizeof(lcrzo_hdrpeth2),
                                          nboctdata, pdata, pdatasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decode_arp(lcrzo_constdata packet,
                            lcrzo_int32 nboctpacket,
                            lcrzo_hdrlarp * phdrlarp)
{
  /*parameters verification */
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);
  /*entete incomplet */
  if (nboctpacket < 0)
    return (LCRZO_ERR_PATOOLOW);
  if ((lcrzo_uint32) nboctpacket < sizeof(lcrzo_hdrparp2))
    return (LCRZO_ERR_OKNOTDECODED);

  /*affectation de l'entete ARP */
  if (phdrlarp != NULL) {
    lcrzo_hdrparp2 hdrparp2;

    memcpy(hdrparp2, packet, sizeof(lcrzo_hdrparp2));
    lcrzo_er(lcrzo_hdrlarp_init_hdrparp2(hdrparp2, phdrlarp));
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_ipoptudpdata(lcrzo_constdata packet,
                                      lcrzo_int32 nboctpacket,
                                      lcrzo_hdrlip * phdrlip,
                                      lcrzo_ipopt ipopt,
                                      lcrzo_uint8 * pnboctipopt,
                                      lcrzo_hdrludp * phdrludp,
                                      lcrzo_data * pdata,
                                      lcrzo_uint16 * pdatasize)
{
  lcrzo_data ipdata;
  lcrzo_uint16 nboctipdata;
  lcrzo_hdrlip hdrliplocal;
  int ret;

  /*decode pour l'entete IP */
  lcrzo_er(lcrzo_packet_decodem_ipoptdata(packet, nboctpacket,
                                          &hdrliplocal, ipopt, pnboctipopt,
                                          &ipdata, &nboctipdata));

  /*si l'entete IP est non NULL on le copie */
  if (phdrlip != NULL) {
    memcpy(phdrlip, &hdrliplocal, sizeof(lcrzo_hdrlip));
  }

  /*decode le contenu */
  if (hdrliplocal.offsetfrag) {
    lcrzo_er(lcrzo_data_free2(&ipdata));
    return (LCRZO_ERR_OKNOTDECODED);    /*ne sait pas decoder les fragments */
  }
  if (hdrliplocal.protocol != LCRZO_HDRLIP_PROTOCOL_UDP) {
    lcrzo_er(lcrzo_data_free2(&ipdata));
    return (LCRZO_ERR_OKNOTDECODED);
  }
  ret = lcrzo_packet_decodem_udpdata(ipdata, nboctipdata,
                                     phdrludp, pdata, pdatasize);
  lcrzo_er(lcrzo_data_free2(&ipdata));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_ipopttcpoptdata(lcrzo_constdata packet,
                                         lcrzo_int32 nboctpacket,
                                         lcrzo_hdrlip * phdrlip,
                                         lcrzo_ipopt ipopt,
                                         lcrzo_uint8 * pnboctipopt,
                                         lcrzo_hdrltcp * phdrltcp,
                                         lcrzo_tcpopt tcpopt,
                                         lcrzo_uint8 * pnbocttcpopt,
                                         lcrzo_data * pdata,
                                         lcrzo_uint16 * pdatasize)
{
  lcrzo_data ipdata;
  lcrzo_uint16 nboctipdata;
  lcrzo_hdrlip hdrliplocal;
  int ret;

  /*decode pour l'entete IP */
  lcrzo_er(lcrzo_packet_decodem_ipoptdata(packet, nboctpacket,
                                          &hdrliplocal,
                                          ipopt, pnboctipopt,
                                          &ipdata, &nboctipdata));

  /*si l'entete IP est non NULL on le copie */
  if (phdrlip != NULL) {
    memcpy(phdrlip, &hdrliplocal, sizeof(lcrzo_hdrlip));
  }

  /*decode le contenu */
  if (hdrliplocal.offsetfrag) {
    lcrzo_er(lcrzo_data_free2(&ipdata));
    return (LCRZO_ERR_OKNOTDECODED);    /*ne sait pas decoder les fragments */
  }
  if (hdrliplocal.protocol != LCRZO_HDRLIP_PROTOCOL_TCP) {
    lcrzo_er(lcrzo_data_free2(&ipdata));
    return (LCRZO_ERR_OKNOTDECODED);
  }
  ret = lcrzo_packet_decodem_tcpoptdata(ipdata, nboctipdata,
                                        phdrltcp, tcpopt, pnbocttcpopt,
                                        pdata, pdatasize);
  lcrzo_er(lcrzo_data_free2(&ipdata));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_ipopticmpdata(lcrzo_constdata packet,
                                       lcrzo_int32 nboctpacket,
                                       lcrzo_hdrlip * phdrlip,
                                       lcrzo_ipopt ipopt,
                                       lcrzo_uint8 * pnboctipopt,
                                       lcrzo_hdrlicmp * phdrlicmp,
                                       lcrzo_data * pdata,
                                       lcrzo_uint16 * pdatasize)
{
  lcrzo_data ipdata;
  lcrzo_uint16 nboctipdata;
  lcrzo_hdrlip hdrliplocal;
  int ret;

  /*decode pour l'entete IP */
  lcrzo_er(lcrzo_packet_decodem_ipoptdata(packet, nboctpacket,
                                          &hdrliplocal, ipopt, pnboctipopt,
                                          &ipdata, &nboctipdata));

  /*si l'entete IP est non NULL on le copie */
  if (phdrlip != NULL) {
    memcpy(phdrlip, &hdrliplocal, sizeof(lcrzo_hdrlip));
  }

  /*decode le contenu */
  if (hdrliplocal.offsetfrag) {
    lcrzo_er(lcrzo_data_free2(&ipdata));
    return (LCRZO_ERR_OKNOTDECODED);    /*ne sait pas decoder les fragments */
  }
  if (hdrliplocal.protocol != LCRZO_HDRLIP_PROTOCOL_ICMP) {
    lcrzo_er(lcrzo_data_free2(&ipdata));
    return (LCRZO_ERR_OKNOTDECODED);
  }
  ret = lcrzo_packet_decodem_icmpdata(ipdata, nboctipdata,
                                      phdrlicmp, pdata, pdatasize);
  lcrzo_er(lcrzo_data_free2(&ipdata));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_ethipoptdata(lcrzo_constdata packet,
                                      lcrzo_int32 nboctpacket,
                                      lcrzo_hdrleth * phdrleth,
                                      lcrzo_hdrlip * phdrlip,
                                      lcrzo_ipopt ipopt,
                                      lcrzo_uint8 * pnboctipopt,
                                      lcrzo_data * pdata,
                                      lcrzo_uint16 * pdatasize)
{
  lcrzo_data ethdata;
  lcrzo_uint16 nboctethdata;
  lcrzo_hdrleth hdrlethlocal;
  int ret;

  /*decode pour l'entete ETH */
  lcrzo_er(lcrzo_packet_decodem_ethdata(packet, nboctpacket,
                                        &hdrlethlocal,
                                        &ethdata, &nboctethdata));

  /*si l'entete ETH est non NULL on le copie */
  if (phdrleth != NULL) {
    memcpy(phdrleth, &hdrlethlocal, sizeof(lcrzo_hdrleth));
  }

  /*decode le contenu */
  if (hdrlethlocal.type != LCRZO_HDRLETH_TYPE_IP) {
    lcrzo_er(lcrzo_data_free2(&ethdata));
    return (LCRZO_ERR_OKNOTDECODED);
  }
  ret = lcrzo_packet_decodem_ipoptdata(ethdata, nboctethdata,
                                       phdrlip, ipopt, pnboctipopt,
                                       pdata, pdatasize);
  lcrzo_er(lcrzo_data_free2(&ethdata));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_ethipoptudpdata(lcrzo_constdata packet,
                                         lcrzo_int32 nboctpacket,
                                         lcrzo_hdrleth * phdrleth,
                                         lcrzo_hdrlip * phdrlip,
                                         lcrzo_ipopt ipopt,
                                         lcrzo_uint8 * pnboctipopt,
                                         lcrzo_hdrludp * phdrludp,
                                         lcrzo_data * pdata,
                                         lcrzo_uint16 * pdatasize)
{
  lcrzo_data ethdata;
  lcrzo_uint16 nboctethdata;
  lcrzo_hdrleth hdrlethlocal;
  int ret;

  /*decode pour l'entete ETH */
  lcrzo_er(lcrzo_packet_decodem_ethdata(packet, nboctpacket,
                                        &hdrlethlocal,
                                        &ethdata, &nboctethdata));

  /*si l'entete ETH est non NULL on le copie */
  if (phdrleth != NULL) {
    memcpy(phdrleth, &hdrlethlocal, sizeof(lcrzo_hdrleth));
  }

  /*decode le contenu */
  if (hdrlethlocal.type != LCRZO_HDRLETH_TYPE_IP) {
    lcrzo_er(lcrzo_data_free2(&ethdata));
    return (LCRZO_ERR_OKNOTDECODED);
  }
  ret = lcrzo_packet_decodem_ipoptudpdata(ethdata, nboctethdata,
                                          phdrlip, ipopt, pnboctipopt,
                                          phdrludp, pdata, pdatasize);
  lcrzo_er(lcrzo_data_free2(&ethdata));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_ethipopttcpoptdata(lcrzo_constdata packet,
                                            lcrzo_int32 nboctpacket,
                                            lcrzo_hdrleth * phdrleth,
                                            lcrzo_hdrlip * phdrlip,
                                            lcrzo_ipopt ipopt,
                                            lcrzo_uint8 * pnboctipopt,
                                            lcrzo_hdrltcp * phdrltcp,
                                            lcrzo_tcpopt tcpopt,
                                            lcrzo_uint8 * pnbocttcpopt,
                                            lcrzo_data * pdata,
                                            lcrzo_uint16 * pdatasize)
{
  lcrzo_data ethdata;
  lcrzo_uint16 nboctethdata;
  lcrzo_hdrleth hdrlethlocal;
  int ret;

  /*decode pour l'entete ETH */
  lcrzo_er(lcrzo_packet_decodem_ethdata(packet, nboctpacket,
                                        &hdrlethlocal,
                                        &ethdata, &nboctethdata));

  /*si l'entete ETH est non NULL on le copie */
  if (phdrleth != NULL) {
    memcpy(phdrleth, &hdrlethlocal, sizeof(lcrzo_hdrleth));
  }

  /*decode le contenu */
  if (hdrlethlocal.type != LCRZO_HDRLETH_TYPE_IP) {
    lcrzo_er(lcrzo_data_free2(&ethdata));
    return (LCRZO_ERR_OKNOTDECODED);
  }
  ret = lcrzo_packet_decodem_ipopttcpoptdata(ethdata, nboctethdata,
                                             phdrlip,
                                             ipopt, pnboctipopt,
                                             phdrltcp,
                                             tcpopt, pnbocttcpopt,
                                             pdata, pdatasize);
  lcrzo_er(lcrzo_data_free2(&ethdata));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decodem_ethipopticmpdata(lcrzo_constdata packet,
                                          lcrzo_int32 nboctpacket,
                                          lcrzo_hdrleth * phdrleth,
                                          lcrzo_hdrlip * phdrlip,
                                          lcrzo_ipopt ipopt,
                                          lcrzo_uint8 * pnboctipopt,
                                          lcrzo_hdrlicmp * phdrlicmp,
                                          lcrzo_data * pdata,
                                          lcrzo_uint16 * pdatasize)
{
  lcrzo_data ethdata;
  lcrzo_uint16 nboctethdata;
  lcrzo_hdrleth hdrlethlocal;
  int ret;

  /*decode pour l'entete ETH */
  lcrzo_er(lcrzo_packet_decodem_ethdata(packet, nboctpacket,
                                        &hdrlethlocal,
                                        &ethdata, &nboctethdata));

  /*si l'entete ETH est non NULL on le copie */
  if (phdrleth != NULL) {
    memcpy(phdrleth, &hdrlethlocal, sizeof(lcrzo_hdrleth));
  }

  /*decode le contenu */
  if (hdrlethlocal.type != LCRZO_HDRLETH_TYPE_IP) {
    lcrzo_er(lcrzo_data_free2(&ethdata));
    return (LCRZO_ERR_OKNOTDECODED);
  }
  ret = lcrzo_packet_decodem_ipopticmpdata(ethdata, nboctethdata,
                                           phdrlip, ipopt, pnboctipopt,
                                           phdrlicmp, pdata, pdatasize);
  lcrzo_er(lcrzo_data_free2(&ethdata));
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_decode_etharp(lcrzo_constdata packet,
                               lcrzo_int32 nboctpacket,
                               lcrzo_hdrleth * phdrleth,
                               lcrzo_hdrlarp * phdrlarp)
{
  lcrzo_data ethdata;
  lcrzo_uint16 nboctethdata;
  lcrzo_hdrleth hdrlethlocal;
  int ret;

  /*decode pour l'entete ETH */
  lcrzo_er(lcrzo_packet_decodem_ethdata(packet, nboctpacket,
                                        &hdrlethlocal,
                                        &ethdata, &nboctethdata));

  /*si l'entete ETH est non NULL on le copie */
  if (phdrleth != NULL) {
    memcpy(phdrleth, &hdrlethlocal, sizeof(lcrzo_hdrleth));
  }

  /*decode le contenu */
  if (hdrlethlocal.type != LCRZO_HDRLETH_TYPE_ARP &&
      hdrlethlocal.type != LCRZO_HDRLETH_TYPE_RARP) {
    lcrzo_er(lcrzo_data_free2(&ethdata));
    return (LCRZO_ERR_OKNOTDECODED);
  }
  ret = lcrzo_packet_decode_arp(ethdata, nboctethdata, phdrlarp);
  lcrzo_er(lcrzo_data_free2(&ethdata));
  return (LCRZO_ERR_OK);
}
