/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_incl.h"


/*-------------------------------------------------------------*/
#define LCRZO_PRINTTYPE_C_entxt    ", comment before"
#define LCRZO_PRINTTYPE_C_frtxt    ", commentaire avant"
#define LCRZO_PRINTTYPE_S_entxt    ", short data lines"
#define LCRZO_PRINTTYPE_S_frtxt    ", lignes courtes"
#define LCRZO_PRINTTYPE_N_entxt    ", newline after"
#define LCRZO_PRINTTYPE_N_frtxt    ", saut de ligne apres"
#define LCRZO_PRINTTYPE_I_entxt    ""
#define LCRZO_PRINTTYPE_I_frtxt    ""

/*-------------------------------------------------------------*/
#define LCRZO_PRIV_HEXA       1
#define LCRZO_PRIV_DEC        2
#define LCRZO_PRIV_CAR        4
#define LCRZO_PRIV_HEXADEC    (LCRZO_PRIV_HEXA|LCRZO_PRIV_DEC)
#define LCRZO_PRIV_HEXACAR    (LCRZO_PRIV_HEXA|LCRZO_PRIV_CAR)
#define LCRZO_PRIV_DECCAR     (LCRZO_PRIV_DEC|LCRZO_PRIV_CAR)
#define LCRZO_PRIV_HEXADECCAR (LCRZO_PRIV_HEXADEC|LCRZO_PRIV_CAR)
int lcrzo_priv_ecrit_tab01(LCRZOFILE * pf,
                           lcrzo_uint8 nb);
int lcrzo_priv_ecrit_tab01(LCRZOFILE * pf,
                           lcrzo_uint8 nb)
{
  fprintf((FILE *) pf, "%1d_", nb);
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_ecrit_tab04(LCRZOFILE * pf,
                           lcrzo_uint8 nb,
                           int type);
int lcrzo_priv_ecrit_tab04(LCRZOFILE * pf,
                           lcrzo_uint8 nb,
                           int type)
{
  if (type == LCRZO_PRIV_HEXA)
    fprintf((FILE *) pf, "___%1Xh___", nb);
  else if (type == LCRZO_PRIV_DEC)
    fprintf((FILE *) pf, "___%2d___", nb);
  else
    return (lcrzo_ieuv("lcrzo_priv_ecrit_tab04", "type"));
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_ecrit_tab08(LCRZOFILE * pf,
                           lcrzo_uint8 nb,
                           int type);
int lcrzo_priv_ecrit_tab08(LCRZOFILE * pf,
                           lcrzo_uint8 nb,
                           int type)
{ /*             "0_1_2_3_.0_1_2_3_:0_1_2_3_.0_1_2_3_|..." */
  if (type == LCRZO_PRIV_HEXA)
    fprintf((FILE *) pf, "_______%02Xh_______", nb);
  else if (type == LCRZO_PRIV_DEC)
    fprintf((FILE *) pf, "_______%3d_______", nb);
  else if (type == LCRZO_PRIV_CAR)
    fprintf((FILE *) pf, "________%c________", lcrzo_priv_char_print(nb));
  else if (type == LCRZO_PRIV_HEXADEC)
    fprintf((FILE *) pf, "_____%02Xh=%3d_____", nb, nb);
  else if (type == LCRZO_PRIV_HEXACAR)
    fprintf((FILE *) pf, "______%02Xh=%c______", nb,
            lcrzo_priv_char_print(nb));
  else if (type == LCRZO_PRIV_DECCAR)
    fprintf((FILE *) pf, "______%3d=%c______", nb, lcrzo_priv_char_print(nb));
  else if (type == LCRZO_PRIV_HEXADECCAR)
    fprintf((FILE *) pf, "____%02Xh=%3d=%c____", nb, nb,
            lcrzo_priv_char_print(nb));
  else
    return (lcrzo_ieuv("lcrzo_priv_ecrit_tab08", "type"));
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_ecrit_tab16(LCRZOFILE * pf,
                           lcrzo_uint16 nb,
                           int type);
int lcrzo_priv_ecrit_tab16(LCRZOFILE * pf,
                           lcrzo_uint16 nb,
                           int type)
{ /*             "0_1_2_3_.0_1_2_3_:0_1_2_3_.0_1_2_3_|..." */
  if (type == LCRZO_PRIV_HEXA)
    fprintf((FILE *) pf, "_______________%04Xh_______________", nb);
  else if (type == LCRZO_PRIV_DEC)
    fprintf((FILE *) pf, "_______________%5d_______________", nb);
  else if (type == LCRZO_PRIV_HEXADEC)
    fprintf((FILE *) pf, "____________%04Xh=%5d____________", nb, nb);
  else
    return (lcrzo_ieuv("lcrzo_priv_ecrit_tab16", "type"));
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_ecrit_tab32(LCRZOFILE * pf,
                           lcrzo_uint32 nb,
                           int type);
int lcrzo_priv_ecrit_tab32(LCRZOFILE * pf,
                           lcrzo_uint32 nb,
                           int type)
{ /*             "0_1_2_3_.0_1_2_3_:0_1_2_3_.0_1_2_3_|..." */
  if (type == LCRZO_PRIV_HEXA) {
    lcrzo_er(lcrzo_uint32_fprint(pf, "_______________________________",
                                 nb, "%08X",
                                 "h_______________________________"));
  } else if (type == LCRZO_PRIV_DEC) {
    lcrzo_er(lcrzo_uint32_fprint(pf, "______________________________",
                                 nb, "%10u",
                                 "_______________________________"));
  } else if (type == LCRZO_PRIV_HEXADEC) {
    lcrzo_er(lcrzo_uint32_fprint(pf, "_________________________",
                                 nb, "%08X", "h"));
    lcrzo_er(lcrzo_uint32_fprint(pf, "=",
                                 nb, "%10u", "__________________________"));
  } else
    return (lcrzo_ieuv("lcrzo_priv_ecrit_tab32", "type"));
  return (LCRZO_ERR_OK);
}

/*
int lcrzo_priv_ecrit_tabxx_test(void)
{ int i;
  printf("0_1_2_3_.0_1_2_3_:0_1_2_3_.0_1_2_3_|");
  puts("0_1_2_3_.0_1_2_3_:0_1_2_3_.0_1_2_3_");
  lcrzo_er(lcrzo_priv_ecrit_tab01(stdout, 1));puts("ZZ");
  for ( i=1 ; i<=2 ; i++ )
  { lcrzo_er(lcrzo_priv_ecrit_tab04(stdout, 0xF, i));puts("ZZ");
  }
  for ( i=1 ; i<=8 ; i++ )
  { lcrzo_er(lcrzo_priv_ecrit_tab08(stdout, 0xFF, i));puts("ZZ");
  }
  for ( i=1 ; i<=8 ; i++ )
  { lcrzo_er(lcrzo_priv_ecrit_tab16(stdout, 0xFFFF, i));puts("ZZ");
  }
  for ( i=1 ; i<=8 ; i++ )
  { lcrzo_er(lcrzo_priv_ecrit_tab32(stdout, 0xFFffffFFU, i));puts("ZZ");
  }
  return(LCRZO_ERR_OK);
}
*/

/*-------------------------------------------------------------*/
int lcrzo_priv_printtype_printcomment(LCRZOFILE * pf,
                                      lcrzo_printtype printtype,
                                      lcrzo_conststring encom,
                                      lcrzo_conststring frcom);
int lcrzo_priv_printtype_printcomment(LCRZOFILE * pf,
                                      lcrzo_printtype printtype,
                                      lcrzo_conststring encom,
                                      lcrzo_conststring frcom)
{
  lcrzo_int16 language;

  lcrzo_er(lcrzo_global_get_language(&language));
  if (printtype & LCRZO_PRINTTYPE_C_) {
    if (language == LCRZO_GLOBAL_LANG_FR) {
      if (frcom != NULL)
        fprintf((FILE *) pf, "#%s\n", frcom);
    } else {
      if (encom != NULL)
        fprintf((FILE *) pf, "#%s\n", encom);
    }
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_printtype_printnewline(LCRZOFILE * pf,
                                      lcrzo_printtype printtype);
int lcrzo_priv_printtype_printnewline(LCRZOFILE * pf,
                                      lcrzo_printtype printtype)
{
  if (printtype & LCRZO_PRINTTYPE_N_) {
    fprintf((FILE *) pf, "\n");
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_ecrittransition(LCRZOFILE * pf,
                               lcrzo_printprofile * pprintprofile,
                               lcrzo_printtype nextprinttype);
int lcrzo_priv_ecrittransition(LCRZOFILE * pf,
                               lcrzo_printprofile * pprintprofile,
                               lcrzo_printtype nextprinttype)
{
  /*fprintf((FILE*)pf, "[Transition de %04X -> %04X]", pprintprofile->lastused,
     nextprinttype); */

  /*rien a ecrire si on ne veut rien ecrire. Ne pas changer
     l'etat de pprintprofile->lastused */
  if ((nextprinttype & 0xFF) == LCRZO_PRINTTYPE_NOTHING_) {
    return (LCRZO_ERR_OK);
  }

  /*if the last used has a newline */
  if ((pprintprofile->lastused & (LCRZO_PRINTTYPE_N_ | LCRZO_PRINTTYPE_I_))
      || (pprintprofile->lastused == LCRZO_PRINTTYPE_START)) {
    /*nothing to print */
  } else if ((pprintprofile->lastused & 0xFF) == LCRZO_PRINTTYPE_NOTHING_) {
    /*nothing to print */
  } else {                      /*if the next must begin with an new line */
    if ((nextprinttype & (LCRZO_PRINTTYPE_C_ | LCRZO_PRINTTYPE_I_)) ||
        (nextprinttype == LCRZO_PRINTTYPE_START)) {
      fprintf((FILE *) pf, "\n");
    } else {
      /*so now, we only have the case where the lastused doesn't end with
        an newline, and the next mustn't begin with an new line */
      switch (pprintprofile->lastused) {
        case LCRZO_PRINTTYPE_SYNTH:
          switch (nextprinttype) {
            case LCRZO_PRINTTYPE_SYNTH:
            case LCRZO_PRINTTYPE_SYNTHN:
            case LCRZO_PRINTTYPE_TEXT:
            case LCRZO_PRINTTYPE_TEXTN:
            case LCRZO_PRINTTYPE_PTEXT:
            case LCRZO_PRINTTYPE_PTEXTN:
            case LCRZO_PRINTTYPE_BASE64:
            case LCRZO_PRINTTYPE_BASE64N:
            case LCRZO_PRINTTYPE_HEXA:
            case LCRZO_PRINTTYPE_HEXAN:
            case LCRZO_PRINTTYPE_MIXED:
            case LCRZO_PRINTTYPE_MIXEDN:
              fprintf((FILE *) pf, " - ");
              break;
            default:
              return (lcrzo_ieuv("lcrzo_priv_ecrittransition",
                                 "nextprinttype 1"));
          }
          break;
        case LCRZO_PRINTTYPE_TEXT:
        case LCRZO_PRINTTYPE_PTEXT:
          switch (nextprinttype) {
            case LCRZO_PRINTTYPE_SYNTH:
            case LCRZO_PRINTTYPE_SYNTHN:
              fprintf((FILE *) pf, " - ");
              break;
            case LCRZO_PRINTTYPE_TEXT:
            case LCRZO_PRINTTYPE_TEXTN:
            case LCRZO_PRINTTYPE_PTEXT:
            case LCRZO_PRINTTYPE_PTEXTN:
              break;
            case LCRZO_PRINTTYPE_BASE64:
            case LCRZO_PRINTTYPE_BASE64N:
            case LCRZO_PRINTTYPE_HEXA:
            case LCRZO_PRINTTYPE_HEXAN:
            case LCRZO_PRINTTYPE_MIXED:
            case LCRZO_PRINTTYPE_MIXEDN:
              break;
            default:
              return (lcrzo_ieuv
                      ("lcrzo_priv_ecrittransition", "nextprinttype 2"));
          }
          break;
        case LCRZO_PRINTTYPE_BASE64:
        case LCRZO_PRINTTYPE_HEXA:
        case LCRZO_PRINTTYPE_MIXED:
          switch (nextprinttype) {
            case LCRZO_PRINTTYPE_SYNTH:
            case LCRZO_PRINTTYPE_SYNTHN:
              fprintf((FILE *) pf, " - ");
              break;
            case LCRZO_PRINTTYPE_TEXT:
            case LCRZO_PRINTTYPE_TEXTN:
            case LCRZO_PRINTTYPE_PTEXT:
            case LCRZO_PRINTTYPE_PTEXTN:
              break;
            case LCRZO_PRINTTYPE_BASE64:
            case LCRZO_PRINTTYPE_BASE64N:
            case LCRZO_PRINTTYPE_HEXA:
            case LCRZO_PRINTTYPE_HEXAN:
            case LCRZO_PRINTTYPE_MIXED:
            case LCRZO_PRINTTYPE_MIXEDN:
              fprintf((FILE *) pf, " ");
              break;
            default:
              return (lcrzo_ieuv
                      ("lcrzo_priv_ecrittransition", "nextprinttype 3"));
          }
          break;
        default:
          return (lcrzo_ieuv("lcrzo_priv_ecrittransition",
                             "pprintprofile->lastused"));
          break;
      }
    }
  }

  /*for the next function call */
  pprintprofile->lastused = nextprinttype;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_ecrittransitionfin(LCRZOFILE * pf,
                                  lcrzo_printprofile * pprintprofile);
int lcrzo_priv_ecrittransitionfin(LCRZOFILE * pf,
                                  lcrzo_printprofile * pprintprofile)
{
  return (lcrzo_priv_ecrittransition
          (pf, pprintprofile, LCRZO_PRINTTYPE_START));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_printprofile_tcdf(LCRZOFILE * pf,
                                 lcrzo_printprofile * pprintprofile,
                                 lcrzo_printtype printtype,
                                 lcrzo_conststring encom,
                                 lcrzo_conststring frcom,
                                 lcrzo_constdata packet,
                                 lcrzo_int32 nboctpacket);
int lcrzo_priv_printprofile_tcdf(LCRZOFILE * pf,
                                 lcrzo_printprofile * pprintprofile,
                                 lcrzo_printtype printtype,
                                 lcrzo_conststring encom,
                                 lcrzo_conststring frcom,
                                 lcrzo_constdata packet,
                                 lcrzo_int32 nboctpacket)
{
  lcrzo_er(lcrzo_priv_ecrittransition(pf, pprintprofile, printtype));
  lcrzo_er(lcrzo_priv_printtype_printcomment(pf, printtype, encom, frcom));
  lcrzo_er(lcrzo_data_fprint(pf, packet, nboctpacket, printtype));
  return (lcrzo_priv_ecrittransitionfin(pf, pprintprofile));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_ecritpacket_datadump(LCRZOFILE * pf,
                                    lcrzo_constdata packet,
                                    lcrzo_int32 nboctpacket);
int lcrzo_priv_ecritpacket_datadump(LCRZOFILE * pf,
                                    lcrzo_constdata packet,
                                    lcrzo_int32 nboctpacket)
{
  char tabgauche[70], tabdroite[17], c, ctmp;
  int i, j, poscour;

  /*parameters verification */
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);

  for (i = 0; i <= (int)nboctpacket / 16; i++) {
    poscour = 0;
    lcrzo_er(lcrzo_string_init_text
             ("                                                     ", 69,
              tabgauche));
    for (j = 0; j < 16; j++) {
      if ((j + (i << 4)) >= (int)nboctpacket)
        break;
      c = packet[j + (i << 4)];
      if (!(j % 4) && (j != 0))
        tabgauche[poscour++] = ' ';
      tabgauche[poscour++] = ' ';
      ctmp = (char)((c >> 4) & 0xF);
      tabgauche[poscour++] = lcrzo_priv_hchar_init_int(ctmp);
      ctmp = (char)(c & 0xF);
      tabgauche[poscour++] = lcrzo_priv_hchar_init_int(ctmp);
      tabdroite[j] = lcrzo_priv_char_print(c);
    }
    tabdroite[j] = '\0';
    if (j) {
      fprintf((FILE *) pf, "%s   # %s\n", tabgauche, tabdroite);
      fflush((FILE *) pf);
    }
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_ecritpacket_datahexmixs(LCRZOFILE * pf,
                                       lcrzo_constdata packet,
                                       lcrzo_int32 nboctpacket,
                                       lcrzo_printtype printtypeichage);
int lcrzo_priv_ecritpacket_datahexmixs(LCRZOFILE * pf,
                                       lcrzo_constdata packet,
                                       lcrzo_int32 nboctpacket,
                                       lcrzo_printtype printtypeichage)
{
  char tabhexa[80], tabmixte[80], c, ctmp;
  int i, j, guillemetecrit;

  /*parameters verification */
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);

  for (i = 0; i <= (int)nboctpacket / 16; i++) {
    lcrzo_er(lcrzo_string_init_text("", 79, tabhexa));
    lcrzo_er(lcrzo_string_init_text("", 79, tabmixte));
    guillemetecrit = 0;
    for (j = 0; j < 16; j++) {
      if ((j + (i << 4)) >= (int)nboctpacket)
        break;
      c = packet[j + (i << 4)];
      /*ecriture de c sous forme hexa */
      lcrzo_er(lcrzo_string_append_char(' ', 1, 79, tabhexa));
      ctmp = (char)((c >> 4) & 0xF);
      lcrzo_er(lcrzo_string_append_char(lcrzo_priv_hchar_init_int(ctmp),
                                        1, 79, tabhexa));
      ctmp = (char)(c & 0xF);
      lcrzo_er(lcrzo_string_append_char(lcrzo_priv_hchar_init_int(ctmp),
                                        1, 79, tabhexa));
      /*ecriture de c sous forme de hexa/chaine_caractere */
      if (lcrzo_priv_mixed_printhexa(c)) {
        if (guillemetecrit) {
          lcrzo_er(lcrzo_string_append_char('\'', 1, 79, tabmixte));
          guillemetecrit = 0;
        }
        lcrzo_er(lcrzo_string_append_char(' ', 1, 79, tabmixte));
        ctmp = (char)((c >> 4) & 0xF);
        lcrzo_er(lcrzo_string_append_char(lcrzo_priv_hchar_init_int(ctmp),
                                          1, 79, tabmixte));
        ctmp = (char)(c & 0xF);
        lcrzo_er(lcrzo_string_append_char(lcrzo_priv_hchar_init_int(ctmp),
                                          1, 79, tabmixte));
      } else {
        if (!guillemetecrit) {
          lcrzo_er(lcrzo_string_append_text(" '", 79, tabmixte));
          guillemetecrit = 1;
        }
        if (c == '\'') {
          lcrzo_er(lcrzo_string_append_text("''", 79, tabmixte));
        } else {
          lcrzo_er(lcrzo_string_append_char(c, 1, 79, tabmixte));
        }
      }
    }
    if (guillemetecrit) {
      lcrzo_er(lcrzo_string_append_char('\'', 1, 79, tabmixte));
    }
    if (j) {
      switch (printtypeichage & 0xFF) {
        case LCRZO_PRINTTYPE_MIXED_:
          fprintf((FILE *) pf, " %s\n", tabmixte);
          break;
        case LCRZO_PRINTTYPE_CMH_:
          fprintf((FILE *) pf, "#%s\n %s\n", tabmixte, tabhexa);
          break;
        case LCRZO_PRINTTYPE_CHM_:
          fprintf((FILE *) pf, "#%s\n %s\n", tabhexa, tabmixte);
          break;
        default:
          return (lcrzo_ieuv("lcrzo_priv_ecritpacket_datahexmixs",
                             "printtypeichage&0xFF"));
      }
      fflush((FILE *) pf);
    }
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_ecritpacket_datamixed(LCRZOFILE * pf,
                                     lcrzo_constdata packet,
                                     lcrzo_int32 nboctpacket);
int lcrzo_priv_ecritpacket_datamixed(LCRZOFILE * pf,
                                     lcrzo_constdata packet,
                                     lcrzo_int32 nboctpacket)
{
  int i;
  lcrzo_bool guillemetecrit, first;
  lcrzo_uint8 c;

  /*parameters verification */
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);

  guillemetecrit = 0;
  first = 1;
  for (i = 0; i < nboctpacket; i++) {
    c = packet[i];
    if (lcrzo_priv_mixed_printhexa(c)) {
      if (guillemetecrit) {
        fputc('\'', (FILE *) pf);
        guillemetecrit = 0;
      }
      if (first)
        first = 0;
      else
        fputc(' ', (FILE *) pf);
      fprintf((FILE *) pf, "%02X", c);
    } else {
      if (!guillemetecrit) {
        if (first)
          first = 0;
        else
          fputc(' ', (FILE *) pf);
        fputc('\'', (FILE *) pf);
        guillemetecrit = 1;
      }
      if (c == '\'') {
        fprintf((FILE *) pf, "''");
      } else {
        fputc(c, (FILE *) pf);
      }
    }
  }
  if (guillemetecrit)
    fputc('\'', (FILE *) pf);

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_affcarbase64(LCRZOFILE * pf,
                            int nb);
int lcrzo_priv_affcarbase64(LCRZOFILE * pf,
                            int nb)
{
  if (nb < 26)
    fprintf((FILE *) pf, "%c", nb + 'A');
  else if (nb < 52)
    fprintf((FILE *) pf, "%c", (nb - 26) + 'a');
  else if (nb < 62)
    fprintf((FILE *) pf, "%c", (nb - 52) + '0');
  else if (nb == 62)
    fprintf((FILE *) pf, "+");
  else if (nb == 63)
    fprintf((FILE *) pf, "/");
  else
    return (lcrzo_ieuv("lcrzo_priv_affcarbase64", "nb"));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_ecrit_ipopttabipl(LCRZOFILE * pf,
                                 lcrzo_uint8 nbiplaffectes,
                                 lcrzo_ipl iplt[]);
int lcrzo_priv_ecrit_ipopttabipl(LCRZOFILE * pf,
                                 lcrzo_uint8 nbiplaffectes,
                                 lcrzo_ipl iplt[])
{
  lcrzo_ipa ipa;
  int i;

  if (nbiplaffectes == 0)
    return (LCRZO_ERR_OK);

  for (i = 0; i < nbiplaffectes; i++) {
    lcrzo_er(lcrzo_ipa_init_ipl(iplt[i], ipa));
    if (i) {
      if (!(i % 4))
        fprintf((FILE *) pf, "\n |    ");
      else
        fprintf((FILE *) pf, ", ");
    } else {
      if (!(i % 4))
        fprintf((FILE *) pf, " |    ");
    }
    lcrzo_er(lcrzo_ipa_fprint(pf, "", ipa, ""));
  }
  fprintf((FILE *) pf, "\n");
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_ecrit_ipopttabtime(LCRZOFILE * pf,
                                  lcrzo_uint8 nbiplaffectes,
                                  lcrzo_ipl iplt[]);
int lcrzo_priv_ecrit_ipopttabtime(LCRZOFILE * pf,
                                  lcrzo_uint8 nbiplaffectes,
                                  lcrzo_ipl iplt[])
{
  int i;

  if (nbiplaffectes == 0)
    return (LCRZO_ERR_OK);

  for (i = 0; i < nbiplaffectes; i++) {
    if (i) {
      if (!(i % 4))
        fprintf((FILE *) pf, "\n |    ");
      else
        fprintf((FILE *) pf, ", ");
    } else {
      if (!(i % 4))
        fprintf((FILE *) pf, " |    ");
    }
    lcrzo_er(lcrzo_uint32_fprint(pf, "", iplt[i], "%u", ""));
  }
  fprintf((FILE *) pf, "\n");
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_ecrit_ipopttabiptime(LCRZOFILE * pf,
                                    lcrzo_uint8 nbiplaffectes,
                                    lcrzo_ipl iplt[]);
int lcrzo_priv_ecrit_ipopttabiptime(LCRZOFILE * pf,
                                    lcrzo_uint8 nbiplaffectes,
                                    lcrzo_ipl iplt[])
{
  lcrzo_ipa ipa;
  int i;

  if (nbiplaffectes == 0)
    return (LCRZO_ERR_OK);

  for (i = 0; i < nbiplaffectes; i++) {
    fprintf((FILE *) pf, " |    ");
    lcrzo_er(lcrzo_ipa_init_ipl(iplt[2 * i], ipa));
    lcrzo_er(lcrzo_ipa_fprint(pf, "", ipa, " : "));
    lcrzo_er(lcrzo_uint32_fprint(pf, "", iplt[2 * i + 1], "%u", "\n"));
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_print_hdrlicmpline(LCRZOFILE * pf,
                                  lcrzo_hdrlicmp hdrlicmp);
int lcrzo_priv_print_hdrlicmpline(LCRZOFILE * pf,
                                  lcrzo_hdrlicmp hdrlicmp)
{
  lcrzo_string string;
  int i;

  lcrzo_er(lcrzo_string_initm_icmptypecode(hdrlicmp.type, hdrlicmp.code,
                                           &string));

  /* print line */
  fprintf((FILE *) pf, " ICMP_(%s)", string);
  for (i = strlen(string); i < 65; i++) {
    fprintf((FILE *) pf, "_");
  }
  fprintf((FILE *) pf, ".\n");
  fflush((FILE *) pf);

  lcrzo_er(lcrzo_string_free2(&string));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/



/*-------------------------------------------------------------*/
int lcrzo_printtype_fprint(LCRZOFILE * pf,
                           lcrzo_printtype printtype)
{
  lcrzo_int16 language;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*printf("[%04x]", printtype); */

  lcrzo_er(lcrzo_global_get_language(&language));
  switch (printtype & 0xFF) {
    case LCRZO_PRINTTYPE_NOTHING_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_NOTHING_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_NOTHING_entxt);
      break;
    case LCRZO_PRINTTYPE_SYNTH_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_SYNTH_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_SYNTH_entxt);
      break;
    case LCRZO_PRINTTYPE_TEXT_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_TEXT_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_TEXT_entxt);
      break;
    case LCRZO_PRINTTYPE_PTEXT_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_PTEXT_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_PTEXT_entxt);
      break;
    case LCRZO_PRINTTYPE_BASE64_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_BASE64_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_BASE64_entxt);
      break;
    case LCRZO_PRINTTYPE_ARRAY_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_ARRAY_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_ARRAY_entxt);
      break;
    case LCRZO_PRINTTYPE_DUMP_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_DUMP_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_DUMP_entxt);
      break;
    case LCRZO_PRINTTYPE_HEXA_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_HEXA_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_HEXA_entxt);
      break;
    case LCRZO_PRINTTYPE_MIXED_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_MIXED_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_MIXED_entxt);
      break;
    case LCRZO_PRINTTYPE_CMH_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_CMH_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_CMH_entxt);
      break;
    case LCRZO_PRINTTYPE_CHM_:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_CHM_frtxt);
      else
        fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_CHM_entxt);
      break;
    default:
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "%s", "Erreur : type d'affichage non reconnu");
      else
        fprintf((FILE *) pf, "%s", "Error : unknown printtype");
      fflush((FILE *) pf);
      return (LCRZO_ERR_PAUNKNOWNTYPE);
      break;
  }

  /*error verification */
  if ((printtype & 0xFF00) & (~(LCRZO_PRINTTYPE_C_ | LCRZO_PRINTTYPE_S_ |
                                LCRZO_PRINTTYPE_N_ | LCRZO_PRINTTYPE_I_))) {
    if (language == LCRZO_GLOBAL_LANG_FR)
      fprintf((FILE *) pf, "%s", "Erreur : type d'affichage non reconnu");
    else
      fprintf((FILE *) pf, "%s", "Error : unknown printtype");
    fflush((FILE *) pf);
    return (LCRZO_ERR_PAUNKNOWNTYPE);
  }

  if (printtype & LCRZO_PRINTTYPE_C_) {
    if (language == LCRZO_GLOBAL_LANG_FR)
      fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_C_frtxt);
    else
      fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_C_entxt);
  }
  if (printtype & LCRZO_PRINTTYPE_S_) {
    if (language == LCRZO_GLOBAL_LANG_FR)
      fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_S_frtxt);
    else
      fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_S_entxt);
  }
  if (printtype & LCRZO_PRINTTYPE_N_) {
    if (language == LCRZO_GLOBAL_LANG_FR)
      fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_N_frtxt);
    else
      fprintf((FILE *) pf, "%s", LCRZO_PRINTTYPE_N_entxt);
  }

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printtype_init_stdin(lcrzo_conststring message,
                               lcrzo_printtype defautprinttype,
                               lcrzo_printtype * pprinttype)
{
  lcrzo_printtype ttypaff[LCRZO_PRIV_PRINTTYPE_COUNT + 1];
  lcrzo_bool valeurdefauttrouvee;
  lcrzo_uint32 numeroprinttypechoisi, numeroprinttypedefaut;
  lcrzo_int32 count, i;
  lcrzo_int16 language;

  /*initialization */
  lcrzo_er(lcrzo_printtype_count(&count));
  valeurdefauttrouvee = 0;
  if (message != NULL)
    if (strlen(message))
      printf("%s\n", message);
  numeroprinttypedefaut = 0;    /*pour faire plaisir au compilateur */
  /*affichage des choix possibles */
  for (i = 1; i <= count; i++) {        /*affichage */
    lcrzo_er(lcrzo_int32_print(" ", i, "%2d", " - "));
    lcrzo_er(lcrzo_printtype_value_pos(i, &(ttypaff[i])));
    lcrzo_er(lcrzo_printtype_print(ttypaff[i]));
    printf("\n");
    /*a t'on trouve la valeur par defaut proposee ? */
    if (ttypaff[i] == defautprinttype) {
      valeurdefauttrouvee = 1;
      numeroprinttypedefaut = i;
    }
  }

  /*si la valeur par defaut n'a pas ete trouvee */
  if (!valeurdefauttrouvee) {
    return (LCRZO_ERR_PADEFAULTNOTINRANGE);
  }

  /*demande a entrer le choix voulu */
  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_uint32_init_stdin((language == LCRZO_GLOBAL_LANG_FR) ?
                                   "Entrez votre choix" : "Enter your choice",
                                   1, count, numeroprinttypedefaut,
                                   &numeroprinttypechoisi));
  if (pprinttype != NULL)
    *pprinttype = ttypaff[numeroprinttypechoisi];
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printtype_count(lcrzo_int32 * pcount)
{
  if (pcount != NULL)
    *pcount = LCRZO_PRIV_PRINTTYPE_COUNT;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printtype_value_pos(lcrzo_int32 pos,
                              lcrzo_printtype * pprinttype)
{
  lcrzo_int32 pospos, count;
  lcrzo_printtype printtype;

  lcrzo_er(lcrzo_printtype_count(&count));
  lcrzo_er(lcrzo_priv_manageelempos
           (count, pos, 0, NULL, &pospos, NULL, NULL));

  switch ((int)pospos) {
    case 1:
      printtype = LCRZO_PRINTTYPE_NOTHING;
      break;
    case 2:
      printtype = LCRZO_PRINTTYPE_SYNTH;
      break;
    case 3:
      printtype = LCRZO_PRINTTYPE_SYNTHN;
      break;
    case 4:
      printtype = LCRZO_PRINTTYPE_SYNTHC;
      break;
    case 5:
      printtype = LCRZO_PRINTTYPE_TEXT;
      break;
    case 6:
      printtype = LCRZO_PRINTTYPE_TEXTN;
      break;
    case 7:
      printtype = LCRZO_PRINTTYPE_TEXTC;
      break;
    case 8:
      printtype = LCRZO_PRINTTYPE_PTEXT;
      break;
    case 9:
      printtype = LCRZO_PRINTTYPE_PTEXTN;
      break;
    case 10:
      printtype = LCRZO_PRINTTYPE_PTEXTC;
      break;
    case 11:
      printtype = LCRZO_PRINTTYPE_BASE64;
      break;
    case 12:
      printtype = LCRZO_PRINTTYPE_BASE64N;
      break;
    case 13:
      printtype = LCRZO_PRINTTYPE_BASE64C;
      break;
    case 14:
      printtype = LCRZO_PRINTTYPE_ARRAY;
      break;
    case 15:
      printtype = LCRZO_PRINTTYPE_ARRAYC;
      break;
    case 16:
      printtype = LCRZO_PRINTTYPE_DUMP;
      break;
    case 17:
      printtype = LCRZO_PRINTTYPE_DUMPC;
      break;
    case 18:
      printtype = LCRZO_PRINTTYPE_HEXA;
      break;
    case 19:
      printtype = LCRZO_PRINTTYPE_HEXAN;
      break;
    case 20:
      printtype = LCRZO_PRINTTYPE_HEXAC;
      break;
    case 21:
      printtype = LCRZO_PRINTTYPE_HEXAS;
      break;
    case 22:
      printtype = LCRZO_PRINTTYPE_HEXACS;
      break;
    case 23:
      printtype = LCRZO_PRINTTYPE_MIXED;
      break;
    case 24:
      printtype = LCRZO_PRINTTYPE_MIXEDN;
      break;
    case 25:
      printtype = LCRZO_PRINTTYPE_MIXEDC;
      break;
    case 26:
      printtype = LCRZO_PRINTTYPE_MIXEDS;
      break;
    case 27:
      printtype = LCRZO_PRINTTYPE_MIXEDCS;
      break;
    case 28:
      printtype = LCRZO_PRINTTYPE_CMH;
      break;
    case 29:
      printtype = LCRZO_PRINTTYPE_CMHC;
      break;
    case 30:
      printtype = LCRZO_PRINTTYPE_CMHS;
      break;
    case 31:
      printtype = LCRZO_PRINTTYPE_CMHCS;
      break;
    case 32:
      printtype = LCRZO_PRINTTYPE_CHM;
      break;
    case 33:
      printtype = LCRZO_PRINTTYPE_CHMC;
      break;
    case 34:
      printtype = LCRZO_PRINTTYPE_CHMS;
      break;
    case 35:
      printtype = LCRZO_PRINTTYPE_CHMCS;
      break;
    default:
      return (LCRZO_ERR_POTOOBIG);
  }

  if (pprinttype != NULL)
    *pprinttype = printtype;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printtype_value_first(lcrzo_printtype * pprinttype)
{
  return (lcrzo_printtype_value_pos(+1, pprinttype));
}

/*-------------------------------------------------------------*/
int lcrzo_printtype_value_last(lcrzo_printtype * pprinttype)
{
  return (lcrzo_printtype_value_pos(-1, pprinttype));
}

/*-------------------------------------------------------------*/
int lcrzo_printtype_loop_range(lcrzo_int32 startpos,
                               lcrzo_int32 endpos,
                               int (*pfonc) (lcrzo_printtype printtype,
                                             lcrzo_ptr pinfos),
                               lcrzo_ptr pinfos)
{
  lcrzo_int32 count, suppos, infpos, i;
  lcrzo_printtype printtype;

  /*parameters verification */
  lcrzo_er(lcrzo_printtype_count(&count));
  if (pfonc == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (count == 0 && startpos == +1 && endpos == -1)
    return (LCRZO_ERR_OK);
  /*affectation des positions */
  lcrzo_er(lcrzo_priv_manageelemrange(count, startpos, endpos,
                                      NULL, &infpos, NULL, &suppos,
                                      NULL, NULL, NULL, NULL));

  /*verification des parametres et affectation des positions */
  if (suppos >= infpos) {
    for (i = infpos; i <= suppos; i++) {
      lcrzo_er(lcrzo_printtype_value_pos(i, &printtype));
      lcrzo_er(((*pfonc) (printtype, pinfos)));
    }
  } else {
    for (i = infpos; i >= suppos; i--) {
      lcrzo_er(lcrzo_printtype_value_pos(i, &printtype));
      lcrzo_er(((*pfonc) (printtype, pinfos)));
    }
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printtype_loop_all(int (*pfonc) (lcrzo_printtype printtype,
                                           lcrzo_ptr pinfos),
                             lcrzo_ptr pinfos)
{
  return (lcrzo_printtype_loop_range(+1, -1, pfonc, pinfos));
}


/*-------------------------------------------------------------*/
lcrzo_printprofile lcrzo_printprofile_returnstruct(lcrzo_printtype hdr,
                                                   lcrzo_printtype hdreth,
                                                   lcrzo_printtype hdrip,
                                                   lcrzo_printtype hdrudp,
                                                   lcrzo_printtype hdrtcp,
                                                   lcrzo_printtype hdricmp,
                                                   lcrzo_printtype hdrarp,
                                                   lcrzo_printtype data,
                                                   lcrzo_printtype dataeth,
                                                   lcrzo_printtype dataip,
                                                   lcrzo_printtype dataudp,
                                                   lcrzo_printtype datatcp,
                                                   lcrzo_printtype dataicmp)
{
  lcrzo_printprofile printprofile;

  printprofile.hdr = hdr;
  printprofile.hdreth = hdreth;
  printprofile.hdrip = hdrip;
  printprofile.hdrudp = hdrudp;
  printprofile.hdrtcp = hdrtcp;
  printprofile.hdricmp = hdricmp;
  printprofile.hdrarp = hdrarp;
  printprofile.data = data;
  printprofile.dataeth = dataeth;
  printprofile.dataip = dataip;
  printprofile.dataudp = dataudp;
  printprofile.datatcp = datatcp;
  printprofile.dataicmp = dataicmp;
  printprofile.lastused = LCRZO_PRINTTYPE_START;
  return (printprofile);
}

/*-------------------------------------------------------------*/
int lcrzo_printprofile_init(lcrzo_printtype hdr,
                            lcrzo_printtype hdreth,
                            lcrzo_printtype hdrip,
                            lcrzo_printtype hdrudp,
                            lcrzo_printtype hdrtcp,
                            lcrzo_printtype hdricmp,
                            lcrzo_printtype hdrarp,
                            lcrzo_printtype data,
                            lcrzo_printtype dataeth,
                            lcrzo_printtype dataip,
                            lcrzo_printtype dataudp,
                            lcrzo_printtype datatcp,
                            lcrzo_printtype dataicmp,
                            lcrzo_printprofile * pprintprofile)
{
  lcrzo_printprofile printprofile;

  printprofile.hdr = hdr;
  printprofile.hdreth = hdreth;
  printprofile.hdrip = hdrip;
  printprofile.hdrudp = hdrudp;
  printprofile.hdrtcp = hdrtcp;
  printprofile.hdricmp = hdricmp;
  printprofile.hdrarp = hdrarp;
  printprofile.data = data;
  printprofile.dataeth = dataeth;
  printprofile.dataip = dataip;
  printprofile.dataudp = dataudp;
  printprofile.datatcp = datatcp;
  printprofile.dataicmp = dataicmp;
  printprofile.lastused = LCRZO_PRINTTYPE_START;
  if (pprintprofile != NULL)
    *pprintprofile = printprofile;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printprofile_fprint(LCRZOFILE * pf,
                              lcrzo_printprofile printprofile)
{
  char msg[20];
  lcrzo_int32 count, i;
  lcrzo_printprofile pp;
  lcrzo_int16 language;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*initialization */
  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_printprofile_count(&count));
  /*affichage */
  printprofile.lastused = LCRZO_PRINTTYPE_START;    /*to be able to compare */
  for (i = 1; i <= count; i++) {
    lcrzo_er(lcrzo_printprofile_value_pos(i, &pp));
    /*a t'on trouve la valeur par defaut proposee ? */
    if (!memcmp(&pp, &printprofile, sizeof(lcrzo_printprofile))) {
      switch ((int)i) {
        case 1:
          if (language == LCRZO_GLOBAL_LANG_FR)
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_SYNTHSYNTH_frtxt);
          else
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_SYNTHSYNTH_entxt);
          break;
        case 2:
          if (language == LCRZO_GLOBAL_LANG_FR)
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_SYNTHESYNTH_frtxt);
          else
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_SYNTHESYNTH_entxt);
          break;
        case 3:
          if (language == LCRZO_GLOBAL_LANG_FR)
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_ARRAYARRAY_frtxt);
          else
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_ARRAYARRAY_entxt);
          break;
        case 4:
          if (language == LCRZO_GLOBAL_LANG_FR)
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_ARRAYDUMP_frtxt);
          else
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_ARRAYDUMP_entxt);
          break;
        case 5:
          if (language == LCRZO_GLOBAL_LANG_FR)
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_ARRAYCHMS_frtxt);
          else
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_ARRAYCHMS_entxt);
          break;
        case 6:
          if (language == LCRZO_GLOBAL_LANG_FR)
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_HEXANHEXAS_frtxt);
          else
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_HEXANHEXAS_entxt);
          break;
        case 7:
          if (language == LCRZO_GLOBAL_LANG_FR)
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_HEXANDUMP_frtxt);
          else
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_HEXANDUMP_entxt);
          break;
        case 8:
          if (language == LCRZO_GLOBAL_LANG_FR)
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_HEXANCHMS_frtxt);
          else
            fprintf((FILE *) pf, "%s", LCRZO_PRINTPROFILE_HEXANCHMS_entxt);
          break;
        default:
          return (lcrzo_ieuv("lcrzo_printprofile_fprint", "i"));
      }
      fflush((FILE *) pf);
      return (LCRZO_ERR_OK);
    }
  }

  /*if its not standard, print detailed information */
  if (language == LCRZO_GLOBAL_LANG_FR) {
    lcrzo_er(lcrzo_string_init_text("Entete", 19, msg));
  } else {
    lcrzo_er(lcrzo_string_init_text("Header", 19, msg));
  }
  fprintf((FILE *) pf, "%ss : ", msg);
  lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.hdr));
  if (printprofile.hdr != printprofile.hdreth) {
    fprintf((FILE *) pf, "\n%s ETH  : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.hdreth));
  }
  if (printprofile.hdr != printprofile.hdrip) {
    fprintf((FILE *) pf, "\n%s IP   : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.hdrip));
  }
  if (printprofile.hdr != printprofile.hdrudp) {
    fprintf((FILE *) pf, "\n%s UDP  : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.hdrudp));
  }
  if (printprofile.hdr != printprofile.hdrtcp) {
    fprintf((FILE *) pf, "\n%s TCP  : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.hdrtcp));
  }
  if (printprofile.hdr != printprofile.hdricmp) {
    fprintf((FILE *) pf, "\n%s ICMP : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.hdricmp));
  }
  if (printprofile.hdr != printprofile.hdrarp) {
    fprintf((FILE *) pf, "\n%s ARP  : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.hdrarp));
  }
  if (language == LCRZO_GLOBAL_LANG_FR) {
    lcrzo_er(lcrzo_string_init_text("Donnees", 19, msg));
  } else {
    lcrzo_er(lcrzo_string_init_text("Data", 19, msg));
  }
  fprintf((FILE *) pf, "\n%s : ", msg);
  lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.data));
  if (printprofile.data != printprofile.dataeth) {
    fprintf((FILE *) pf, "\n%s ETH  : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.dataeth));
  }
  if (printprofile.data != printprofile.dataip) {
    fprintf((FILE *) pf, "\n%s IP   : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.dataip));
  }
  if (printprofile.data != printprofile.dataudp) {
    fprintf((FILE *) pf, "\n%s UDP  : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.dataudp));
  }
  if (printprofile.data != printprofile.datatcp) {
    fprintf((FILE *) pf, "\n%s TCP  : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.datatcp));
  }
  if (printprofile.data != printprofile.dataicmp) {
    fprintf((FILE *) pf, "\n%s ICMP : ", msg);
    lcrzo_er(lcrzo_printtype_fprint(pf, printprofile.dataicmp));
  }
  fprintf((FILE *) pf, "\n");

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printprofile_init_stdin(lcrzo_conststring message,
                                  lcrzo_printprofile defaultprintprofile,
                                  lcrzo_printprofile * ppp)
{
  lcrzo_printprofile tpp[LCRZO_PRIV_PRINTPROFILE_COUNT + 1], printprofile;
  lcrzo_bool valeurdefauttrouvee;
  lcrzo_uint32 numeroprintprofilechoisi, numeroprintprofiledefaut;
  lcrzo_int32 count, i;
  lcrzo_int16 language;
  char c;

  /*initialization */
  lcrzo_er(lcrzo_printprofile_count(&count));
  valeurdefauttrouvee = 0;
  if (message != NULL)
    if (strlen(message))
      printf("%s\n", message);
  numeroprintprofiledefaut = 0; /*pour faire plaisir au compilateur */

  /*affichage des choix possibles */
  defaultprintprofile.lastused = LCRZO_PRINTTYPE_START;
  for (i = 1; i <= count; i++) {
    /*affichage */
    lcrzo_er(lcrzo_int32_print(" ", i, "%2d", " - "));
    lcrzo_er(lcrzo_printprofile_value_pos(i, &(tpp[i])));
    lcrzo_er(lcrzo_printprofile_print(tpp[i]));
    printf("\n");
    /*a t'on trouve la valeur par defaut proposee ? */
    if (!memcmp(&(tpp[i]), &defaultprintprofile, sizeof(lcrzo_printprofile))) {
      valeurdefauttrouvee = 1;
      numeroprintprofiledefaut = i;
    }
  }

  /*si la valeur par defaut n'a pas ete trouvee */
  if (!valeurdefauttrouvee) {
    return (LCRZO_ERR_PADEFAULTNOTINRANGE);
  }

  /*demande a entrer le choix voulu */
  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_int32_print(" ", count + 1, "%2d", " - "));
  if (language == LCRZO_GLOBAL_LANG_FR)
    printf("profil personnalise\n");
  else
    printf("personalized profile\n");
  lcrzo_er(lcrzo_uint32_init_stdin((language == LCRZO_GLOBAL_LANG_FR) ?
                                   "Choisissez le profil voulu" :
                                   "Choose the profile",
                                   1, count + 1, numeroprintprofiledefaut,
                                   &numeroprintprofilechoisi));

  /*standard profile */
  if ((lcrzo_int32) numeroprintprofilechoisi <= count) {
    if (ppp != NULL)
      *ppp = tpp[numeroprintprofilechoisi];
    return (LCRZO_ERR_OK);
  }

  /*personalized profile */
  if (language == LCRZO_GLOBAL_LANG_FR) {
    lcrzo_er(lcrzo_printtype_init_stdin
             ("Choisissez le type d'affichage pour les entetes",
              LCRZO_PRINTTYPE_ARRAY, &printprofile.hdr));
    lcrzo_er(lcrzo_printtype_init_stdin
             ("Choisissez le type d'affichage pour les donnees",
              LCRZO_PRINTTYPE_DUMP, &printprofile.data));
    lcrzo_er(lcrzo_stdin_char
             ("Voulez-vous affiner la definition du profil",
              (const lcrzo_uint8 *)"oOnN", 'n', (lcrzo_uint8 *) & c));
  } else {
    lcrzo_er(lcrzo_printtype_init_stdin("Choose the printtype for headers",
                                        LCRZO_PRINTTYPE_ARRAY,
                                        &printprofile.hdr));
    lcrzo_er(lcrzo_printtype_init_stdin
             ("Choose the printtype for data", LCRZO_PRINTTYPE_DUMP,
              &printprofile.data));
    lcrzo_er(lcrzo_stdin_char
             ("Do you want to refine the profile",
              (const lcrzo_uint8 *)"yYnN", 'n', (lcrzo_uint8 *) & c));
  }
  if (c == 'o' || c == 'O' || c == 'y' || c == 'Y') {
    if (language == LCRZO_GLOBAL_LANG_FR) {
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les entetes ETH",
                printprofile.hdr, &printprofile.hdreth));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les entetes IP",
                printprofile.hdr, &printprofile.hdrip));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les entetes UDP",
                printprofile.hdr, &printprofile.hdrudp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les entetes TCP",
                printprofile.hdr, &printprofile.hdrtcp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les entetes ICMP",
                printprofile.hdr, &printprofile.hdricmp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les entetes ARP",
                printprofile.hdr, &printprofile.hdrarp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les donnees ETH",
                printprofile.data, &printprofile.dataeth));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les donnees IP",
                printprofile.data, &printprofile.dataip));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les donnees UDP",
                printprofile.data, &printprofile.dataudp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les donnees TCP",
                printprofile.data, &printprofile.datatcp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choisissez le type d'affichage pour les donnees ICMP",
                printprofile.data, &printprofile.dataicmp));
    } else {
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for headers ETH", printprofile.hdr,
                &printprofile.hdreth));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for headers IP", printprofile.hdr,
                &printprofile.hdrip));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for headers UDP", printprofile.hdr,
                &printprofile.hdrudp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for headers TCP", printprofile.hdr,
                &printprofile.hdrtcp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for headers ICMP", printprofile.hdr,
                &printprofile.hdricmp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for headers ARP", printprofile.hdr,
                &printprofile.hdrarp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for data ETH", printprofile.data,
                &printprofile.dataeth));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for data IP", printprofile.data,
                &printprofile.dataip));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for data UDP", printprofile.data,
                &printprofile.dataudp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for data TCP", printprofile.data,
                &printprofile.datatcp));
      lcrzo_er(lcrzo_printtype_init_stdin
               ("Choose the printtype for data ICMP", printprofile.data,
                &printprofile.dataicmp));
    }
  } else {
    printprofile.hdreth = printprofile.hdr;
    printprofile.hdrip = printprofile.hdr;
    printprofile.hdrudp = printprofile.hdr;
    printprofile.hdrtcp = printprofile.hdr;
    printprofile.hdricmp = printprofile.hdr;
    printprofile.hdrarp = printprofile.hdr;
    printprofile.dataeth = printprofile.data;
    printprofile.dataip = printprofile.data;
    printprofile.dataudp = printprofile.data;
    printprofile.datatcp = printprofile.data;
    printprofile.dataicmp = printprofile.data;
  }
  printprofile.lastused = LCRZO_PRINTTYPE_START;
  if (ppp != NULL)
    *ppp = printprofile;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printprofile_count(lcrzo_int32 * pcount)
{
  if (pcount != NULL)
    *pcount = LCRZO_PRIV_PRINTPROFILE_COUNT;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printprofile_value_pos(lcrzo_int32 pos,
                                 lcrzo_printprofile * pprintprofile)
{
  lcrzo_int32 pospos, count;
  lcrzo_printprofile printprofile;

  lcrzo_er(lcrzo_printprofile_count(&count));
  lcrzo_er(lcrzo_priv_manageelempos
           (count, pos, 0, NULL, &pospos, NULL, NULL));

  switch ((int)pospos) {
    case 1:
      printprofile = LCRZO_PRINTPROFILE_SYNTHSYNTH;
      break;
    case 2:
      printprofile = LCRZO_PRINTPROFILE_SYNTHESYNTH;
      break;
    case 3:
      printprofile = LCRZO_PRINTPROFILE_ARRAYARRAY;
      break;
    case 4:
      printprofile = LCRZO_PRINTPROFILE_ARRAYDUMP;
      break;
    case 5:
      printprofile = LCRZO_PRINTPROFILE_ARRAYCHMS;
      break;
    case 6:
      printprofile = LCRZO_PRINTPROFILE_HEXANHEXAS;
      break;
    case 7:
      printprofile = LCRZO_PRINTPROFILE_HEXANDUMP;
      break;
    case 8:
      printprofile = LCRZO_PRINTPROFILE_HEXANCHMS;
      break;
    default:
      return (LCRZO_ERR_POTOOBIG);
  }

  if (pprintprofile != NULL)
    *pprintprofile = printprofile;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printprofile_value_first(lcrzo_printprofile * pprintprofile)
{
  return (lcrzo_printprofile_value_pos(+1, pprintprofile));
}

/*-------------------------------------------------------------*/
int lcrzo_printprofile_value_last(lcrzo_printprofile * pprintprofile)
{
  return (lcrzo_printprofile_value_pos(-1, pprintprofile));
}

/*-------------------------------------------------------------*/
int lcrzo_printprofile_loop_range(lcrzo_int32 startpos,
                                  lcrzo_int32 endpos,
                                  int (*pfonc) (lcrzo_printprofile
                                                printprofile,
                                                lcrzo_ptr pinfos),
                                  lcrzo_ptr pinfos)
{
  lcrzo_int32 count, suppos, infpos, i;
  lcrzo_printprofile printprofile;

  /*parameters verification */
  if (pfonc == NULL)
    return (LCRZO_ERR_PANULLPTR);
  /*verification des parametres et affectation des positions */
  lcrzo_er(lcrzo_printprofile_count(&count));
  if (count == 0 && startpos == +1 && endpos == -1)
    return (LCRZO_ERR_OK);
  lcrzo_er(lcrzo_priv_manageelemrange(count, startpos, endpos,
                                      NULL, &infpos, NULL, &suppos,
                                      NULL, NULL, NULL, NULL));

  /*verification des parametres et affectation des positions */
  if (suppos >= infpos) {
    for (i = infpos; i <= suppos; i++) {
      lcrzo_er(lcrzo_printprofile_value_pos(i, &printprofile));
      lcrzo_er(((*pfonc) (printprofile, pinfos)));
    }
  } else {
    for (i = infpos; i >= suppos; i--) {
      lcrzo_er(lcrzo_printprofile_value_pos(i, &printprofile));
      lcrzo_er(((*pfonc) (printprofile, pinfos)));
    }
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_printprofile_loop_all(int (*pfonc) (lcrzo_printprofile printprofile,
                                              lcrzo_ptr pinfos),
                                lcrzo_ptr pinfos)
{
  return (lcrzo_printprofile_loop_range(+1, -1, pfonc, pinfos));
}


/*-------------------------------------------------------------*/
int lcrzo_hs_fprint(LCRZOFILE * pf,
                    lcrzo_conststring deb,
                    lcrzo_consths hs,
                    lcrzo_conststring fin)
{
  lcrzo_hs hs_copie;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  lcrzo_er(lcrzo_string_init_coretext(hs, LCRZO_HS_MAXBYTES, hs_copie));
  if (deb != NULL)
    fprintf((FILE *) pf, "%s", deb);
  fprintf((FILE *) pf, "%s", hs_copie);
  if (fin != NULL)
    fprintf((FILE *) pf, "%s", fin);
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

int lcrzo_ips_fprint(LCRZOFILE * pf,
                     lcrzo_conststring deb,
                     lcrzo_constips ips,
                     lcrzo_conststring fin)
{
  lcrzo_ips ips_copie;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_string_init_coretext(ips, LCRZO_IPS_MAXBYTES, ips_copie));
  if (deb != NULL)
    fprintf((FILE *) pf, "%s", deb);
  fprintf((FILE *) pf, "%s", ips_copie);
  if (fin != NULL)
    fprintf((FILE *) pf, "%s", fin);
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

int lcrzo_ipa_fprint(LCRZOFILE * pf,
                     lcrzo_conststring deb,
                     lcrzo_constipa ipa,
                     lcrzo_conststring fin)
{
  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (ipa == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (deb != NULL)
    fprintf((FILE *) pf, "%s", deb);
  fprintf((FILE *) pf, "%d.%d.%d.%d", (lcrzo_uint8) ipa[0],
          (lcrzo_uint8) ipa[1], (lcrzo_uint8) ipa[2], (lcrzo_uint8) ipa[3]);
  if (fin != NULL)
    fprintf((FILE *) pf, "%s", fin);
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

int lcrzo_ipl_fprint(LCRZOFILE * pf,
                     lcrzo_conststring deb,
                     lcrzo_ipl ipl,
                     lcrzo_conststring fin)
{
  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (deb != NULL)
    fprintf((FILE *) pf, "%s", deb);
  fprintf((FILE *) pf, "%d.%d.%d.%d",
          (lcrzo_uint8) (ipl >> 24),
          (lcrzo_uint8) ((ipl >> 16) & 0xFF),
          (lcrzo_uint8) ((ipl >> 8) & 0xFF), (lcrzo_uint8) (ipl & 0xFF));
  if (fin != NULL)
    fprintf((FILE *) pf, "%s", fin);
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

int lcrzo_eths_fprint(LCRZOFILE * pf,
                      lcrzo_conststring deb,
                      lcrzo_consteths eths,
                      lcrzo_conststring fin)
{
  lcrzo_hs eths_copie;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_string_init_coretext(eths, LCRZO_HS_MAXBYTES, eths_copie));
  if (deb != NULL)
    fprintf((FILE *) pf, "%s", deb);
  fprintf((FILE *) pf, "%s", eths_copie);
  if (fin != NULL)
    fprintf((FILE *) pf, "%s", fin);
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

int lcrzo_etha_fprint(LCRZOFILE * pf,
                      lcrzo_conststring deb,
                      lcrzo_constetha etha,
                      lcrzo_conststring fin)
{
  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (etha == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (deb != NULL)
    fprintf((FILE *) pf, "%s", deb);
  fprintf((FILE *) pf, "%02X:%02X:%02X:%02X:%02X:%02X", (lcrzo_uint8) etha[0],
          (lcrzo_uint8) etha[1], (lcrzo_uint8) etha[2],
          (lcrzo_uint8) etha[3], (lcrzo_uint8) etha[4],
          (lcrzo_uint8) etha[5]);
  if (fin != NULL)
    fprintf((FILE *) pf, "%s", fin);
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_device_fprint(LCRZOFILE * pf,
                        lcrzo_conststring deb,
                        lcrzo_constdevice device,
                        lcrzo_conststring fin)
{
  lcrzo_device device_copie;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_string_init_coretext(device,
                                      LCRZO_DEVICE_MAXBYTES, device_copie));
  if (deb != NULL)
    fprintf((FILE *) pf, "%s", deb);
  fprintf((FILE *) pf, "%s", device_copie);
  if (fin != NULL)
    fprintf((FILE *) pf, "%s", fin);
  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hs_fprint_infos(LCRZOFILE * pf,
                          lcrzo_consths hs)
{
  lcrzo_ipl ipl;
  lcrzo_eths eths;
  lcrzo_int16 language;
  int retour;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_hs_fprint(pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                           "nom de machine :    " : "computer name :     ",
                           hs, "\n"));

  retour = lcrzo_ipl_init_hs(hs, &ipl);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ip :       " : " ip address :       ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_ipl_fprint(pf, "", ipl, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  if (retour == LCRZO_ERR_OK)
    retour = lcrzo_eths_init_ipl(ipl, eths);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ethernet : " : " ethernet address : ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_eths_fprint(pf, "", eths, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_ips_fprint_infos(LCRZOFILE * pf,
                           lcrzo_constips ips)
{
  lcrzo_hs hs;
  lcrzo_ipl ipl;
  lcrzo_eths eths;
  lcrzo_int16 language;
  int retour;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_ips_fprint(pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                            "adresse ip :        " : "ip address :        ",
                            ips, "\n"));

  retour = lcrzo_hs_init_ips(ips, hs);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " nom de machine :   " : " computer name :    ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_hs_fprint(pf, "", hs, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolu\n" : "unresolved\n");
  }

  retour = lcrzo_ipl_init_ips(ips, &ipl);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ip :       " : " ip address :       ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_ipl_fprint(pf, "", ipl, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  retour = lcrzo_eths_init_ips(ips, eths);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ethernet : " : " ethernet address : ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_eths_fprint(pf, "", eths, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_ipa_fprint_infos(LCRZOFILE * pf,
                           lcrzo_constipa ipa)
{
  lcrzo_hs hs;
  lcrzo_ipl ipl;
  lcrzo_eths eths;
  lcrzo_int16 language;
  int retour;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_ipa_fprint(pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                            "adresse ip :        " : "ip address :        ",
                            ipa, "\n"));

  retour = lcrzo_hs_init_ipa(ipa, hs);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " nom de machine :   " : " computer name :    ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_hs_fprint(pf, "", hs, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolu\n" : "unresolved\n");
  }

  retour = lcrzo_ipl_init_ipa(ipa, &ipl);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ip :       " : " ip address :       ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_ipl_fprint(pf, "", ipl, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  retour = lcrzo_eths_init_ipa(ipa, eths);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ethernet : " : " ethernet address : ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_eths_fprint(pf, "", eths, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_ipl_fprint_infos(LCRZOFILE * pf,
                           lcrzo_ipl ipl)
{
  lcrzo_hs hs;
  lcrzo_eths eths;
  lcrzo_int16 language;
  int retour;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_ipl_fprint(pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                            "adresse ip :        " : "ip address :        ",
                            ipl, "\n"));

  retour = lcrzo_hs_init_ipl(ipl, hs);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " nom de machine :   " : " computer name :    ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_hs_fprint(pf, "", hs, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolu\n" : "unresolved\n");
  }

  retour = lcrzo_eths_init_ipl(ipl, eths);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ethernet : " : " ethernet address : ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_eths_fprint(pf, "", eths, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_eths_fprint_infos(LCRZOFILE * pf,
                            lcrzo_consteths eths)
{
  lcrzo_hs hs;
  lcrzo_ipl ipl;
  lcrzo_int16 language;
  int retour1, retour2;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_eths_fprint(pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                             "adresse ethernet :  " : "ethernet address :  ",
                             eths, "\n"));

  retour1 = lcrzo_ipl_init_eths(eths, &ipl);
  if (retour1 == LCRZO_ERR_OK)
    retour2 = lcrzo_hs_init_ipl(ipl, hs);
  else
    retour2 = retour1;

  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " nom de machine :   " : " computer name :    ");
  if (retour2 == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_hs_fprint(pf, "", hs, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolu\n" : "unresolved\n");
  }

  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ip :       " : " ip address :       ");
  if (retour1 == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_ipl_fprint(pf, "", ipl, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_etha_fprint_infos(LCRZOFILE * pf,
                            lcrzo_constetha etha)
{
  lcrzo_hs hs;
  lcrzo_ipl ipl;
  lcrzo_int16 language;
  int retour1, retour2;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_etha_fprint(pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                             "adresse ethernet :  " : "ethernet address :  ",
                             etha, "\n"));

  retour1 = lcrzo_ipl_init_etha(etha, &ipl);
  if (retour1 == LCRZO_ERR_OK)
    retour2 = lcrzo_hs_init_ipl(ipl, hs);
  else
    retour2 = retour1;


  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " nom de machine :   " : " computer name :    ");
  if (retour2 == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_hs_fprint(pf, "", hs, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolu\n" : "unresolved\n");
  }

  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ip :       " : " ip address :       ");
  if (retour2 == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_ipl_fprint(pf, "", ipl, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_device_fprint_infos(LCRZOFILE * pf,
                              lcrzo_constdevice device)
{
  lcrzo_hs hs;
  lcrzo_ipl ipl, mask;
  lcrzo_etha etha;
  lcrzo_int16 language;
  int retour;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_global_get_language(&language));
  lcrzo_er(lcrzo_device_fprint(pf, "device :            ", device, "\n"));

  retour = lcrzo_hs_init_device(device, hs);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " nom de machine :   " : " computer name :    ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_hs_fprint(pf, "", hs, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolu\n" : "unresolved\n");
  }

  retour = lcrzo_ipl_init_device(device, &ipl, &mask);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ip :       " : " ip address :       ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_ipl_fprint(pf, "", ipl, ""));
    lcrzo_er(lcrzo_ipl_fprint(pf, "/", mask, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  retour = lcrzo_etha_init_device(device, etha);
  fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
          " adresse ethernet : " : " ethernet address : ");
  if (retour == LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_etha_fprint(pf, "", etha, "\n"));
  } else {
    lcrzo_er(lcrzo_err_purge());
    fprintf((FILE *) pf, "%s", (language == LCRZO_GLOBAL_LANG_FR) ?
            "non resolue\n" : "unresolved\n");
  }

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_data_fprint(LCRZOFILE * pf,
                      lcrzo_constdata data,
                      lcrzo_int32 datasize,
                      lcrzo_printtype printtype)
{
  lcrzo_int32 i, nbblocscomplets, nbcaracrestant;
  lcrzo_uint8 c;
  lcrzo_int16 language;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (data == NULL && datasize)
    return (LCRZO_ERR_PANULLPTRSIZE);

  switch (printtype & (0x00FF | LCRZO_PRINTTYPE_S_)) {
    case LCRZO_PRINTTYPE_NOTHING_:
      break;
    case LCRZO_PRINTTYPE_SYNTH_:
      lcrzo_er(lcrzo_global_get_language(&language));
      lcrzo_er(lcrzo_int32_fprint(pf, "", datasize, "%d",
                                  (language ==
                                   LCRZO_GLOBAL_LANG_FR) ? " octets" :
                                  " bytes"));
      break;
    case LCRZO_PRINTTYPE_TEXT_:
      for (i = 0; i < datasize; i++)
        fputc(data[i], (FILE *) pf);
      break;
    case LCRZO_PRINTTYPE_PTEXT_:
      for (i = 0; i < datasize; i++) {
        c = data[i];
        fputc(lcrzo_priv_char_printall(c), (FILE *) pf);
      }
      break;
    case LCRZO_PRINTTYPE_BASE64_:
      nbblocscomplets = datasize / 3;
      nbcaracrestant = datasize - 3 * nbblocscomplets;
      /*traite les blocs complets */
      for (i = 0; i < nbblocscomplets; i++) {
        lcrzo_er(lcrzo_priv_affcarbase64(pf, data[3 * i] >> 2));
        lcrzo_er(lcrzo_priv_affcarbase64(pf, ((data[3 * i] & 3) << 4) |
                                         (data[3 * i + 1] >> 4)));
        lcrzo_er(lcrzo_priv_affcarbase64(pf, ((data[3 * i + 1] & 0xF) << 2) |
                                         (data[3 * i + 2] >> 6)));
        lcrzo_er(lcrzo_priv_affcarbase64(pf, data[3 * i + 2] & 0x3F));
      }
      /*traite selon le nombre de caracteres restants */
      if (nbcaracrestant == 1) {
        lcrzo_er(lcrzo_priv_affcarbase64(pf, data[3 * i] >> 2));
        lcrzo_er(lcrzo_priv_affcarbase64(pf, ((data[3 * i] & 3) << 4)));
        fprintf((FILE *) pf, "==");
      } else if (nbcaracrestant == 2) {
        lcrzo_er(lcrzo_priv_affcarbase64(pf, data[3 * i] >> 2));
        lcrzo_er(lcrzo_priv_affcarbase64(pf, ((data[3 * i] & 3) << 4) |
                                         (data[3 * i + 1] >> 4)));
        lcrzo_er(lcrzo_priv_affcarbase64(pf, ((data[3 * i + 1] & 0xF) << 2)));
        fprintf((FILE *) pf, "=");
      }
      break;
    case LCRZO_PRINTTYPE_ARRAY_:
      if (datasize) {
        fprintf((FILE *) pf, " .___________________________________");
        fprintf((FILE *) pf, "____________________________________.\n");
      }
      for (i = 0; i < datasize / 4; i++) {
        fprintf((FILE *) pf, " |");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i + 1],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i + 2],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i + 3],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|\n");
      }
      if (datasize % 4 == 3) {
        fprintf((FILE *) pf, " |");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i + 1],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i + 2],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|\n");
      } else if (datasize % 4 == 2) {
        fprintf((FILE *) pf, " |");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i + 1],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|\n");
      } else if (datasize % 4 == 1) {
        fprintf((FILE *) pf, " |");
        lcrzo_er(lcrzo_priv_ecrit_tab08(pf, data[4 * i],
                                        LCRZO_PRIV_HEXADECCAR));
        fprintf((FILE *) pf, "|\n");
      }
      break;
    case LCRZO_PRINTTYPE_DUMP_:
      lcrzo_er(lcrzo_priv_ecritpacket_datadump(pf, data, datasize));
      break;
    case LCRZO_PRINTTYPE_HEXA_:
      for (i = 0; i < datasize; i++)
        fprintf((FILE *) pf, "%02X", data[i]);
      break;
    case LCRZO_PRINTTYPE_HEXAS & (~LCRZO_PRINTTYPE_I_):
      for (i = 0; i < datasize;) {
        fprintf((FILE *) pf, " %02X", data[i]);
        i++;
        if (!(i % 24))
          fprintf((FILE *) pf, "\n");
      }
      if (i % 24)
        fprintf((FILE *) pf, "\n");
      break;
    case LCRZO_PRINTTYPE_MIXEDS & (~LCRZO_PRINTTYPE_I_):
    case LCRZO_PRINTTYPE_CMHS & (~LCRZO_PRINTTYPE_I_):
    case LCRZO_PRINTTYPE_CHMS & (~LCRZO_PRINTTYPE_I_):
      lcrzo_er(lcrzo_priv_ecritpacket_datahexmixs(pf, data, datasize,
                                                  printtype));
      break;
    case LCRZO_PRINTTYPE_MIXED_:
      lcrzo_er(lcrzo_priv_ecritpacket_datamixed(pf, data, datasize));
      break;
    case LCRZO_PRINTTYPE_CMH_:
      fprintf((FILE *) pf, "# ");
      lcrzo_er(lcrzo_priv_ecritpacket_datamixed(pf, data, datasize));
      fprintf((FILE *) pf, "\n  ");
      for (i = 0; i < datasize; i++)
        fprintf((FILE *) pf, "%02X", data[i]);
      fprintf((FILE *) pf, "\n");
      break;
    case LCRZO_PRINTTYPE_CHM_:
      fprintf((FILE *) pf, "# ");
      for (i = 0; i < datasize; i++)
        fprintf((FILE *) pf, "%02X", data[i]);
      fprintf((FILE *) pf, "\n ");
      lcrzo_er(lcrzo_priv_ecritpacket_datamixed(pf, data, datasize));
      fprintf((FILE *) pf, "\n");
      break;
    default:
      return (lcrzo_ieuv("lcrzo_data_fprint",
                         "printtype & ( 0x00FF | LCRZO_PRINTTYPE_S_ )"));
      break;
  }

  /*eventually print a newline */
  lcrzo_er(lcrzo_priv_printtype_printnewline(pf, printtype));

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrleth_fprint(LCRZOFILE * pf,
                         lcrzo_hdrleth hdrleth,
                         lcrzo_printtype printtype)
{
  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*affichage eventuel du commentaire */
  lcrzo_er(lcrzo_priv_printtype_printcomment(pf, printtype, "Ethernet header",
                                             "Entete Ethernet"));

  /*affichage de l'entete */
  switch (printtype & 0xFF) {
    case LCRZO_PRINTTYPE_NOTHING_:
      break;
    case LCRZO_PRINTTYPE_SYNTH_:
      fprintf((FILE *) pf, "%02x:%02x:%02x:%02x:%02x:%02x",
              (lcrzo_uint8) hdrleth.src[0], (lcrzo_uint8) hdrleth.src[1],
              (lcrzo_uint8) hdrleth.src[2], (lcrzo_uint8) hdrleth.src[3],
              (lcrzo_uint8) hdrleth.src[4], (lcrzo_uint8) hdrleth.src[5]);
      fprintf((FILE *) pf, "->%02x:%02x:%02x:%02x:%02x:%02x",
              (lcrzo_uint8) hdrleth.dst[0], (lcrzo_uint8) hdrleth.dst[1],
              (lcrzo_uint8) hdrleth.dst[2], (lcrzo_uint8) hdrleth.dst[3],
              (lcrzo_uint8) hdrleth.dst[4], (lcrzo_uint8) hdrleth.dst[5]);
      break;
    case LCRZO_PRINTTYPE_TEXT_:
    case LCRZO_PRINTTYPE_PTEXT_:
    case LCRZO_PRINTTYPE_BASE64_:
    case LCRZO_PRINTTYPE_DUMP_:
    case LCRZO_PRINTTYPE_HEXA_:
    case LCRZO_PRINTTYPE_MIXED_:
    case LCRZO_PRINTTYPE_CMH_:
    case LCRZO_PRINTTYPE_CHM_:
      {
        lcrzo_hdrpeth2 hdrpeth2;

        lcrzo_er(lcrzo_hdrpeth2_init_hdrleth(hdrleth, hdrpeth2));
        lcrzo_er(lcrzo_data_fprint
                 (pf, hdrpeth2, sizeof(hdrpeth2), printtype));
        return (LCRZO_ERR_OK);  /*return here to not print newline twice */
      }
      break;
    case LCRZO_PRINTTYPE_ARRAY_:
      fprintf((FILE *) pf, " ETH________________________________________");
      fprintf((FILE *) pf, "_____________________________.\n");
      fprintf((FILE *) pf, " | %02x:%02x:%02x:%02x:%02x:%02x",
              (lcrzo_uint8) hdrleth.src[0], (lcrzo_uint8) hdrleth.src[1],
              (lcrzo_uint8) hdrleth.src[2], (lcrzo_uint8) hdrleth.src[3],
              (lcrzo_uint8) hdrleth.src[4], (lcrzo_uint8) hdrleth.src[5]);
      fprintf((FILE *) pf, " vers %02x:%02x:%02x:%02x:%02x:%02x",
              (lcrzo_uint8) hdrleth.dst[0], (lcrzo_uint8) hdrleth.dst[1],
              (lcrzo_uint8) hdrleth.dst[2], (lcrzo_uint8) hdrleth.dst[3],
              (lcrzo_uint8) hdrleth.dst[4], (lcrzo_uint8) hdrleth.dst[5]);
      fprintf((FILE *) pf, "         type : 0x%04x        |\n", hdrleth.type);
      fprintf((FILE *) pf,
              " |_______________________________________________");
      fprintf((FILE *) pf, "________________________|\n");
      break;
    default:
      return (lcrzo_ieuv("lcrzo_hdrleth_fprint", "printtype&0xFF"));
  }

  /*eventually print a newline */
  lcrzo_er(lcrzo_priv_printtype_printnewline(pf, printtype));

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlip_fprint(LCRZOFILE * pf,
                        lcrzo_hdrlip hdrlip,
                        lcrzo_printtype printtype)
{
  lcrzo_ips ips;
  char tab[20];                 /*va servir a tabuler les adresses ip */
  int posgauche;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*affichage eventuel du commentaire */
  lcrzo_er(lcrzo_priv_printtype_printcomment
           (pf, printtype, "IP header", "Entete IP"));

  /*affichage de l'entete */
  switch (printtype & 0xFF) {
    case LCRZO_PRINTTYPE_NOTHING_:
      break;
    case LCRZO_PRINTTYPE_SYNTH_:
      lcrzo_er(lcrzo_ips_init_ipl(hdrlip.saddr, ips));
      fprintf((FILE *) pf, "%s", ips);
      lcrzo_er(lcrzo_ips_init_ipl(hdrlip.daddr, ips));
      fprintf((FILE *) pf, "->%s", ips);
      break;
    case LCRZO_PRINTTYPE_TEXT_:
    case LCRZO_PRINTTYPE_PTEXT_:
    case LCRZO_PRINTTYPE_BASE64_:
    case LCRZO_PRINTTYPE_DUMP_:
    case LCRZO_PRINTTYPE_HEXA_:
    case LCRZO_PRINTTYPE_MIXED_:
    case LCRZO_PRINTTYPE_CMH_:
    case LCRZO_PRINTTYPE_CHM_:
      {
        lcrzo_hdrpip2 hdrpip2;

        lcrzo_er(lcrzo_hdrpip2_init_hdrlip(hdrlip, hdrpip2));
        lcrzo_er(lcrzo_data_fprint(pf, hdrpip2, sizeof(hdrpip2), printtype));
        return (LCRZO_ERR_OK);  /*return here to not print newline twice */
      }
      break;
    case LCRZO_PRINTTYPE_ARRAY_:
      fprintf((FILE *) pf, " IP____________________________________________");
      fprintf((FILE *) pf, "__________________________.\n");
      fprintf((FILE *) pf, " |version |  ihl   |       tos       |");
      fprintf((FILE *) pf, "              totlen               |\n");
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab04(pf, hdrlip.version, LCRZO_PRIV_DEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab04(pf, hdrlip.ihl, LCRZO_PRIV_DEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab08(pf, hdrlip.tos, LCRZO_PRIV_DEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrlip.totlen, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|\n");
      fprintf((FILE *) pf, " |                id                 |");
      fprintf((FILE *) pf, "xxDfMf         fragoffset          |\n");
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrlip.id, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf, hdrlip.reserve));
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf, hdrlip.dontfrag));
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf, hdrlip.morefrag));
      fprintf((FILE *) pf, "_________%04Xh=%5d_________|\n",
              hdrlip.offsetfrag, hdrlip.offsetfrag);
      fprintf((FILE *) pf, " |       ttl       |    protocol     |");
      fprintf((FILE *) pf, "          header checksum          |\n");
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab08(pf, hdrlip.ttl, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab08(pf, hdrlip.protocol,
                                      LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrlip.check, LCRZO_PRIV_HEXA));
      fprintf((FILE *) pf, "|\n");
      fprintf((FILE *) pf, " |                                source");
      fprintf((FILE *) pf, "                                 |\n");
      lcrzo_er(lcrzo_ips_init_ipl(hdrlip.saddr, ips));
      posgauche = (17 - strlen(ips)) / 2;
      lcrzo_er(lcrzo_string_init_char('_', posgauche, 19, tab));
      lcrzo_er(lcrzo_string_append_text(ips, 19, tab));
      lcrzo_er(lcrzo_string_append_char('_', 17 - strlen(ips) - posgauche,
                                        19, tab));
      fprintf((FILE *) pf, " |___________________________%s", tab);
      fprintf((FILE *) pf, "___________________________|\n");
      fprintf((FILE *) pf, " |                              destination");
      fprintf((FILE *) pf, "                              |\n");
      lcrzo_er(lcrzo_ips_init_ipl(hdrlip.daddr, ips));
      posgauche = (17 - strlen(ips)) / 2;
      lcrzo_er(lcrzo_string_init_char('_', posgauche, 19, tab));
      lcrzo_er(lcrzo_string_append_text(ips, 19, tab));
      lcrzo_er(lcrzo_string_append_char('_', 17 - strlen(ips) - posgauche,
                                        19, tab));
      fprintf((FILE *) pf, " |___________________________%s", tab);
      fprintf((FILE *) pf, "___________________________|\n");
      break;
    default:
      return (lcrzo_ieuv("lcrzo_hdrlip_fprint", "printtype&0xFF"));
  }

  /*eventually print a newline */
  lcrzo_er(lcrzo_priv_printtype_printnewline(pf, printtype));

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrludp_fprint(LCRZOFILE * pf,
                         lcrzo_hdrludp hdrludp,
                         lcrzo_printtype printtype)
{
  lcrzo_int16 language;

  /*parameters verification */

  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  /*affichage eventuel du commentaire */
  lcrzo_er(lcrzo_priv_printtype_printcomment(pf, printtype, "UDP header",
                                             "Entete UDP"));

  /*affichage de l'entete */
  switch (printtype & 0xFF) {
    case LCRZO_PRINTTYPE_NOTHING_:
      break;
    case LCRZO_PRINTTYPE_SYNTH_:
      fprintf((FILE *) pf, "UDP%d->%d", hdrludp.sport, hdrludp.dport);
      break;
    case LCRZO_PRINTTYPE_TEXT_:
    case LCRZO_PRINTTYPE_PTEXT_:
    case LCRZO_PRINTTYPE_BASE64_:
    case LCRZO_PRINTTYPE_DUMP_:
    case LCRZO_PRINTTYPE_HEXA_:
    case LCRZO_PRINTTYPE_MIXED_:
    case LCRZO_PRINTTYPE_CMH_:
    case LCRZO_PRINTTYPE_CHM_:
      {
        lcrzo_hdrpudp2 hdrpudp2;

        lcrzo_er(lcrzo_hdrpudp2_init_hdrludp(hdrludp, hdrpudp2));
        lcrzo_er(lcrzo_data_fprint
                 (pf, hdrpudp2, sizeof(hdrpudp2), printtype));
        return (LCRZO_ERR_OK);  /*return here to not print newline twice */
      }
      break;
    case LCRZO_PRINTTYPE_ARRAY_:
      lcrzo_er(lcrzo_global_get_language(&language));
      fprintf((FILE *) pf, " UDP_________________________________");
      fprintf((FILE *) pf, "____________________________________.\n");
      if (language == LCRZO_GLOBAL_LANG_FR) {
        fprintf((FILE *) pf, " |            port source            |");
        fprintf((FILE *) pf, "         port destination          |\n");
      } else {
        fprintf((FILE *) pf, " |            source port            |");
        fprintf((FILE *) pf, "         destination port          |\n");
      }
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrludp.sport, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrludp.dport, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|\n");
      fprintf((FILE *) pf, " |              length               |");
      fprintf((FILE *) pf, "             checksum              |\n");
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrludp.len, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrludp.check, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|\n");
      break;
    default:
      return (lcrzo_ieuv("lcrzo_hdrludp_fprint", "printtype&0xFF"));
  }

  /*eventually print a newline */
  lcrzo_er(lcrzo_priv_printtype_printnewline(pf, printtype));

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrltcp_fprint(LCRZOFILE * pf,
                         lcrzo_hdrltcp hdrltcp,
                         lcrzo_printtype printtype)
{
  lcrzo_int16 language;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*affichage eventuel du commentaire */
  lcrzo_er(lcrzo_priv_printtype_printcomment(pf, printtype, "TCP header",
                                             "Entete TCP"));

  /*affichage de l'entete */
  switch (printtype & 0xFF) {
    case LCRZO_PRINTTYPE_NOTHING_:
      break;
    case LCRZO_PRINTTYPE_SYNTH_:
      fprintf((FILE *) pf, "TCP");
      if (hdrltcp.syn)
        fprintf((FILE *) pf, "s");
      if (hdrltcp.ack)
        fprintf((FILE *) pf, "a");
      if (hdrltcp.fin)
        fprintf((FILE *) pf, "f");
      if (hdrltcp.rst)
        fprintf((FILE *) pf, "r");
      if (hdrltcp.psh)
        fprintf((FILE *) pf, "p");
      if (hdrltcp.urg)
        fprintf((FILE *) pf, "u");
      fprintf((FILE *) pf, "%d->%d", hdrltcp.sport, hdrltcp.dport);
      break;
    case LCRZO_PRINTTYPE_TEXT_:
    case LCRZO_PRINTTYPE_PTEXT_:
    case LCRZO_PRINTTYPE_BASE64_:
    case LCRZO_PRINTTYPE_DUMP_:
    case LCRZO_PRINTTYPE_HEXA_:
    case LCRZO_PRINTTYPE_MIXED_:
    case LCRZO_PRINTTYPE_CMH_:
    case LCRZO_PRINTTYPE_CHM_:
      {
        lcrzo_hdrptcp2 hdrptcp2;

        lcrzo_er(lcrzo_hdrptcp2_init_hdrltcp(hdrltcp, hdrptcp2));
        lcrzo_er(lcrzo_data_fprint
                 (pf, hdrptcp2, sizeof(hdrptcp2), printtype));
        return (LCRZO_ERR_OK);  /*return here to not print newline twice */
      }
      break;
    case LCRZO_PRINTTYPE_ARRAY_:
      lcrzo_er(lcrzo_global_get_language(&language));
      fprintf((FILE *) pf, " TCP______________________________");
      fprintf((FILE *) pf, "_______________________________________.\n");
      if (language == LCRZO_GLOBAL_LANG_FR) {
        fprintf((FILE *) pf, " |            port source            |");
        fprintf((FILE *) pf, "         port destination          |\n");
      } else {
        fprintf((FILE *) pf, " |            source port            |");
        fprintf((FILE *) pf, "         destination port          |\n");
      }
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrltcp.sport, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrltcp.dport, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|\n");
      fprintf((FILE *) pf, " |                                seq");
      fprintf((FILE *) pf, " num                                |\n");
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab32(pf, hdrltcp.seqnum,
                                      LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|\n");
      fprintf((FILE *) pf, " |                                ack ");
      fprintf((FILE *) pf, "num                                |\n");
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab32(pf, hdrltcp.acknum,
                                      LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|\n");
      fprintf((FILE *) pf, " |dataoff |         . . UrAk PuRsSyFi|");
      fprintf((FILE *) pf, "               window              |\n");
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab04(pf, hdrltcp.doff, LCRZO_PRIV_DEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab04
               (pf, (lcrzo_uint8) (hdrltcp.reserve >> 2), LCRZO_PRIV_DEC));
      fprintf((FILE *) pf, "_");
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf,
                                      (lcrzo_uint8) (1 &
                                                     (hdrltcp.
                                                      reserve >> 1))));
      lcrzo_er(lcrzo_priv_ecrit_tab01
               (pf, (lcrzo_uint8) (1 & hdrltcp.reserve)));
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf, hdrltcp.urg));
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf, hdrltcp.ack));
      fprintf((FILE *) pf, "_");
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf, hdrltcp.psh));
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf, hdrltcp.rst));
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf, hdrltcp.syn));
      lcrzo_er(lcrzo_priv_ecrit_tab01(pf, hdrltcp.fin));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrltcp.window,
                                      LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|\n");
      fprintf((FILE *) pf, " |             checksum              |");
      if (language == LCRZO_GLOBAL_LANG_FR)
        fprintf((FILE *) pf, "          pointeur urgent          |\n");
      else
        fprintf((FILE *) pf, "          urgent pointer           |\n");
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrltcp.check, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrltcp.urgptr,
                                      LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|\n");
      break;
    default:
      return (lcrzo_ieuv("lcrzo_hdrltcp_fprint", "printtype&0xFF"));
  }

  /*eventually print a newline */
  lcrzo_er(lcrzo_priv_printtype_printnewline(pf, printtype));

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlicmp_fprint(LCRZOFILE * pf,
                          lcrzo_hdrlicmp hdrlicmp,
                          lcrzo_printtype printtype)
{
  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*affichage eventuel du commentaire */
  lcrzo_er(lcrzo_priv_printtype_printcomment(pf, printtype, "ICMP header",
                                             "Entete ICMP"));

  /*affichage de l'entete */
  switch (printtype & 0xFF) {
    case LCRZO_PRINTTYPE_NOTHING_:
      break;
    case LCRZO_PRINTTYPE_SYNTH_:
      fprintf((FILE *) pf, "ICMP%d", hdrlicmp.type);
      break;
    case LCRZO_PRINTTYPE_TEXT_:
    case LCRZO_PRINTTYPE_PTEXT_:
    case LCRZO_PRINTTYPE_BASE64_:
    case LCRZO_PRINTTYPE_DUMP_:
    case LCRZO_PRINTTYPE_HEXA_:
    case LCRZO_PRINTTYPE_MIXED_:
    case LCRZO_PRINTTYPE_CMH_:
    case LCRZO_PRINTTYPE_CHM_:
      {
        lcrzo_hdrpicmp2 hdrpicmp2;

        lcrzo_er(lcrzo_hdrpicmp2_init_hdrlicmp(hdrlicmp, hdrpicmp2));
        lcrzo_er(lcrzo_data_fprint
                 (pf, hdrpicmp2, sizeof(hdrpicmp2), printtype));
        return (LCRZO_ERR_OK);  /*return here to not print newline twice */
      }
      break;
    case LCRZO_PRINTTYPE_ARRAY_:
      lcrzo_er(lcrzo_priv_print_hdrlicmpline(pf, hdrlicmp));
      fprintf((FILE *) pf, " |      type       |      code       |");
      fprintf((FILE *) pf, "             checksum              |\n");
      fprintf((FILE *) pf, " |");
      lcrzo_er(lcrzo_priv_ecrit_tab08(pf, hdrlicmp.type, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab08(pf, hdrlicmp.code, LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|");
      lcrzo_er(lcrzo_priv_ecrit_tab16(pf, hdrlicmp.check,
                                      LCRZO_PRIV_HEXADEC));
      fprintf((FILE *) pf, "|\n");
      break;
    default:
      return (lcrzo_ieuv("lcrzo_hdrlicmp_fprint", "printtype&0xFF"));
  }

  /*eventually print a newline */
  lcrzo_er(lcrzo_priv_printtype_printnewline(pf, printtype));

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_hdrlarp_fprint(LCRZOFILE * pf,
                         lcrzo_hdrlarp hdrlarp,
                         lcrzo_printtype printtype)
{
  int i;
  lcrzo_int16 language;
  char tab[40];                 /* pour tabuler le | de droite */
  char chpsrc[40], chpdst[40];

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*affichage eventuel du commentaire */
  lcrzo_er(lcrzo_priv_printtype_printcomment(pf, printtype, "ARP header",
                                             "Entete ARP"));

  /*affichage de l'entete */
  switch (printtype & 0xFF) {
    case LCRZO_PRINTTYPE_NOTHING_:
      break;
    case LCRZO_PRINTTYPE_SYNTH_:
      switch (hdrlarp.op) {
        case 1:
          fprintf((FILE *) pf, "ARPRequest");
          lcrzo_er(lcrzo_ipa_fprint(pf, "_", hdrlarp.prot_dst, ""));
          break;
        case 2:
          fprintf((FILE *) pf, "ARPReply");
          lcrzo_er(lcrzo_etha_fprint(pf, "_", hdrlarp.hw_src, ""));
          break;
        case 3:
          fprintf((FILE *) pf, "RARPRequest");
          lcrzo_er(lcrzo_etha_fprint(pf, "_", hdrlarp.hw_dst, ""));
          break;
        case 4:
          fprintf((FILE *) pf, "RARPReply");
          lcrzo_er(lcrzo_ipa_fprint(pf, "_", hdrlarp.prot_dst, ""));
          break;
        default:
          fprintf((FILE *) pf, "ARP??");
          break;
      }
      break;
    case LCRZO_PRINTTYPE_TEXT_:
    case LCRZO_PRINTTYPE_PTEXT_:
    case LCRZO_PRINTTYPE_BASE64_:
    case LCRZO_PRINTTYPE_DUMP_:
    case LCRZO_PRINTTYPE_HEXA_:
    case LCRZO_PRINTTYPE_MIXED_:
    case LCRZO_PRINTTYPE_CMH_:
    case LCRZO_PRINTTYPE_CHM_:
      {
        lcrzo_hdrparp2 hdrparp2;

        lcrzo_er(lcrzo_hdrparp2_init_hdrlarp(hdrlarp, hdrparp2));
        lcrzo_er(lcrzo_data_fprint
                 (pf, hdrparp2, sizeof(hdrparp2), printtype));
        return (LCRZO_ERR_OK);  /*return here to not print newline twice */
      }
      break;
    case LCRZO_PRINTTYPE_ARRAY_:
      lcrzo_er(lcrzo_global_get_language(&language));
      switch (hdrlarp.op) {
        case 1:
          fprintf((FILE *) pf, " ARP Request_");
          if (language == LCRZO_GLOBAL_LANG_FR) {
            lcrzo_er(lcrzo_string_init_text("demandeur :      ", 39, chpsrc));
            lcrzo_er(lcrzo_string_init_text("info demandee :  ", 39, chpdst));
          } else {
            lcrzo_er(lcrzo_string_init_text("this address :   ", 39, chpsrc));
            lcrzo_er(lcrzo_string_init_text("asks that :      ", 39, chpdst));
          }
          break;
        case 2:
          fprintf((FILE *) pf, " ARP Reply___");
          if (language == LCRZO_GLOBAL_LANG_FR) {
            lcrzo_er(lcrzo_string_init_text("info demandee :  ", 39, chpsrc));
            lcrzo_er(lcrzo_string_init_text("pour qui c'est : ", 39, chpdst));
          } else {
            lcrzo_er(lcrzo_string_init_text("that answer :    ", 39, chpsrc));
            lcrzo_er(lcrzo_string_init_text("is for this :    ", 39, chpdst));
          }
          break;
        case 3:
          fprintf((FILE *) pf, " RARP Request");
          if (language == LCRZO_GLOBAL_LANG_FR) {
            lcrzo_er(lcrzo_string_init_text("demandeur :      ", 39, chpsrc));
            lcrzo_er(lcrzo_string_init_text("info demandee :  ", 39, chpdst));
          } else {
            lcrzo_er(lcrzo_string_init_text("this address :   ", 39, chpsrc));
            lcrzo_er(lcrzo_string_init_text("asks that :      ", 39, chpdst));
          }
          break;
        case 4:
          fprintf((FILE *) pf, " RARP Reply__");
          if (language == LCRZO_GLOBAL_LANG_FR) {
            lcrzo_er(lcrzo_string_init_text("repondeur :      ", 39, chpsrc));
            lcrzo_er(lcrzo_string_init_text("info repondue :  ", 39, chpdst));
          } else {
            lcrzo_er(lcrzo_string_init_text("this address :   ", 39, chpsrc));
            lcrzo_er(lcrzo_string_init_text("answered :       ", 39, chpdst));
          }
          break;
        default:
          fprintf((FILE *) pf, " ARP ??______");
          lcrzo_er(lcrzo_string_init_text("source :         ", 39, chpsrc));
          lcrzo_er(lcrzo_string_init_text("destination :    ", 39, chpdst));
          break;
      }
      fprintf((FILE *) pf, "______________________________");
      fprintf((FILE *) pf, "______________________________.\n");
      fprintf((FILE *) pf, " | hw_type=%04xh, prot_type=%04xh,",
              hdrlarp.hw_type, hdrlarp.prot_type);
      fprintf((FILE *) pf, " hw_size=%02xh, prot_size=%02xh,",
              hdrlarp.hw_size, hdrlarp.prot_size);
      fprintf((FILE *) pf, " op=%04xh  |\n", hdrlarp.op);
      fprintf((FILE *) pf, " | %s%02x:%02x:%02x:%02x:%02x:%02x", chpsrc,
              (lcrzo_uint8) hdrlarp.hw_src[0],
              (lcrzo_uint8) hdrlarp.hw_src[1],
              (lcrzo_uint8) hdrlarp.hw_src[2],
              (lcrzo_uint8) hdrlarp.hw_src[3],
              (lcrzo_uint8) hdrlarp.hw_src[4],
              (lcrzo_uint8) hdrlarp.hw_src[5]);
      fprintf((FILE *) pf, " - %d.%d.%d.%d %n",
              (lcrzo_uint8) hdrlarp.prot_src[0],
              (lcrzo_uint8) hdrlarp.prot_src[1],
              (lcrzo_uint8) hdrlarp.prot_src[2],
              (lcrzo_uint8) hdrlarp.prot_src[3], &i);
      lcrzo_er(lcrzo_string_init_text
               ("                                     ", 39, tab));
      tab[30 - i] = '\0';
      fprintf((FILE *) pf, "%s      |\n", tab);
      fprintf((FILE *) pf, " | %s%02x:%02x:%02x:%02x:%02x:%02x", chpdst,
              (lcrzo_uint8) hdrlarp.hw_dst[0],
              (lcrzo_uint8) hdrlarp.hw_dst[1],
              (lcrzo_uint8) hdrlarp.hw_dst[2],
              (lcrzo_uint8) hdrlarp.hw_dst[3],
              (lcrzo_uint8) hdrlarp.hw_dst[4],
              (lcrzo_uint8) hdrlarp.hw_dst[5]);
      fprintf((FILE *) pf, " - %d.%d.%d.%d %n",
              (lcrzo_uint8) hdrlarp.prot_dst[0],
              (lcrzo_uint8) hdrlarp.prot_dst[1],
              (lcrzo_uint8) hdrlarp.prot_dst[2],
              (lcrzo_uint8) hdrlarp.prot_dst[3], &i);
      lcrzo_er(lcrzo_string_init_text
               ("                                     ", 39, tab));
      tab[30 - i] = '\0';
      fprintf((FILE *) pf, "%s      |\n", tab);
      fprintf((FILE *) pf, " |______________________________________");
      fprintf((FILE *) pf, "_________________________________|\n");
      break;
    default:
      return (lcrzo_ieuv("lcrzo_hdrlarp_fprint", "printtype&0xFF"));
  }

  /*eventually print a newline */
  lcrzo_er(lcrzo_priv_printtype_printnewline(pf, printtype));

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_ipopt_fprint(LCRZOFILE * pf,
                       lcrzo_constipopt ipopt,
                       lcrzo_uint8 nboctipopt,
                       lcrzo_printtype printtype)
{
  int retour;
  lcrzo_int16 language;
  lcrzo_uint8 pointeur, nboctnoop, overflow, flag;
  lcrzo_ipl iplt[9];
  lcrzo_ipopt optioninconnue;
  lcrzo_uint8 nboctoptioninconnue;
  lcrzo_uint8 nbiplaffectes;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (ipopt == NULL && nboctipopt)
    return (LCRZO_ERR_PANULLPTRSIZE);

  /*affichage eventuel du commentaire */
  lcrzo_er(lcrzo_priv_printtype_printcomment(pf, printtype, "IP options",
                                             "Options IP"));

  /*affichage de l'entete */
  switch (printtype & 0xFF) {
    case LCRZO_PRINTTYPE_NOTHING_:
      break;
    case LCRZO_PRINTTYPE_SYNTH_:
    case LCRZO_PRINTTYPE_TEXT_:
    case LCRZO_PRINTTYPE_PTEXT_:
    case LCRZO_PRINTTYPE_BASE64_:
    case LCRZO_PRINTTYPE_DUMP_:
    case LCRZO_PRINTTYPE_HEXA_:
    case LCRZO_PRINTTYPE_MIXED_:
    case LCRZO_PRINTTYPE_CMH_:
    case LCRZO_PRINTTYPE_CHM_:
      lcrzo_er(lcrzo_data_fprint(pf, ipopt, nboctipopt, printtype));
      return (LCRZO_ERR_OK);    /*return here to not print newline twice */
      break;
    case LCRZO_PRINTTYPE_ARRAY_:
      lcrzo_er(lcrzo_global_get_language(&language));
      fprintf((FILE *) pf,
              (language ==
               LCRZO_GLOBAL_LANG_FR) ? " OPTIONS IP" : " IP OPTIONS");
      fprintf((FILE *) pf, "____________________________________");
      fprintf((FILE *) pf, "__________________________.\n");
      /*cherche lsrr */
      retour = lcrzo_ipopt_decode_lsrr(ipopt, nboctipopt,
                                       &nbiplaffectes, &pointeur,
                                       &(iplt[0]), &(iplt[1]), &(iplt[2]),
                                       &(iplt[3]), &(iplt[4]), &(iplt[5]),
                                       &(iplt[6]), &(iplt[7]), &(iplt[8]));
      if (retour == LCRZO_ERR_OK) {
        fprintf((FILE *) pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                " | Loose source routing : nombre_de_routes %d," :
                " | Loose source routing : number_of_routes %d,",
                nbiplaffectes);
        fprintf((FILE *) pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                "  pointeur %d                |\n" :
                "  pointer %d                 |\n", pointeur);
        lcrzo_er(lcrzo_priv_ecrit_ipopttabipl(pf, nbiplaffectes, iplt));
      }
      /*cherche ssrr */
      retour = lcrzo_ipopt_decode_ssrr(ipopt, nboctipopt,
                                       &nbiplaffectes, &pointeur,
                                       &(iplt[0]), &(iplt[1]), &(iplt[2]),
                                       &(iplt[3]), &(iplt[4]), &(iplt[5]),
                                       &(iplt[6]), &(iplt[7]), &(iplt[8]));
      if (retour == LCRZO_ERR_OK) {
        fprintf((FILE *) pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                " | Strict source routing : nombre_de_routes %d," :
                " | Strict source routing : number_of_routes %d,",
                nbiplaffectes);
        fprintf((FILE *) pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                "  pointeur %d               |\n" :
                "  pointer %d                |\n", pointeur);
        lcrzo_er(lcrzo_priv_ecrit_ipopttabipl(pf, nbiplaffectes, iplt));
      }
      /*cherche rr */
      retour = lcrzo_ipopt_decode_rr(ipopt, nboctipopt,
                                     &nbiplaffectes, &pointeur,
                                     &(iplt[0]), &(iplt[1]), &(iplt[2]),
                                     &(iplt[3]), &(iplt[4]), &(iplt[5]),
                                     &(iplt[6]), &(iplt[7]), &(iplt[8]));
      if (retour == LCRZO_ERR_OK) {
        fprintf((FILE *) pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                " | Record route : nombre_de_routes %d, pointeur %d" :
                " | Record route : number_of_routes %d, pointer %d ",
                nbiplaffectes, pointeur);
        fprintf((FILE *) pf, "                         |\n");
        lcrzo_er(lcrzo_priv_ecrit_ipopttabipl(pf, nbiplaffectes, iplt));
      }
      /*cherche timestamp */
      retour = lcrzo_ipopt_decode_time(ipopt, nboctipopt,
                                       &nbiplaffectes, &pointeur,
                                       &overflow, &flag,
                                       &(iplt[0]), &(iplt[1]), &(iplt[2]),
                                       &(iplt[3]), &(iplt[4]), &(iplt[5]),
                                       &(iplt[6]), &(iplt[7]), &(iplt[8]));
      if (retour == LCRZO_ERR_OK) {
        fprintf((FILE *) pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                " | Timestamp : nombre_de_timestamp %d, pointeur %d, " :
                " | Timestamp : number_of_routes %d, pointer %d,     ",
                nbiplaffectes, pointeur);
        fprintf((FILE *) pf, "overflow %d, flag %d     |\n", overflow, flag);
        lcrzo_er(lcrzo_priv_ecrit_ipopttabtime(pf, nbiplaffectes, iplt));
      }
      /*cherche timestamp */
      retour = lcrzo_ipopt_decode_timeip(ipopt, nboctipopt,
                                         &nbiplaffectes, &pointeur,
                                         &overflow, &flag,
                                         &(iplt[0]), &(iplt[1]), &(iplt[2]),
                                         &(iplt[3]), &(iplt[4]), &(iplt[5]),
                                         &(iplt[6]), &(iplt[7]));
      if (retour == LCRZO_ERR_OK) {
        fprintf((FILE *) pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                " | Timestamp : nombre_de_timestamp %d, pointeur %d, " :
                " | Timestamp : number_of_timestamp %d, pointer %d,  ",
                nbiplaffectes, pointeur);
        fprintf((FILE *) pf, "overflow %d, flag %d     |\n", overflow, flag);
        lcrzo_er(lcrzo_priv_ecrit_ipopttabiptime(pf, nbiplaffectes, iplt));
      }
      /*cherche noop */
      retour = lcrzo_ipopt_decode_noop(ipopt, nboctipopt, &nboctnoop);
      if (retour == LCRZO_ERR_OK) {
        fprintf((FILE *) pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                " | Nombre d'octets NOOP : %d\n" :
                " | Number of NOOP bytes: %d\n", nboctnoop);
      }
      /*cherche inconnu */
      retour = lcrzo_ipopt_decode_unknown(ipopt, nboctipopt,
                                          optioninconnue,
                                          &nboctoptioninconnue);
      if (retour == LCRZO_ERR_OK) {
        fprintf((FILE *) pf, (language == LCRZO_GLOBAL_LANG_FR) ?
                " | Option non reconnue : \n" :
                " | Unrecognized option : \n");
        lcrzo_er(lcrzo_data_fprint(pf, optioninconnue,
                                   nboctoptioninconnue,
                                   LCRZO_PRINTTYPE_HEXAS));
      }
      fprintf((FILE *) pf, " |____________________________________________");
      fprintf((FILE *) pf, "___________________________|\n");
      break;
    default:
      return (lcrzo_ieuv("lcrzo_ipopt_fprint", "printtype&0xFF"));
  }

  /*eventually print a newline */
  lcrzo_er(lcrzo_priv_printtype_printnewline(pf, printtype));

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_fprint_eth(LCRZOFILE * pf,
                            lcrzo_constdata packet,
                            lcrzo_int32 nboctpacket,
                            lcrzo_printprofile printprofile)
{
  lcrzo_data data;
  lcrzo_hdrleth hdrleth;
  lcrzo_uint16 nboctdata;
  int ret;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);

  /*tentative de decodage */
  ret = lcrzo_packet_decodem_ethdata(packet, nboctpacket,
                                     &hdrleth, &data, &nboctdata);
  if (ret != LCRZO_ERR_OK) {
    return (lcrzo_priv_printprofile_tcdf(pf, &printprofile, printprofile.data,
                                         "Data", "Donnees",
                                         packet, nboctpacket));
  }

  /*affichage de l'entete */
  ret = lcrzo_priv_ecrittransition(pf, &printprofile, printprofile.hdreth);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }
  ret = lcrzo_hdrleth_fprint(pf, hdrleth, printprofile.hdreth);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }

  /*affichage des donnees ethernet */
  switch (hdrleth.type) {
    case LCRZO_HDRLETH_TYPE_IP:
      ret = lcrzo_packet_fprint_ip(pf, data, nboctdata, printprofile);
      break;
    case LCRZO_HDRLETH_TYPE_ARP:
    case LCRZO_HDRLETH_TYPE_RARP:
      ret = lcrzo_packet_fprint_arp(pf, data, nboctdata, printprofile);
      break;
    default:
      ret = lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                         printprofile.dataeth,
                                         "ETH data", "Donnees ETH",
                                         data, nboctdata);
      break;
  }
  lcrzo_er(lcrzo_data_free2(&data));
  fflush((FILE *) pf);
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_fprint_ip(LCRZOFILE * pf,
                           lcrzo_constdata packet,
                           lcrzo_int32 nboctpacket,
                           lcrzo_printprofile printprofile)
{
  lcrzo_data data;
  lcrzo_ipopt opt;
  lcrzo_hdrlip hdrlip;
  lcrzo_uint8 nboctopt;
  lcrzo_uint16 nboctdata;
  int ret;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);

  /*tentative de decodage */
  ret = lcrzo_packet_decodem_ipoptdata(packet, nboctpacket, &hdrlip,
                                       opt, &nboctopt, &data, &nboctdata);
  if (ret != LCRZO_ERR_OK) {
    return (lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                         printprofile.dataeth,
                                         "ETH data", "Donnees ETH",
                                         packet, nboctpacket));
  }

  /*affichage de l'entete */
  ret = lcrzo_priv_ecrittransition(pf, &printprofile, printprofile.hdrip);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }
  ret = lcrzo_hdrlip_fprint(pf, hdrlip, printprofile.hdrip);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }
  if (nboctopt) {
    ret = lcrzo_priv_ecrittransition(pf, &printprofile, printprofile.hdrip);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_data_free2(&data));
      return (ret);
    }
    ret = lcrzo_ipopt_fprint(pf, opt, nboctopt, printprofile.hdrip);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_data_free2(&data));
      return (ret);
    }
  }

  if (hdrlip.offsetfrag) {
    /*ne sait pas decoder les fragments */
    ret = lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                       printprofile.dataip,
                                       "IP data", "Donnees IP",
                                       data, nboctdata);
    lcrzo_er(lcrzo_data_free2(&data));
    fflush((FILE *) pf);
    return (ret);
  }

  /*affichage des donnees IP */
  switch (hdrlip.protocol) {
    case LCRZO_HDRLIP_PROTOCOL_UDP:
      ret = lcrzo_packet_fprint_udp(pf, data, nboctdata, printprofile);
      break;
    case LCRZO_HDRLIP_PROTOCOL_TCP:
      ret = lcrzo_packet_fprint_tcp(pf, data, nboctdata, printprofile);
      break;
    case LCRZO_HDRLIP_PROTOCOL_ICMP:
      ret = lcrzo_packet_fprint_icmp(pf, data, nboctdata, printprofile);
      break;
    default:
      ret = lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                         printprofile.dataip,
                                         "IP data", "Donnees IP",
                                         data, nboctdata);
      break;
  }

  lcrzo_er(lcrzo_data_free2(&data));
  fflush((FILE *) pf);
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_fprint_udp(LCRZOFILE * pf,
                            lcrzo_constdata packet,
                            lcrzo_int32 nboctpacket,
                            lcrzo_printprofile printprofile)
{
  lcrzo_data data;
  lcrzo_hdrludp hdrludp;
  lcrzo_uint16 nboctdata;
  int ret;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);

  /*tentative de decodage */
  ret = lcrzo_packet_decodem_udpdata(packet, nboctpacket, &hdrludp,
                                     &data, &nboctdata);
  if (ret != LCRZO_ERR_OK) {
    return (lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                         printprofile.dataip,
                                         "IP data", "Donnees IP",
                                         packet, nboctpacket));
  }

  /*affichage de l'entete */
  ret = lcrzo_priv_ecrittransition(pf, &printprofile, printprofile.hdrudp);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }
  ret = lcrzo_hdrludp_fprint(pf, hdrludp, printprofile.hdrudp);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }

  /*affichage des donnees UDP */
  ret = lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                     printprofile.dataudp,
                                     "UDP data", "Donnees UDP",
                                     data, nboctdata);
  lcrzo_er(lcrzo_data_free2(&data));
  fflush((FILE *) pf);
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_fprint_tcp(LCRZOFILE * pf,
                            lcrzo_constdata packet,
                            lcrzo_int32 nboctpacket,
                            lcrzo_printprofile printprofile)
{
  lcrzo_data data;
  lcrzo_tcpopt tcpopt;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_uint16 nboctdata;
  lcrzo_uint8 nbocttcpopt;
  int ret;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);

  /*tentative de decodage */
  ret = lcrzo_packet_decodem_tcpoptdata(packet, nboctpacket,
                                        &hdrltcp, tcpopt, &nbocttcpopt,
                                        &data, &nboctdata);
  if (ret != LCRZO_ERR_OK) {
    return (lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                         printprofile.dataip,
                                         "IP data", "Donnees IP",
                                         packet, nboctpacket));
  }

  /*affichage de l'entete */
  ret = lcrzo_priv_ecrittransition(pf, &printprofile, printprofile.hdrtcp);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }
  ret = lcrzo_hdrltcp_fprint(pf, hdrltcp, printprofile.hdrtcp);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }
  if (nbocttcpopt) {
    ret = lcrzo_priv_ecrittransition(pf, &printprofile, printprofile.hdrtcp);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_data_free2(&data));
      return (ret);
    }
    ret = lcrzo_priv_printtype_printcomment(pf, printprofile.hdrtcp,
                                            "TCP options", "Options TCP");
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_data_free2(&data));
      return (ret);
    }
    ret = lcrzo_data_fprint(pf, tcpopt, nbocttcpopt, printprofile.hdrtcp);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_data_free2(&data));
      return (ret);
    }
  }

  /*affichage des donnees TCP */
  ret = lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                     printprofile.datatcp,
                                     "TCP data", "Donnees TCP",
                                     data, nboctdata);
  lcrzo_er(lcrzo_data_free2(&data));
  fflush((FILE *) pf);
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_fprint_icmp(LCRZOFILE * pf,
                             lcrzo_constdata packet,
                             lcrzo_int32 nboctpacket,
                             lcrzo_printprofile printprofile)
{
  lcrzo_data data;
  lcrzo_hdrlicmp hdrlicmp;
  lcrzo_uint16 nboctdata;
  int ret;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);

  /*tentative de decodage */
  ret = lcrzo_packet_decodem_icmpdata(packet, nboctpacket, &hdrlicmp,
                                      &data, &nboctdata);
  if (ret != LCRZO_ERR_OK) {
    return (lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                         printprofile.dataip,
                                         "IP data", "Donnees IP",
                                         packet, nboctpacket));
  }

  /*affichage de l'entete */
  ret = lcrzo_priv_ecrittransition(pf, &printprofile, printprofile.hdricmp);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }
  ret = lcrzo_hdrlicmp_fprint(pf, hdrlicmp, printprofile.hdricmp);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_data_free2(&data));
    return (ret);
  }

  /*affichage des donnees ICMP */
  ret = lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                     printprofile.dataicmp,
                                     "ICMP data", "Donnees ICMP",
                                     data, nboctdata);
  lcrzo_er(lcrzo_data_free2(&data));
  fflush((FILE *) pf);
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_packet_fprint_arp(LCRZOFILE * pf,
                            lcrzo_constdata packet,
                            lcrzo_int32 nboctpacket,
                            lcrzo_printprofile printprofile)
{
  lcrzo_hdrlarp hdrlarp;
  int ret;

  /*parameters verification */
  if (pf == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (packet == NULL && nboctpacket)
    return (LCRZO_ERR_PANULLPTRSIZE);

  /*tentative de decodage */
  ret = lcrzo_packet_decode_arp(packet, nboctpacket, &hdrlarp);
  if (ret != LCRZO_ERR_OK) {
    return (lcrzo_priv_printprofile_tcdf(pf, &printprofile,
                                         printprofile.dataeth,
                                         "Ethernet data", "Donnees Ethernet",
                                         packet, nboctpacket));
  }

  /*affichage de l'entete */
  lcrzo_er(lcrzo_priv_ecrittransition
           (pf, &printprofile, printprofile.hdrarp));
  lcrzo_er(lcrzo_hdrlarp_fprint(pf, hdrlarp, printprofile.hdrarp));
  lcrzo_er(lcrzo_priv_ecrittransitionfin(pf, &printprofile));

  fflush((FILE *) pf);
  return (LCRZO_ERR_OK);
}
