/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/


#include "lcrzo_incl.h"

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

/*-------------------------------------------------------------*/
typedef struct {
  lcrzo_int32 index;
  lcrzo_wait item;
  lcrzo_bool event;
} lcrzo_priv_waitlistitem;

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_independent_wait_read(lcrzo_wait * pwait1,
                                          lcrzo_time * pmaxwaitingtime,
                                          lcrzo_bool * pevent1);
int lcrzo_priv_wait_independent_wait_read(lcrzo_wait * pwait1,
                                          lcrzo_time * pmaxwaitingtime,
                                          lcrzo_bool * pevent1)
{
  lcrzo_bool event1;
  int ret;

  /* first, only wait */
  switch (pwait1->source) {
    case LCRZO_PRIV_WAIT_SOURCE_FILE:
      lcrzo_er(lcrzo_priv_wait_file_wait((lcrzo_file *) pwait1->psource,
                                         pmaxwaitingtime, &event1));
      break;
    case LCRZO_PRIV_WAIT_SOURCE_IPC:
      lcrzo_er(lcrzo_priv_wait_ipc_wait((lcrzo_ipc *) pwait1->psource,
                                        pmaxwaitingtime, &event1));
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SNIFF:
      lcrzo_er(lcrzo_priv_wait_sniff_wait((lcrzo_sniff *) pwait1->psource,
                                          pmaxwaitingtime, &event1));
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SOCK:
      lcrzo_er(lcrzo_priv_wait_sock_wait((lcrzo_sock *) pwait1->psource,
                                         pmaxwaitingtime, &event1));
      break;
    case LCRZO_PRIV_WAIT_SOURCE_PROCESS:
      lcrzo_er(lcrzo_priv_wait_process_wait((lcrzo_process *) pwait1->psource,
                                            pmaxwaitingtime, &event1));
      break;
    case LCRZO_PRIV_WAIT_SOURCE_FD2:
      lcrzo_er(lcrzo_priv_wait_fd2_wait((lcrzo_fd2 *) pwait1->psource,
                                        pmaxwaitingtime, &event1));
      break;
    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE:
      lcrzo_er(lcrzo_priv_wait_winhandle_wait
               ((lcrzo_winhandle *) pwait1->psource, pmaxwaitingtime,
                &event1));
      break;
    case LCRZO_PRIV_WAIT_SOURCE_FUNCTION:
      /* call the internal function */
      lcrzo_er((*pwait1->pfunc) (pwait1->infos, pwait1->infossize,
                                 pmaxwaitingtime, pevent1));
      /* leave here */
      return (LCRZO_ERR_OK);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_KBD:
      lcrzo_er(lcrzo_priv_wait_kbd_wait((lcrzo_kbd *) pwait1->psource,
                                        pwait1->action,
                                        pmaxwaitingtime, &event1));
      break;
    default:
      return (lcrzo_ieuv
              ("lcrzo_priv_wait_independent_wait_read", "pwait1->source"));
  }

#if LCRZO_DEBUG_WAIT_WAIT==1
  printf("Event1=%d\n", event1);
#endif

  /* nothing occured */
  if (!event1) {
    if (pevent1 != NULL)
      *pevent1 = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  /* read to obtain the promised data */
  switch (pwait1->source) {
    case LCRZO_PRIV_WAIT_SOURCE_FILE:
      ret = lcrzo_priv_wait_file_read((lcrzo_file *) pwait1->psource);
      if (ret == LCRZO_ERR_OKDATAEND) {
        ((lcrzo_file *) (pwait1->psource))->dataend = LCRZO_TRUE;
        ret = LCRZO_ERR_OK;
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_IPC:
      ret = lcrzo_priv_wait_ipc_read((lcrzo_ipc *) pwait1->psource);
      if (ret == LCRZO_ERR_OKDATAEND) {
        ((lcrzo_ipc *) (pwait1->psource))->dataend = LCRZO_TRUE;
        ret = LCRZO_ERR_OK;
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SNIFF:
      ret = lcrzo_priv_wait_sniff_read((lcrzo_sniff *) pwait1->psource);
      if (ret == LCRZO_ERR_OKDATAEND) {
        ((lcrzo_sniff *) (pwait1->psource))->dataend = LCRZO_TRUE;
        ret = LCRZO_ERR_OK;
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SOCK:
      ret = lcrzo_priv_wait_sock_read((lcrzo_sock *) pwait1->psource);
      if (ret == LCRZO_ERR_OKDATAEND) {
        ((lcrzo_sock *) (pwait1->psource))->dataend = LCRZO_TRUE;
        ret = LCRZO_ERR_OK;
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_PROCESS:
      /* nothing to read for a process */
      ret = LCRZO_ERR_OK;
      break;
    case LCRZO_PRIV_WAIT_SOURCE_FD2:
      ret = lcrzo_priv_wait_fd2_read((lcrzo_fd2 *) pwait1->psource);
      if (ret == LCRZO_ERR_OKDATAEND) {
        ((lcrzo_fd2 *) (pwait1->psource))->dataend = LCRZO_TRUE;
        ret = LCRZO_ERR_OK;
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE:
      ret =
        lcrzo_priv_wait_winhandle_read((lcrzo_winhandle *) pwait1->psource);
      if (ret == LCRZO_ERR_OKDATAEND) {
        ((lcrzo_winhandle *) (pwait1->psource))->dataend = LCRZO_TRUE;
        ret = LCRZO_ERR_OK;
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_KBD:
      ret = lcrzo_priv_wait_kbd_read((lcrzo_kbd *) pwait1->psource,
                                     pwait1->action);
      if (ret == LCRZO_ERR_OKDATAEND) {
        ((lcrzo_kbd *) (pwait1->psource))->dataend = LCRZO_TRUE;
        ret = LCRZO_ERR_OK;
      }
      break;
    default:
      return (lcrzo_ieuv
              ("lcrzo_priv_wait_independent_wait_read", "pwait1->source"));
  }

#if LCRZO_DEBUG_WAIT_WAIT==1
  printf("End read (%d)\n", ret);
#endif

  if (ret == LCRZO_ERR_OK) {    /* data was read */
    if (pevent1 != NULL)
      *pevent1 = LCRZO_TRUE;
    return (LCRZO_ERR_OK);
  } else if (ret == LCRZO_ERR_OKTEMPDATAEND) {  /* no read data */
    if (pevent1 != NULL)
      *pevent1 = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  } else {
    return (ret);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_check_read(lcrzo_wait * pwait,
                               lcrzo_bool * pcanread);
int lcrzo_priv_wait_check_read(lcrzo_wait * pwait,
                               lcrzo_bool * pcanread)
{
  int ret;

  if (pwait->action != LCRZO_PRIV_WAIT_ACTION_READ) {
    if (pcanread != NULL)
      *pcanread = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  switch (pwait->source) {
    case LCRZO_PRIV_WAIT_SOURCE_FILE:
      ret =
        lcrzo_priv_storeddata_contain(((lcrzo_file *) (pwait->psource))->
                                      storeddata,
                                      ((lcrzo_file *) (pwait->psource))->
                                      storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_IPC:
      ret =
        lcrzo_priv_storedtlvdata_contain(((lcrzo_ipc *) (pwait->psource))->
                                         storeddata,
                                         ((lcrzo_ipc *) (pwait->psource))->
                                         storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SNIFF:
      ret =
        lcrzo_priv_storeddata_contain(((lcrzo_sniff *) (pwait->psource))->
                                      storeddata,
                                      ((lcrzo_sniff *) (pwait->psource))->
                                      storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SOCK:
      ret =
        lcrzo_priv_storedtlvdata_contain(((lcrzo_sock *) (pwait->psource))->
                                         storeddata,
                                         ((lcrzo_sock *) (pwait->psource))->
                                         storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_FD2:
      ret =
        lcrzo_priv_storeddata_contain(((lcrzo_fd2 *) (pwait->psource))->
                                      storeddata,
                                      ((lcrzo_fd2 *) (pwait->psource))->
                                      storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE:
      ret =
        lcrzo_priv_storeddata_contain(((lcrzo_winhandle *) (pwait->psource))->
                                      storeddata,
                                      ((lcrzo_winhandle *) (pwait->psource))->
                                      storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_KBD:
      ret =
        lcrzo_priv_storeddata_contain(((lcrzo_kbd *) (pwait->psource))->
                                      storeddata,
                                      ((lcrzo_kbd *) (pwait->psource))->
                                      storeddatasize, pcanread);
      break;
    default:
      ret = lcrzo_ieuv("lcrzo_priv_wait_check_read", "pwait->source");
  }

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_check_read_fixed(lcrzo_wait * pwait,
                                     lcrzo_bool * pcanread);
int lcrzo_priv_wait_check_read_fixed(lcrzo_wait * pwait,
                                     lcrzo_bool * pcanread)
{
  int ret;

  if (pwait->action != LCRZO_PRIV_WAIT_ACTION_READ_FIXED) {
    if (pcanread != NULL)
      *pcanread = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  if (pwait->wantedfixedsize == 0) {
    if (pcanread != NULL)
      *pcanread = LCRZO_TRUE;
    return (LCRZO_ERR_OK);
  }

  switch (pwait->source) {
    case LCRZO_PRIV_WAIT_SOURCE_FILE:
      ret =
        lcrzo_priv_storeddata_contain_fixed(((lcrzo_file *) (pwait->
                                                             psource))->
                                            storeddata,
                                            ((lcrzo_file *) (pwait->
                                                             psource))->
                                            storeddatasize,
                                            pwait->wantedfixedsize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_IPC:
      ret =
        lcrzo_priv_storedtlvdata_contain_fixed(((lcrzo_ipc *) (pwait->
                                                               psource))->
                                               storeddata,
                                               ((lcrzo_ipc *) (pwait->
                                                               psource))->
                                               storeddatasize,
                                               pwait->wantedfixedsize,
                                               pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SNIFF:
      ret = lcrzo_ie("lcrzo_priv_wait_fixedok",
                     "should not wait for fixed in sniff",
                     "on ne devrait pas attendre pour du fixed lors du sniff");
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SOCK:
      ret =
        lcrzo_priv_storedtlvdata_contain_fixed(((lcrzo_sock *) (pwait->
                                                                psource))->
                                               storeddata,
                                               ((lcrzo_sock *) (pwait->
                                                                psource))->
                                               storeddatasize,
                                               pwait->wantedfixedsize,
                                               pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_FD2:
      ret =
        lcrzo_priv_storeddata_contain_fixed(((lcrzo_fd2 *) (pwait->psource))->
                                            storeddata,
                                            ((lcrzo_fd2 *) (pwait->psource))->
                                            storeddatasize,
                                            pwait->wantedfixedsize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE:
      ret =
        lcrzo_priv_storeddata_contain_fixed(((lcrzo_winhandle *) (pwait->
                                                                  psource))->
                                            storeddata,
                                            ((lcrzo_winhandle *) (pwait->
                                                                  psource))->
                                            storeddatasize,
                                            pwait->wantedfixedsize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_KBD:
      ret =
        lcrzo_priv_storeddata_contain_fixed(((lcrzo_kbd *) (pwait->psource))->
                                            storeddata,
                                            ((lcrzo_kbd *) (pwait->psource))->
                                            storeddatasize,
                                            pwait->wantedfixedsize, pcanread);
      break;
    default:
      ret = lcrzo_ieuv("lcrzo_priv_wait_check_read_fixed", "pwait->source");
  }

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_check_read_line(lcrzo_wait * pwait,
                                    lcrzo_bool * pcanread);
int lcrzo_priv_wait_check_read_line(lcrzo_wait * pwait,
                                    lcrzo_bool * pcanread)
{
  int ret;

  if (pwait->action != LCRZO_PRIV_WAIT_ACTION_READ_LINE) {
    if (pcanread != NULL)
      *pcanread = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  switch (pwait->source) {
    case LCRZO_PRIV_WAIT_SOURCE_FILE:
      ret =
        lcrzo_priv_storeddata_contain_line(((lcrzo_file *) (pwait->psource))->
                                           storeddata,
                                           ((lcrzo_file *) (pwait->psource))->
                                           storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_IPC:
      ret =
        lcrzo_priv_storedtlvdata_contain_line(((lcrzo_ipc *) (pwait->
                                                              psource))->
                                              storeddata,
                                              ((lcrzo_ipc *) (pwait->
                                                              psource))->
                                              storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SNIFF:
      ret = lcrzo_ie("lcrzo_priv_wait_lineok",
                     "should not wait for line in sniff",
                     "on ne devrait pas attendre pour une ligne lors du sniff");
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SOCK:
      ret =
        lcrzo_priv_storedtlvdata_contain_line(((lcrzo_sock *) (pwait->
                                                               psource))->
                                              storeddata,
                                              ((lcrzo_sock *) (pwait->
                                                               psource))->
                                              storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_FD2:
      ret =
        lcrzo_priv_storeddata_contain_line(((lcrzo_fd2 *) (pwait->psource))->
                                           storeddata,
                                           ((lcrzo_fd2 *) (pwait->psource))->
                                           storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE:
      ret =
        lcrzo_priv_storeddata_contain_line(((lcrzo_winhandle *) (pwait->
                                                                 psource))->
                                           storeddata,
                                           ((lcrzo_winhandle *) (pwait->
                                                                 psource))->
                                           storeddatasize, pcanread);
      break;
    case LCRZO_PRIV_WAIT_SOURCE_KBD:
      ret =
        lcrzo_priv_storeddata_contain_line(((lcrzo_kbd *) (pwait->psource))->
                                           storeddata,
                                           ((lcrzo_kbd *) (pwait->psource))->
                                           storeddatasize, pcanread);
      break;
    default:
      ret = lcrzo_ieuv("lcrzo_priv_wait_check_read_line", "pwait->source");
  }

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_check_end(lcrzo_wait * pwait,
                              lcrzo_bool * pisatend);
int lcrzo_priv_wait_check_end(lcrzo_wait * pwait,
                              lcrzo_bool * pisatend)
{
  lcrzo_bool contain, isatend;

  switch (pwait->source) {
    case LCRZO_PRIV_WAIT_SOURCE_FILE:
      if (((lcrzo_file *) (pwait->psource))->dataend &&
          ((lcrzo_file *) (pwait->psource))->storeddatasize == 0) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      if (((lcrzo_file *) (pwait->psource))->dataend &&
          pwait->action == LCRZO_PRIV_WAIT_ACTION_READ_LINE) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      if (((lcrzo_file *) (pwait->psource))->dataend &&
          pwait->action == LCRZO_PRIV_WAIT_ACTION_READ_FIXED) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_IPC:
      lcrzo_er(lcrzo_priv_storedtlvdata_contain
               (((lcrzo_ipc *) (pwait->psource))->storeddata,
                ((lcrzo_ipc *) (pwait->psource))->storeddatasize, &contain));
      if (((lcrzo_ipc *) (pwait->psource))->dataend && !contain) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      lcrzo_er(lcrzo_priv_storedtlvdata_isatend
               (((lcrzo_ipc *) (pwait->psource))->storeddata,
                ((lcrzo_ipc *) (pwait->psource))->storeddatasize, &isatend));
      if (isatend) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SNIFF:
      if (((lcrzo_sniff *) (pwait->psource))->dataend &&
          ((lcrzo_file *) (pwait->psource))->storeddatasize == 0) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_SOCK:
      lcrzo_er(lcrzo_priv_storedtlvdata_contain
               (((lcrzo_sock *) (pwait->psource))->storeddata,
                ((lcrzo_sock *) (pwait->psource))->storeddatasize, &contain));
      if (((lcrzo_sock *) (pwait->psource))->dataend && !contain) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      lcrzo_er(lcrzo_priv_storedtlvdata_isatend
               (((lcrzo_sock *) (pwait->psource))->storeddata,
                ((lcrzo_sock *) (pwait->psource))->storeddatasize, &isatend));
      if (isatend) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_PROCESS:
      if (((lcrzo_process *) (pwait->psource))->hasended) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_FD2:
      if (((lcrzo_fd2 *) (pwait->psource))->dataend &&
          ((lcrzo_fd2 *) (pwait->psource))->storeddatasize == 0) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      if (((lcrzo_fd2 *) (pwait->psource))->dataend &&
          pwait->action == LCRZO_PRIV_WAIT_ACTION_READ_LINE) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      if (((lcrzo_fd2 *) (pwait->psource))->dataend &&
          pwait->action == LCRZO_PRIV_WAIT_ACTION_READ_FIXED) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_WINHANDLE:
      if (((lcrzo_winhandle *) (pwait->psource))->dataend &&
          ((lcrzo_winhandle *) (pwait->psource))->storeddatasize == 0) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      if (((lcrzo_winhandle *) (pwait->psource))->dataend &&
          pwait->action == LCRZO_PRIV_WAIT_ACTION_READ_LINE) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      if (((lcrzo_winhandle *) (pwait->psource))->dataend &&
          pwait->action == LCRZO_PRIV_WAIT_ACTION_READ_FIXED) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      break;
    case LCRZO_PRIV_WAIT_SOURCE_KBD:
      if (((lcrzo_kbd *) (pwait->psource))->dataend &&
          ((lcrzo_kbd *) (pwait->psource))->storeddatasize == 0) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      if (((lcrzo_kbd *) (pwait->psource))->dataend &&
          pwait->action == LCRZO_PRIV_WAIT_ACTION_READ_LINE) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      if (((lcrzo_kbd *) (pwait->psource))->dataend &&
          pwait->action == LCRZO_PRIV_WAIT_ACTION_READ_FIXED) {
        if (pisatend != NULL)
          *pisatend = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      break;
    default:
      lcrzo_er(lcrzo_ieuv("lcrzo_priv_wait_check_end", "pwait->source"));
  }

  if (pisatend != NULL)
    *pisatend = LCRZO_FALSE;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_wait(lcrzo_wait * pwait,
                         lcrzo_time * pmaxwaitingtime,
                         lcrzo_bool * pevent);
int lcrzo_priv_wait_wait(lcrzo_wait * pwait,
                         lcrzo_time * pmaxwaitingtime,
                         lcrzo_bool * pevent)
{
  lcrzo_bool event, continuetowait, b1, b2, b3, b4;
  lcrzo_time starttime, timediff, waitingtime;

  /* parameters verification */
  if (pwait == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pmaxwaitingtime == NULL)
    return (LCRZO_ERR_PANULLPTR);
  switch (pwait->action) {
    case LCRZO_PRIV_WAIT_ACTION_READ:
    case LCRZO_PRIV_WAIT_ACTION_READ_FIXED:
    case LCRZO_PRIV_WAIT_ACTION_READ_LINE:
    case LCRZO_PRIV_WAIT_ACTION_END:
    case LCRZO_PRIV_WAIT_ACTION_EVENT:
      break;
    default:
      return (lcrzo_ieuv("lcrzo_priv_wait_wait", "pwait->action"));
  }

  /* initialize time */
  lcrzo_er(lcrzo_time_init_currenttime(&starttime));
  lcrzo_er(lcrzo_time_init_time(pmaxwaitingtime, &waitingtime));

  /* first, check if it's not already ok */
  if (pwait->action != LCRZO_PRIV_WAIT_ACTION_EVENT) {
    lcrzo_er(lcrzo_priv_wait_check_end(pwait, &b1));
    lcrzo_er(lcrzo_priv_wait_check_read(pwait, &b2));
    lcrzo_er(lcrzo_priv_wait_check_read_fixed(pwait, &b3));
    lcrzo_er(lcrzo_priv_wait_check_read_line(pwait, &b4));
    if (b1 || b2 || b3 || b4) { /* sufficient data was read */
      if (pevent != NULL)
        *pevent = LCRZO_TRUE;
#if LCRZO_DEBUG_WAIT_WAIT==1
      printf("Already OK %d %d %d %d\n", b1, b2, b3, b4);
#endif
      return (LCRZO_ERR_OK);
    }
  }

  /* then, loop */
  continuetowait = LCRZO_TRUE;
  do {                          
    /* try to wait and read data on specific devices */
    lcrzo_er(lcrzo_priv_wait_independent_wait_read
             (pwait, &waitingtime, &event));
    /* data was read */
    if (event) {
      if (pwait->action == LCRZO_PRIV_WAIT_ACTION_EVENT) {      
        /* special case, can stop here */
        if (pevent != NULL)
          *pevent = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      } else {
        lcrzo_er(lcrzo_priv_wait_check_end(pwait, &b1));
        lcrzo_er(lcrzo_priv_wait_check_read(pwait, &b2));
        lcrzo_er(lcrzo_priv_wait_check_read_fixed(pwait, &b3));
        lcrzo_er(lcrzo_priv_wait_check_read_line(pwait, &b4));
        if (b1 || b2 || b3 || b4) {     
          /* sufficient data was read */
          if (pevent != NULL)
            *pevent = LCRZO_TRUE;
#if LCRZO_DEBUG_WAIT_WAIT==1
          printf("Finally OK %d %d %d %d\n", b1, b2, b3, b4);
#endif
          return (LCRZO_ERR_OK);
        }
      }
    }
    /* check if we can exit */
    if (pmaxwaitingtime->sec == LCRZO_WAIT_NODELAY_VAL && pmaxwaitingtime->usec == LCRZO_WAIT_NODELAY_VAL) {    
      /* just one test */
      continuetowait = LCRZO_FALSE;
    } else if (!(pmaxwaitingtime->sec == LCRZO_WAIT_INFINITE_VAL && pmaxwaitingtime->usec == LCRZO_WAIT_INFINITE_VAL)) {        
      /* it's time to exit */
      lcrzo_er(lcrzo_time_diffcurrent_time(&starttime, &timediff));
      if (timediff.sec > pmaxwaitingtime->sec)
        continuetowait = LCRZO_FALSE;
      if (timediff.sec == pmaxwaitingtime->sec &&
          timediff.usec >= pmaxwaitingtime->usec)
        continuetowait = LCRZO_FALSE;
      /* set new waiting time */
      if (continuetowait) {
        lcrzo_er(lcrzo_time_minus_time(pmaxwaitingtime, &timediff,
                                       &waitingtime));
      }
    }
    if (continuetowait) {       
      /* really, it can sleep more than one millisecond */
      lcrzo_er(lcrzo_time_sleep_msec(1));
    }
  }
  while (continuetowait);

  /* if we are here, no sufficient data was read */
  if (pevent != NULL)
    *pevent = LCRZO_FALSE;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_list_wait(lcrzo_list * pwaitlist,
                              lcrzo_time * pmaxwaitingtime,
                              lcrzo_bool waitforall,
                              lcrzo_bool * pevent,
                              lcrzo_int32 * pfirsteventindex,
                              lcrzo_list * plistofalleventindexes);
int lcrzo_priv_wait_list_wait(lcrzo_list * pwaitlist,
                              lcrzo_time * pmaxwaitingtime,
                              lcrzo_bool waitforall,
                              lcrzo_bool * pevent,
                              lcrzo_int32 * pfirsteventindex,
                              lcrzo_list * plistofalleventindexes)
{
  lcrzo_time starttime, timediff;
  lcrzo_priv_waitlistitem wli;
  lcrzo_bool event, continuetocheck, eventonall, atleastoneevent;
  lcrzo_int32 i, waitlistcount;

  /* parameters verification */
  if (pmaxwaitingtime == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*Developper note : this algorithm it not as good as using 
     WaitForMultipleObjects.
     However, at the time of this writing, I don't know how to mix handles
     from lcrzo_sniff, or lcrzo_sock, because these modules are not yet 
     ported to Windows. 
     So this is an intermediate solution which should be sufficient for the 
     earliest versions of the Windows port.
   */

  /* initialize the number of data sources */
  lcrzo_er(lcrzo_list_count(pwaitlist, &waitlistcount));
  if (waitlistcount == 0)
    return (LCRZO_ERR_LOWAITLISTEMPTY);

  /* initialize time */
  lcrzo_er(lcrzo_time_init_currenttime(&starttime));

  /* check */
  atleastoneevent = LCRZO_FALSE;
  continuetocheck = LCRZO_TRUE;
  eventonall = LCRZO_FALSE;
  while (continuetocheck) {
    /* loop through each data source */
    eventonall = LCRZO_TRUE;
    for (i = 1; i <= waitlistcount; i++) {      
      /* select on which event to work */
      lcrzo_er(lcrzo_list_value_pos(pwaitlist, i, &wli));
      /* verify if it contains something to read */
      if (waitlistcount == 1) {
        lcrzo_er(lcrzo_priv_wait_wait(&wli.item, pmaxwaitingtime, &event));
      } else {
        lcrzo_er(lcrzo_priv_wait_wait(&wli.item, LCRZO_WAIT_NODELAY, &event));
      }
#if LCRZO_DEBUG_WAIT_WAIT==1
      printf("Item %ld : event=%d\n", i, event);
#endif
      if (event)
        atleastoneevent = LCRZO_TRUE;
      if (waitforall && !event)
        eventonall = LCRZO_FALSE;
      /* replace list item (even if event is false, because
         it might previously have been true) */
      wli.event = event;
      lcrzo_er(lcrzo_list_replace_pos(pwaitlist, i, &wli));
    }

    /* treat special exit cases */
    if (!waitforall && atleastoneevent) {       
      /* wait for one device succeded */
      continuetocheck = LCRZO_FALSE;
    } else if (waitforall && eventonall) {      
      /* wait for all devices succeded */
      continuetocheck = LCRZO_FALSE;
    } else if (pmaxwaitingtime->sec == LCRZO_WAIT_NODELAY_VAL && pmaxwaitingtime->usec == LCRZO_WAIT_NODELAY_VAL) {
      /* just one test */
      continuetocheck = LCRZO_FALSE;
    } else if (!(pmaxwaitingtime->sec == LCRZO_WAIT_INFINITE_VAL && pmaxwaitingtime->usec == LCRZO_WAIT_INFINITE_VAL)) {
      /* it's time to exit */
      lcrzo_er(lcrzo_time_diffcurrent_time(&starttime, &timediff));
      if (timediff.sec > pmaxwaitingtime->sec)
        continuetocheck = LCRZO_FALSE;
      if (timediff.sec == pmaxwaitingtime->sec
          && timediff.usec >= pmaxwaitingtime->usec)
        continuetocheck = LCRZO_FALSE;
    } else {
      /* infinite */
    }
    /* just a little sleep (otherwise, this loop may consume
       a lot of ressources) */
    if (continuetocheck) {
      /* really, it can sleep more than one millisecond */
      lcrzo_er(lcrzo_time_sleep_msec(1));
    }
  }

  /* set output values */
  if (!waitforall && atleastoneevent) {
    if (pevent != NULL)
      *pevent = LCRZO_TRUE;
    /* loop to set the eventindexes list */
    if (plistofalleventindexes != NULL) {
      lcrzo_er(lcrzo_list_init
               (plistofalleventindexes, sizeof(lcrzo_int32), NULL));
    }
    for (i = waitlistcount; i >= 1; i--) {
      /* select on which event to work */
      lcrzo_er(lcrzo_list_value_pos(pwaitlist, i, &wli));
      if (wli.event) {
        if (pfirsteventindex != NULL)
          *pfirsteventindex = wli.index;
        if (plistofalleventindexes != NULL) {
          lcrzo_er(lcrzo_list_add_first(plistofalleventindexes, &wli.index));
        }
      }
    }
  } else if (waitforall && eventonall) {
    if (pevent != NULL)
      *pevent = LCRZO_TRUE;
  } else {
    if (pevent != NULL)
      *pevent = LCRZO_FALSE;
    /* do not initialize pfirsteventindex or plistofalleventindexes */
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

/*-------------------------------------------------------------*/
int lcrzo_wait_init_process_end(lcrzo_process * pprocess,
                                lcrzo_wait * pwait)
{
  if (pprocess == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_PROCESS;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_END;
    pwait->psource = pprocess;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_file_read(lcrzo_file * pfile,
                              lcrzo_wait * pwait)
{
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if file is open for reading */
  if (!pfile->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_FILE;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ;
    pwait->psource = pfile;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_file_read_fixed(lcrzo_file * pfile,
                                    lcrzo_int32 wantedfixedsize,
                                    lcrzo_wait * pwait)
{
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if file is open for reading */
  if (!pfile->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_FILE;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ_FIXED;
    pwait->wantedfixedsize = wantedfixedsize;
    pwait->psource = pfile;
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_file_read_line(lcrzo_file * pfile,
                                   lcrzo_wait * pwait)
{
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if file is open for reading */
  if (!pfile->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_FILE;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ_LINE;
    pwait->psource = pfile;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_file_end(lcrzo_file * pfile,
                             lcrzo_wait * pwait)
{
  if (pfile == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_FILE;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_END;
    pwait->psource = pfile;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_ipc_read(lcrzo_ipc * pipc,
                             lcrzo_wait * pwaitread)
{
  if (pipc == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if ipc is open for reading */
  if (!pipc->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwaitread != NULL) {
    pwaitread->source = LCRZO_PRIV_WAIT_SOURCE_IPC;
    pwaitread->action = LCRZO_PRIV_WAIT_ACTION_READ;
    pwaitread->psource = pipc;
    lcrzo_debug_notset_int32(pwaitread->wantedfixedsize);
    pwaitread->pfunc = NULL;
    pwaitread->infos = NULL;
    pwaitread->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_ipc_read_fixed(lcrzo_ipc * pipc,
                                   lcrzo_int32 wantedfixedsize,
                                   lcrzo_wait * pwaitread)
{
  if (pipc == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if ipc is open for reading */
  if (!pipc->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwaitread != NULL) {
    pwaitread->source = LCRZO_PRIV_WAIT_SOURCE_IPC;
    pwaitread->action = LCRZO_PRIV_WAIT_ACTION_READ_FIXED;
    pwaitread->wantedfixedsize = wantedfixedsize;
    pwaitread->psource = pipc;
    pwaitread->pfunc = NULL;
    pwaitread->infos = NULL;
    pwaitread->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_ipc_read_line(lcrzo_ipc * pipc,
                                  lcrzo_wait * pwaitread)
{
  if (pipc == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if ipc is open for reading */
  if (!pipc->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwaitread != NULL) {
    pwaitread->source = LCRZO_PRIV_WAIT_SOURCE_IPC;
    pwaitread->action = LCRZO_PRIV_WAIT_ACTION_READ_LINE;
    pwaitread->psource = pipc;
    lcrzo_debug_notset_int32(pwaitread->wantedfixedsize);
    pwaitread->pfunc = NULL;
    pwaitread->infos = NULL;
    pwaitread->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_ipc_end(lcrzo_ipc * pipc,
                            lcrzo_wait * pwaitread)
{
  if (pipc == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwaitread != NULL) {
    pwaitread->source = LCRZO_PRIV_WAIT_SOURCE_IPC;
    pwaitread->action = LCRZO_PRIV_WAIT_ACTION_END;
    pwaitread->psource = pipc;
    lcrzo_debug_notset_int32(pwaitread->wantedfixedsize);
    pwaitread->pfunc = NULL;
    pwaitread->infos = NULL;
    pwaitread->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_sniff_read(lcrzo_sniff * psniff,
                               lcrzo_wait * pwait)
{
  if (psniff == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_SNIFF;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ;
    pwait->psource = psniff;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_sock_read(lcrzo_sock * psock,
                              lcrzo_wait * pwaitread)
{
  if (psock == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwaitread != NULL) {
    pwaitread->source = LCRZO_PRIV_WAIT_SOURCE_SOCK;
    pwaitread->action = LCRZO_PRIV_WAIT_ACTION_READ;
    pwaitread->psource = psock;
    lcrzo_debug_notset_int32(pwaitread->wantedfixedsize);
    pwaitread->pfunc = NULL;
    pwaitread->infos = NULL;
    pwaitread->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_sock_read_fixed(lcrzo_sock * psock,
                                    lcrzo_int32 wantedfixedsize,
                                    lcrzo_wait * pwaitread)
{
  if (psock == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwaitread != NULL) {
    pwaitread->source = LCRZO_PRIV_WAIT_SOURCE_SOCK;
    pwaitread->action = LCRZO_PRIV_WAIT_ACTION_READ_FIXED;
    pwaitread->wantedfixedsize = wantedfixedsize;
    pwaitread->psource = psock;
    pwaitread->pfunc = NULL;
    pwaitread->infos = NULL;
    pwaitread->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_sock_read_line(lcrzo_sock * psock,
                                   lcrzo_wait * pwaitread)
{
  if (psock == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwaitread != NULL) {
    pwaitread->source = LCRZO_PRIV_WAIT_SOURCE_SOCK;
    pwaitread->action = LCRZO_PRIV_WAIT_ACTION_READ_LINE;
    pwaitread->psource = psock;
    lcrzo_debug_notset_int32(pwaitread->wantedfixedsize);
    pwaitread->pfunc = NULL;
    pwaitread->infos = NULL;
    pwaitread->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_sock_end(lcrzo_sock * psock,
                             lcrzo_wait * pwaitread)
{
  if (psock == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwaitread != NULL) {
    pwaitread->source = LCRZO_PRIV_WAIT_SOURCE_SOCK;
    pwaitread->action = LCRZO_PRIV_WAIT_ACTION_END;
    pwaitread->psource = psock;
    lcrzo_debug_notset_int32(pwaitread->wantedfixedsize);
    pwaitread->pfunc = NULL;
    pwaitread->infos = NULL;
    pwaitread->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_fd2_read(lcrzo_fd2 * pfd2,
                             lcrzo_wait * pwait)
{
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if fd2 is open for reading */
  if (!pfd2->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_FD2;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ;
    pwait->psource = pfd2;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_fd2_read_fixed(lcrzo_fd2 * pfd2,
                                   lcrzo_int32 wantedfixedsize,
                                   lcrzo_wait * pwait)
{
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if fd2 is open for reading */
  if (!pfd2->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_FD2;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ_FIXED;
    pwait->wantedfixedsize = wantedfixedsize;
    pwait->psource = pfd2;
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_fd2_read_line(lcrzo_fd2 * pfd2,
                                  lcrzo_wait * pwait)
{
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if fd2 is open for reading */
  if (!pfd2->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_FD2;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ_LINE;
    pwait->psource = pfd2;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_fd2_end(lcrzo_fd2 * pfd2,
                            lcrzo_wait * pwait)
{
  if (pfd2 == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_FD2;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_END;
    pwait->psource = pfd2;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_winhandle_read(lcrzo_winhandle * pwinhandle,
                                   lcrzo_wait * pwait)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  pwait = pwait;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if winhandle is open for reading */
  if (!pwinhandle->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_WINHANDLE;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ;
    pwait->psource = pwinhandle;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
/*-------------------------------------------------------------*/
     int lcrzo_wait_init_winhandle_read_fixed(lcrzo_winhandle * pwinhandle,
                                              lcrzo_int32 wantedfixedsize,
                                              lcrzo_wait * pwait)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  wantedfixedsize = wantedfixedsize;
  pwait = pwait;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if winhandle is open for reading */
  if (!pwinhandle->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_WINHANDLE;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ_FIXED;
    pwait->wantedfixedsize = wantedfixedsize;
    pwait->psource = pwinhandle;
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
/*-------------------------------------------------------------*/
     int lcrzo_wait_init_winhandle_read_line(lcrzo_winhandle * pwinhandle,
                                             lcrzo_wait * pwait)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  pwait = pwait;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if winhandle is open for reading */
  if (!pwinhandle->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_WINHANDLE;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ_LINE;
    pwait->psource = pwinhandle;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
/*-------------------------------------------------------------*/
     int lcrzo_wait_init_winhandle_end(lcrzo_winhandle * pwinhandle,
                                       lcrzo_wait * pwait)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  pwait = pwait;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_WINHANDLE;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_END;
    pwait->psource = pwinhandle;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
/*-------------------------------------------------------------*/
     int lcrzo_wait_init_kbd_read(lcrzo_kbd * pkbd,
                                  lcrzo_wait * pwait)
{
  if (pkbd == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if kbd is open for reading */
  if (!pkbd->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_KBD;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ;
    pwait->psource = pkbd;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_kbd_read_fixed(lcrzo_kbd * pkbd,
                                   lcrzo_int32 wantedfixedsize,
                                   lcrzo_wait * pwait)
{
  if (pkbd == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if kbd is open for reading */
  if (!pkbd->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_KBD;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ_FIXED;
    pwait->wantedfixedsize = wantedfixedsize;
    pwait->psource = pkbd;
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_kbd_read_line(lcrzo_kbd * pkbd,
                                  lcrzo_wait * pwait)
{
  if (pkbd == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* check if kbd is open for reading */
  if (!pkbd->openforreading) {
    return (LCRZO_ERR_LONOTOPENFORREADING);
  }

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_KBD;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_READ_LINE;
    pwait->psource = pkbd;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_kbd_end(lcrzo_kbd * pkbd,
                            lcrzo_wait * pwait)
{
  if (pkbd == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_KBD;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_END;
    pwait->psource = pkbd;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = NULL;
    pwait->infos = NULL;
    pwait->infossize = 0;
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#include "sysdep/wait_init_fd_event.c"

/*-------------------------------------------------------------*/
#include "sysdep/wait_init_handle_event.c"

/*-------------------------------------------------------------*/
int lcrzo_wait_init_function_event(int (*pfunc) (lcrzo_constptr infos,
                                                 lcrzo_int32 infossize,
                                                 lcrzo_time * pmaxwaitingtime,
                                                 lcrzo_bool * phasevent),
                                   lcrzo_constptr infos,
                                   lcrzo_int32 infossize,
                                   lcrzo_wait * pwait)
{
  if (pfunc == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (infos == NULL && infossize)
    return (LCRZO_ERR_PANULLPTRSIZE);

  if (pwait != NULL) {
    pwait->source = LCRZO_PRIV_WAIT_SOURCE_FUNCTION;
    pwait->action = LCRZO_PRIV_WAIT_ACTION_EVENT;
    pwait->psource = NULL;
    lcrzo_debug_notset_int32(pwait->wantedfixedsize);
    pwait->pfunc = pfunc;
    lcrzo_er(lcrzo_data_initm_data((lcrzo_constdata) infos, infossize,
                                   &pwait->infos, &pwait->infossize));
  }
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_wait_close(lcrzo_wait * pwait)
{
  if (pwait == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pwait->infos == NULL && pwait->infossize)
    return (LCRZO_ERR_PANULLPTRSIZE);

  pwait->source = 0;
  pwait->action = 0;
  pwait->psource = NULL;
  lcrzo_debug_notset_int32(pwait->wantedfixedsize);
  if (pwait->infos != NULL) {
    lcrzo_er(lcrzo_data_free2(&pwait->infos));
  }

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
lcrzo_time *lcrzo__time_init_wait_nodelay(void)
{
  static lcrzo_time none;

  /* Can use static even in multithread because store always the same value */
  none.sec = LCRZO_WAIT_NODELAY_VAL;
  none.usec = LCRZO_WAIT_NODELAY_VAL;
  return (&none);
}

/*-------------------------------------------------------------*/
lcrzo_time *lcrzo__time_init_wait_infinite(void)
{
  static lcrzo_time infinite;

  /* Can use static even in multithread because store always the same value */
  infinite.sec = LCRZO_WAIT_INFINITE_VAL;
  infinite.usec = LCRZO_WAIT_INFINITE_VAL;
  return (&infinite);
}


/*-------------------------------------------------------------*/
/*int lcrzo_priv_wait_list_init(lcrzo_ptr pelem);
int lcrzo_priv_wait_list_init(lcrzo_ptr pelem)
{ lcrzo_priv_waitlistitem *pwli=(lcrzo_priv_waitlistitem *)pelem;
  return(lcrzo_wait_close(&pwli->item));
}*/
int lcrzo_wait_list_init(lcrzo_list * pwaitlist)
{
  lcrzo_er(lcrzo_list_init(pwaitlist, sizeof(lcrzo_priv_waitlistitem), NULL));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_list_add(lcrzo_list * pwaitlist,
                        lcrzo_int32 itemindex,
                        lcrzo_wait * pwait)
{
  lcrzo_priv_waitlistitem wli;

  if (pwaitlist == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pwait == NULL)
    return (LCRZO_ERR_PANULLPTR);

  wli.index = itemindex;
  wli.item = *pwait;
  wli.event = LCRZO_FALSE;

  lcrzo_er(lcrzo_list_add_last(pwaitlist, &wli));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_list_del(lcrzo_constptr pelem,
                             lcrzo_ptr pinfos,
                             lcrzo_bool * pyes);
int lcrzo_priv_wait_list_del(lcrzo_constptr pelem,
                             lcrzo_ptr pinfos,
                             lcrzo_bool * pyes)
{
  const lcrzo_priv_waitlistitem *pwli =
    (const lcrzo_priv_waitlistitem *)pelem;
  lcrzo_int32 *pi = (lcrzo_int32 *) pinfos;
  lcrzo_bool yes;

  yes = LCRZO_FALSE;
  if (pwli->index == *pi)
    yes = LCRZO_TRUE;
  if (pyes != NULL)
    *pyes = yes;
  return (LCRZO_ERR_OK);
}

int lcrzo_wait_list_del(lcrzo_list * pwaitlist,
                        lcrzo_int32 itemindex)
{
  lcrzo_er(lcrzo_list_remove_criteria_all2(pwaitlist,
                                           &lcrzo_priv_wait_list_del,
                                           &itemindex));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_list_wait(lcrzo_list * pwaitlist,
                         lcrzo_time * pmaxwaitingtime,
                         lcrzo_bool * pevent,
                         lcrzo_int32 * pfirsteventindex,
                         lcrzo_list * plistofalleventindexes)
{
  return (lcrzo_priv_wait_list_wait(pwaitlist, pmaxwaitingtime, LCRZO_FALSE,
                                    pevent, pfirsteventindex,
                                    plistofalleventindexes));
}

/*-------------------------------------------------------------*/
int lcrzo_wait_list_wait_all(lcrzo_list * pwaitlist,
                             lcrzo_time * pmaxwaitingtime,
                             lcrzo_bool * pevent)
{
  return (lcrzo_priv_wait_list_wait(pwaitlist, pmaxwaitingtime, LCRZO_TRUE,
                                    pevent, NULL, NULL));
}


/*-------------------------------------------------------------*/
int lcrzo_wait_wait1(lcrzo_wait * pwait1,
                     lcrzo_time * pmaxwaitingtime,
                     lcrzo_bool * pevent1)
{
  return (lcrzo_wait_wait5(pwait1, NULL, NULL, NULL, NULL,
                           pmaxwaitingtime, pevent1, NULL, NULL, NULL, NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_wait_wait2(lcrzo_wait * pwait1,
                     lcrzo_wait * pwait2,
                     lcrzo_time * pmaxwaitingtime,
                     lcrzo_bool * pevent1,
                     lcrzo_bool * pevent2)
{
  return (lcrzo_wait_wait5(pwait1, pwait2, NULL, NULL, NULL,
                           pmaxwaitingtime, pevent1, pevent2, NULL, NULL,
                           NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_wait_wait2_all(lcrzo_wait * pwait1,
                         lcrzo_wait * pwait2,
                         lcrzo_time * pmaxwaitingtime,
                         lcrzo_bool * pevent)
{
  return (lcrzo_wait_wait5_all(pwait1, pwait2, NULL, NULL, NULL,
                               pmaxwaitingtime, pevent));
}

/*-------------------------------------------------------------*/
int lcrzo_wait_wait3(lcrzo_wait * pwait1,
                     lcrzo_wait * pwait2,
                     lcrzo_wait * pwait3,
                     lcrzo_time * pmaxwaitingtime,
                     lcrzo_bool * pevent1,
                     lcrzo_bool * pevent2,
                     lcrzo_bool * pevent3)
{
  return (lcrzo_wait_wait5(pwait1, pwait2, pwait3, NULL, NULL,
                           pmaxwaitingtime, pevent1, pevent2, pevent3, NULL,
                           NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_wait_wait3_all(lcrzo_wait * pwait1,
                         lcrzo_wait * pwait2,
                         lcrzo_wait * pwait3,
                         lcrzo_time * pmaxwaitingtime,
                         lcrzo_bool * pevent)
{
  return (lcrzo_wait_wait5_all(pwait1, pwait2, pwait3, NULL, NULL,
                               pmaxwaitingtime, pevent));
}

/*-------------------------------------------------------------*/
int lcrzo_wait_wait4(lcrzo_wait * pwait1,
                     lcrzo_wait * pwait2,
                     lcrzo_wait * pwait3,
                     lcrzo_wait * pwait4,
                     lcrzo_time * pmaxwaitingtime,
                     lcrzo_bool * pevent1,
                     lcrzo_bool * pevent2,
                     lcrzo_bool * pevent3,
                     lcrzo_bool * pevent4)
{
  return (lcrzo_wait_wait5(pwait1, pwait2, pwait3, pwait4, NULL,
                           pmaxwaitingtime, pevent1, pevent2, pevent3,
                           pevent4, NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_wait_wait4_all(lcrzo_wait * pwait1,
                         lcrzo_wait * pwait2,
                         lcrzo_wait * pwait3,
                         lcrzo_wait * pwait4,
                         lcrzo_time * pmaxwaitingtime,
                         lcrzo_bool * pevent)
{
  return (lcrzo_wait_wait5_all(pwait1, pwait2, pwait3, pwait4, NULL,
                               pmaxwaitingtime, pevent));
}

/*-------------------------------------------------------------*/
int lcrzo_wait_wait5(lcrzo_wait * pwait1,
                     lcrzo_wait * pwait2,
                     lcrzo_wait * pwait3,
                     lcrzo_wait * pwait4,
                     lcrzo_wait * pwait5,
                     lcrzo_time * pmaxwaitingtime,
                     lcrzo_bool * pevent1,
                     lcrzo_bool * pevent2,
                     lcrzo_bool * pevent3,
                     lcrzo_bool * pevent4,
                     lcrzo_bool * pevent5)
{
  lcrzo_list list, eventlist;
  lcrzo_bool event;
  lcrzo_int32 count, i, itemindex;

  lcrzo_er(lcrzo_wait_list_init(&list));
  if (pwait1 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 1, pwait1));
  }
  if (pwait2 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 2, pwait2));
  }
  if (pwait3 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 3, pwait3));
  }
  if (pwait4 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 4, pwait4));
  }
  if (pwait5 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 5, pwait5));
  }
  lcrzo_er(lcrzo_priv_wait_list_wait(&list, pmaxwaitingtime, LCRZO_FALSE,
                                     &event, NULL, &eventlist));
  /* set default output values */
  if (pevent1 != NULL)
    *pevent1 = LCRZO_FALSE;
  if (pevent2 != NULL)
    *pevent2 = LCRZO_FALSE;
  if (pevent3 != NULL)
    *pevent3 = LCRZO_FALSE;
  if (pevent4 != NULL)
    *pevent4 = LCRZO_FALSE;
  if (pevent5 != NULL)
    *pevent5 = LCRZO_FALSE;
  /* set true events */
  if (event) {
    /* loop through eventlist */
    lcrzo_er(lcrzo_list_count(&eventlist, &count));
    for (i = 1; i <= count; i++) {
      lcrzo_er(lcrzo_list_value_pos(&eventlist, i, &itemindex));
      switch ((int)itemindex) {
        case 1:
          if (pevent1 != NULL)
            *pevent1 = LCRZO_TRUE;
          break;
        case 2:
          if (pevent2 != NULL)
            *pevent2 = LCRZO_TRUE;
          break;
        case 3:
          if (pevent3 != NULL)
            *pevent3 = LCRZO_TRUE;
          break;
        case 4:
          if (pevent4 != NULL)
            *pevent4 = LCRZO_TRUE;
          break;
        case 5:
          if (pevent5 != NULL)
            *pevent5 = LCRZO_TRUE;
          break;
        default:
          return (lcrzo_ieuv("lcrzo_wait_wait5", "itemindex"));
      }
    }
    lcrzo_er(lcrzo_list_close(&eventlist));
  }

  lcrzo_er(lcrzo_list_close(&list));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_wait5_all(lcrzo_wait * pwait1,
                         lcrzo_wait * pwait2,
                         lcrzo_wait * pwait3,
                         lcrzo_wait * pwait4,
                         lcrzo_wait * pwait5,
                         lcrzo_time * pmaxwaitingtime,
                         lcrzo_bool * pevent)
{
  lcrzo_list list;

  lcrzo_er(lcrzo_wait_list_init(&list));
  if (pwait1 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 1, pwait1));
  }
  if (pwait2 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 2, pwait2));
  }
  if (pwait3 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 3, pwait3));
  }
  if (pwait4 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 4, pwait4));
  }
  if (pwait5 != NULL) {
    lcrzo_er(lcrzo_wait_list_add(&list, 5, pwait5));
  }
  lcrzo_er(lcrzo_priv_wait_list_wait(&list, pmaxwaitingtime, LCRZO_TRUE,
                                     pevent, NULL, NULL));
  lcrzo_er(lcrzo_list_close(&list));
  return (LCRZO_ERR_OK);
}
