/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001,2002 Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
                   http://laurentconstantin.free.fr/
                   http://membres.lycos.fr/lauconstantin/
  [my current email address is on the web servers]

                                  -----
  This file is part of Lcrzo.

  Lcrzo is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Lcrzo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_incl.h"

/*-------------------------------------------------------------*/
#include "sysdep/priv_wait_winhandle_wait.c"

/*-------------------------------------------------------------*/
#include "sysdep/priv_wait_winhandle_read.c"

/*-------------------------------------------------------------*/
#include "sysdep/winhandle_init_handle.c"

/*-------------------------------------------------------------*/
int lcrzo_winhandle_close(lcrzo_winhandle * pwinhandle)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  if (pwinhandle == NULL)
    return (LCRZO_ERR_OK);

  /* file can not be used */
  pwinhandle->thehandle = NULL;
  pwinhandle->dataend = LCRZO_TRUE;
  pwinhandle->openforreading = LCRZO_FALSE;
  pwinhandle->openforwriting = LCRZO_FALSE;
  lcrzo_debug_notset_int16(pwinhandle->type);

  /* free data */
  if (pwinhandle->storeddata == NULL && pwinhandle->storeddatasize)
    return (LCRZO_ERR_PANULLPTRSIZE);
  if (pwinhandle->storeddatasize) {
    lcrzo_er(lcrzo_data_free2(&(pwinhandle->storeddata)));
    pwinhandle->storeddata = NULL;
    pwinhandle->storeddatasize = 0;
  }

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int lcrzo_handle_init_winhandle(lcrzo_winhandle * pwinhandle,
                                LCRZOHANDLE * phandle)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  phandle = phandle;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  /*parameters verification */
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (phandle != NULL) {
    *phandle = pwinhandle->thehandle;
  }

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int lcrzo_winhandle_write(lcrzo_winhandle * pwinhandle,
                          lcrzo_constdata data,
                          lcrzo_int32 datasize)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  data = data;
  datasize = datasize;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  /*parameters verification */
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (!pwinhandle->openforwriting)
    return (LCRZO_ERR_LONOTOPENFORWRITING);

  lcrzo_er(lcrzo_priv_handle_write(pwinhandle->thehandle, data, datasize));

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int lcrzo_winhandle_write_msg(lcrzo_winhandle * pwinhandle,
                              lcrzo_constdata data,
                              lcrzo_int32 datasize)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  data = data;
  datasize = datasize;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  /*parameters verification */
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  if (!pwinhandle->openforwriting)
    return (LCRZO_ERR_LONOTOPENFORWRITING);

  lcrzo_er(lcrzo_priv_handle_write_msg
           (pwinhandle->thehandle, data, datasize));

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int lcrzo_winhandle_readm(lcrzo_winhandle * pwinhandle,
                          lcrzo_time * pmaxwaitingtime,
                          lcrzo_data * pdataout,
                          lcrzo_int32 * pdataoutsize)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  pmaxwaitingtime = pmaxwaitingtime;
  pdataout = pdataout;
  pdataoutsize = pdataoutsize;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_winhandle_read(pwinhandle, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract
           (&(pwinhandle->storeddata), &(pwinhandle->storeddatasize),
            pwinhandle->dataend, pdataout, pdataoutsize));

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int lcrzo_winhandle_readm_fixed(lcrzo_winhandle * pwinhandle,
                                lcrzo_time * pmaxwaitingtime,
                                lcrzo_int32 datasizetoread,
                                lcrzo_data * pdataout)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  pmaxwaitingtime = pmaxwaitingtime;
  datasizetoread = datasizetoread;
  pdataout = pdataout;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_winhandle_read_fixed
           (pwinhandle, datasizetoread, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract_fixed(&(pwinhandle->storeddata),
                                               &(pwinhandle->storeddatasize),
                                               pwinhandle->dataend,
                                               datasizetoread, pdataout));

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
int lcrzo_winhandle_readm_line(lcrzo_winhandle * pwinhandle,
                               lcrzo_time * pmaxwaitingtime,
                               lcrzo_data * pdataout,
                               lcrzo_int32 * pdataoutsize)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pwinhandle = pwinhandle;
  pmaxwaitingtime = pmaxwaitingtime;
  pdataout = pdataout;
  pdataoutsize = pdataoutsize;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  lcrzo_wait waitread;
  lcrzo_bool mayread;

  /* parameter verification */
  if (pwinhandle == NULL)
    return (LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_wait_init_winhandle_read_line(pwinhandle, &waitread));
  lcrzo_er(lcrzo_wait_wait1(&waitread, pmaxwaitingtime, &mayread));
  lcrzo_er(lcrzo_wait_close(&waitread));
  if (!mayread)
    return (LCRZO_ERR_OKTEMPDATAEND);

  /* exit with data */
  lcrzo_er(lcrzo_priv_storeddata_extract_line(&(pwinhandle->storeddata),
                                              &(pwinhandle->storeddatasize),
                                              pwinhandle->dataend, pdataout,
                                              pdataoutsize));

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
