/* This file is part of Lcrzo.
*/

int lcrzo_dir_count(lcrzo_conststring dirname,
                    lcrzo_int32 * pcount)
#if defined LCRZODEF_SYSTEM_Unix
{
  DIR *thisdir;
  struct dirent *file;
  int ret;
  lcrzo_string filename;
  lcrzo_int32 count;
  lcrzo_string cleandirname;
  lcrzo_bool direxist;

  /* verify if directory exists */
  lcrzo_er(lcrzo_dir_exist(dirname, &direxist));
  if (!direxist)
    return (LCRZO_ERR_OKFILENOTFOUND);

  /*open the directory */
  lcrzo_er(lcrzo_path_clean(dirname, &cleandirname));
  thisdir = opendir(cleandirname);
  lcrzo_er(lcrzo_string_free2(&cleandirname));
  if (thisdir == NULL)
    return (LCRZO_ERR_FUOPENDIR);

  file = readdir(thisdir);
  count = 0;
  while (file != NULL) {
    lcrzo_er(lcrzo_string_initm_text(file->d_name, &filename));
    if (strcmp(filename, ".") && strcmp(filename, "..")) {
      /*printf("%s\n", filename); */
      count++;
    }
    lcrzo_er(lcrzo_string_free2(&filename));
    file = readdir(thisdir);
  }
  ret = closedir(thisdir);

  if (pcount != NULL)
    *pcount = count;
  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  struct _finddata_t fileinfos;
  lcrzo_int32 count;
  lcrzo_string filename;
  long hFile;
  int ret;
  lcrzo_bool direxist;

  /* verify if directory exists */
  lcrzo_er(lcrzo_dir_exist(dirname, &direxist));
  if (!direxist)
    return (LCRZO_ERR_OKFILENOTFOUND);

  /* search the first file */
  lcrzo_er(lcrzo_path_clean(dirname, &filename));
  if (filename[strlen(filename) - 1] == '/') {
    lcrzo_er(lcrzo_string_appendm_text("*", &filename));
  } else {
    lcrzo_er(lcrzo_string_appendm_text("\\*", &filename));
  }
  hFile = _findfirst(filename, &fileinfos);
  lcrzo_er(lcrzo_string_free2(&filename));
  if (hFile == -1)
    return (LCRZO_ERR_FUFINDFIRST);

  /* count files */
  count = 0;
  do {
    if (strcmp(fileinfos.name, ".") && strcmp(fileinfos.name, "..")) {
      count++;
    }
    ret = _findnext(hFile, &fileinfos);
  }
  while (ret == 0);
  lcrzo_er(lcrzo_priv_err_purge_int());

  /* close */
  ret = _findclose(hFile);
  if (ret == -1)
    return (LCRZO_ERR_FUFINDCLOSE);

  /* set output value */
  if (pcount != NULL)
    *pcount = count;
  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
