/* This file is part of Lcrzo.
*/

int lcrzo_dir_value_pos(lcrzo_conststring dirname,
                        lcrzo_int32 pos,
                        lcrzo_string * pfilename)
#if defined LCRZODEF_SYSTEM_Unix
{
  DIR *thisdir;
  struct dirent *file;
  int ret;
  lcrzo_string filename;
  lcrzo_string cleandirname;
  lcrzo_int32 count, pospos;
  lcrzo_bool direxist;

  /* verify if directory exists */
  lcrzo_er(lcrzo_dir_exist(dirname, &direxist));
  if (!direxist)
    return (LCRZO_ERR_OKFILENOTFOUND);

  if (pos > 0) {
    pospos = pos;
  } else {
    lcrzo_er(lcrzo_dir_count(dirname, &count));
    lcrzo_er(lcrzo_priv_manageelempos
             (count, pos, 0, NULL, &pospos, NULL, NULL));
  }

  /* open the directory */
  lcrzo_er(lcrzo_path_clean(dirname, &cleandirname));
  thisdir = opendir(cleandirname);
  lcrzo_debug_err_ok_activate(LCRZO_FALSE);
  lcrzo_er(lcrzo_string_free2(&cleandirname));
  lcrzo_debug_err_ok_activate(LCRZO_TRUE);
  if (thisdir == NULL)
    return (LCRZO_ERR_FUOPENDIR);

  file = readdir(thisdir);
  count = 0;
  while (file != NULL) {
    lcrzo_er(lcrzo_string_initm_text(file->d_name, &filename));
    if (strcmp(filename, ".") && strcmp(filename, "..")) {
      /*printf("%s\n", filename); */
      count++;
      if (count == pospos) {
        if (pfilename == NULL) {
          lcrzo_er(lcrzo_string_free2(&filename));
        } else
          *pfilename = filename;
        break;
      }
    }
    lcrzo_er(lcrzo_string_free2(&filename));
    file = readdir(thisdir);
  }
  ret = closedir(thisdir);
  if (file == NULL) {
    if (pos > 0)
      return (LCRZO_ERR_POTOOBIG);
    else
      return (LCRZO_ERR_POTOOLOW);
  }

  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  struct _finddata_t fileinfos;
  lcrzo_int32 count, pospos;
  lcrzo_string filename;
  lcrzo_bool direxist;
  long hFile;
  int ret, retf;

  /* verify if directory exists */
  lcrzo_er(lcrzo_dir_exist(dirname, &direxist));
  if (!direxist)
    return (LCRZO_ERR_OKFILENOTFOUND);

  if (pos > 0) {
    pospos = pos;
  } else {
    lcrzo_er(lcrzo_dir_count(dirname, &count));
    lcrzo_er(lcrzo_priv_manageelempos
             (count, pos, 0, NULL, &pospos, NULL, NULL));
  }

  /* open the directory */
  lcrzo_er(lcrzo_path_clean(dirname, &filename));
  if (filename[strlen(filename) - 1] == '/') {
    lcrzo_er(lcrzo_string_appendm_text("*", &filename));
  } else {
    lcrzo_er(lcrzo_string_appendm_text("\\*", &filename));
  }
  hFile = _findfirst(filename, &fileinfos);
  lcrzo_debug_err_ok_activate(LCRZO_FALSE);
  lcrzo_er(lcrzo_string_free2(&filename));
  lcrzo_debug_err_ok_activate(LCRZO_TRUE);
  if (hFile == -1)
    return (LCRZO_ERR_FUFINDFIRST);

  /* count files */
  count = 0;
  retf = 0;
  do {
    if (strcmp(fileinfos.name, ".") && strcmp(fileinfos.name, "..")) {
      count++;
      if (count == pospos) {
        lcrzo_er(lcrzo_string_initm_text(fileinfos.name, pfilename));
        break;
      }
    }
    retf = _findnext(hFile, &fileinfos);
  }
  while (retf == 0);
  lcrzo_er(lcrzo_priv_err_purge_int());

  /* close */
  ret = _findclose(hFile);
  if (ret == -1)
    return (LCRZO_ERR_FUFINDCLOSE);

  if (retf != 0) {
    if (pos > 0)
      return (LCRZO_ERR_POTOOBIG);
    else
      return (LCRZO_ERR_POTOOLOW);
  }

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
