/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_file_open_append(lcrzo_conststring filename,
                           lcrzo_bool mustbeanexistentfile,
                           lcrzo_file * pfile)
{
  int ret, fd;
  lcrzo_int16 language;
  struct stat st1, st2;
  lcrzo_bool fileexists;

  /*parameters verification */
  if (filename == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /*before opening, we verify if the file seems OK */
  ret = lcrzo_priv_path_stat_get(filename, &st1);
  if (ret == LCRZO_ERR_OK) {
    fileexists = LCRZO_TRUE;
    lcrzo_er(lcrzo_priv_stat_forbidslink(st1));
    lcrzo_er(lcrzo_priv_stat_forbidsnotreg(st1));
  } else if (ret == LCRZO_ERR_OKFILENOTFOUND) {
    fileexists = LCRZO_FALSE;
    lcrzo_er(lcrzo_priv_err_purge_text());
  } else {
    return (ret);
  }

  /*error cases */
  if (mustbeanexistentfile && !fileexists) {
    lcrzo_er(lcrzo_global_get_language(&language));
    if (language == LCRZO_GLOBAL_LANG_FR) {
      lcrzo_er(lcrzo_priv_global_set_errmsglcrzo
               ("Fichier existant voulu, mais il n'existe pas"));
    } else {
      lcrzo_er(lcrzo_priv_global_set_errmsglcrzo
               ("We want an existent file, but file doesn't exists"));
    }
    return (LCRZO_ERR_SPFILESECURITY);
  }

  /*open */
  if (fileexists) {
    fd = LAURENT_open(filename, O_WRONLY | O_APPEND | O_BINARY);
    if (fd == -1) {
#if defined LCRZODEF_SYSTEM_Unix
#elif defined LCRZODEF_SYSTEM_Windows
      if (errno == ENOENT) {
        lcrzo_er(lcrzo_priv_err_purge_int());
        lcrzo_er(lcrzo_priv_winpath240_check(filename));
      }
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
      lcrzo_er(lcrzo_priv_global_set_errmsglcrzo(filename));
      return (LCRZO_ERR_FUOPEN);
    }
    /*verify if the characteristics are the same */
    ret = lcrzo_priv_fd_stat_get(fd, &st2);
    if (ret != LCRZO_ERR_OK) {
      LAURENT_close(fd);
      return (ret);
    }
    ret = lcrzo_priv_stat_forbidschange(st1, st2);
    if (ret != LCRZO_ERR_OK) {
      LAURENT_close(fd);
      return (ret);
    }
  } else {
    lcrzo_er(lcrzo_priv_path_create_parentdirs(filename, LCRZO_FALSE));
    fd =
      LAURENT_open3(filename, O_WRONLY | O_CREAT | O_EXCL | O_BINARY, 0600);
    if (fd == -1) {
#if defined LCRZODEF_SYSTEM_Unix
#elif defined LCRZODEF_SYSTEM_Windows
      if (errno == ENOENT) {
        lcrzo_er(lcrzo_priv_err_purge_int());
        lcrzo_er(lcrzo_priv_winpath240_check(filename));
      }
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
      return (LCRZO_ERR_FUOPEN);
    }
  }

  /*eventually close the file descriptor */
  if (pfile == NULL) {
    ret = LAURENT_close(fd);
    if (ret == -1)
      return (LCRZO_ERR_FUCLOSE);
  } else {
    pfile->fd = fd;
    pfile->canclosefd = LCRZO_TRUE;
    pfile->openforreading = LCRZO_FALSE;
    pfile->openforwriting = LCRZO_TRUE;
    pfile->dataend = LCRZO_FALSE;
    pfile->storeddata = NULL;
    pfile->storeddatasize = 0;
  }

  return (LCRZO_ERR_OK);
}
