/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_kbd_initdefault(lcrzo_kbd * pkbd)
{
#if defined LCRZODEF_SYSTEM_Unix
  return (lcrzo_kbd_init_fd(0, pkbd));
#elif defined LCRZODEF_SYSTEM_Windows
  return (lcrzo_kbd_init_handle(GetStdHandle(STD_INPUT_HANDLE), pkbd));
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
}

/*-------------------------------------------------------------*/
int lcrzo_kbd_init_fd(int fd,
                      lcrzo_kbd * pkbd)
#if defined LCRZODEF_SYSTEM_Unix
{
  struct termios thetermios;

  if (pkbd == NULL)
    return (LCRZO_ERR_OK);

  pkbd->fd = fd;
  lcrzo_debug_notset_ptr(pkbd->thehandle);
  pkbd->openforreading = LCRZO_TRUE;
  pkbd->openforwriting = LCRZO_FALSE;
  pkbd->dataend = LCRZO_FALSE;
  pkbd->storeddata = NULL;
  pkbd->storeddatasize = 0;

  /* check if it's a tty */
  pkbd->type = LCRZO_PRIV_HANDLE_TYPE_UNKNOWN;
  pkbd->echokeypresses = LCRZO_FALSE;
  if (isatty(fd)) {
    pkbd->type = LCRZO_PRIV_HANDLE_TYPE_CONSOLE;
    /* check if characters are echoed */
    if (tcgetattr(fd, &thetermios))
      return (LCRZO_ERR_FUTCGETATTR);
    if (thetermios.c_lflag & ECHO) {
      pkbd->echokeypresses = LCRZO_TRUE;
    }
  } else {
    errno = 0;
  }

  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pkbd = pkbd;
  fd = fd;
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
/*-------------------------------------------------------------*/
     int lcrzo_kbd_init_handle(LCRZOHANDLE thehandle,
                               lcrzo_kbd * pkbd)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  pkbd = pkbd;
  thehandle = thehandle;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  DWORD ftype, conmode;
  BOOL err;

  if (pkbd == NULL)
    return (LCRZO_ERR_OK);

  lcrzo_debug_notset_int(pkbd->fd);
  pkbd->thehandle = thehandle;
  pkbd->openforreading = LCRZO_TRUE;
  pkbd->openforwriting = LCRZO_FALSE;
  pkbd->dataend = LCRZO_FALSE;
  pkbd->storeddata = NULL;
  pkbd->storeddatasize = 0;

  /* check if it's a tty */
  pkbd->type = LCRZO_PRIV_HANDLE_TYPE_UNKNOWN;
  ftype = GetFileType(thehandle);
  if (ftype == FILE_TYPE_CHAR) {
    pkbd->type = LCRZO_PRIV_HANDLE_TYPE_CONSOLE;
  } else if (ftype == FILE_TYPE_DISK) {
    pkbd->type = LCRZO_PRIV_HANDLE_TYPE_FILE;
  } else if (ftype == FILE_TYPE_PIPE) {
    pkbd->type = LCRZO_PRIV_HANDLE_TYPE_PIPE;
  }

  /* check if characters are echoed */
  pkbd->echokeypresses = LCRZO_TRUE;
  if (pkbd->type == LCRZO_PRIV_HANDLE_TYPE_CONSOLE) {
    err = GetConsoleMode(thehandle, &conmode);
    if (!err)
      return (LCRZO_ERR_FUGETCONSOLEMODE);
    if (!(conmode & ENABLE_ECHO_INPUT)) {
      pkbd->echokeypresses = LCRZO_FALSE;
    }
  }
  lcrzo_er(lcrzo_priv_err_purge_int());

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
