/* This file is part of Lcrzo.
*/

int lcrzo_priv_cleanpath_stat_get(lcrzo_conststring pathname,
                                  struct stat *pst)
{
  int ret;

  if (pathname == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pst == NULL)
    return (LCRZO_ERR_PANULLPTR);

#if defined LCRZODEF_SYSTEM_Unix
  ret = lstat(pathname, pst);
#elif defined LCRZODEF_SYSTEM_Windows
  ret = _stat(pathname, (struct _stat *)pst);
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
  if (ret == -1) {
    if (errno == ENOENT) {
      lcrzo_er(lcrzo_priv_err_purge_int());
      lcrzo_er(lcrzo_priv_global_set_errmsglcrzo(pathname));
      return (LCRZO_ERR_OKFILENOTFOUND);
    }
    return (LCRZO_ERR_FULSTAT);
  }
  lcrzo_er(lcrzo_priv_err_purge_int());
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_path_stat_get(lcrzo_conststring pathname,
                             struct stat *pst)
{
  lcrzo_string cleanpathname;
  int ret;

  lcrzo_er(lcrzo_path_clean(pathname, &cleanpathname));
  ret = lcrzo_priv_cleanpath_stat_get(cleanpathname, pst);
  lcrzo_debug_err_ok_activate(LCRZO_FALSE);
  lcrzo_er(lcrzo_string_free2(&cleanpathname));
  lcrzo_debug_err_ok_activate(LCRZO_TRUE);
  return (ret);
}
