/* This file is part of Lcrzo.
*/

#ifdef LCRZODEF_SYSTEM_Linux
#elif defined LCRZODEF_SYSTEM_FreeBSD || defined LCRZODEF_SYSTEM_OpenBSD | defined LCRZODEF_SYSTEM_MacOS
#elif defined LCRZODEF_SYSTEM_Solaris

/*-------------------------------------------------------------*/
int lcrzo_priv_priv_etha_init_device(lcrzo_constdevice device,
                                     lcrzo_int16 * phwtype,
                                     lcrzo_etha etha);
int lcrzo_priv_priv_etha_init_device(lcrzo_constdevice device,
                                     lcrzo_int16 * phwtype,
                                     lcrzo_etha etha)
{
  int fd;
  dl_info_req_t ireq;
  dl_phys_addr_req_t pareq;
  lcrzo_uint8 *petha;
  lcrzo_etha ethalocal;
  lcrzo_int16 hwtype;
  lcrzo_bool deviceequal;
  lcrzo_ptr buf;     /* use of void* otherwise SIGBUS on Solaris 2.5 */

  lcrzo_er(lcrzo_device_equal2(device, "lo0", &deviceequal));
  if (deviceequal) {
    if (phwtype != NULL)
      *phwtype = LCRZO_DEVICE_HWTYPE_LOOPBACK;
    lcrzo_er(lcrzo_etha_init(0, 0, 0, 0, 0, 0, etha));
    return (LCRZO_ERR_OK);
  }

  lcrzo_er(lcrzo_priv_dlpi_open(device, &fd));

  /* ask type */
  ireq.dl_primitive = DL_INFO_REQ;
  lcrzo_er(lcrzo_priv_dlpi_putmsg(fd, (char *)&ireq, sizeof(ireq), 0));
  lcrzo_er(lcrzo_ptr_alloc(2048, &buf));
  lcrzo_er(lcrzo_priv_dlpi_getmsg(fd, DL_INFO_ACK_SIZE, (char *)buf));
  lcrzo_er(lcrzo_priv_hwtype_init_hwtype(((dl_info_ack_t *) buf)->dl_mac_type,
                                         &hwtype));
  lcrzo_er(lcrzo_ptr_free2(&buf));

  /* ask Ethernet address */
  if (hwtype == LCRZO_DEVICE_HWTYPE_ETHER) {
    pareq.dl_primitive = DL_PHYS_ADDR_REQ;
    pareq.dl_addr_type = DL_CURR_PHYS_ADDR;
    lcrzo_er(lcrzo_priv_dlpi_putmsg(fd, (char *)&pareq, sizeof(pareq), 0));
    lcrzo_er(lcrzo_ptr_alloc(2048, &buf));
    lcrzo_er(lcrzo_priv_dlpi_getmsg(fd, DL_PHYS_ADDR_ACK_SIZE, (char *)buf));
    petha = (lcrzo_data) buf + ((dl_phys_addr_ack_t *) buf)->dl_addr_offset;
    lcrzo_er(lcrzo_etha_init_etha(petha, etha));
    lcrzo_er(lcrzo_etha_init_etha(petha, ethalocal));
    lcrzo_er(lcrzo_ptr_free2(&buf));
    if (ethalocal[0] == 0 && ethalocal[1] == 0 && ethalocal[2] == 0 &&
        ethalocal[3] == 0 && ethalocal[4] == 0 && ethalocal[5] == 0) {
      hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
    }
  } else {
    lcrzo_er(lcrzo_etha_initdefault(etha));
  }

  close(fd);
  if (phwtype != NULL)
    *phwtype = hwtype;
  return (LCRZO_ERR_OK);
}

#elif defined LCRZODEF_SYSTEM_Unsupported
#elif defined LCRZODEF_SYSTEM_Windows

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_NTregistry(lcrzo_bool setdevices,
                                    lcrzo_bool setroutes,
                                    lcrzo_bool searchipl,
                                    lcrzo_ipl iplwanted,
                                    lcrzo_device founddevice)
{
  lcrzo_list list, deviplist, devmasklist;
  lcrzo_string devname, ipaddress, netmask, gwaddress;
  lcrzo_int32 devipcount, devmaskcount, devi;
  lcrzo_string shortdevname, devkey;
  lcrzo_int32 cardnumber, cardnumbercount, devicenumber;
  lcrzo_uint32 dhcpenabled;
  lcrzo_device device;
  lcrzo_int16 hwtype;
  lcrzo_etha etha;
  lcrzo_ipl ipl, mask;
  lcrzo_bool isanalias, staticworked, dhcpworked, found;
  lcrzo_list gwlist;
  lcrzo_int32 gwcount, gwi;
  lcrzo_ipl gw;
  char *pc;
  int ret;

  /* First, add loopback */
  if (setdevices) {
    lcrzo_er(lcrzo_etha_initdefault(etha));
    lcrzo_er(lcrzo_priv_conf_devices_add_system
             ("Loopback", LCRZO_DEVICE_HWTYPE_LOOPBACK, etha, 0x7F000001,
              0xFF000000u, 1500, LCRZO_TRUE, LCRZO_FALSE, LCRZO_FALSE, 0));
  }

  /* get device infos */
  found = LCRZO_FALSE;
  ret = LCRZO_ERR_OKSEARCHNOTFOUND;
  lcrzo_er(lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE,
                                            "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Linkage",
                                            "Bind", &list));
  lcrzo_er(lcrzo_list_count(&list, &cardnumbercount));
  devicenumber = 0;
  for (cardnumber = 1;
       cardnumber <= cardnumbercount && (!searchipl || !found);
       cardnumber++) {
    devicenumber++;

    /* get the device full name */
    lcrzo_er(lcrzo_stringlist_valuem_pos(&list, cardnumber, NULL, &devname));

    /* compute the short device name */
    pc = strrchr(devname, '\\');
    if (pc == NULL)
      pc = devname;
    lcrzo_er(lcrzo_string_initm_text(pc + 1, &shortdevname));

    /* construct the key needed to access its parameters */
    lcrzo_er(lcrzo_string_initm_text
             ("SYSTEM\\CurrentControlSet\\Services\\", &devkey));
    lcrzo_er(lcrzo_string_appendm_text(shortdevname, &devkey));
    lcrzo_er(lcrzo_string_appendm_text("\\Parameters\\Tcpip", &devkey));

    /* check if DHCP is enabled */
    ret =
      lcrzo_priv_winreg_uint32_get(HKEY_LOCAL_MACHINE, devkey, "EnableDHCP",
                                   &dhcpenabled);
    if (ret != LCRZO_ERR_OK)
      dhcpenabled = LCRZO_FALSE;
    staticworked = LCRZO_FALSE;
    dhcpworked = LCRZO_FALSE;

    if (!dhcpenabled) {
      /* ask the IPAddress/SubnetMask value for this key */
      ret =
        lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE, devkey,
                                         "IPAddress", &deviplist);
      if (ret == LCRZO_ERR_OK) {
        ret =
          lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE, devkey,
                                           "SubnetMask", &devmasklist);
        if (ret == LCRZO_ERR_OK) {
          ret = lcrzo_list_count(&deviplist, &devipcount);
          devmaskcount = 0;     /* for the compiler warning */
          if (ret == LCRZO_ERR_OK) {
            ret = lcrzo_list_count(&devmasklist, &devmaskcount);
          }
          if (ret == LCRZO_ERR_OK) {
            /* get values */
            for (devi = 1; 
                 devi <= devipcount && devi <= devmaskcount 
                        && (!searchipl || !found); devi++) {
              /* get the IP address */
              ret =
                lcrzo_stringlist_valuem_pos(&deviplist, devi, NULL,
                                            &ipaddress);
              if (ret == LCRZO_ERR_OK) {
                ret =
                  lcrzo_stringlist_valuem_pos(&devmasklist, devi, NULL,
                                              &netmask);
                if (ret == LCRZO_ERR_OK) {
                  /* update conf */
                  ret = lcrzo_ipl_init_ips(ipaddress, &ipl);
                  mask = 0;     /* for the compiler warning */
                  if (ret == LCRZO_ERR_OK) {
                    ret = lcrzo_ipl_init_ips(netmask, &mask);
                  }
                  if (ret == LCRZO_ERR_OK && ipl != 0 && mask != 0) {
                    staticworked = LCRZO_TRUE;
                    if (setdevices) {
                      ret =
                        lcrzo_priv_conf_device_init_devicealias(shortdevname,
                                                                devi, device);
                      if (ret == LCRZO_ERR_OK) {
                        if (devi == 1) {
                          isanalias = LCRZO_FALSE;
                        } else {
                          devicenumber++;
                          isanalias = LCRZO_TRUE;
                        }
                        ret = lcrzo_priv_conf_etha_init_cardnumberdevice(device, devicenumber, cardnumber, isanalias, &hwtype, etha);
                        if (ret == LCRZO_ERR_OK) {
                          ret = lcrzo_priv_conf_winras_devices_add_system(device, hwtype, etha, ipl, mask, 1500, LCRZO_TRUE, isanalias);
                        } else {
                          ret = lcrzo_priv_conf_incomplete_add_5(device, ipl,
                                                             mask, isanalias);
                        }
                      }
                    }
                    if (searchipl) {
                      if (ipl == iplwanted) {
                        ret =
                          lcrzo_priv_conf_device_init_devicealias
                          (shortdevname, devi, founddevice);
                        if (ret == LCRZO_ERR_OK)
                          found = LCRZO_TRUE;
                      }
                    }
                  }
                  lcrzo_er(lcrzo_string_free2(&netmask));
                }
                lcrzo_er(lcrzo_string_free2(&ipaddress));
              }
            }
          }
          lcrzo_er(lcrzo_list_close(&devmasklist));
        }
        lcrzo_er(lcrzo_list_close(&deviplist));
      }
    }

    if (dhcpenabled || !staticworked) {
      /* for ras, dhcp is not enabled, but values are in DhcpIPAddress */
      ret =
        lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, devkey,
                                     "DhcpIPAddress", &ipaddress);
      if (ret == LCRZO_ERR_OK) {
        ret =
          lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, devkey,
                                       "DhcpSubnetMask", &netmask);
        if (ret == LCRZO_ERR_OK) {
          /* update conf */
          ret = lcrzo_ipl_init_ips(ipaddress, &ipl);
          mask = 0;             /* for the compiler warning */
          if (ret == LCRZO_ERR_OK) {
            ret = lcrzo_ipl_init_ips(netmask, &mask);
          }
          if (ret == LCRZO_ERR_OK && ipl != 0 && mask != 0) {
            dhcpworked = LCRZO_TRUE;
            if (setdevices) {
              ret =
                lcrzo_priv_conf_device_init_devicealias(shortdevname, 1,
                                                        device);
              if (ret == LCRZO_ERR_OK) {
                ret =
                  lcrzo_priv_conf_etha_init_cardnumberdevice(device,
                                                             devicenumber,
                                                             cardnumber,
                                                             LCRZO_FALSE,
                                                             &hwtype, etha);
                if (ret == LCRZO_ERR_OK) {
                  ret =
                    lcrzo_priv_conf_winras_devices_add_system(device, hwtype,
                                                              etha, ipl, mask,
                                                              1500,
                                                              LCRZO_TRUE,
                                                              LCRZO_FALSE);
                } else {
                  ret =
                    lcrzo_priv_conf_incomplete_add_5(device, ipl, mask,
                                                     LCRZO_FALSE);
                }
              }
            }
            if (searchipl) {
              if (ipl == iplwanted) {
                ret =
                  lcrzo_priv_conf_device_init_devicealias(shortdevname, 1,
                                                          founddevice);
                if (ret == LCRZO_ERR_OK)
                  found = LCRZO_TRUE;
              }
            }
          }
          /* free */
          lcrzo_er(lcrzo_string_free2(&netmask));
        } else {
          /* ignore this error (allow missing key) */
          ret = LCRZO_ERR_OK;
        }
        lcrzo_er(lcrzo_string_free2(&ipaddress));
      } else {
        /* ignore this error (allow missing key) */
        ret = LCRZO_ERR_OK;
      }
    }

    /* eventually add the routes */
    if (setroutes && (staticworked || dhcpworked)) {
      if (dhcpworked) {
        ret =
          lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE, devkey,
                                           "DhcpDefaultGateway", &gwlist);
      } else if (staticworked) {
        ret =
          lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE, devkey,
                                           "DefaultGateway", &gwlist);
      }
      if (ret == LCRZO_ERR_OK) {
        ret = lcrzo_list_count(&gwlist, &gwcount);
        if (ret == LCRZO_ERR_OK) {
          /* get values */
          for (gwi = 1; gwi <= gwcount; gwi++) {
            /* get the GW address */
            ret = lcrzo_stringlist_valuem_pos(&gwlist, gwi, NULL, &gwaddress);
            if (ret == LCRZO_ERR_OK) {
              ret = lcrzo_ipl_init_ips(gwaddress, &gw);
              if (ret == LCRZO_ERR_OK) {
                ret =
                  lcrzo_priv_conf_device_init_devicealias(shortdevname, 1,
                                                          device);
                if (ret == LCRZO_ERR_OK) {
                  ret =
                    lcrzo_priv_conf_routes_add_system(device, 0, 0, 0, gw, 1,
                                                      LCRZO_TRUE);
                }
              }
              /* free */
              lcrzo_er(lcrzo_string_free2(&gwaddress));
            }
          }
        }
        lcrzo_er(lcrzo_list_close(&gwlist));
      } else {
        /* ignore this error (allow missing key) */
        ret = LCRZO_ERR_OK;
      }
    }

    /* free */
    lcrzo_er(lcrzo_string_free2(&devkey));
    lcrzo_er(lcrzo_string_free2(&shortdevname));
    lcrzo_er(lcrzo_string_free2(&devname));
  }
  lcrzo_er(lcrzo_list_close(&list));

  if (searchipl && !found) {
    return (LCRZO_ERR_OKUNRESOLVED);
  }

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_dev_NTregistry(void);
int lcrzo_priv_conf_init_dev_NTregistry(void)
{
  return (lcrzo_priv_conf_init_NTregistry(LCRZO_TRUE, LCRZO_FALSE,
                                          LCRZO_FALSE, 0, NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_2kregistry2(lcrzo_conststring shortdevname,
                                     lcrzo_int32 * pdevicenumber,
                                     lcrzo_int32 * pcardnumber,
                                     lcrzo_conststring ipconfigdata,
                                     lcrzo_int32 ipconfignum,
                                     lcrzo_bool setdevices,
                                     lcrzo_bool setroutes,
                                     lcrzo_bool searchipl,
                                     lcrzo_ipl iplwanted,
                                     lcrzo_bool * piplfound,
                                     lcrzo_device founddevice);
int lcrzo_priv_conf_init_2kregistry2(lcrzo_conststring shortdevname,
                                     lcrzo_int32 * pdevicenumber,
                                     lcrzo_int32 * pcardnumber,
                                     lcrzo_conststring ipconfigdata,
                                     lcrzo_int32 ipconfignum,
                                     lcrzo_bool setdevices,
                                     lcrzo_bool setroutes,
                                     lcrzo_bool searchipl,
                                     lcrzo_ipl iplwanted,
                                     lcrzo_bool * piplfound,
                                     lcrzo_device founddevice)
{
  lcrzo_list deviplist, devmasklist;
  lcrzo_string ipaddress, netmask, gwaddress;
  lcrzo_int32 devipcount, devmaskcount, devi;
  lcrzo_string devkey;
  lcrzo_uint32 dhcpenabled;
  lcrzo_device device;
  lcrzo_int16 hwtype;
  lcrzo_etha etha;
  lcrzo_ipl ipl, mask;
  lcrzo_bool isanalias, staticworked, dhcpworked;
  lcrzo_list gwlist;
  lcrzo_int32 gwcount, gwi;
  lcrzo_ipl gw;
  int ret;

  /* construct the key needed to access its parameters */
  lcrzo_er(lcrzo_string_initm_text
           ("SYSTEM\\CurrentControlSet\\Services\\", &devkey));
  lcrzo_er(lcrzo_string_appendm_text(ipconfigdata, &devkey));

  /* check if DHCP is enabled */
  ret =
    lcrzo_priv_winreg_uint32_get(HKEY_LOCAL_MACHINE, devkey, "EnableDHCP",
                                 &dhcpenabled);
  if (ret != LCRZO_ERR_OK)
    dhcpenabled = LCRZO_FALSE;

  /* init */
  staticworked = LCRZO_FALSE;
  dhcpworked = LCRZO_FALSE;

  if (!dhcpenabled) {
    /* ask the IPAddress/SubnetMask value for this key */
    ret =
      lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE, devkey,
                                       "IPAddress", &deviplist);
    if (ret == LCRZO_ERR_OK) {
      ret =
        lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE, devkey,
                                         "SubnetMask", &devmasklist);
      if (ret == LCRZO_ERR_OK) {
        ret = lcrzo_list_count(&deviplist, &devipcount);
        if (ret == LCRZO_ERR_OK) {
          ret = lcrzo_list_count(&devmasklist, &devmaskcount);
          if (ret == LCRZO_ERR_OK) {
            /* get values */
            for (devi = 1;
                 devi <= devipcount && devi <= devmaskcount
                        && (!searchipl || !*piplfound); devi++) {
              /* get the IP address */
              ret =
                lcrzo_stringlist_valuem_pos(&deviplist, devi, NULL,
                                            &ipaddress);
              if (ret == LCRZO_ERR_OK) {
                ret =
                  lcrzo_stringlist_valuem_pos(&devmasklist, devi, NULL,
                                              &netmask);
                if (ret == LCRZO_ERR_OK) {
                  ret = lcrzo_ipl_init_ips(ipaddress, &ipl);
                  if (ret == LCRZO_ERR_OK) {
                    ret = lcrzo_ipl_init_ips(netmask, &mask);
                    if (ret == LCRZO_ERR_OK && ipl != 0 && mask != 0) {
                      staticworked = LCRZO_TRUE;
                      if (setdevices) {
                        ret =
                          lcrzo_priv_conf_device_init_devicealias
                          (shortdevname, devi, device);
                        if (ret == LCRZO_ERR_OK) {
                          if (devi == 1) {
                            isanalias = LCRZO_FALSE;
                          } else {
                            (*pdevicenumber)++;
                            isanalias = LCRZO_TRUE;
                          }
                          ret = lcrzo_priv_conf_etha_init_cardnumberdevice(device, *pdevicenumber, *pcardnumber, isanalias, &hwtype, etha);
                          if (ret == LCRZO_ERR_OK) {
                            ret =
                              lcrzo_priv_conf_winras_devices_add_system
                              (device, hwtype, etha, ipl, mask, 1500,
                               LCRZO_TRUE, isanalias);
                          } else {
                            ret =
                              lcrzo_priv_conf_incomplete_add_5(device, ipl,
                                                               mask,
                                                               isanalias);
                          }
                        }
                      }
                      if (searchipl) {
                        if (ipl == iplwanted) {
                          ret =
                            lcrzo_priv_conf_device_init_devicealias
                            (shortdevname, devi, founddevice);
                          if (ret == LCRZO_ERR_OK)
                            *piplfound = LCRZO_TRUE;
                        }
                      }
                    }
                  }
                  lcrzo_er(lcrzo_string_free2(&netmask));
                }
                lcrzo_er(lcrzo_string_free2(&ipaddress));
              }
            }
          }
        }
        lcrzo_er(lcrzo_list_close(&devmasklist));
      }
      lcrzo_er(lcrzo_list_close(&deviplist));
    }
  }

  /* try with dhcp */
  if (dhcpenabled || !staticworked) {
    /* for ras, dhcp is not enabled, but values are in DhcpIPAddress */
    ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, devkey,
                                       "DhcpIPAddress", &ipaddress);
    if (ret == LCRZO_ERR_OK) {
      ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, devkey,
                                         "DhcpSubnetMask", &netmask);
      if (ret == LCRZO_ERR_OK) {
        ret = lcrzo_ipl_init_ips(ipaddress, &ipl);
        if (ret == LCRZO_ERR_OK) {
          ret = lcrzo_ipl_init_ips(netmask, &mask);
          if (ret == LCRZO_ERR_OK) {
            if (ipl != 0 && mask != 0) {
              /* ok, so we really got an address from a DHCP server */
              dhcpworked = LCRZO_TRUE;
              if (setdevices) {
                ret =
                  lcrzo_priv_conf_device_init_devicealias(shortdevname,
                                                          ipconfignum,
                                                          device);
                if (ret == LCRZO_ERR_OK) {
                  ret =
                    lcrzo_priv_conf_etha_init_cardnumberdevice(device,
                                                               *pdevicenumber,
                                                               *pcardnumber,
                                                               LCRZO_FALSE,
                                                               &hwtype, etha);
                  if (ret == LCRZO_ERR_OK) {
                    ret =
                      lcrzo_priv_conf_winras_devices_add_system(device,
                                                                hwtype, etha,
                                                                ipl, mask,
                                                                1500,
                                                                LCRZO_TRUE,
                                                                LCRZO_FALSE);
                  } else {
                    ret =
                      lcrzo_priv_conf_incomplete_add_5(device, ipl, mask,
                                                       LCRZO_FALSE);
                  }
                }
              }
              if (searchipl) {
                if (ipl == iplwanted) {
                  ret =
                    lcrzo_priv_conf_device_init_devicealias(shortdevname,
                                                            ipconfignum,
                                                            founddevice);
                  if (ret == LCRZO_ERR_OK)
                    *piplfound = LCRZO_TRUE;
                }
              }
            }
          }
        }
        lcrzo_er(lcrzo_string_free2(&netmask));
      }
      lcrzo_er(lcrzo_string_free2(&ipaddress));
    }
  }

  /* eventually add the routes */
  if (setroutes && (staticworked || dhcpworked)) {
    if (dhcpworked) {
      ret =
        lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE, devkey,
                                         "DhcpDefaultGateway", &gwlist);
    } else if (staticworked) {
      ret =
        lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE, devkey,
                                         "DefaultGateway", &gwlist);
    }
    if (ret == LCRZO_ERR_OK) {
      ret = lcrzo_list_count(&gwlist, &gwcount);
      if (ret == LCRZO_ERR_OK) {
        /* get values */
        for (gwi = 1; gwi <= gwcount; gwi++) {
          /* get the GW address */
          ret = lcrzo_stringlist_valuem_pos(&gwlist, gwi, NULL, &gwaddress);
          if (ret == LCRZO_ERR_OK) {
            ret = lcrzo_ipl_init_ips(gwaddress, &gw);
            if (ret == LCRZO_ERR_OK) {
              ret =
                lcrzo_priv_conf_device_init_devicealias(shortdevname, 1,
                                                        device);
              if (ret == LCRZO_ERR_OK) {
                ret =
                  lcrzo_priv_conf_routes_add_system(device, 0, 0, 0, gw, 1,
                                                    LCRZO_TRUE);
              }
            }
            /* free */
            lcrzo_er(lcrzo_string_free2(&gwaddress));
          }
        }
      }
      lcrzo_er(lcrzo_list_close(&gwlist));
    } else {
      /* ignore this error (allow missing key) */
      ret = LCRZO_ERR_OK;
    }
  }

  lcrzo_er(lcrzo_string_free2(&devkey));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_2kregistry(lcrzo_bool setdevices,
                                    lcrzo_bool setroutes,
                                    lcrzo_bool searchipl,
                                    lcrzo_ipl iplwanted,
                                    lcrzo_device founddevice)
{
  lcrzo_list list, ipconfiglist;
  lcrzo_string devname, ipconfigdata, shortdevname, ipconfig;
  lcrzo_int32 cardnumber, cardnumbercount, devicenumber, ipconfigcount, j;
  lcrzo_bool iplfound;
  char *pc;
  int ret;

  /* First, add loopback */
  if (setdevices) {
    lcrzo_etha etha;

    lcrzo_er(lcrzo_etha_initdefault(etha));
    lcrzo_er(lcrzo_priv_conf_devices_add_system
             ("Loopback", LCRZO_DEVICE_HWTYPE_LOOPBACK, etha, 0x7F000001,
              0xFF000000u, 1500, LCRZO_TRUE, LCRZO_FALSE, LCRZO_FALSE, 0));
  }

  /* get device infos */
  iplfound = LCRZO_FALSE;
  lcrzo_er(lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE,
                                            "SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Linkage",
                                            "Bind", &list));
  lcrzo_er(lcrzo_list_count(&list, &cardnumbercount));
  devicenumber = 0;
  for (cardnumber = 1;
       cardnumber <= cardnumbercount && (!searchipl || !iplfound);
       cardnumber++) {
    devicenumber++;

    /* get the device full name */
    ret = lcrzo_stringlist_valuem_pos(&list, cardnumber, NULL, &devname);
    if (ret == LCRZO_ERR_OK) {
      /* compute the short device name */
      pc = strrchr(devname, '\\');
      if (pc == NULL)
        pc = devname;
      ret = lcrzo_string_initm_text(pc + 1, &shortdevname);
      if (ret == LCRZO_ERR_OK) {
        /*printf("%s\n", shortdevname); */
        /* get the configuration associated with this device */
        ret =
          lcrzo_string_initm_text
          ("SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Adapters\\",
           &ipconfig);
        if (ret == LCRZO_ERR_OK) {
          ret = lcrzo_string_appendm_text(shortdevname, &ipconfig);
          if (ret == LCRZO_ERR_OK) {
            ret =
              lcrzo_priv_winreg_stringlist_get(HKEY_LOCAL_MACHINE, ipconfig,
                                               "IpConfig", &ipconfiglist);
            if (ret == LCRZO_ERR_OK) {
              ret = lcrzo_list_count(&ipconfiglist, &ipconfigcount);
              if (ret == LCRZO_ERR_OK) {
                for (j = 1; j <= ipconfigcount && (!searchipl || !iplfound);
                     j++) {
                  ret =
                    lcrzo_stringlist_valuem_pos(&ipconfiglist, j, NULL,
                                                &ipconfigdata);
                  if (ret == LCRZO_ERR_OK) {
                    /*printf("--> %s\n", ipconfigdata); */
                    ret =
                      lcrzo_priv_conf_init_2kregistry2(shortdevname,
                                                       &devicenumber,
                                                       &cardnumber,
                                                       ipconfigdata, j,
                                                       setdevices, setroutes,
                                                       searchipl, iplwanted,
                                                       &iplfound,
                                                       founddevice);
                    lcrzo_er(lcrzo_string_free2(&ipconfigdata));
                  }
                }
              }
              lcrzo_er(lcrzo_list_close(&ipconfiglist));
            }
          }
          lcrzo_er(lcrzo_string_free2(&ipconfig));
        }
        lcrzo_er(lcrzo_string_free2(&shortdevname));
      }
      lcrzo_er(lcrzo_string_free2(&devname));
    }
  }
  lcrzo_er(lcrzo_list_close(&list));

  if (searchipl && !iplfound) {
    return (LCRZO_ERR_OKUNRESOLVED);
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_dev_2kregistry(void);
int lcrzo_priv_conf_init_dev_2kregistry(void)
{
  return (lcrzo_priv_conf_init_2kregistry(LCRZO_TRUE, LCRZO_FALSE,
                                          LCRZO_FALSE, 0, NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_W9xregistry5_dev(lcrzo_constdevice device,
                                          lcrzo_ipl ip,
                                          lcrzo_ipl mask,
                                          lcrzo_int32 * pdevicenumber,
                                          lcrzo_int32 cardnumber,
                                          lcrzo_int32 * paddressnumber,
                                          lcrzo_bool setdevices,
                                          lcrzo_bool searchipl,
                                          lcrzo_ipl iplwanted,
                                          lcrzo_bool * pfound,
                                          lcrzo_device founddevice);
int lcrzo_priv_conf_init_W9xregistry5_dev(lcrzo_constdevice device,
                                          lcrzo_ipl ip,
                                          lcrzo_ipl mask,
                                          lcrzo_int32 * pdevicenumber,
                                          lcrzo_int32 cardnumber,
                                          lcrzo_int32 * paddressnumber,
                                          lcrzo_bool setdevices,
                                          lcrzo_bool searchipl,
                                          lcrzo_ipl iplwanted,
                                          lcrzo_bool * pfound,
                                          lcrzo_device founddevice)
{
  lcrzo_etha etha;
  lcrzo_device dev;
  lcrzo_bool isanalias;
  lcrzo_int16 hwtype;
  int ret;

  /* compute the corresponding device */
  ret = LCRZO_ERR_OK;
  if (setdevices) {
    ret =
      lcrzo_priv_conf_device_init_devicealias(device, *paddressnumber, dev);
    if (ret == LCRZO_ERR_OK) {
      isanalias =
        (lcrzo_bool) ((*paddressnumber == 1) ? LCRZO_FALSE : LCRZO_TRUE);
      ret =
        lcrzo_priv_conf_etha_init_cardnumberdevice(dev, *pdevicenumber,
                                                   cardnumber, isanalias,
                                                   &hwtype, etha);
      if (ret == LCRZO_ERR_OK) {
        ret =
          lcrzo_priv_conf_winras_devices_add_system(dev, hwtype, etha, ip,
                                                    mask, 1500, LCRZO_TRUE,
                                                    isanalias);
      } else {
        ret = lcrzo_priv_conf_incomplete_add_5(dev, ip, mask, isanalias);
      }
    }
  }
  if (searchipl) {
    if (ip != 0 && ip == iplwanted) {
      ret =
        lcrzo_priv_conf_device_init_devicealias(device, *paddressnumber,
                                                founddevice);
      *pfound = LCRZO_TRUE;
    }
  }
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_W9xregistry5_routes(lcrzo_constdevice device,
                                             lcrzo_ipl ip,
                                             lcrzo_string routers);
int lcrzo_priv_conf_init_W9xregistry5_routes(lcrzo_constdevice device,
                                             lcrzo_ipl ip,
                                             lcrzo_string routers)
{
  lcrzo_ipl gw;
  char *pc, *pcstart;
  lcrzo_device dev;
  int i, ret;

  /* analyze value which may contain a comma separated list of gateways */
  i = 1;
  pcstart = pc = routers;
  ret = LCRZO_ERR_OK;
  while (pc != NULL && ret == LCRZO_ERR_OK) {
    pc = strchr(pcstart, ',');
    if (pc == NULL) {
      ret = lcrzo_ipl_init_ips(pcstart, &gw);
    } else {
      *pc = '\0';
      ret = lcrzo_ipl_init_ips(pcstart, &gw);
      pcstart = pc + 1;
    }
    if (ret == LCRZO_ERR_OK) {
      /* add the route */
      ret = lcrzo_priv_device_init_ascii(device, dev);
      if (ret == LCRZO_ERR_OK) {
        ret =
          lcrzo_priv_conf_routes_add_system(dev, 0, 0, ip, gw, i, LCRZO_TRUE);
      }
    }
    i++;
    if (pc != NULL)
      *pc = '\0';
  }
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_W9xregistry5_dev2(lcrzo_constdevice device,
                                           lcrzo_string ipaddress,
                                           lcrzo_string ipmask,
                                           lcrzo_int32 * pdevicenumber,
                                           lcrzo_int32 cardnumber,
                                           lcrzo_int32 * paddressnumber,
                                           lcrzo_bool setdevices,
                                           lcrzo_bool searchipl,
                                           lcrzo_ipl iplwanted,
                                           lcrzo_bool * piplfound,
                                           lcrzo_device founddevice);
int lcrzo_priv_conf_init_W9xregistry5_dev2(lcrzo_constdevice device,
                                           lcrzo_string ipaddress,
                                           lcrzo_string ipmask,
                                           lcrzo_int32 * pdevicenumber,
                                           lcrzo_int32 cardnumber,
                                           lcrzo_int32 * paddressnumber,
                                           lcrzo_bool setdevices,
                                           lcrzo_bool searchipl,
                                           lcrzo_ipl iplwanted,
                                           lcrzo_bool * piplfound,
                                           lcrzo_device founddevice)
{
  lcrzo_ipl ip, mask;
  char *pcip, *pcmask;
  int ret, i;

  /* Note : this is strange, but values have to be read from
     right to left (checked with winipcfg) */
  pcip = ipaddress;
  pcmask = ipmask;
  mask = 0;
  ret = LCRZO_ERR_OK;
  i = 0;
  while (pcip != NULL && pcmask != NULL 
         && ret == LCRZO_ERR_OK && (!searchipl || !*piplfound)) {
    /* find ip */
    pcip = strrchr(ipaddress, ',');
    if (pcip == NULL) {
      ret = lcrzo_ipl_init_ips(ipaddress, &ip);
    } else {
      ret = lcrzo_ipl_init_ips(pcip + 1, &ip);
      *pcip = '\0';
    }
    if (ret == LCRZO_ERR_OK) {
      /* find mask */
      pcmask = strrchr(ipmask, ',');
      if (pcmask == NULL) {
        ret = lcrzo_ipl_init_ips(ipmask, &mask);
      } else {
        ret = lcrzo_ipl_init_ips(pcmask + 1, &mask);
        *pcmask = '\0';
      }
    }
    if (ret == LCRZO_ERR_OK) {
      i++;
      /* add the device */
      if (i > 1) {
        (*pdevicenumber)++;
        (*paddressnumber)++;
      }
      if (setdevices) {
        ret = lcrzo_priv_conf_init_W9xregistry5_dev(device, ip, mask,
                                                    pdevicenumber, cardnumber,
                                                    paddressnumber,
                                                    setdevices, searchipl,
                                                    iplwanted, piplfound,
                                                    founddevice);
      }
      if (searchipl) {
        if (ip != 0 && ip == iplwanted) {
          ret =
            lcrzo_priv_conf_device_init_devicealias(device, *paddressnumber,
                                                    founddevice);
          *piplfound = LCRZO_TRUE;
        }
      }
    }
  }

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_W9xregistry5(lcrzo_constdevice device,
                                      lcrzo_string ipaddress,
                                      lcrzo_string ipmask,
                                      lcrzo_string routers,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 cardnumber,
                                      lcrzo_int32 * paddressnumber,
                                      lcrzo_bool * ptrywithdhcp,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice);
int lcrzo_priv_conf_init_W9xregistry5(lcrzo_constdevice device,
                                      lcrzo_string ipaddress,
                                      lcrzo_string ipmask,
                                      lcrzo_string routers,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 cardnumber,
                                      lcrzo_int32 * paddressnumber,
                                      lcrzo_bool * ptrywithdhcp,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice)
{
  lcrzo_ipl ip, mask;
  int ret;

  /* first, check with one address only */
  ret = lcrzo_ipl_init_ips(ipaddress, &ip);
  if (ret == LCRZO_ERR_OK) {
    ret = lcrzo_ipl_init_ips(ipmask, &mask);
    if (ret == LCRZO_ERR_OK) {
      /* when both are 0, perhaps use DHCP */
      if (ip == 0 && mask == 0) {
        *ptrywithdhcp = LCRZO_TRUE;
        return (LCRZO_ERR_OK);
      }
      if (setdevices || searchipl) {
        /* add the device */
        ret = lcrzo_priv_conf_init_W9xregistry5_dev(device, ip, mask,
                                                    pdevicenumber, cardnumber,
                                                    paddressnumber, setdevices,
                                                    searchipl, iplwanted,
                                                    piplfound, founddevice);
      }
      if (setroutes) {
        /* also add the routes */
        ret = lcrzo_priv_conf_init_W9xregistry5_routes(device, ip, routers);
      }
      /* ok, we exit now */
      return (ret);
    }
  }

  /* if we are here, ipaddress and ipmask are lists */
  if (setdevices || searchipl) {
    ret = lcrzo_priv_conf_init_W9xregistry5_dev2(device, ipaddress, ipmask,
                                                 pdevicenumber, cardnumber,
                                                 paddressnumber, setdevices,
                                                 searchipl, iplwanted,
                                                 piplfound, founddevice);
  }
  if (setroutes) {
    /* also add the routes, with 0 as source address */
    ret = lcrzo_priv_conf_init_W9xregistry5_routes(device, 0, routers);
  }
  return (ret);
}


/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_W9xregistry4(lcrzo_constdevice device,
                                      lcrzo_conststring keynettrans,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 cardnumber,
                                      lcrzo_int32 * paddressnumber,
                                      lcrzo_bool * ptrywithdhcp,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice);
int lcrzo_priv_conf_init_W9xregistry4(lcrzo_constdevice device,
                                      lcrzo_conststring keynettrans,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 cardnumber,
                                      lcrzo_int32 * paddressnumber,
                                      lcrzo_bool * ptrywithdhcp,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice)
{
  lcrzo_string value, value2, value3;
  int ret;

  /* get ip address */
  ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, keynettrans,
                                     "IPAddress", &value);
  if (ret != LCRZO_ERR_OK)
    return (ret);

  /* get netmask */
  ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, keynettrans, "IPMask",
                                     &value2);
  if (ret != LCRZO_ERR_OK) {
    lcrzo_er(lcrzo_string_free2(&value));
    return (ret);
  }

  /* get routers */
  ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, keynettrans,
                                     "DefaultGateway", &value3);
  if (ret != LCRZO_ERR_OK) {
    ret = lcrzo_string_initm_text("", &value3);
    if (ret != LCRZO_ERR_OK) {
      lcrzo_er(lcrzo_string_free2(&value));
      lcrzo_er(lcrzo_string_free2(&value2));
      return (ret);
    }
  }

  /* set all IP addresses */
  ret = lcrzo_priv_conf_init_W9xregistry5(device, value, value2, value3,
                                          pdevicenumber, cardnumber,
                                          paddressnumber, ptrywithdhcp,
                                          setdevices, setroutes, searchipl,
                                          iplwanted, piplfound, founddevice);
  lcrzo_er(lcrzo_string_free2(&value));
  lcrzo_er(lcrzo_string_free2(&value2));
  lcrzo_er(lcrzo_string_free2(&value3));

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_W9xregistry3(lcrzo_constdevice device,
                                      lcrzo_conststring keylongmstcpi,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 cardnumber,
                                      lcrzo_int32 * paddressnumber,
                                      lcrzo_bool * ptrywithdhcp,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice);
int lcrzo_priv_conf_init_W9xregistry3(lcrzo_constdevice device,
                                      lcrzo_conststring keylongmstcpi,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 cardnumber,
                                      lcrzo_int32 * paddressnumber,
                                      lcrzo_bool * ptrywithdhcp,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice)
{
  lcrzo_string value, keynettrans;
  int ret;

  /*printf("%s : %s\n", device, keylongmstcpi); */

  /* check it is TCP/IP */
  ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, keylongmstcpi,
                                     "DeviceDesc", &value);
  if (ret == LCRZO_ERR_OK) {
    if (strcmp(value, "TCP/IP"))
      ret = LCRZO_ERR_OKUNRESOLVED;
    lcrzo_er(lcrzo_string_free2(&value));
  }
  /* simply ignore previous errors */
  if (ret != LCRZO_ERR_OK)
    return (LCRZO_ERR_OK);
  /*puts("It is TCP/IP"); */

  /* get the nettrans key */
  ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, keylongmstcpi, 
                                     "Driver", &value);
  if (ret == LCRZO_ERR_OK) {
    ret = lcrzo_string_initm_text("System\\CurrentControlSet\\Services\\Class\\", &keynettrans);
    if (ret == LCRZO_ERR_OK) {
      ret = lcrzo_string_appendm_text(value, &keynettrans);
      if (ret == LCRZO_ERR_OK) {
        /*printf("nettrans=%s\n", keynettrans); */
        ret = lcrzo_priv_conf_init_W9xregistry4(device, keynettrans,
                                                pdevicenumber, cardnumber,
                                                paddressnumber, ptrywithdhcp,
                                                setdevices, setroutes,
                                                searchipl,
                                                iplwanted, piplfound,
                                                founddevice);
      }
      lcrzo_er(lcrzo_string_free2(&keynettrans));
    }
    lcrzo_er(lcrzo_string_free2(&value));
  }

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_W9xregistry2(lcrzo_conststring keyname,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 * pcardnumber,
                                      lcrzo_int32 * pdhcpaddressnumber,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice);
int lcrzo_priv_conf_init_W9xregistry2(lcrzo_conststring keyname,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 * pcardnumber,
                                      lcrzo_int32 * pdhcpaddressnumber,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice)
{
  lcrzo_string keynet, keymstcp, keylongmstcpi;
  lcrzo_string value, value2, keymstcpi;
  lcrzo_int32 count, i, addressnumber;
  lcrzo_list list;
  lcrzo_device device, dev;
  lcrzo_bool trywithdhcp, isppp;
  int ret;

  isppp = LCRZO_FALSE;

  /* obtain the name of the key net */
  ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, keyname, "Driver",
                                     &value);
  if (ret == LCRZO_ERR_OK) {
    /* construct the key containing the device name */
    ret =
      lcrzo_string_initm_text("System\\CurrentControlSet\\Services\\Class\\",
                              &keynet);
    if (ret == LCRZO_ERR_OK) {
      ret = lcrzo_string_appendm_text(value, &keynet);
      if (ret == LCRZO_ERR_OK) {
        /* test for PPP */
        ret = lcrzo_priv_winreg_data_get(HKEY_LOCAL_MACHINE, keynet,
                                         "PointToPointIP", NULL, NULL);
        if (ret == LCRZO_ERR_OK) {
          isppp = LCRZO_TRUE;
        }
        ret = lcrzo_string_appendm_text("\\NDIS", &keynet);
        if (ret == LCRZO_ERR_OK) {
          /* obtain the device name */
          /*printf("keynet=%s\n", keynet); */
          ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE, keynet,
                                             "LogDriverName", &value2);
          if (ret == LCRZO_ERR_OK) {
            ret = lcrzo_device_init(value2, device);
            lcrzo_er(lcrzo_string_free2(&value2));
          }
        }
      }
      lcrzo_er(lcrzo_string_free2(&keynet));
    }
    lcrzo_er(lcrzo_string_free2(&value));
  }
  /* this device is counted even if it is bad */
  /*printf("Device = %s\n", device); */
  (*pdevicenumber)++;
  (*pcardnumber)++;
  addressnumber = 1;
  if (ret != LCRZO_ERR_OK) {
    /* skip it */
    return (LCRZO_ERR_OK);
  }

  /* unfortunately, there is no information about PPP is the registry */
  if (isppp) {
    if (setdevices) {
      ret = lcrzo_priv_conf_incomplete_add_7(device);
    }
    /* ignore the errors */
    return (LCRZO_ERR_OK);
  }

  /* add info corresponding to each device */
  trywithdhcp = LCRZO_FALSE;
  ret = lcrzo_string_initm_text(keyname, &keymstcp);
  if (ret == LCRZO_ERR_OK) {
    ret = lcrzo_string_appendm_text("\\Bindings", &keymstcp);
    if (ret == LCRZO_ERR_OK) {
      ret =
        lcrzo_priv_winreg_subvallist_get(HKEY_LOCAL_MACHINE, keymstcp, &list);
      if (ret == LCRZO_ERR_OK) {
        ret = lcrzo_list_count(&list, &count);
        /*printf("count=%d\n", count); */
        if (ret == LCRZO_ERR_OK) {
          for (i = 1;
               i <= count && ret == LCRZO_ERR_OK && (!searchipl
                                                     || !*piplfound); i++) {
            ret = lcrzo_stringlist_valuem_pos(&list, i, NULL, &keymstcpi);
            if (ret == LCRZO_ERR_OK) {
              /*printf("keymstcpi=%s\n", keymstcpi); */
              ret = lcrzo_string_initm_text("Enum\\Network\\", &keylongmstcpi);
              if (ret == LCRZO_ERR_OK) {
                ret = lcrzo_string_appendm_text(keymstcpi, &keylongmstcpi);
                if (ret == LCRZO_ERR_OK) {
                  /* verify it is TCP/IP and set values */
                  ret =
                    lcrzo_priv_conf_init_W9xregistry3(device, keylongmstcpi,
                                                      pdevicenumber,
                                                      *pcardnumber,
                                                      &addressnumber,
                                                      &trywithdhcp,
                                                      setdevices, setroutes,
                                                      searchipl, iplwanted,
                                                      piplfound, founddevice);
                }
                lcrzo_er(lcrzo_string_free2(&keylongmstcpi));
              }
              lcrzo_er(lcrzo_string_free2(&keymstcpi));
            }
          }
        }
        lcrzo_er(lcrzo_list_close(&list));
      }
    }
    lcrzo_er(lcrzo_string_free2(&keymstcp));
  }

  if (trywithdhcp) {
    lcrzo_priv_conf_validdhcpinfo vdi;

    ret = lcrzo_priv_device_init_ascii(device, dev);
    if (ret == LCRZO_ERR_OK) {
      ret = lcrzo_priv_conf_validdhcpinfo_get(dev, *pdhcpaddressnumber, &vdi);
      if (ret == LCRZO_ERR_OK) {
        (*pdhcpaddressnumber)++;
        if (setdevices) {
          /* add device */
          ret = lcrzo_priv_conf_winras_devices_add_system(dev,
                                                  LCRZO_DEVICE_HWTYPE_ETHER,
                                                          vdi.eth, vdi.ip,
                                                          vdi.mask, 1500,
                                                          LCRZO_TRUE,
                                                          LCRZO_FALSE);
        }
        if (setroutes) {
          /* add routes */
          for (i = 0; i < LCRZO_PRIV_CONF_VALIDIPINFO_MAXGW && vdi.gw[i] != 0;
               i++) {
            ret = lcrzo_priv_conf_routes_add_system(dev, 0, 0, vdi.ip,
                                                    vdi.gw[i],
                                                    i + 1, LCRZO_TRUE);
          }
        }
        if (searchipl) {
          if (vdi.ip != 0 && vdi.ip == iplwanted) {
            lcrzo_er(lcrzo_device_init(dev, founddevice));
            *piplfound = LCRZO_TRUE;
          }
        }
      }
    }
  }

  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_W9xregistry1(lcrzo_conststring keyname1,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 * pcardnumber,
                                      lcrzo_int32 * pdhcpaddressnumber,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice);
int lcrzo_priv_conf_init_W9xregistry1(lcrzo_conststring keyname1,
                                      lcrzo_int32 * pdevicenumber,
                                      lcrzo_int32 * pcardnumber,
                                      lcrzo_int32 * pdhcpaddressnumber,
                                      lcrzo_bool setdevices,
                                      lcrzo_bool setroutes,
                                      lcrzo_bool searchipl,
                                      lcrzo_ipl iplwanted,
                                      lcrzo_bool * piplfound,
                                      lcrzo_device founddevice)
{
  lcrzo_list list2;
  lcrzo_string keyname2, value;
  lcrzo_int32 j, count2;
  int ret, retf;

  retf = LCRZO_ERR_OK;
  /* go one step further */
  ret = lcrzo_priv_winreg_subkeylist_get(HKEY_LOCAL_MACHINE, keyname1, &list2);
  if (ret == LCRZO_ERR_OK) {
    ret = lcrzo_list_count(&list2, &count2);
    if (ret == LCRZO_ERR_OK) {
      for (j = 1;
           j <= count2 && ret == LCRZO_ERR_OK && (!searchipl || !*piplfound);
           j++) {
        ret = lcrzo_stringlist_valuem_pos(&list2, j, NULL, &keyname2);
        if (ret == LCRZO_ERR_OK) {
          /*printf("%s\n", keyname2); */

          ret = lcrzo_priv_winreg_string_get(HKEY_LOCAL_MACHINE,
                                             (lcrzo_string) keyname2, "Class",
                                             &value);
          if (ret == LCRZO_ERR_OK) {
            /*printf("%s\n", value); */
            if (!strcmp(value, "Net")) {
              /*printf("%s\n", keyname2); */
              retf = lcrzo_priv_conf_init_W9xregistry2(keyname2,
                                                       pdevicenumber,
                                                       pcardnumber,
                                                       pdhcpaddressnumber,
                                                       setdevices, setroutes,
                                                       searchipl, iplwanted,
                                                       piplfound,
                                                       founddevice);
            }
            lcrzo_er(lcrzo_string_free2(&value));
          }
          lcrzo_er(lcrzo_string_free2(&keyname2));
        }
      }
    }
    lcrzo_er(lcrzo_list_close(&list2));
  }

  /* use retf to differentiate errors on keys (which might be absent,
     so ignored), and errors of functions */
  return (retf);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_W9xregistry(lcrzo_bool setdevices,
                                     lcrzo_bool setroutes,
                                     lcrzo_bool searchipl,
                                     lcrzo_ipl iplwanted,
                                     lcrzo_device founddevice);
int lcrzo_priv_conf_init_W9xregistry(lcrzo_bool setdevices,
                                     lcrzo_bool setroutes,
                                     lcrzo_bool searchipl,
                                     lcrzo_ipl iplwanted,
                                     lcrzo_device founddevice)
{
  lcrzo_list list0, list1;
  lcrzo_string keyname0, keyname1;
  lcrzo_int32 h, i, count0, count1;
  lcrzo_int32 devicenumber, cardnumber, dhcpaddressnumber;
  lcrzo_bool iplfound;
  int ret, retf;

  /* First, add loopback */
  if (setdevices) {
    lcrzo_etha etha;

    lcrzo_er(lcrzo_etha_initdefault(etha));
    lcrzo_er(lcrzo_priv_conf_devices_add_system
             ("Loopback", LCRZO_DEVICE_HWTYPE_LOOPBACK, etha, 0x7F000001,
              0xFF000000u, 1500, LCRZO_TRUE, LCRZO_FALSE, LCRZO_FALSE, 0));
  }

  /* init global variables */
  devicenumber = 0;
  cardnumber = 0;
  dhcpaddressnumber = 1;
  iplfound = LCRZO_FALSE;

  /* loop */
  retf = LCRZO_ERR_OK;
  lcrzo_er(lcrzo_priv_winreg_subkeylist_get
           (HKEY_LOCAL_MACHINE, "Enum", &list0));
  ret = lcrzo_list_count(&list0, &count0);
  if (ret == LCRZO_ERR_OK) {
    for (h = 1;
         h <= count0 && ret == LCRZO_ERR_OK && (!searchipl || !iplfound);
         h++) {
      ret = lcrzo_stringlist_valuem_pos(&list0, h, NULL, &keyname0);
      if (ret == LCRZO_ERR_OK) {
        /*printf("%s\n", keyname0); */

        /* go one step further */
        ret =
          lcrzo_priv_winreg_subkeylist_get(HKEY_LOCAL_MACHINE, keyname0,
                                           &list1);
        if (ret == LCRZO_ERR_OK) {
          ret = lcrzo_list_count(&list1, &count1);
          if (ret == LCRZO_ERR_OK) {
            for (i = 1;
                 i <= count1 && ret == LCRZO_ERR_OK && (!searchipl
                                                        || !iplfound); i++) {
              ret = lcrzo_stringlist_valuem_pos(&list1, i, NULL, &keyname1);
              if (ret == LCRZO_ERR_OK) {
                /*printf("%s\n", keyname1); */
                retf = lcrzo_priv_conf_init_W9xregistry1(keyname1,
                                                         &devicenumber,
                                                         &cardnumber,
                                                         &dhcpaddressnumber,
                                                         setdevices,
                                                         setroutes, searchipl,
                                                         iplwanted, &iplfound,
                                                         founddevice);
                lcrzo_er(lcrzo_string_free2(&keyname1));
              }
            }
          }
          lcrzo_er(lcrzo_list_close(&list1));
        }
        lcrzo_er(lcrzo_string_free2(&keyname0));
      }
    }
  }
  lcrzo_er(lcrzo_list_close(&list0));

  if (searchipl && !iplfound) {
    return (LCRZO_ERR_OKUNRESOLVED);
  }
  /* use retf to differentiate errors on keys (which might
     be absent, so ignored), and errors of functions */
  return (retf);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_dev_W9xregistry(void);
int lcrzo_priv_conf_init_dev_W9xregistry(void)
{
  return (lcrzo_priv_conf_init_W9xregistry(LCRZO_TRUE, LCRZO_FALSE,
                                           LCRZO_FALSE, 0, NULL));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_dev_snmpdll(void);
int lcrzo_priv_conf_init_dev_snmpdll(void)
{
  lcrzo_priv_dll_snmpreq snmpdllreqip, snmpdllreqdev;
  AsnObjectIdentifier oid0, oid1, oid2, oid3, oid4;
  lcrzo_uint32 i, numberofcards, mtu, mactype, ifindexip, ifindexdev,
    ifindexprevious, speed;
  lcrzo_ipl ipaddress, ipnetmask;
  lcrzo_etha etha;
  lcrzo_bool foundinterface;
  lcrzo_int16 hwtype, addressnumber;

  if (!lcrzo_global_dll_snmp.isloaded)
    return (LCRZO_ERR_IEUNIMPLEMENTED);

  /*lcrzo_priv_snmpdll_printall(); */

  /* get the number of interfaces */
  lcrzo_er(lcrzo_priv_dll_snmpoid_init(&oid0, 1, 3, 6, 1, 2, 1, 2, 1, -1));
  lcrzo_er(lcrzo_priv_dll_snmpreq_init
           (&snmpdllreqdev, &oid0, NULL, NULL, NULL, NULL));
  lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid0));
  lcrzo_er(lcrzo_priv_dll_snmpreq_getnext(&snmpdllreqdev));
  /*lcrzo_er(lcrzo_priv_snmpdllreq_print(&snmpdllreqdev)); */
  lcrzo_er(lcrzo_priv_dll_snmpreq__uint32(&snmpdllreqdev, 0, &numberofcards));
  /*printf("Number of cards = %d\n", numberofcards); */
  lcrzo_er(lcrzo_priv_dll_snmpreq_close(&snmpdllreqdev));

  /* just for the compiler warning */
  ifindexdev = 0;
  mtu = 0;
  hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  addressnumber = 0;

  /* get characteristics of the IP addresses */
  lcrzo_er(lcrzo_priv_dll_snmpoid_init
           (&oid0, 1, 3, 6, 1, 2, 1, 4, 20, 1, 1, -1));
  lcrzo_er(lcrzo_priv_dll_snmpoid_init
           (&oid1, 1, 3, 6, 1, 2, 1, 4, 20, 1, 2, -1));
  lcrzo_er(lcrzo_priv_dll_snmpoid_init
           (&oid2, 1, 3, 6, 1, 2, 1, 4, 20, 1, 3, -1));
  lcrzo_er(lcrzo_priv_dll_snmpreq_init
           (&snmpdllreqip, &oid0, &oid1, &oid2, NULL, NULL));
  lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid0));
  lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid1));
  lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid2));
  ifindexprevious = 0xFFFFFFFFu;
  lcrzo_er(lcrzo_priv_dll_snmpreq_getnext(&snmpdllreqip));
  while (snmpdllreqip.varBindList.list[0].name.ids[7] == 20
         && snmpdllreqip.varBindList.list[0].name.ids[9] == 1) {
    /*lcrzo_er(lcrzo_priv_snmpdllreq_print(&snmpdllreqip)); */
    lcrzo_er(lcrzo_priv_dll_snmpreq__uint32(&snmpdllreqip, 0, &ipaddress));
    lcrzo_er(lcrzo_priv_dll_snmpreq__uint32(&snmpdllreqip, 1, &ifindexip));
    lcrzo_er(lcrzo_priv_dll_snmpreq__uint32(&snmpdllreqip, 2, &ipnetmask));

    /* now find information corresponding to the interface number */
    /* do not ask twice */
    if (ifindexip == ifindexprevious) {
      foundinterface = LCRZO_TRUE;
      addressnumber++;
    } else {
      foundinterface = LCRZO_FALSE;
      addressnumber = 1;
    }

    if (!foundinterface) {
      lcrzo_er(lcrzo_priv_dll_snmpoid_init
               (&oid0, 1, 3, 6, 1, 2, 1, 2, 2, 1, 1, -1));
      lcrzo_er(lcrzo_priv_dll_snmpoid_init
               (&oid1, 1, 3, 6, 1, 2, 1, 2, 2, 1, 3, -1));
      lcrzo_er(lcrzo_priv_dll_snmpoid_init
               (&oid2, 1, 3, 6, 1, 2, 1, 2, 2, 1, 4, -1));
      lcrzo_er(lcrzo_priv_dll_snmpoid_init
               (&oid3, 1, 3, 6, 1, 2, 1, 2, 2, 1, 5, -1));
      lcrzo_er(lcrzo_priv_dll_snmpoid_init
               (&oid4, 1, 3, 6, 1, 2, 1, 2, 2, 1, 6, -1));
      lcrzo_er(lcrzo_priv_dll_snmpreq_init
               (&snmpdllreqdev, &oid0, &oid1, &oid2, &oid3, &oid4));
      lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid0));
      lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid1));
      lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid2));
      lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid3));
      lcrzo_er(lcrzo_priv_dll_snmpoid_close(&oid4));
      for (i = 0; i < numberofcards; i++) {
        lcrzo_er(lcrzo_priv_dll_snmpreq_getnext(&snmpdllreqdev));
        /*lcrzo_er(lcrzo_priv_snmpdllreq_print(&snmpdllreqdev)); */
        lcrzo_er(lcrzo_priv_dll_snmpreq__uint32
                 (&snmpdllreqdev, 0, &ifindexdev));
        lcrzo_er(lcrzo_priv_dll_snmpreq__uint32(&snmpdllreqdev, 1, &mactype));
        lcrzo_er(lcrzo_priv_dll_snmpreq__uint32(&snmpdllreqdev, 2, &mtu));
        lcrzo_er(lcrzo_priv_dll_snmpreq__uint32(&snmpdllreqdev, 3, &speed));
        lcrzo_er(lcrzo_priv_dll_snmpreq__etha(&snmpdllreqdev, 4, etha));
        if (ifindexdev == ifindexip) {
          foundinterface = LCRZO_TRUE;
          lcrzo_er(lcrzo_priv_hwtype_init_hwtype_snmp
                   (mactype, speed, &hwtype));
          break;
        }
      }
      lcrzo_er(lcrzo_priv_dll_snmpreq_close(&snmpdllreqdev));
    }

    if (foundinterface) {
      ifindexprevious = ifindexdev;
      /* add it */
      lcrzo_er(lcrzo_priv_conf_devicenumbername_devices_add_system
               (ifindexip, hwtype, etha, ipaddress, ipnetmask, mtu,
                addressnumber));
    } else {
      lcrzo_er(lcrzo_priv_conf_incomplete_add_6
               (ifindexip, ipaddress, ipnetmask,
                (lcrzo_bool) ((addressnumber ==
                               1) ? LCRZO_FALSE : LCRZO_TRUE)));
    }

    lcrzo_er(lcrzo_priv_dll_snmpreq_getnext(&snmpdllreqip));
  }
  lcrzo_er(lcrzo_priv_dll_snmpreq_close(&snmpdllreqip));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_dev_iphlpapidll(void);
int lcrzo_priv_conf_init_dev_iphlpapidll(void)
{
  lcrzo_uint32 bufsize, i;
  PMIB_IPADDRTABLE pipaddrtable;
  PMIB_IFROW pifrow;
  lcrzo_uint32 mtu, ifindexprevious, devicenumber;
  lcrzo_etha etha;
  lcrzo_bool foundinterface;
  lcrzo_int16 hwtype, addressnumber;
  lcrzo_ipl ipaddress, ipnetmask;
  int ret;

  if (!lcrzo_global_dll_iphlpapi.isloaded)
    return (LCRZO_ERR_IEUNIMPLEMENTED);

  /* get the IP table size */
  bufsize = 0;
  ret = (lcrzo_global_dll_iphlpapi.GetIpAddrTable) (NULL, &bufsize, FALSE);
  if (ret != ERROR_INSUFFICIENT_BUFFER) {
    return (LCRZO_ERR_FUGETIPADDRTABLE);
  }
  lcrzo_er(lcrzo_priv_err_purge_int());

  /* allocate data */
  lcrzo_er(lcrzo_data_alloc(bufsize, (lcrzo_data *) & pipaddrtable));

  /* get the IP table */
  ret = lcrzo_global_dll_iphlpapi.GetIpAddrTable(pipaddrtable,
                                                 &bufsize, FALSE);
  if (ret != NO_ERROR) {
    lcrzo_er(lcrzo_data_free2((lcrzo_data *) & pipaddrtable));
    return (LCRZO_ERR_FUGETIPADDRTABLE);
  }
  lcrzo_er(lcrzo_priv_err_purge_int());

  /* loop through each entry */
  ifindexprevious = 0xFFFFFFFFu;
  addressnumber = 0;
  hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  mtu = 0;
  for (i = 0; i < pipaddrtable->dwNumEntries; i++) {
    devicenumber = pipaddrtable->table[i].dwIndex;
    ipaddress = lcrzo_htonl(pipaddrtable->table[i].dwAddr);
    ipnetmask = lcrzo_htonl(pipaddrtable->table[i].dwMask);

    /* do not ask twice */
    if (devicenumber == ifindexprevious) {
      foundinterface = LCRZO_TRUE;
      addressnumber++;
    } else {
      foundinterface = LCRZO_FALSE;
      addressnumber = 1;
    }

    if (!foundinterface) {
      /* get interface information */
      lcrzo_er(lcrzo_data_alloc(sizeof(MIB_IFROW), (lcrzo_data *) & pifrow));
      /* search for this index */
      pifrow->dwIndex = devicenumber;
      ret = lcrzo_global_dll_iphlpapi.GetIfEntry(pifrow);
      if (ret == NO_ERROR) {
        /* found the wanted interface */
        lcrzo_er(lcrzo_priv_err_purge_int());
        foundinterface = LCRZO_TRUE;
        lcrzo_er(lcrzo_priv_hwtype_init_hwtype_snmp
                 (pifrow->dwType, pifrow->dwSpeed, &hwtype));
        if (hwtype == LCRZO_DEVICE_HWTYPE_ETHER) {
          lcrzo_er(lcrzo_etha_init
                   (pifrow->bPhysAddr[0], pifrow->bPhysAddr[1],
                    pifrow->bPhysAddr[2], pifrow->bPhysAddr[3],
                    pifrow->bPhysAddr[4], pifrow->bPhysAddr[5], etha));
        } else {                /* if it's not Ethernet */
          lcrzo_er(lcrzo_etha_initdefault(etha));
        }
        mtu = pifrow->dwMtu;
      }
      lcrzo_er(lcrzo_data_free2((lcrzo_data *) & pifrow));
    }

    if (foundinterface) {
      ifindexprevious = devicenumber;
      /* add it */
      ret = lcrzo_priv_conf_devicenumbername_devices_add_system(devicenumber,
                                                                hwtype, etha,
                                                                ipaddress,
                                                                ipnetmask, mtu,
                                                                addressnumber);
    } else {
      ret = lcrzo_priv_conf_incomplete_add_6(devicenumber, ipaddress,
                                             ipnetmask,
                                             (lcrzo_bool) ((addressnumber ==
                                                            1) ? LCRZO_FALSE :
                                                           LCRZO_TRUE));
    }

  }

  lcrzo_er(lcrzo_data_free2((lcrzo_data *) & pipaddrtable));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_init_dev_gethostbyname(void);
int lcrzo_priv_conf_init_dev_gethostbyname(void)
{
  lcrzo_hs hs;
  struct hostent *phe;
  lcrzo_device device;
  lcrzo_etha etha;
  lcrzo_ipl ip, mask;
  lcrzo_int16 hwtype;
  int ret, i;

  /* First, add loopback */
  lcrzo_er(lcrzo_etha_initdefault(etha));
  lcrzo_er(lcrzo_priv_conf_devices_add_system
           ("Loopback", LCRZO_DEVICE_HWTYPE_LOOPBACK, etha, 0x7F000001,
            0xFF000000u, 1500, LCRZO_TRUE, LCRZO_FALSE, LCRZO_FALSE, 0));

  /* note: we do not start this function with
     lcrzo_priv_conf_localip_init_gethostbyname,
     otherwise we loose the order (needed by
     lcrzo_priv_conf_etha_init_cardnumberdevice) */

  /* get the local hostname */
  ret = gethostname(hs, LCRZO_HS_MAXBYTES);
  if (ret != 0)
    return (LCRZO_ERR_OKUNRESOLVED);

  /* get the corresponding IP addresses */
  phe = gethostbyname(hs);
  if (phe == NULL)
    return (LCRZO_ERR_OKUNRESOLVED);
  if (phe->h_addrtype != AF_INET)
    return (LCRZO_ERR_OKUNRESOLVED);
  if (phe->h_length != LCRZO_IPA_MAXBYTES)
    return (LCRZO_ERR_OKUNRESOLVED);

  /* loop through each device */
  i = 0;
  while (phe->h_addr_list[i] != NULL) {
    ip = ((lcrzo_uint8) phe->h_addr_list[i][0]) << 24;
    ip |= ((lcrzo_uint8) phe->h_addr_list[i][1]) << 16;
    ip |= ((lcrzo_uint8) phe->h_addr_list[i][2]) << 8;
    ip |= ((lcrzo_uint8) phe->h_addr_list[i][3]);
    if ((ip & 0x80000000u) == 0)
      mask = 0xFF000000u;
    else if ((ip & 0xC0000000u) == 0x80000000u)
      mask = 0xFFFF0000u;
    else if ((ip & 0xE0000000u) == 0xC0000000u)
      mask = 0xFFFFFF00u;
    else
      mask = 0xFFFFFF00u;
    ret = lcrzo_priv_conf_device_init_ipl(ip, device);
    if (ret == LCRZO_ERR_OK) {
      ret =
        lcrzo_priv_conf_etha_init_cardnumberdevice(device, i + 1, i + 1,
                                                   LCRZO_FALSE, &hwtype,
                                                   etha);
      if (ret == LCRZO_ERR_OK) {
        ret =
          lcrzo_priv_conf_winras_devices_add_system(device, hwtype, etha, ip,
                                                    mask, 1500, LCRZO_TRUE,
                                                    LCRZO_FALSE);
      } else {
        ret = lcrzo_priv_conf_incomplete_add_2(device, ip);
      }
    } else {
      ret =
        lcrzo_priv_conf_etha_init_cardnumberdevice("", i + 1, i + 1,
                                                   LCRZO_FALSE, &hwtype,
                                                   etha);
      if (ret == LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_incomplete_add_3(i, hwtype, etha, ip);
      } else {
        ret = lcrzo_priv_conf_incomplete_add_4(i, ip);
      }
    }
    i++;
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/* this function doesn't work correctly for RAS. Moreover there are
   bugs in versions <= 2.3Beta (WinPcap developers contacted, and code
   corrected) */
int lcrzo_priv_conf_init_dev_packetdll_notused(void);
int lcrzo_priv_conf_init_dev_packetdll_notused(void)
{
  lcrzo_priv_confvars *ptrcv;
  lcrzo_priv_conf_deviceeth de;
  lcrzo_int32 cardnumber, i, count;
  lcrzo_ipl ip, mask;
  lcrzo_device dev;
  lcrzo_etha etha;
  lcrzo_int16 hwtype;
  BOOL err;
  int ret;

  lcrzo_er(lcrzo_priv_conf_localdevice_eventuallyinit());
  lcrzo_er(lcrzo_priv_confvars_ptr_get(&ptrcv));

  /* loop for each adapter name */
  cardnumber = 0;
  ret = lcrzo_list_count(&ptrcv->list_localdevice, &count);
  if (ret == LCRZO_ERR_OK) {
    for (i = 1; i <= count; i++) {
      ret = lcrzo_list_value_pos(&ptrcv->list_localdevice, i, &de);
      if (ret == LCRZO_ERR_OK) {
        /* get the IP address for this device */
        cardnumber++;
        ret = lcrzo_priv_ascii_init_device(de.device, dev);
        if (ret == LCRZO_ERR_OK) {
          err = PacketGetNetInfo(dev, &ip, &mask);
          if (err) {
            ret =
              lcrzo_priv_conf_etha_init_cardnumberdevice(de.device,
                                                         cardnumber,
                                                         cardnumber,
                                                         LCRZO_FALSE, &hwtype,
                                                         etha);
            if (ret == LCRZO_ERR_OK) {
              ret =
                lcrzo_priv_conf_winras_devices_add_system(de.device, hwtype,
                                                          etha, ip, mask,
                                                          1500, LCRZO_TRUE,
                                                          LCRZO_FALSE);
            }
          }
        }
      }
    }
  }
  if (cardnumber == 0)
    return (LCRZO_ERR_OKUNRESOLVED);
  return (LCRZO_ERR_OK);
}

#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/

int lcrzo_priv_conf_init_dev(void)
/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Linux || defined LCRZODEF_SYSTEM_Solaris
{
  int fd, i, nbinterface, ret;
  struct ifconf ifc;
  struct ifreq ireqbuf[100];
  struct sockaddr_in soin;
  lcrzo_bool isup, isanalias, isptp;
  lcrzo_device device;
  struct ifreq ifr;
  lcrzo_etha etha;
  lcrzo_ipl ipl, netmask, ptpipl;
  lcrzo_uint32 mtu;
  lcrzo_int16 hwtype;

  /*on ouvre une socket car c'est la dessus que l'on fait un ioctl */
  fd = socket(AF_INET, SOCK_DGRAM, 0);
  if (fd < 0)
    return (LCRZO_ERR_FUSOCKET);

  /*le buffer va comporter toutes les interfaces */
  ifc.ifc_len = sizeof(ireqbuf);        /*taille totale */
  ifc.ifc_ifcu.ifcu_req = ireqbuf;      /*position de depart */
  memset((char *)ireqbuf, 0, sizeof(ireqbuf));

  /*met dans ifc la config locale */
  if (ioctl(fd, SIOCGIFCONF, (char *)&ifc) < 0) {
    close(fd);
    return (LCRZO_ERR_FUIOCTL);
  }

  nbinterface = ifc.ifc_len / sizeof(struct ifreq);

  ptpipl = 0;                   /* for the compiler warning */
  /*parcours la boucle */
  for (i = 0; i < nbinterface; i++) {
    ret = lcrzo_device_init(ireqbuf[i].ifr_name, device);
    if (ret != LCRZO_ERR_OK) {
      close(fd);
      return (ret);
    }
    /* get the info from the device */
    memset(&ifr, 0, sizeof(ifr));
    lcrzo_er(lcrzo_string_init_text
             (device, sizeof(ifr.ifr_name) - 1, ifr.ifr_name));
    ifr.ifr_addr.sa_family = AF_INET;
    if (ioctl(fd, SIOCGIFFLAGS, (char *)&ifr) < 0) {
      close(fd);
      return (LCRZO_ERR_FUIOCTL);
    }
    /* check if up */
    if (ifr.ifr_flags & IFF_UP)
      isup = 1;
    else
      isup = 0;
    /* check if is point to point */
    if (ifr.ifr_flags & IFF_POINTOPOINT)
      isptp = 1;
    else
      isptp = 0;
    /* check if it's an alias */
    ret = lcrzo_string_search_char(device, 0, +1, -1, ':', NULL, NULL);
    if (ret == LCRZO_ERR_OKSEARCHNOTFOUND)
      isanalias = 0;
    else
      isanalias = 1;
    /* get the Ethernet address */
#if defined LCRZODEF_SYSTEM_Solaris
    ret = lcrzo_priv_priv_etha_init_device(device, &hwtype, etha);
    if (ret != LCRZO_ERR_OK) {
      close(fd);
      return (ret);
    }
#elif defined LCRZODEF_SYSTEM_Linux
    memset(&ifr, 0, sizeof(ifr));
    lcrzo_er(lcrzo_string_init_text
             (device, sizeof(ifr.ifr_name) - 1, ifr.ifr_name));
    ifr.ifr_addr.sa_family = AF_INET;
    if (ioctl(fd, SIOCGIFHWADDR, (char *)&ifr) < 0) {
      close(fd);
      return (LCRZO_ERR_FUIOCTL);
    }
    lcrzo_er(lcrzo_priv_hwtype_init_hwtype
             (ifr.ifr_hwaddr.sa_family, &hwtype));
    if (hwtype == LCRZO_DEVICE_HWTYPE_ETHER) {
      memcpy(etha, &ifr.ifr_hwaddr.sa_data, LCRZO_ETHA_MAXBYTES);
    } else {
      lcrzo_er(lcrzo_etha_initdefault(etha));
    }
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
    /* get the IP address */
    memset(&ifr, 0, sizeof(ifr));
    lcrzo_er(lcrzo_string_init_text
             (device, sizeof(ifr.ifr_name) - 1, ifr.ifr_name));
    ifr.ifr_addr.sa_family = AF_INET;
    if (ioctl(fd, SIOCGIFADDR, (char *)&ifr) < 0) {
      close(fd);
      return (LCRZO_ERR_FUIOCTL);
    }
    memcpy(&soin, &(ifr.ifr_addr), sizeof(struct sockaddr_in));
    ipl = lcrzo_ntohl(soin.sin_addr.s_addr);
    /* get the netmask */
    memset(&ifr, 0, sizeof(ifr));
    lcrzo_er(lcrzo_string_init_text
             (device, sizeof(ifr.ifr_name) - 1, ifr.ifr_name));
    ifr.ifr_addr.sa_family = AF_INET;
    if (ioctl(fd, SIOCGIFNETMASK, (char *)&ifr) < 0) {
      close(fd);
      return (LCRZO_ERR_FUIOCTL);
    }
    memcpy(&soin, &(ifr.ifr_addr), sizeof(struct sockaddr_in));
    netmask = lcrzo_ntohl(soin.sin_addr.s_addr);
    /* get the ptpipl */
    if (isptp) {
      memset(&ifr, 0, sizeof(ifr));
      lcrzo_er(lcrzo_string_init_text
               (device, sizeof(ifr.ifr_name) - 1, ifr.ifr_name));
      ifr.ifr_addr.sa_family = AF_INET;
      if (ioctl(fd, SIOCGIFDSTADDR, (char *)&ifr) < 0) {
        close(fd);
        return (LCRZO_ERR_FUIOCTL);
      }
      memcpy(&soin, &(ifr.ifr_addr), sizeof(struct sockaddr_in));
      ptpipl = lcrzo_ntohl(soin.sin_addr.s_addr);
    }
    /* get the mtu */
    memset(&ifr, 0, sizeof(ifr));
    lcrzo_er(lcrzo_string_init_text
             (device, sizeof(ifr.ifr_name) - 1, ifr.ifr_name));
    ifr.ifr_addr.sa_family = AF_INET;
    if (ioctl(fd, SIOCGIFMTU, (char *)&ifr) < 0) {
      close(fd);
      return (LCRZO_ERR_FUIOCTL);
    }
#if defined LCRZODEF_SYSTEM_Solaris
    mtu = ifr.ifr_metric;
#else
    mtu = ifr.ifr_mtu;
#endif

    /* save it in the configuration */
    ret = lcrzo_priv_conf_devices_add_system(device, hwtype, etha,
                                             ipl, netmask, mtu,
                                             isup, isanalias, isptp, ptpipl);
    if (ret != LCRZO_ERR_OK) {
      close(fd);
      return (ret);
    }
  }

  ret = close(fd);
  if (ret == -1)
    return (LCRZO_ERR_FUCLOSE);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#elif defined LCRZODEF_SYSTEM_FreeBSD || defined LCRZODEF_SYSTEM_OpenBSD || defined LCRZODEF_SYSTEM_MacOS
{
  lcrzo_uint32 correctsize, mtu;
  lcrzo_device device;
  lcrzo_etha etha;
  lcrzo_bool isup, isanalias, isptp, lastwasaifinfo;
  lcrzo_ipl ipl, netmask, ptpipl;
  lcrzo_int16 hwtype;
  int mib[6], flags;
  char *tmpbuf, *lim, *next;
  struct if_msghdr *ifm;
  struct sockaddr_dl *sdl;
  LAURENT_size_t len;

  mib[0] = CTL_NET;             /*reseau */
  mib[1] = PF_ROUTE;            /*famille de protocole */
  mib[2] = 0;                   /*numero de protocole */
  mib[3] = AF_INET;             /*famille d'adresse */
  mib[4] = NET_RT_IFLIST;       /*liste des interfaces */
  mib[5] = 0;

  /*on fait d'abord un premier appel pour connaitre la taille a allouer */
  if (sysctl(mib, 6, NULL, &len, NULL, 0) < 0) {
    return (LCRZO_ERR_FUSYSCTL);
  }

  /* This is a trick : we have to allocate more, because struct if_msghdr
     is defined as :
     struct if_msghdr {
     u_short ifm_msglen;
     etc. (lot of data)
     }
     The item ifm_msglen permits to skip over unrecognized entries.
     Unfortunately, "sizeof(struct if_msghdr)>size_of_last_block".
     So, the call "lcrzo_er(lcrzo_data_initm_data((lcrzo_data)next, 
     sizeof(struct if_msghdr), (lcrzo_data*)&ifm, NULL));" fails
     for the last block of NET_RT_IFLIST (because we try to read data
     past the end of tmpbuf).
     FreeBSD should have defined for example :
     struct if_msghdr_skip {
     u_short ifm_msglen;
     }
     So, the call posing problem could be replaced by :
     "lcrzo_er(lcrzo_data_initm_data((lcrzo_data)next, 
     sizeof(struct if_msghdr_skip), (lcrzo_data*)&ifm, NULL));"
     Moreover, in future versions of FreeBSD, they could use :
     struct if_msghdr {
     u_""long"" ifm_msglen;
     etc. (lot of data)
     }
     So, the only way to create a code which will also work in future
     versions of FreeBSD is to allocate more data than needed :
   */
  len += sizeof(struct if_msghdr);

  /*on alloue la taille necessaire */
  lcrzo_er(lcrzo_string_alloc(len, &tmpbuf));

  /*on fait le vrai appel */
  if (sysctl(mib, 6, tmpbuf, &len, NULL, 0) < 0) {
    lcrzo_er(lcrzo_string_free2(&tmpbuf));
    return (LCRZO_ERR_FUSYSCTL);
  }
  lim = tmpbuf + len;

  /* just for the compiler warning (should always start with RTM_IFINFO) */
  hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  mtu = 0;
  isup = LCRZO_FALSE;
  isanalias = LCRZO_FALSE;
  isptp = LCRZO_FALSE;
  lastwasaifinfo = LCRZO_FALSE;
  /*on parcours les elements de la liste trouvee */
  next = tmpbuf;
  while (next < lim) {
    /*405 ifm = (struct if_msghdr *)next; */
    lcrzo_er(lcrzo_data_initm_data
             ((lcrzo_data) next, sizeof(struct if_msghdr),
              (lcrzo_data *) & ifm, NULL));
    correctsize = ifm->ifm_msglen;
    lcrzo_er(lcrzo_data_free2((lcrzo_data *) & ifm));
    lcrzo_er(lcrzo_data_initm_data((lcrzo_data) next, correctsize,
                                   (lcrzo_data *) & ifm, NULL));
    next += ifm->ifm_msglen;

    /* update device information (device, hwtype, etha, mtu, up) */
    if (ifm->ifm_type == RTM_IFINFO) {
      /* special case of a down device do not having IP/net address */
      if (lastwasaifinfo) {
        lcrzo_er(lcrzo_priv_conf_devices_add_system(device, hwtype,
                                                    etha, 0, 0, mtu,
                                                    LCRZO_FALSE, LCRZO_FALSE,
                                                    LCRZO_FALSE, 0));
      }
      /* continue */
      sdl = (struct sockaddr_dl *)(ifm + 1);
      /* sdl->sdl_nlen contains the device name len, not terminated by 0 */
      if (sdl->sdl_nlen > LCRZO_DEVICE_MAXBYTES)
        return (LCRZO_ERR_PAOVERFLOW);
      lcrzo_er(lcrzo_string_init_text
               (sdl->sdl_data, LCRZO_DEVICE_MAXBYTES, device));
      device[sdl->sdl_nlen] = '\0';
      /* get the Ethernet address */
      lcrzo_er(lcrzo_priv_hwtype_init_hwtype(sdl->sdl_type, &hwtype));
      if (hwtype == LCRZO_DEVICE_HWTYPE_ETHER) {
        memcpy(etha, LLADDR(sdl), LCRZO_ETHA_MAXBYTES);
      } else {
        lcrzo_er(lcrzo_etha_initdefault(etha));
      }
      /* get the mtu */
      mtu = (ifm->ifm_data).ifi_mtu;
      /* get the up flag */
      flags = ifm->ifm_flags;
      if (flags & IFF_UP)
        isup = LCRZO_TRUE;
      else
        isup = LCRZO_FALSE;
      if (flags & IFF_POINTOPOINT)
        isptp = LCRZO_TRUE;
      else
        isptp = LCRZO_FALSE;
      /* reset alias count */
      isanalias = LCRZO_FALSE;
      lastwasaifinfo = LCRZO_TRUE;
    } else {
      lastwasaifinfo = LCRZO_FALSE;
    }

    /* found address information */
    if (ifm->ifm_type == RTM_NEWADDR) {
      struct ifa_msghdr *ifam;
      struct sockaddr *psa;
      struct sockaddr_in *psai;
      int ifamaddr, i;

      /* it contains ifa_msghdr followed by sockaddrs */
      ifam = (struct ifa_msghdr *)ifm;
      /* ifamaddr contains a bitwise set of sockaddrs types 
         which are in ifm */
      ifamaddr = ifam->ifam_addrs;
      /* initialize the pointer to the first sockaddr */
      psa = (struct sockaddr *)((char *)ifm + sizeof(struct ifa_msghdr));
      /* we look if one of the sockaddr contains ipl or netmask */
      netmask = 0;              /* just for the compiler */
      ipl = 0;                  /* just for the compiler */
      ptpipl = 0;               /* just for the compiler */
      for (i = 0; i < RTAX_MAX; i++) {
        if (ifamaddr & (1 << i)) {
          /* we are only interested by NETMASK and IFA */
          if (i == RTAX_NETMASK || i == RTAX_IFA || i == RTAX_BRD) {
            /*405 psai=(struct sockaddr_in *)psa; */
            lcrzo_er(lcrzo_data_initm_data((lcrzo_data) psa,
                                           FB_ROUNDUP(psa->sa_len),
                                           (lcrzo_data *) & psai, NULL));
            if (i == RTAX_NETMASK) {
              netmask = lcrzo_ntohl(psai->sin_addr.s_addr);
            } else if (i == RTAX_IFA) {
              ipl = lcrzo_ntohl(psai->sin_addr.s_addr);
            } else if (i == RTAX_BRD) {
              ptpipl = lcrzo_ntohl(psai->sin_addr.s_addr);
            }
            lcrzo_er(lcrzo_data_free2((lcrzo_data *) & psai));
          }
          /* psa points to the next item */
          psa = (struct sockaddr *)(((char *)psa) + FB_ROUNDUP(psa->sa_len));
        }
      }
      /* we add the device */
      lcrzo_er(lcrzo_priv_conf_devices_add_system(device, hwtype,
                                                  etha, ipl, netmask, mtu,
                                                  isup, isanalias, isptp,
                                                  ptpipl));
      isanalias = LCRZO_TRUE;
    }

    lcrzo_er(lcrzo_data_free2((lcrzo_data *) & ifm));
  }

  lcrzo_er(lcrzo_string_free2(&tmpbuf));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#elif defined LCRZODEF_SYSTEM_Unsupported
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#elif defined LCRZODEF_SYSTEM_Windows
{
  int ret;
  lcrzo_priv_globalvars *ptrglo;

  lcrzo_er(lcrzo_priv_globalvars_ptr_get(&ptrglo));
  switch ((int)ptrglo->windows_systemversion) {
    case LCRZO_WINDOWS_SYSTEMVERSION_nt4:
      ret = lcrzo_priv_conf_init_dev_iphlpapidll();
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_snmpdll();
      }
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_NTregistry();
      }
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_gethostbyname();
      }
      break;

    case LCRZO_WINDOWS_SYSTEMVERSION_2k:
    case LCRZO_WINDOWS_SYSTEMVERSION_xp:
      ret = lcrzo_priv_conf_init_dev_iphlpapidll();
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_snmpdll();
      }
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_2kregistry();
      }
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_gethostbyname();
      }
      break;

    case LCRZO_WINDOWS_SYSTEMVERSION_95:
    case LCRZO_WINDOWS_SYSTEMVERSION_98:
    case LCRZO_WINDOWS_SYSTEMVERSION_me:
      ret = lcrzo_priv_conf_init_dev_iphlpapidll();
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_snmpdll();
      }
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_gethostbyname();
      }
      if (ret != LCRZO_ERR_OK) {
        /* does not always work */
        ret = lcrzo_priv_conf_init_dev_W9xregistry();
      }
      break;

    default:
      ret = lcrzo_priv_conf_init_dev_iphlpapidll();
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_snmpdll();
      }
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_2kregistry();
      }
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_NTregistry();
      }
      if (ret != LCRZO_ERR_OK) {
        ret = lcrzo_priv_conf_init_dev_gethostbyname();
      }
      if (ret != LCRZO_ERR_OK) {
        /* does not always work */
        ret = lcrzo_priv_conf_init_dev_W9xregistry();
      }
      break;
  }

  /* return OK even if nothing was initialized */
  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
/*-------------------------------------------------------------*/
     int lcrzo_priv_conf_dev_debug_fprint(LCRZOFILE * pf)
#ifdef LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_OK);
  pf = pf;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  lcrzo_priv_confvars *ptrcv;

  lcrzo_erpcw1o(pf, lcrzo_priv_confvars_ptr_get(&ptrcv),
                "lcrzo_priv_confvars_ptr_get");

  lcrzo_erpcw1o(pf, lcrzo_list_remove_all(&ptrcv->list_devicenumbername),
                "lcrzo_list_remove_all");
  fprintf(pf, "\n\n++++++++++++++++++++++++++++++++\n");
  fprintf(pf, "Configuration obtained with lcrzo_priv_conf_init_dev_iphlpapidll\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_before(),
                "lcrzo_priv_conf_incomplete_job_before");
  lcrzo_erpcw2o(pf, lcrzo_priv_conf_init_dev_iphlpapidll(),
                "lcrzo_priv_conf_init_dev_iphlpapidll",
                LCRZO_ERR_IEUNIMPLEMENTED);
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_debug_fprint(pf),
                "lcrzo_priv_conf_incomplete_job_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_device_init_ipl_debug_fprint(pf),
                "lcrzo_priv_conf_device_init_ipl_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_etha_init_device_debug_fprint(pf),
                "lcrzo_priv_conf_etha_init_device_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_devicenumbername_debug_fprint(pf),
                "lcrzo_priv_conf_devicenumbername_debug_fprint");

  lcrzo_erpcw1o(pf, lcrzo_list_remove_all(&ptrcv->list_devicenumbername),
                "lcrzo_list_remove_all");
  fprintf(pf, "\n\n++++++++++++++++++++++++++++++++\n");
  fprintf(pf,
          "Configuration obtained with lcrzo_priv_conf_init_dev_snmpdll\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_before(),
                "lcrzo_priv_conf_incomplete_job_before");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_init_dev_snmpdll(),
                "lcrzo_priv_conf_init_dev_snmpdll");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_debug_fprint(pf),
                "lcrzo_priv_conf_incomplete_job_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_device_init_ipl_debug_fprint(pf),
                "lcrzo_priv_conf_device_init_ipl_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_etha_init_device_debug_fprint(pf),
                "lcrzo_priv_conf_etha_init_device_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_devicenumbername_debug_fprint(pf),
                "lcrzo_priv_conf_devicenumbername_debug_fprint");

  fprintf(pf, "\n\n++++++++++++++++++++++++++++++++\n");
  fprintf(pf,
          "Configuration obtained with lcrzo_priv_conf_init_dev_NTregistry\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_before(),
                "lcrzo_priv_conf_incomplete_job_before");
  lcrzo_erpcw2o(pf, lcrzo_priv_conf_init_dev_NTregistry(),
                "lcrzo_priv_conf_init_dev_NTregistry",,
                LCRZO_ERR_FUREGOPENKEYEX);
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_debug_fprint(pf),
                "lcrzo_priv_conf_incomplete_job_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_device_init_ipl_debug_fprint(pf),
                "lcrzo_priv_conf_device_init_ipl_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_etha_init_device_debug_fprint(pf),
                "lcrzo_priv_conf_etha_init_device_debug_fprint");

  fprintf(pf, "\n\n++++++++++++++++++++++++++++++++\n");
  fprintf(pf,
          "Configuration obtained with lcrzo_priv_conf_init_dev_2kregistry\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_before(),
                "lcrzo_priv_conf_incomplete_job_before");
  lcrzo_erpcw2o(pf, lcrzo_priv_conf_init_dev_2kregistry(),
                "lcrzo_priv_conf_init_dev_2kregistry",,
                LCRZO_ERR_FUREGOPENKEYEX);
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_debug_fprint(pf),
                "lcrzo_priv_conf_incomplete_job_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_device_init_ipl_debug_fprint(pf),
                "lcrzo_priv_conf_device_init_ipl_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_etha_init_device_debug_fprint(pf),
                "lcrzo_priv_conf_etha_init_device_debug_fprint");

  fprintf(pf, "\n\n++++++++++++++++++++++++++++++++\n");
  fprintf(pf,
          "Configuration obtained with lcrzo_priv_conf_init_dev_gethostbyname\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_before(),
                "lcrzo_priv_conf_incomplete_job_before");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_init_dev_gethostbyname(),
                "lcrzo_priv_conf_init_dev_gethostbyname");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_debug_fprint(pf),
                "lcrzo_priv_conf_incomplete_job_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_device_init_ipl_debug_fprint(pf),
                "lcrzo_priv_conf_device_init_ipl_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_etha_init_device_debug_fprint(pf),
                "lcrzo_priv_conf_etha_init_device_debug_fprint");

  fprintf(pf, "\n\n++++++++++++++++++++++++++++++++\n");
  fprintf(pf,
          "Configuration obtained with lcrzo_priv_conf_init_dev_W9xregistry\n");
  lcrzo_erpcw1o(pf, lcrzo_conf_flush_system(), "lcrzo_conf_flush_system");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_before(),
                "lcrzo_priv_conf_incomplete_job_before");
  lcrzo_erpcw2o(pf, lcrzo_priv_conf_init_dev_W9xregistry(),
                "lcrzo_priv_conf_init_dev_W9xregistry",
                LCRZO_ERR_FUREGOPENKEYEX);
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_incomplete_job_debug_fprint(pf),
                "lcrzo_priv_conf_incomplete_job_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_conf_fprint(pf), "lcrzo_conf_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_device_init_ipl_debug_fprint(pf),
                "lcrzo_priv_conf_device_init_ipl_debug_fprint");
  lcrzo_erpcw1o(pf, lcrzo_priv_conf_etha_init_device_debug_fprint(pf),
                "lcrzo_priv_conf_etha_init_device_debug_fprint");

  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
