/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_system_verify_routes(lcrzo_priv_confvars * ptr)
{
  lcrzo_int32 count, i, cdcount, cdi;
  lcrzo_priv_conf_devices cd;
  lcrzo_priv_conf_routes cr;
  lcrzo_device device;
  lcrzo_bool needtoupdateip, needtoupdatedev, needtodeletedev, hasaptp;
  lcrzo_bool deviceequal;

  /* eventually update gateway and device */

  lcrzo_er(lcrzo_list_count(&(ptr->list_system_routes), &count));
  for (i = 1; i <= count; i++) {
    needtoupdatedev = LCRZO_FALSE;
    needtoupdateip = LCRZO_FALSE;
    needtodeletedev = LCRZO_FALSE;
    hasaptp = LCRZO_FALSE;
    lcrzo_er(lcrzo_list_value_pos(&(ptr->list_system_routes), i, &cr));
    lcrzo_er(lcrzo_list_count(&(ptr->list_system_devices), &cdcount));
    for (cdi = 1; cdi <= cdcount; cdi++) {
      lcrzo_er(lcrzo_list_value_pos(&(ptr->list_system_devices), cdi, &cd));
      if (!cd.isup)
        continue;
      if (cd.ispointtopoint)
        hasaptp = LCRZO_TRUE;
      /* complete fields */
      if (cr.ipsource != 0) {
        /* need to set device */
        if (cr.ipsource == cd.ip) {
          /* ipsource was found in the device list */
          lcrzo_er(lcrzo_device_init(cd.device, device));
          needtoupdatedev = LCRZO_TRUE;
          /* test if it's not a broadcast (set on Windows) : to delete */
          if (cr.mask == 0xFFFFFFFFu) {
            lcrzo_uint32 theip;
            /* ex: 1.0.0.0 | ~255.0.0.0 = 1.0.0.0 
               | 0.255.255.255 = 1.255.255.255 */
            theip = cd.ip | (~cd.netmask);
            if (theip == cr.dest) {
              needtodeletedev = LCRZO_TRUE;
            }
          }
          break;
        }
      } else {
        /* need to set ipsource and device */
        if (cr.gateway != 0) {
          /* need to find the route to this gateway */
          if ((cd.ip & cd.netmask) == (cr.gateway & cd.netmask)) {
            cr.ipsource = cd.ip;
            lcrzo_er(lcrzo_device_init(cd.device, device));
            needtoupdatedev = LCRZO_TRUE;
            needtoupdateip = LCRZO_TRUE;
            break;
          }
        } else {
          /* need to find the route to ... */
          if (cd.ip == cr.dest) {
            /* ... the device */
            cr.ipsource = 0;
            lcrzo_er(lcrzo_device_init(cd.device, device));
            needtoupdatedev = LCRZO_TRUE;
            needtoupdateip = LCRZO_TRUE;
            break;
          }
          if (cd.netmask == cr.mask && (cd.ip & cd.netmask) == cr.dest) {
            /* ... the lan */
            cr.ipsource = cd.ip;
            lcrzo_er(lcrzo_device_init(cd.device, device));
            needtoupdatedev = LCRZO_TRUE;
            needtoupdateip = LCRZO_TRUE;
            break;
          }
        }
      }
    }

    /*special case for ptp */
    if (hasaptp) {
      for (cdi = 1; cdi <= cdcount; cdi++) {
        lcrzo_er(lcrzo_list_value_pos(&(ptr->list_system_devices), cdi, &cd));
        if (!cd.isup)
          continue;
        if (!strcmp(cr.device, cd.device) &&
            cd.ispointtopoint && cr.dest != cd.ip) {
          cr.ipsource = cd.ip;
          cr.gateway = cd.pointtopointipl;
          needtoupdateip = LCRZO_TRUE;
        }
      }
    }

    /* verify the device we want to set */
    if (needtoupdatedev) {
      /* if they are equal, do not update */
      lcrzo_er(lcrzo_device_equal2(device, cr.device, &deviceequal));
      if (deviceequal) {
        needtoupdatedev = LCRZO_FALSE;
      } else {
        if (strlen(device) == 0) {
          needtoupdatedev = LCRZO_FALSE;
        } else {
          lcrzo_er(lcrzo_device_init(device, cr.device));
        }
      }
    }

    /* eventually update */
    if (needtoupdatedev || needtoupdateip) {
      lcrzo_er(lcrzo_list_replace_pos(&(ptr->list_system_routes), i, &cr));
    }

    /* eventually remove */
    if (needtodeletedev) {
      lcrzo_er(lcrzo_list_remove_pos(&(ptr->list_system_routes), i));
      i--;
      count--;
    }

  }

  /* sort the routes : from "host routes" to "default route" */
  lcrzo_er(lcrzo_list_sort_all2(&(ptr->list_system_routes),
                                lcrzo_priv_conf_routes_cmp, NULL));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_conf_system_renew(lcrzo_priv_confvars * ptr)
{
  int ret;

  /* remove all permanent entries */
  lcrzo_er(lcrzo_list_remove_all(&ptr->list_system_devices));
  lcrzo_er(lcrzo_list_remove_all(&ptr->list_system_arpperm));
  lcrzo_er(lcrzo_list_remove_all(&ptr->list_system_routes));

  /* remove all intermediate values */
  lcrzo_er(lcrzo_list_remove_all(&ptr->list_devicenumbername));
  lcrzo_er(lcrzo_list_remove_all(&ptr->list_localeth));
  lcrzo_er(lcrzo_list_remove_all(&ptr->list_localip));
  lcrzo_er(lcrzo_list_remove_all(&ptr->list_validdhcpinfo));

  /* remove previous incomplete values */
  lcrzo_er(lcrzo_list_remove_all(&ptr->list_incomplete));

  /* set the new permanent values */
#if LCRZO_DEBUG_CONF_FAKE==0

#if defined LCRZODEF_SYSTEM_Windows
  lcrzo_er(lcrzo_priv_conf_incomplete_job_before());
#endif
  ret = lcrzo_priv_conf_init_dev();
  if (ret != LCRZO_ERR_OK && ret != LCRZO_ERR_IEUNIMPLEMENTED)
    return (ret);
#if defined LCRZODEF_SYSTEM_Windows
  lcrzo_er(lcrzo_priv_conf_incomplete_job_after());
#endif
  ret = lcrzo_priv_conf_init_arpcache();
  if (ret != LCRZO_ERR_OK && ret != LCRZO_ERR_IEUNIMPLEMENTED)
    return (ret);
  ret = lcrzo_priv_conf_init_rarpcache();
  if (ret != LCRZO_ERR_OK && ret != LCRZO_ERR_IEUNIMPLEMENTED)
    return (ret);
  ret = lcrzo_priv_conf_init_routes();
  if (ret != LCRZO_ERR_OK && ret != LCRZO_ERR_IEUNIMPLEMENTED)
    return (ret);
#else
  /* just an example for debugging */
  {
    lcrzo_etha etha;
    lcrzo_ipl ip, mask;

    lcrzo_er(lcrzo_etha_init(0, 0x40, 0x95, 0x46, 0x14, 0xCB, etha));
    lcrzo_er(lcrzo_ipl_init_ips("192.168.10.20", &ip));
    lcrzo_er(lcrzo_ipl_init_ips("255.255.255.0", &mask));
    lcrzo_er(lcrzo_priv_conf_devices_add_system("RTL8029", etha, ip, mask,
                                                1500, LCRZO_TRUE,
                                                LCRZO_FALSE));
  }
#endif

  /* ignore internal errors. For example, under Windows, if a DLL wasn't
     found, do not display GetLastError. */
  lcrzo_er(lcrzo_err_purge());

  /* We compute ipsource if it is != 0 (not often set by
     system dependant functions)
     We eventually update device (Linux never set aliases, Solaris
     set local devices to "")
     Moreover, sort the routes accordingly to the netmask (FreeBSD
     doesn't do it) */
  lcrzo_er(lcrzo_priv_conf_system_verify_routes(ptr));

  return (LCRZO_ERR_OK);
}
