/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_priv_fd_wait(int fd,
                       lcrzo_time * pmaxwaitingtime,
                       lcrzo_bool * pevent)
#if LCRZODEF_HAVEFONC_POLL==1
#include <sys/poll.h>
{
  struct pollfd pollarray[1];
  lcrzo_uint32 timeoutuint;
  lcrzo_int32 timeoutint;
  int ret, mask;

  /* set wanted mask */
#if ! defined POLLRDNORM
#define POLLRDNORM 0
#endif
#if ! defined POLLRDBAND
#define POLLRDBAND 0
#endif
  mask = POLLIN | POLLPRI | POLLRDNORM | POLLRDBAND | POLLHUP;

  /* set fd */
  pollarray[0].fd = fd;
  pollarray[0].events = mask;
  pollarray[0].revents = 0;

  /* set timeout */
  if (pmaxwaitingtime->sec == LCRZO_WAIT_NODELAY_VAL &&
      pmaxwaitingtime->usec == LCRZO_WAIT_NODELAY_VAL) {
    timeoutint = 0;
  } else if (pmaxwaitingtime->sec == LCRZO_WAIT_INFINITE_VAL &&
             pmaxwaitingtime->usec == LCRZO_WAIT_INFINITE_VAL) {
    timeoutint = -1;
  } else {
    ret = lcrzo_time_get_msec(pmaxwaitingtime, &timeoutuint);
    if (ret == LCRZO_ERR_OK) {
      if (timeoutuint > 0x7FFFFFFF)
        timeoutint = -1;
      else
        timeoutint = timeoutuint;
    } else if (ret == LCRZO_ERR_SPTIMEMSECOVER) {
      timeoutint = -1;
    } else
      return (ret);
  }

  /* wait */
  ret = poll(pollarray, 1, timeoutint);
  if (ret < 0)
    return (LCRZO_ERR_FUPOLL);

  /* set error values */
  if (ret == 0) {
    if (pevent != NULL)
      *pevent = LCRZO_FALSE;
    return (LCRZO_ERR_OK);
  }

  /* set normal values */
  if (pevent != NULL) {
    if (pollarray[0].revents & mask)
      *pevent = LCRZO_TRUE;
    else
      *pevent = LCRZO_FALSE;
  }
  return (LCRZO_ERR_OK);
}
#else
{
  lcrzo_bool event;
  fd_set readfdset, exceptfdset;
  struct timeval timeouttv, *ptv;
  int ret;

  /* initialize fd */
  FD_ZERO(&readfdset);
  FD_ZERO(&exceptfdset);
  FD_SET(fd, &readfdset);
  FD_SET(fd, &exceptfdset);

  /* set timeout */
  ptv = &timeouttv;
  if (pmaxwaitingtime->sec == LCRZO_WAIT_NODELAY_VAL &&
      pmaxwaitingtime->usec == LCRZO_WAIT_NODELAY_VAL) {
    timeouttv.tv_sec = 0;
    timeouttv.tv_usec = 0;
  } else if (pmaxwaitingtime->sec == LCRZO_WAIT_INFINITE_VAL &&
             pmaxwaitingtime->usec == LCRZO_WAIT_INFINITE_VAL) {
    ptv = NULL;
  } else {
    timeouttv.tv_sec = pmaxwaitingtime->sec;
    timeouttv.tv_usec = pmaxwaitingtime->usec;
  }

  /* select */
  ret = select(fd + 1, &readfdset, NULL, &exceptfdset, ptv);
  if (ret < 0)
    return (LCRZO_ERR_FUSELECT);

  /* check output */
  event = LCRZO_FALSE;
  if (FD_ISSET(fd, &readfdset) || FD_ISSET(fd, &exceptfdset)) {
    event = LCRZO_TRUE;
  }

  if (pevent != NULL)
    *pevent = event;
  return (LCRZO_ERR_OK);
}
#endif
