/* This file is part of Lcrzo.
*/

#ifdef LCRZODEF_SYSTEM_Linux
int lcrzo_priv_hwtype_init_hwtype(lcrzo_uint32 systemhwtype,
                                  lcrzo_int16 * phwtype)
{
  lcrzo_int16 hwtype;

  if (systemhwtype == ARPHRD_ETHER)
    hwtype = LCRZO_DEVICE_HWTYPE_ETHER;
  else if (systemhwtype == ARPHRD_LOOPBACK)
    hwtype = LCRZO_DEVICE_HWTYPE_LOOPBACK;
  else if (systemhwtype == ARPHRD_PPP)
    hwtype = LCRZO_DEVICE_HWTYPE_PPP;
  else {
    /*lcrzo_er(lcrzo_uint32_print("Warning : hardware type unknown = ", 
      systemhwtype, "%d", ""));
      lcrzo_er(lcrzo_uint32_print(" = 0x", systemhwtype , "%04X", "\n")); */
    hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  }

  if (phwtype != NULL)
    *phwtype = hwtype;
  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_FreeBSD || defined LCRZODEF_SYSTEM_OpenBSD | defined LCRZODEF_SYSTEM_MacOS
int lcrzo_priv_hwtype_init_hwtype(lcrzo_uint32 systemhwtype,
                                  lcrzo_int16 * phwtype)
{
  lcrzo_int16 hwtype;

  if (systemhwtype == IFT_ETHER)
    hwtype = LCRZO_DEVICE_HWTYPE_ETHER;
  else if (systemhwtype == IFT_LOOP)
    hwtype = LCRZO_DEVICE_HWTYPE_LOOPBACK;
  else if (systemhwtype == IFT_PPP)
    hwtype = LCRZO_DEVICE_HWTYPE_PPP;
  else if (systemhwtype == IFT_PARA)
    hwtype = LCRZO_DEVICE_HWTYPE_PLIP;
  else if (systemhwtype == IFT_SLIP)
    hwtype = LCRZO_DEVICE_HWTYPE_SLIP;
  else {
    /*lcrzo_er(lcrzo_uint32_print("Warning : hardware type unknown = ", 
      systemhwtype, "%d", ""));
      lcrzo_er(lcrzo_uint32_print(" = 0x", systemhwtype , "%04X", "\n")); */
    hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  }

  if (phwtype != NULL)
    *phwtype = hwtype;
  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Solaris
int lcrzo_priv_hwtype_init_hwtype(lcrzo_uint32 systemhwtype,
                                  lcrzo_int16 * phwtype)
{
  lcrzo_int16 hwtype;

  if (systemhwtype == DL_ETHER)
    hwtype = LCRZO_DEVICE_HWTYPE_ETHER;
  else {
    /*lcrzo_er(lcrzo_uint32_print("Warning : hardware type unknown = ", 
      systemhwtype, "%d", ""));
      lcrzo_er(lcrzo_uint32_print(" = 0x", systemhwtype , "%04X", "\n")); */
    hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  }

  if (phwtype != NULL)
    *phwtype = hwtype;
  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Unsupported
int lcrzo_priv_hwtype_init_hwtype(lcrzo_uint32 systemhwtype,
                                  lcrzo_int16 * phwtype)
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  systemhwtype = systemhwtype;
  phwtype = phwtype;
}
#elif defined LCRZODEF_SYSTEM_Windows
int lcrzo_priv_hwtype_init_hwtype_ndis(lcrzo_uint32 systemhwtype,
                                       lcrzo_uint32 speed_bps,
                                       lcrzo_int16 * phwtype)
{
  lcrzo_int16 hwtype;

  if (systemhwtype == NdisMedium802_3) {
    hwtype = LCRZO_DEVICE_HWTYPE_ETHER;
    if (speed_bps < LCRZO_PRIV_DEVICE_HWTYPE_ETHER_MINSPEED)
      hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  } else {
    /*lcrzo_er(lcrzo_uint32_print("Warning : NDIS hardware type unknown = ", 
      systemhwtype, "%d", ""));
      lcrzo_er(lcrzo_uint32_print(" = 0x", systemhwtype , "%04X", "\n")); */
    hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  }

  if (phwtype != NULL)
    *phwtype = hwtype;
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_hwtype_init_hwtype_snmp(lcrzo_uint32 systemhwtype,
                                       lcrzo_uint32 speed_bps,
                                       lcrzo_int16 * phwtype)
{
  lcrzo_int16 hwtype;

  if (systemhwtype == MIB_IF_TYPE_ETHERNET) {
    hwtype = LCRZO_DEVICE_HWTYPE_ETHER;
    if (speed_bps < LCRZO_PRIV_DEVICE_HWTYPE_ETHER_MINSPEED)
      hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  } else if (systemhwtype == MIB_IF_TYPE_LOOPBACK)
    hwtype = LCRZO_DEVICE_HWTYPE_LOOPBACK;
  else {
    /*lcrzo_er(lcrzo_uint32_print("Warning : SNMP hardware type unknown = ", 
      systemhwtype, "%d", ""));
      lcrzo_er(lcrzo_uint32_print(" = 0x", systemhwtype , "%04X", "\n")); */
    hwtype = LCRZO_DEVICE_HWTYPE_UNKNOWN;
  }

  if (phwtype != NULL)
    *phwtype = hwtype;
  return (LCRZO_ERR_OK);
}

int lcrzo_priv_hwtype_completewithdevice(lcrzo_int16 precomputedhwtype,
                                         lcrzo_constdevice device,
                                         lcrzo_bool devicenameisshort,
                                         lcrzo_int16 * phwtype)
{
  lcrzo_priv_globalvars *ptrglo;
  lcrzo_int16 hwtype;

  hwtype = precomputedhwtype;
  if (phwtype != NULL)
    *phwtype = hwtype;

  /* can not compute */
  if (strlen(device) == 0)
    return (LCRZO_ERR_OK);

  /* already computed */
  if (hwtype != LCRZO_DEVICE_HWTYPE_UNKNOWN)
    return (LCRZO_ERR_OK);

  /* loopback */
  if (!strncmp(device, "Loopback", 8)) {
    if (phwtype != NULL)
      *phwtype = LCRZO_DEVICE_HWTYPE_LOOPBACK;
    return (LCRZO_ERR_OK);
  }

  /* depends on system */
  lcrzo_er(lcrzo_priv_globalvars_ptr_get(&ptrglo));
  switch ((int)ptrglo->windows_systemversion) {
    case LCRZO_WINDOWS_SYSTEMVERSION_nt4:
    case LCRZO_WINDOWS_SYSTEMVERSION_2k:
    case LCRZO_WINDOWS_SYSTEMVERSION_xp:
      if (devicenameisshort && !strncmp(device, "NdisWan", 7))
        hwtype = LCRZO_DEVICE_HWTYPE_PPP;
      else if (!devicenameisshort &&
               !strncmp(device, "\\Device\\Packet_NdisWan", 22))
        hwtype = LCRZO_DEVICE_HWTYPE_PPP;
      else if (!devicenameisshort &&
               !strncmp(device, "\\Device\\NPF_NdisWan", 19))
        hwtype = LCRZO_DEVICE_HWTYPE_PPP;
      break;
    case LCRZO_WINDOWS_SYSTEMVERSION_95:
    case LCRZO_WINDOWS_SYSTEMVERSION_98:
    case LCRZO_WINDOWS_SYSTEMVERSION_me:
      if (!strncmp(device, "PPPMAC", 6))
        hwtype = LCRZO_DEVICE_HWTYPE_PPP;
      break;
    default:
      break;
  }

  /* set output */
  if (phwtype != NULL)
    *phwtype = hwtype;
  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
