/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_priv_kbd_key_echo2(lcrzo_kbd * pkbd,
                             lcrzo_uint8 * pcarac);
int lcrzo_priv_kbd_key_echo2(lcrzo_kbd * pkbd,
                             lcrzo_uint8 * pcarac)
{
  lcrzo_data data;
  int ret;

  if (pkbd->echokeypresses) {
    ret = lcrzo_kbd_readm_fixed(pkbd, LCRZO_WAIT_INFINITE, 1, &data);
  } else {
    lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_TRUE));
    ret = lcrzo_kbd_readm_fixed(pkbd, LCRZO_WAIT_INFINITE, 1, &data);
    lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_FALSE));
  }

  if (ret == LCRZO_ERR_OK) {
    if (pcarac != NULL)
      *pcarac = data[0];
    lcrzo_er(lcrzo_data_free2(&data));
  }
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_kbd_key_echo(lcrzo_kbd * pkbd,
                            lcrzo_uint8 * pcarac)
#if defined LCRZODEF_SYSTEM_Unix
{
  lcrzo_uint8 carac;
  struct termios old_termios, new_termios;

  lcrzo_er(lcrzo_kbd_purge(pkbd));

  /* cases which cannot be treated here */
  if (pkbd->type != LCRZO_PRIV_HANDLE_TYPE_CONSOLE) {
    return (lcrzo_priv_kbd_key_echo2(pkbd, pcarac));
  }
  if (pkbd->fd != 0) {
    return (lcrzo_priv_kbd_key_echo2(pkbd, pcarac));
  }

  /* change mode */
  if (tcgetattr(pkbd->fd, &old_termios))
    return (LCRZO_ERR_FUTCGETATTR);
  new_termios = old_termios;
  new_termios.c_lflag &= (~ICANON);
  new_termios.c_cc[VMIN] = 1;   /* need one char */
  new_termios.c_cc[VTIME] = 0;  /* wait forever */
  new_termios.c_lflag |= ECHO;
  if (tcsetattr(pkbd->fd, TCSANOW, &new_termios))
    return (LCRZO_ERR_FUTCSETATTR);

  /* read */
  carac = (lcrzo_uint8) fgetc(stdin);

  /* reset mode */
  if (tcsetattr(pkbd->fd, TCSANOW, &old_termios))
    return (LCRZO_ERR_FUTCSETATTR);

  if (pcarac != NULL)
    *pcarac = carac;
  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  lcrzo_uint8 carac;
  BOOL err;

  lcrzo_er(lcrzo_kbd_purge(pkbd));

  /* cases which cannot be treated here */
  if (pkbd->type != LCRZO_PRIV_HANDLE_TYPE_CONSOLE) {
    return (lcrzo_priv_kbd_key_echo2(pkbd, pcarac));
  }

  carac = 0;
  while (LCRZO_TRUE) {
    carac = (lcrzo_uint8) _getch();
    lcrzo_er(lcrzo_priv_err_purge_int());
    if (carac == 0x08) {
      /* ignore backspace */
    } else if (carac == 0x00) {
      /* ignore Function-i, but get the second part */
      _getch();
    } else if (carac == 0xE0) {
      /* ignore arrow, but get the second part */
      _getch();
    } else if (carac == 0x03) {
      /* on control-c, generate the event (then the program should exit) */
      err = GenerateConsoleCtrlEvent(CTRL_C_EVENT, 0);
      if (!err) {
        return (LCRZO_ERR_FUGENERATECONSOLECTRLEVENT);
      }
    } else {
      /* ok */
      break;
    }
  }

  /* display pressed key */
  if (carac == 0x0D) {
    /* because the newline is suppressed */
    putchar(0x0D);
    putchar(0x0A);
  } else {
    putchar(carac);
  }

  if (pcarac != NULL)
    *pcarac = carac;
  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
