/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_priv_kbddefault_line_echo(lcrzo_string * pstring)
{
  lcrzo_kbd kbd;

  lcrzo_er(lcrzo_kbd_initdefault(&kbd));
  lcrzo_er(lcrzo_priv_kbd_line_echo(&kbd, pstring));
  lcrzo_er(lcrzo_kbd_close(&kbd));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_kbd_line_echo2(lcrzo_kbd * pkbd,
                              lcrzo_string * pstring);
int lcrzo_priv_kbd_line_echo2(lcrzo_kbd * pkbd,
                              lcrzo_string * pstring)
{
  lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  if (pkbd->echokeypresses) {
    ret = lcrzo_kbd_readm_line(pkbd, LCRZO_WAIT_INFINITE, &data, &datasize);
  } else {
    lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_TRUE));
    ret = lcrzo_kbd_readm_line(pkbd, LCRZO_WAIT_INFINITE, &data, &datasize);
    lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_FALSE));
  }

  if (ret == LCRZO_ERR_OK) {
    data[datasize] = '\0';
    if (pstring == NULL) {
      lcrzo_er(lcrzo_data_free2(&data));
    } else {
      *pstring = (lcrzo_string) data;
    }
  }
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_kbd_line_echo3(lcrzo_kbd * pkbd,
                              lcrzo_string * pstring);
int lcrzo_priv_kbd_line_echo3(lcrzo_kbd * pkbd,
                              lcrzo_string * pstring)
#if defined LCRZODEF_SYSTEM_Unix
{
  char *ptr;
  lcrzo_int32 fgetstep, ptrallocsize, userstringsize;
  lcrzo_bool changeecho;
  lcrzo_bool resettc;
  struct termios old_termios, new_termios;
  int c;

  fgetstep = 80;
  ptrallocsize = fgetstep;
  lcrzo_er(lcrzo_string_alloc(ptrallocsize, &ptr));

  changeecho = LCRZO_FALSE;
  if (!pkbd->echokeypresses) {
    lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_TRUE));
    changeecho = LCRZO_TRUE;
  }

  /* ensures kbd is in ICANON (for example this is not the case
     with Solaris) */
  resettc = LCRZO_FALSE;
  if (tcgetattr(pkbd->fd, &old_termios))
    return (LCRZO_ERR_FUTCGETATTR);
  if (!(new_termios.c_lflag & ICANON)) {
    new_termios = old_termios;
    new_termios.c_lflag |= ICANON;
    if (tcsetattr(pkbd->fd, TCSANOW, &new_termios))
      return (LCRZO_ERR_FUTCSETATTR);
    resettc = LCRZO_TRUE;
  }

  userstringsize = 0;
  while (LCRZO_TRUE) {
    c = getc(stdin);
    if (c == 0x0D || c == 0x0A || c == EOF)
      break;
    ptr[userstringsize] = (char)c;
    userstringsize++;
    if (userstringsize == ptrallocsize) {
      ptrallocsize += fgetstep;
      lcrzo_er(lcrzo_string_realloc(ptrallocsize, &ptr));
    }
  }
  ptr[userstringsize] = '\0';

  /* restore attributes */
  if (resettc) {
    if (tcsetattr(pkbd->fd, TCSANOW, &old_termios))
      return (LCRZO_ERR_FUTCSETATTR);
  }

  if (changeecho) {
    lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_FALSE));
  }

  if (pstring != NULL) {
    *pstring = ptr;
  } else {
    lcrzo_er(lcrzo_string_free2(&ptr));
  }

  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  char *ptr;
  lcrzo_int32 fgetstep, ptrallocsize, userstringsize;
  int c;
  BOOL err;

  fgetstep = 80;
  ptrallocsize = fgetstep;
  lcrzo_er(lcrzo_string_alloc(ptrallocsize, &ptr));

  userstringsize = 0;
  while (LCRZO_TRUE) {
    /* cannot use getche because backspace can erase data
       printed before calling the function */
    c = _getch();
    if (c == 0x0D || c == 0x0A) {
      putchar(0x0D);
      putchar(0x0A);
      break;
    } else if (c == 0x08) {
      /* backspace */
      if (userstringsize) {
        userstringsize--;
        putchar(0x08);
        putchar(' ');
        putchar(0x08);
      }
    } else if (c == 0x00) {
      /* Function-i ignore, but get the second part */
      _getch();
    } else if (c == 0xE0) {
      /* ignore arrow, but get the second part */
      _getch();
    } else if (c == 0x03) {
      /* on control-c, generate the event (then the program should exit) */
      err = GenerateConsoleCtrlEvent(CTRL_C_EVENT, 0);
      if (!err) {
        lcrzo_er(lcrzo_string_free2(&ptr));
        return (LCRZO_ERR_FUGENERATECONSOLECTRLEVENT);
      }
    } else {
      ptr[userstringsize] = (char)c;
      userstringsize++;
      if (userstringsize == ptrallocsize) {
        ptrallocsize += fgetstep;
        lcrzo_er(lcrzo_string_realloc(ptrallocsize, &ptr));
      }
      putchar(c);
    }
  }
  lcrzo_er(lcrzo_priv_err_purge_int());
  ptr[userstringsize] = '\0';

  if (pstring != NULL) {
    *pstring = ptr;
  } else {
    lcrzo_er(lcrzo_string_free2(&ptr));
  }

  return (LCRZO_ERR_OK);
  pkbd = pkbd;
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
/*-------------------------------------------------------------*/
     int lcrzo_priv_kbd_line_echo(lcrzo_kbd * pkbd,
                                  lcrzo_string * pstring)
#if defined LCRZODEF_SYSTEM_Unix
{
  lcrzo_er(lcrzo_kbd_purge(pkbd));

  /* cases which cannot be treated here */
  if (pkbd->type != LCRZO_PRIV_HANDLE_TYPE_CONSOLE) {
    return (lcrzo_priv_kbd_line_echo2(pkbd, pstring));
  }
  if (pkbd->fd != 0) {
    return (lcrzo_priv_kbd_line_echo2(pkbd, pstring));
  }

  lcrzo_er(lcrzo_priv_kbd_line_echo3(pkbd, pstring));
  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
{

  lcrzo_er(lcrzo_kbd_purge(pkbd));

  /* cases which cannot be treated here */
  if (pkbd->type != LCRZO_PRIV_HANDLE_TYPE_CONSOLE) {
    return (lcrzo_priv_kbd_line_echo2(pkbd, pstring));
  }

  return (lcrzo_priv_kbd_line_echo3(pkbd, pstring));
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
