/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_priv_kbddefault_line_noecho(lcrzo_string * pstring)
{
  lcrzo_kbd kbd;

  lcrzo_er(lcrzo_kbd_initdefault(&kbd));
  lcrzo_er(lcrzo_priv_kbd_line_noecho(&kbd, pstring));
  lcrzo_er(lcrzo_kbd_close(&kbd));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_kbd_line_noecho2(lcrzo_kbd * pkbd,
                                lcrzo_string * pstring);
int lcrzo_priv_kbd_line_noecho2(lcrzo_kbd * pkbd,
                                lcrzo_string * pstring)
{
  lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  if (pkbd->echokeypresses) {
    lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_FALSE));
    ret = lcrzo_kbd_readm_line(pkbd, LCRZO_WAIT_INFINITE, &data, &datasize);
    lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_TRUE));
  } else {
    ret = lcrzo_kbd_readm_line(pkbd, LCRZO_WAIT_INFINITE, &data, &datasize);
  }

  if (ret == LCRZO_ERR_OK) {
    data[datasize] = '\0';
    if (pstring == NULL) {
      lcrzo_er(lcrzo_data_free2(&data));
    } else {
      *pstring = (lcrzo_string) data;
    }
  }
  return (ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_kbd_line_noecho3(lcrzo_kbd * pkbd,
                                lcrzo_string * pstring);
int lcrzo_priv_kbd_line_noecho3(lcrzo_kbd * pkbd,
                                lcrzo_string * pstring)
{
  char *ptr;
  lcrzo_int32 fgetstep, ptrallocsize, userstringsize;
  int c;

  fgetstep = 80;
  ptrallocsize = fgetstep;
  lcrzo_er(lcrzo_string_alloc(ptrallocsize, &ptr));

  userstringsize = 0;
#if defined LCRZODEF_SYSTEM_Unix
  {
    lcrzo_bool changeecho;
    lcrzo_bool resettc;
    struct termios old_termios, new_termios;

    changeecho = LCRZO_FALSE;
    if (pkbd->echokeypresses) {
      lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_FALSE));
      changeecho = LCRZO_TRUE;
    }
    /* ensures kbd is in ICANON (for example this is not the case
       with Solaris) */
    resettc = LCRZO_FALSE;
    if (tcgetattr(pkbd->fd, &old_termios))
      return (LCRZO_ERR_FUTCGETATTR);
    if (!(new_termios.c_lflag & ICANON)) {
      new_termios = old_termios;
      new_termios.c_lflag |= ICANON;
      if (tcsetattr(pkbd->fd, TCSANOW, &new_termios))
        return (LCRZO_ERR_FUTCSETATTR);
      resettc = LCRZO_TRUE;
    }
    while (LCRZO_TRUE) {
      c = getc(stdin);
      if (c == 0x0D || c == 0x0A || c == EOF)
        break;
      ptr[userstringsize] = (char)c;
      userstringsize++;
      if (userstringsize == ptrallocsize) {
        ptrallocsize += fgetstep;
        lcrzo_er(lcrzo_string_realloc(ptrallocsize, &ptr));
      }
    }
    /* restore attributes */
    if (resettc) {
      if (tcsetattr(pkbd->fd, TCSANOW, &old_termios))
        return (LCRZO_ERR_FUTCSETATTR);
    }
    if (changeecho) {
      lcrzo_er(lcrzo_kbd_set_echo(pkbd, LCRZO_TRUE));
    }
  }
#elif defined LCRZODEF_SYSTEM_Windows
  while (LCRZO_TRUE) {
    BOOL err;

    c = _getch();
    if (c == 0x0D || c == 0x0A)
      break;
    if (c == 0x08) {
      /* backspace */
      if (userstringsize)
        userstringsize--;
    } else if (c == 0x00) {
      /* ignore Function-i, but get the second part */
      _getch();
    } else if (c == 0xE0) {
      /* ignore arrow, but get the second part */
      _getch();
    } else if (c == 0x03) {
      /* on control-c, generate the event (then the program should exit) */
      err = GenerateConsoleCtrlEvent(CTRL_C_EVENT, 0);
      if (!err) {
        lcrzo_er(lcrzo_string_free2(&ptr));
        return (LCRZO_ERR_FUGENERATECONSOLECTRLEVENT);
      }
    } else {
      ptr[userstringsize] = (char)c;
      userstringsize++;
      if (userstringsize == ptrallocsize) {
        ptrallocsize += fgetstep;
        lcrzo_er(lcrzo_string_realloc(ptrallocsize, &ptr));
      }
    }
  }
  lcrzo_er(lcrzo_priv_err_purge_int());
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
  ptr[userstringsize] = '\0';
  puts("");

  if (pstring != NULL) {
    *pstring = ptr;
  } else {
    lcrzo_er(lcrzo_string_free2(&ptr));
  }

  return (LCRZO_ERR_OK);
  pkbd = pkbd;
}

/*-------------------------------------------------------------*/
int lcrzo_priv_kbd_line_noecho(lcrzo_kbd * pkbd,
                               lcrzo_string * pstring)
#if defined LCRZODEF_SYSTEM_Unix
{
  lcrzo_er(lcrzo_kbd_purge(pkbd));

  /* cases which cannot be treated here */
  if (pkbd->type != LCRZO_PRIV_HANDLE_TYPE_CONSOLE) {
    return (lcrzo_priv_kbd_line_noecho2(pkbd, pstring));
  }
  if (pkbd->fd != 0) {
    return (lcrzo_priv_kbd_line_noecho2(pkbd, pstring));
  }

  lcrzo_er(lcrzo_priv_kbd_line_noecho3(pkbd, pstring));

  return (LCRZO_ERR_OK);
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  lcrzo_er(lcrzo_kbd_purge(pkbd));

  /* cases which cannot be treated here */
  if (pkbd->type != LCRZO_PRIV_HANDLE_TYPE_CONSOLE) {
    return (lcrzo_priv_kbd_line_noecho2(pkbd, pstring));
  }

  return (lcrzo_priv_kbd_line_noecho3(pkbd, pstring));
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
