/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
typedef struct {
  lcrzo_uint32 firstprocess;
  lcrzo_uint32 *pchildprocesses;
  lcrzo_unix_signalhandler pfunc_sigint;
  lcrzo_unix_signalhandler pfunc_sigterm;
} lcrzo_priv_unix_signalhandler;
lcrzo_priv_unix_signalhandler lcrzo_global_signalhandler;

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_child_killall2(int num);
int lcrzo_priv_unix_signalhandler_child_killall2(int num)
{
  lcrzo_uint32 i, nbitems;

  nbitems = lcrzo_global_signalhandler.pchildprocesses[0];

  for (i = 1; i <= nbitems; i++) {
    if (lcrzo_global_signalhandler.pchildprocesses[i] != 0) {
#if LCRZO_DEBUG_UNIX_SIGNALHANDLER==1
      printf("I(%d) kill process ", getpid());
      lcrzo_er(lcrzo_uint32_print("",
                                  lcrzo_global_signalhandler.
                                  pchildprocesses[i], "%u", " at position "));
      lcrzo_er(lcrzo_uint32_print("", i, "%u", "\n"));
#endif
      kill(lcrzo_global_signalhandler.pchildprocesses[i], num);
    }
  }
  errno = 0;                    /* BSD sets ESRCH if it does not find a pid */
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
void lcrzo_priv_unix_signalhandler_sigreceived(int num);
void lcrzo_priv_unix_signalhandler_sigreceived(int num)
{
  int ret;
  lcrzo_unix_signalhandler pfunc_sig;
  lcrzo_bool wanttoresume;

#if LCRZO_DEBUG_UNIX_SIGNALHANDLER==1
  printf("SIG%d received by pid %d\n", num, getpid());
#endif

  /* obtain the internal function */
  pfunc_sig = LCRZO_UNIX_SIGNALHANDLER_DFL;
  if (num == SIGINT) {
    pfunc_sig = lcrzo_global_signalhandler.pfunc_sigint;
  } else if (num == SIGTERM) {
    pfunc_sig = lcrzo_global_signalhandler.pfunc_sigterm;
  }

  /* "Since libc6, signal uses BSD semantics and the default
     behaviour is not reset when the signal is raised. You may
     use sysv_signal to get SysV semantics." (Linux signal manpage)
     So set again signal to have portable code. */
  if (pfunc_sig == LCRZO_UNIX_SIGNALHANDLER_IGN) {
    /* do nothing */
    signal(num, lcrzo_priv_unix_signalhandler_sigreceived);
    return;
  } else if (pfunc_sig == LCRZO_UNIX_SIGNALHANDLER_DFL) {
    /* we end the process, so we continue */
  } else {
    wanttoresume = LCRZO_FALSE; /*is case not set by user */
    ret = (*pfunc_sig) (num, &wanttoresume);
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_priv_err_print_noglo(ret);
    if (wanttoresume) {
      signal(num, lcrzo_priv_unix_signalhandler_sigreceived);
      return;
    }
  }

  /* kill childs */
  ret = lcrzo_priv_unix_signalhandler_child_killall2(num);
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);

  /* we cannot debug memory leaks everytime, because we do not
     know the state of process before been killed. */
  if (num == SIGINT) {
    ret = lcrzo_debug_memleak_report();
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_priv_err_print_noglo(ret);
    ret = lcrzo_debug_memcorrupt_check_all();
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_priv_err_print_noglo(ret);
    ret = lcrzo_debug_memcorrupt_close();
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_priv_err_print_noglo(ret);
    ret = lcrzo_debug_memleak_close();
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_priv_err_print_noglo(ret);
    ret = lcrzo_debug_resfdopen_check();
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_priv_err_print_noglo(ret);
  }

  /* to be sure every message is displayed */
  fflush(stdout);
  fflush(stderr);

  raise(SIGKILL);
  return;
}

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_init(void)
{
  lcrzo_global_signalhandler.firstprocess = 0;
  lcrzo_global_signalhandler.pchildprocesses = NULL;
  lcrzo_global_signalhandler.pfunc_sigint = LCRZO_UNIX_SIGNALHANDLER_DFL;
  lcrzo_global_signalhandler.pfunc_sigterm = LCRZO_UNIX_SIGNALHANDLER_DFL;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_firstprocess_init(void)
{
  lcrzo_global_signalhandler.firstprocess = getpid();
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_firstprocess_kill(void)
{
  int firstpid;

  firstpid = lcrzo_global_signalhandler.firstprocess;
  if (firstpid != 0) {
    if (firstpid != getpid()) {
      kill(firstpid, SIGTERM);
    }
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#define LCRZO_PRIV_SIGNALHANDLER_CHILD_STEP 100
int lcrzo_priv_unix_signalhandler_child_init(void)
{
  lcrzo_uint32 allocsize, i;

  /* eventually free data from previous process */
  if (lcrzo_global_signalhandler.pchildprocesses != NULL) {
    free(lcrzo_global_signalhandler.pchildprocesses);
  }
  /* alloc new zone */
  allocsize =
    (1 + LCRZO_PRIV_SIGNALHANDLER_CHILD_STEP) * sizeof(lcrzo_uint32);
  lcrzo_global_signalhandler.pchildprocesses =
    (lcrzo_uint32 *) malloc(allocsize);
  if (lcrzo_global_signalhandler.pchildprocesses == NULL) {
    return (LCRZO_ERR_FUMALLOC);
  }
  /* indicates the nbitems */
  lcrzo_global_signalhandler.pchildprocesses[0] =
    LCRZO_PRIV_SIGNALHANDLER_CHILD_STEP;
  /* set the items to 0 */
  for (i = 1; i <= LCRZO_PRIV_SIGNALHANDLER_CHILD_STEP; i++) {
    lcrzo_global_signalhandler.pchildprocesses[i] = 0;
  }
  /* set signal handlers */
  signal(SIGINT, lcrzo_priv_unix_signalhandler_sigreceived);
  signal(SIGTERM, lcrzo_priv_unix_signalhandler_sigreceived);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_child_add(lcrzo_uint32 processnumber)
{
  lcrzo_uint32 i, nbitems, allocsize;

  /* first try to find a hole */
  nbitems = lcrzo_global_signalhandler.pchildprocesses[0];
  for (i = 1; i <= nbitems; i++) {
    if (lcrzo_global_signalhandler.pchildprocesses[i] == 0) {
      lcrzo_global_signalhandler.pchildprocesses[i] = processnumber;
#if LCRZO_DEBUG_UNIX_SIGNALHANDLER==1
      printf("I(%d) add process ", getpid());
      lcrzo_er(lcrzo_uint32_print("", processnumber, "%u", " at position "));
      lcrzo_er(lcrzo_uint32_print("", i, "%u", "\n"));
#endif
      return (LCRZO_ERR_OK);
    }
  }

  /* if we are here, we need to allocate more space */
  allocsize =
    (1 + nbitems +
     LCRZO_PRIV_SIGNALHANDLER_CHILD_STEP) * sizeof(lcrzo_uint32);
  lcrzo_global_signalhandler.pchildprocesses =
    (lcrzo_uint32 *) realloc(lcrzo_global_signalhandler.pchildprocesses,
                             allocsize);
  if (lcrzo_global_signalhandler.pchildprocesses == NULL) {
    return (LCRZO_ERR_FUREALLOC);
  }
  lcrzo_global_signalhandler.pchildprocesses[0] =
    nbitems + LCRZO_PRIV_SIGNALHANDLER_CHILD_STEP;
  for (i = nbitems + 1; i <= nbitems + LCRZO_PRIV_SIGNALHANDLER_CHILD_STEP;
       i++) {
    lcrzo_global_signalhandler.pchildprocesses[i] = 0;
  }
#if LCRZO_DEBUG_UNIX_SIGNALHANDLER==1
  printf("I(%d) add more space from ", getpid());
  lcrzo_er(lcrzo_uint32_print("", nbitems + 1, "%u", " to "));
  lcrzo_er(lcrzo_uint32_print("",
                              nbitems + LCRZO_PRIV_SIGNALHANDLER_CHILD_STEP,
                              "%u", "\n"));
#endif
  /* set processnumber */
  lcrzo_global_signalhandler.pchildprocesses[nbitems + 1] = processnumber;
#if LCRZO_DEBUG_UNIX_SIGNALHANDLER==1
  printf("I(%d) add process ", getpid());
  lcrzo_er(lcrzo_uint32_print("", processnumber, "%u", " at position "));
  lcrzo_er(lcrzo_uint32_print("", nbitems + 1, "%u", "\n"));
#endif
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_child_del(lcrzo_uint32 processnumber)
{
  lcrzo_uint32 i, nbitems;

  /* we ignore if process is not found (because this function might
     be called twice with lcrzo_process_other_wait_end) */
  nbitems = lcrzo_global_signalhandler.pchildprocesses[0];
  for (i = 1; i <= nbitems; i++) {
    if (lcrzo_global_signalhandler.pchildprocesses[i] == processnumber) {
      lcrzo_global_signalhandler.pchildprocesses[i] = 0;
#if LCRZO_DEBUG_UNIX_SIGNALHANDLER==1
      printf("I(%d) remove process ", getpid());
      lcrzo_er(lcrzo_uint32_print("", processnumber, "%u", " at position "));
      lcrzo_er(lcrzo_uint32_print("", i, "%u", "\n"));
#endif
    }
  }
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_child_killall(void)
{
  lcrzo_er(lcrzo_priv_unix_signalhandler_child_killall2(SIGTERM));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_set(int signumber,
                                      lcrzo_unix_signalhandler sh)
{
  if (signumber == SIGINT) {
    lcrzo_global_signalhandler.pfunc_sigint = sh;
  } else if (signumber == SIGTERM) {
    lcrzo_global_signalhandler.pfunc_sigterm = sh;
  } else {
    return (LCRZO_ERR_SPUNIXSIGHANDLERBADSIG);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_unix_signalhandler_get(int signumber,
                                      lcrzo_unix_signalhandler * psh)
{
  lcrzo_unix_signalhandler sh;

  if (signumber == SIGINT) {
    sh = lcrzo_global_signalhandler.pfunc_sigint;
  } else if (signumber == SIGTERM) {
    sh = lcrzo_global_signalhandler.pfunc_sigterm;
  } else {
    return (LCRZO_ERR_SPUNIXSIGHANDLERBADSIG);
  }

  if (psh != NULL)
    *psh = sh;
  return (LCRZO_ERR_OK);
}
