/* This file is part of Lcrzo.
*/

int lcrzo_priv_wait_fd_read(int fd,
                            lcrzo_data * pstoreddata,
                            lcrzo_int32 * pstoreddatasize)
{
  lcrzo_int32 readsize;
  lcrzo_data ptr;

  lcrzo_er(lcrzo_data_alloc(LCRZO_PRIV_READMAXMSGSIZE, &ptr));

  /* read */
#if defined LCRZODEF_SYSTEM_Unix
  readsize = read(fd, ptr, LCRZO_PRIV_READMAXMSGSIZE);
#elif defined LCRZODEF_SYSTEM_Windows
  readsize = _read(fd, ptr, LCRZO_PRIV_READMAXMSGSIZE);
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

  /* an error occured */
  if (readsize == -1) {
    lcrzo_er(lcrzo_data_free2(&ptr));
    if (errno == EAGAIN) {
      /* nothing was available */
      errno = 0;
      return (LCRZO_ERR_OKTEMPDATAEND);
    } else if (errno == EBADF) {
      /* reached end */
      errno = 0;
      return (LCRZO_ERR_OKDATAEND);
    } else {
      /* fatal error */
      return (LCRZO_ERR_FUREAD);
    }
  }

  /* if we read 0, we are at end of file */
  if (readsize == 0) {
    lcrzo_er(lcrzo_data_free2(&ptr));
    return (LCRZO_ERR_OKDATAEND);
  }

  /* put read data in stored data */
  lcrzo_er(lcrzo_priv_storeddata_append(ptr, readsize, pstoreddata,
                                        pstoreddatasize));
  lcrzo_er(lcrzo_data_free2(&ptr));

  return (LCRZO_ERR_OK);
}
