/* This file is part of Lcrzo.
*/

int lcrzo_priv_wait_handle_wait(LCRZOHANDLE thehandle,
                                lcrzo_int16 handletype,
                                lcrzo_time * pmaxwaitingtime,
                                lcrzo_bool * pevent)
#if defined LCRZODEF_SYSTEM_Unix
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  thehandle = thehandle;
  pmaxwaitingtime = pmaxwaitingtime;
  handletype = handletype;
  pevent = pevent;
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  DWORD err, msec;
  lcrzo_bool event;
  DWORD nbevent;
  INPUT_RECORD ir[50];
  DWORD n, i;
  DWORD readbytes, bytesavail;

  event = LCRZO_FALSE;
  switch (handletype) {
    case LCRZO_PRIV_HANDLE_TYPE_FILE:
      /* it's a filehandle for a disk file, for which there is
         only data, or endoffile. So there is no need to wait. */
      event = LCRZO_TRUE;
      break;

    case LCRZO_PRIV_HANDLE_TYPE_UNKNOWN:
      /* wait during the specified time */
      lcrzo_er(lcrzo_priv_wait_winmsec_init_time(pmaxwaitingtime, &msec));
      err = WaitForSingleObject(thehandle, msec);
      if (err == WAIT_FAILED)
        return (LCRZO_ERR_FUWAITFORSINGLEOBJECT);
      /* no data to read */
      if (err == WAIT_TIMEOUT)
        event = LCRZO_FALSE;
      else if (err == WAIT_OBJECT_0)
        event = LCRZO_TRUE;
      else
        return (LCRZO_ERR_FUWAITFORSINGLEOBJECT);
      break;

    case LCRZO_PRIV_HANDLE_TYPE_CONSOLE:
      /* wait during the specified time */
      lcrzo_er(lcrzo_priv_wait_winmsec_init_time(pmaxwaitingtime, &msec));
      err = WaitForSingleObject(thehandle, msec);
      if (err == WAIT_FAILED)
        return (LCRZO_ERR_FUWAITFORSINGLEOBJECT);
      /* no data to read */
      if (err == WAIT_TIMEOUT)
        event = LCRZO_FALSE;
      else if (err == WAIT_OBJECT_0)
        event = LCRZO_TRUE;
      else
        return (LCRZO_ERR_FUWAITFORSINGLEOBJECT);
      if (event) {
        /* check there is really data */
        err = GetNumberOfConsoleInputEvents(thehandle, &nbevent);
        if (err) {
          /*printf("nbevent=%d\n", nbevent); */
          err = PeekConsoleInput(thehandle, ir, 50, &n);
          if (!err)
            return (LCRZO_ERR_FUPEEKCONSOLEINPUT);
          event = LCRZO_FALSE;
          for (i = 0; i < n; i++) {
            /*printf("Type=%d\t", ir[i].EventType);
              printf("c=%c=%02x b=%d\n", ir[i].Event.KeyEvent.uChar.AsciiChar,
              ir[i].Event.KeyEvent.uChar.AsciiChar,
              ir[i].Event.KeyEvent.bKeyDown); */
            if (ir[i].EventType == KEY_EVENT && ir[i].Event.KeyEvent.bKeyDown) {
              event = LCRZO_TRUE;
              break;
            }
          }
        }
      }
      break;

    case LCRZO_PRIV_HANDLE_TYPE_PIPE:
      err =
        PeekNamedPipe(thehandle, NULL, 0xFFFF, &readbytes, &bytesavail, NULL);
      if (!err) {
        /* end event */
        event = LCRZO_TRUE;
      } else {
        if (readbytes || bytesavail)
          event = LCRZO_TRUE;
      }
      break;

    default:
      return (lcrzo_ieuv("lcrzo_priv_wait_handle_wait", "handletype"));
  }

  if (pevent != NULL)
    *pevent = event;
  return (LCRZO_ERR_OK);
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
