/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
int lcrzo_priv_wait_kbd_wait(lcrzo_kbd * pkbd,
                             int waitaction,
                             lcrzo_time * pmaxwaitingtime,
                             lcrzo_bool * pevent)
#if defined LCRZODEF_SYSTEM_Unix
{
  struct termios old_termios, new_termios;
  lcrzo_bool resettc;
  int ret;

  resettc = LCRZO_FALSE;
  if (pkbd->type == LCRZO_PRIV_HANDLE_TYPE_CONSOLE &&
      waitaction != LCRZO_PRIV_WAIT_ACTION_READ_LINE) {
    resettc = LCRZO_TRUE;
    /* change attributes */
    if (tcgetattr(pkbd->fd, &old_termios))
      return (LCRZO_ERR_FUTCGETATTR);
    new_termios = old_termios;
    new_termios.c_lflag &= (~ICANON);
    new_termios.c_cc[VMIN] = 1; /* need at least one char */
    new_termios.c_cc[VTIME] = 0;        /* wait forever */
    /* Only the first access to fd can print the char. So, we cannot
       use ~ECHO here and ECHO in function lcrzo_priv_wait_kbd_read.
       So, if user wants to echo the character, it has to be done here.
       So, we do not change the bit ECHO of c_lflag which already
       corresponds to pkbd->echokeypresses. Then, in read function,
       we'll set it to ~ECHO (otherwise might print the char twice). */
    /* set the new value */
    if (tcsetattr(pkbd->fd, TCSANOW, &new_termios))
      return (LCRZO_ERR_FUTCSETATTR);
  }
  if (pkbd->type == LCRZO_PRIV_HANDLE_TYPE_CONSOLE
      && waitaction == LCRZO_PRIV_WAIT_ACTION_READ_LINE) {
    /* ensures kbd is in ICANON (for example this is not the case
       with Solaris) */
    if (tcgetattr(pkbd->fd, &old_termios))
      return (LCRZO_ERR_FUTCGETATTR);
    if (!(new_termios.c_lflag & ICANON)) {
      new_termios = old_termios;
      new_termios.c_lflag |= ICANON;
      if (tcsetattr(pkbd->fd, TCSANOW, &new_termios))
        return (LCRZO_ERR_FUTCSETATTR);
      resettc = LCRZO_TRUE;
    }
  }

  /* wait */
  ret = lcrzo_priv_fd_wait(pkbd->fd, pmaxwaitingtime, pevent);

  /* restore attributes */
  if (resettc) {
    if (tcsetattr(pkbd->fd, TCSANOW, &old_termios))
      return (LCRZO_ERR_FUTCSETATTR);
  }

  return (ret);
}
#elif defined LCRZODEF_SYSTEM_Windows
{
  lcrzo_er(lcrzo_priv_wait_handle_wait(pkbd->thehandle, pkbd->type,
                                       pmaxwaitingtime, pevent));
  return (LCRZO_ERR_OK);
  /* just for the compiler warning */
  waitaction = waitaction;
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
