/* This file is part of Lcrzo.
*/

int lcrzo_process_current_exit(int returnvalue)
{
  int ret;

#if defined LCRZODEF_SYSTEM_Unix
  lcrzo_priv_globalvars *ptr;

  lcrzo_uint8(*int2uint8) (int retval);

  ret = lcrzo_priv_unix_signalhandler_child_killall();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);

  ret = lcrzo_priv_globalvars_ptr_get(&ptr);
  if (ret != LCRZO_ERR_OK) {
    int2uint8 = NULL;
    /*ignore */ lcrzo_priv_err_print_noglo(ret);
  } else {
    int2uint8 = ptr->pprocess_int2uint8;
  }

  ret = lcrzo_priv_globalvars_close();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);
#elif defined LCRZODEF_SYSTEM_Windows
  ret = lcrzo_close_forthreadonly();    /* this function calls lcrzo_priv_globalvars_close */
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

  /* debug memory leaks */
  ret = lcrzo_debug_memleak_report();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);
  ret = lcrzo_debug_memcorrupt_check_all();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);
  ret = lcrzo_debug_memcorrupt_close();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);
  ret = lcrzo_debug_memleak_close();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);

  /* debug open descriptors */
  ret = lcrzo_debug_resfdopen_check();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);

  /* to be sure every message is displayed */
  fflush(stdout);
  fflush(stderr);

#if defined LCRZODEF_SYSTEM_Unix
  if (int2uint8 == NULL)
    _exit(returnvalue);
  _exit((*int2uint8) (returnvalue));
#elif defined LCRZODEF_SYSTEM_Windows
  ret = lcrzo_debug_globalvars_close_process();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_priv_err_print_noglo(ret);
  _endthreadex(returnvalue);
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

  return (lcrzo_ierp("lcrzo_process_current_exit"));
}
