/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Unix
#elif defined LCRZODEF_SYSTEM_Windows

typedef struct {
  int (*pfunc) (lcrzo_ptr infos,
                lcrzo_int32 infossize,
                lcrzo_process * pparentprocess);
  lcrzo_data infos;
  lcrzo_int32 infossize;
  lcrzo_process parentprocess;
  lcrzo_priv_globalvars *globalptr;
} lcrzo_priv_process_funcinfos;

unsigned __stdcall lcrzo_priv_process_other_init(lcrzo_ptr pfuncinfos);
unsigned __stdcall lcrzo_priv_process_other_init(lcrzo_ptr pfuncinfos)
{
  lcrzo_priv_process_funcinfos *plppfi =
    (lcrzo_priv_process_funcinfos *) pfuncinfos;
  lcrzo_priv_globalvars *gloptr;
  int ret;

  /* Developper note : we should ensure that the global environnement is not
     changed in the father before the end of the copy (by using an IPC blocking
     the father). It should be done in next versions. */
  ret = lcrzo_priv_globalvars_copy(plppfi->globalptr);
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_process_current_exit(ret);

  /* for memory debug */
  ret = lcrzo_debug_globalvars_init_process();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_process_current_exit(ret);
  ret = lcrzo_debug_memleak_init();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_process_current_exit(ret);
  ret = lcrzo_debug_memcorrupt_init();
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_process_current_exit(ret);
  /* this memory might have been set in previous thread,
     and we do not want to detect it as free of unallocated memory */
  ret = lcrzo_priv_globalvars_ptr_get(&gloptr);
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_process_current_exit(ret);
  if (gloptr->errmsglcrzo != NULL) {
    ret = lcrzo_debug_memleak_add((lcrzo_data) gloptr->errmsglcrzo);
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_process_current_exit(ret);
    ret = lcrzo_debug_memcorrupt_add((lcrzo_data) gloptr->errmsglcrzo);
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_process_current_exit(ret);
  }
  if (gloptr->errmsgsys != NULL) {
    ret = lcrzo_debug_memleak_add((lcrzo_data) gloptr->errmsgsys);
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_process_current_exit(ret);
    ret = lcrzo_debug_memcorrupt_add((lcrzo_data) gloptr->errmsgsys);
    if (ret != LCRZO_ERR_OK)    /*ignore */
      lcrzo_process_current_exit(ret);
  }

  /*  launch the child */
  ret = (*(plppfi->pfunc)) (plppfi->infos, plppfi->infossize,
                            &(plppfi->parentprocess));
  if (ret != LCRZO_ERR_OK)      /*ignore */
    lcrzo_process_current_exit(ret);

  /*ignore */ lcrzo_process_current_exit(ret);
  /* never reached */
  return (lcrzo_ierp("lcrzo_priv_process_other_init"));
}
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif


/*-------------------------------------------------------------*/
int lcrzo_process_other_init2(int (*pfunc) (lcrzo_ptr infos,
                                            lcrzo_int32 infossize,
                                            lcrzo_process * pparentprocess),
                              lcrzo_ptr infos,
                              lcrzo_int32 infossize,
                              lcrzo_uint8(*punixreturn_int2uint8) (int ret),
                              int (*punixreturn_uint82int) (lcrzo_uint8 ret),
                              lcrzo_process * pchildprocess)
{
  /* parameter verification */
  if (pfunc == NULL)
    return (LCRZO_ERR_PANULLPTR);
  if (pchildprocess == NULL)
    return (LCRZO_ERR_PANULLPTR);

#if defined LCRZODEF_SYSTEM_Unix
  {
    lcrzo_process currentprocess;

    lcrzo_er(lcrzo_process_current_init(&currentprocess));
    pchildprocess->threadid = 0;
    pchildprocess->threadhandle = 0;
    /* prepare for return of the child */
    if (punixreturn_uint82int != NULL) {
      pchildprocess->punixreturn_uint82int = punixreturn_uint82int;
    } else {
      pchildprocess->punixreturn_uint82int = &lcrzo_process_uint82int_std;
    }
    /* create child */
    pchildprocess->processid = fork();
    if ((int)pchildprocess->processid < 0)
      return (LCRZO_ERR_FUFORK);
    if (pchildprocess->processid == 0) {
      lcrzo_priv_globalvars *ptr;
      int ret;

      /* debug memory leaks */
      ret = lcrzo_debug_memleak_close();
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      ret = lcrzo_debug_memcorrupt_close();
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      ret = lcrzo_debug_memleak_init();
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      ret = lcrzo_debug_memcorrupt_init();
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      /* change the return politics */
      ret = lcrzo_priv_globalvars_ptr_get(&ptr);
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      if (punixreturn_int2uint8 != NULL)
        ptr->pprocess_int2uint8 = punixreturn_int2uint8;
      else
        ptr->pprocess_int2uint8 = &lcrzo_process_int2uint8_std;
      /* this memory might have been set in previous process,
         and we do not want to detect it as free of unallocated memory */
      if (ptr->errmsglcrzo != NULL) {
        ret = lcrzo_debug_memleak_add((lcrzo_data) ptr->errmsglcrzo);
        if (ret != LCRZO_ERR_OK)        /*ignore */
          lcrzo_process_current_exit(ret);
        ret = lcrzo_debug_memcorrupt_add((lcrzo_data) ptr->errmsglcrzo);
        if (ret != LCRZO_ERR_OK)        /*ignore */
          lcrzo_process_current_exit(ret);
      }
      if (ptr->errmsgsys != NULL) {
        ret = lcrzo_debug_memleak_add((lcrzo_data) ptr->errmsgsys);
        if (ret != LCRZO_ERR_OK)        /*ignore */
          lcrzo_process_current_exit(ret);
        ret = lcrzo_debug_memcorrupt_add((lcrzo_data) ptr->errmsgsys);
        if (ret != LCRZO_ERR_OK)        /*ignore */
          lcrzo_process_current_exit(ret);
      }
      ret = lcrzo_debug_memleak_add((lcrzo_data) ptr);
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      ret = lcrzo_debug_memcorrupt_add((lcrzo_data) ptr);
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      /* reinit random seed */
      ret = lcrzo_rand_init(0);
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      /* prepare for killing its own childs */
      ret = lcrzo_priv_unix_signalhandler_child_init();
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      /* call the main function */
      ret = (*pfunc) (infos, infossize, &currentprocess);
      if (ret != LCRZO_ERR_OK)  /*ignore */
        lcrzo_process_current_exit(ret);
      /*ignore */ lcrzo_process_current_exit(ret);
      /* never reached */
    }
    /* never reached by child */
    lcrzo_er(lcrzo_priv_unix_signalhandler_child_add
             (pchildprocess->processid));
  }
#elif defined LCRZODEF_SYSTEM_Windows
  {
    lcrzo_priv_process_funcinfos *plppfi;
    unsigned long h;
    unsigned t;

    /* launch process */
    lcrzo_er(lcrzo_data_alloc(sizeof(lcrzo_priv_process_funcinfos),
                              &(pchildprocess->pfuncinfos)));
    plppfi = (lcrzo_priv_process_funcinfos *) (pchildprocess->pfuncinfos);
    plppfi->pfunc = pfunc;
    lcrzo_er(lcrzo_data_initm_data(infos, infossize, &(plppfi->infos),
                                   &(plppfi->infossize)));
    lcrzo_er(lcrzo_process_current_init(&(plppfi->parentprocess)));
    lcrzo_er(lcrzo_priv_globalvars_ptr_get(&(plppfi->globalptr)));
    h = _beginthreadex(NULL, 0, lcrzo_priv_process_other_init, plppfi, 0, &t);
    pchildprocess->threadid = t;
    /* never reached by child */
    if (h == 0)
      return (LCRZO_ERR_FUCREATETHREAD);
    pchildprocess->threadhandle = (HANDLE) h;

    /* set process id */
    pchildprocess->processid = _getpid();
  }
#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif

  /* for return value */
  pchildprocess->hasended = LCRZO_FALSE;
  lcrzo_debug_notset_int(pchildprocess->returnedvalue);

  return (LCRZO_ERR_OK);
  /* just for the compiler warning */
  punixreturn_int2uint8 = NULL;
  punixreturn_uint82int = NULL;
}
