/* This file is part of Lcrzo.
*/

/*-------------------------------------------------------------*/
typedef struct {
  LCRZOHANDLE thehandle;
  lcrzo_bool useex;
} lcrzo_priv_handle_event;

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#if defined LCRZODEF_SYSTEM_Unix

int lcrzo_wait_init_handle_event(LCRZOHANDLE thehandle,
                                 lcrzo_bool useex,
                                 lcrzo_wait * pwait)
{
  return (LCRZO_ERR_IEUNIMPLEMENTED);
  thehandle = thehandle;
  useex = useex;
  pwait = pwait;
}

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#elif defined LCRZODEF_SYSTEM_Windows

int lcrzo_priv_wait_init_handle_event(lcrzo_constptr infos,
                                      lcrzo_int32 infossize,
                                      lcrzo_time * pmaxwaitingtime,
                                      lcrzo_bool * phasevent);
int lcrzo_priv_wait_init_handle_event(lcrzo_constptr infos,
                                      lcrzo_int32 infossize,
                                      lcrzo_time * pmaxwaitingtime,
                                      lcrzo_bool * phasevent)
{
  DWORD err, msec;
  lcrzo_bool event;
  lcrzo_priv_handle_event *phe = (lcrzo_priv_handle_event *) infos;

  if (infossize != sizeof(lcrzo_priv_handle_event))
    return (LCRZO_ERR_IEINTERNALERROR);
  if (infos == NULL)
    return (LCRZO_ERR_PANULLPTR);

  /* wait during the specified time */
  lcrzo_er(lcrzo_priv_wait_winmsec_init_time(pmaxwaitingtime, &msec));
  if (phe->useex) {
    err = WaitForSingleObjectEx(phe->thehandle, msec, TRUE);
  } else {
    err = WaitForSingleObject(phe->thehandle, msec);
  }
  if (err == WAIT_FAILED)
    return (LCRZO_ERR_FUWAITFORSINGLEOBJECT);

  /* check error */
  event = LCRZO_FALSE;
  if (err == WAIT_TIMEOUT)
    event = LCRZO_FALSE;
  else if (err == WAIT_OBJECT_0)
    event = LCRZO_TRUE;
  else if (err == WAIT_ABANDONED)
    event = LCRZO_TRUE;
  else if (err == WAIT_IO_COMPLETION && phe->useex)
    event = LCRZO_TRUE;
  else
    return (LCRZO_ERR_FUWAITFORSINGLEOBJECT);

  if (phasevent != NULL)
    *phasevent = event;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_wait_init_handle_event(LCRZOHANDLE thehandle,
                                 lcrzo_bool useex,
                                 lcrzo_wait * pwait)
{
  lcrzo_priv_handle_event he;

  he.thehandle = thehandle;
  he.useex = useex;
  lcrzo_er(lcrzo_wait_init_function_event(&lcrzo_priv_wait_init_handle_event,
                                          (lcrzo_constdata) & he, sizeof(he),
                                          pwait));
  return (LCRZO_ERR_OK);
}

#else
#error "Unknown value for LCRZODEF_SYSTEM"
#endif
